# $Header:   //Archive/PVCSROOT/Archive_NLS/Mgmt/HMSQueries.ksv   1.32   Jul 24 2001 14:30:38   seirlbec  $

# Default variables

# Base registry path for profile subkeys
PROFILE_BASE_KEY="HKEY_LOCAL_MACHINE\\SOFTWARE\\Accessible Software, Inc.\\Access1\\1.0\\Health Monitor\\Profiles\\"

# Base path for shell scripts
INSTALL_DIR=$(REG QUERY "SOFTWARE\Accessible Software, Inc.\Access1\1.0\InstallDirectory" | cut -f3)
# Making sure there is a '\' on the end
INSTALL_DIR=$(echo $INSTALL_DIR | awk '/[^\]$/ { $0 = $0 "\\" ; print $0 ; next} // { print $0 }')
BASE_DIR="$INSTALL_DIR"Mgmt\\HMS\\ShellScripts\\

# The local host name, converted to all caps
localhost=$(hostname | tr '[a-z]' '[A-Z]')

typeset -i nCycleInterval=60
typeset -i nNumCycles=0

export nNumCycles

# Root directory
ROOTDIR=//$localhost/HMS

# Required component host names
DBHost=       # Central database host
MonitorHost=  # Host running the component monitoring services
MVSHost=      # Host running MVS listeners and event handlers
PAHost=       # Host running Propagation Agents

# Database login parameters
DBUser=sa
DBPassword=

LastDBBlocked=1 # Keeps track of the last status of the GetDBBlocks function

test=0    # If 1 then test by outputting the configuration parameters and exiting
verbose=0 # If 1 then verbose messages are generated to standard output during run

# Prints output if verbose option is set, otherwise swallows it.
vprint()
{
  now=$(date +"%Y/%m/%d %H:%M:%S")
  print -R >${Input}LASTACTION.TXT $nNumCycles "| " $now "| " $*
  if [ $verbose = 0 ]; then return; fi
  print -u2 -R >&2 $nNumCycles "| " $now "| " $*
}

Usage()
{
  print "USAGE:"
  print "  HMSQueries <options>"
  print "  The following options are supported:"
  print "    -k<profile-key>"
  print "        Required registry profile key that defines the environment being"
  print "        monitored. This is a subkey of "
  print "            HKEY_LOCAL_MACHINE"
  print "              SOFTWARE"
  print "                Accessible Software, Inc."
  print "                  Access1"
  print "                    1.0"
  print "                      Health Monitor"
  print "                        Profiles"
  print "    -t  Test mode. Prints configuration variables and exits"
  print "    -v  Verbose mode. Status messages are sent to stdout during run."
}

# Defines a variable that must be set
typeset -i nRequiredVariables=0
DefineRequiredVariable()
{
  var=$1
  nRequiredVariables=$[$nRequiredVariables+1]
  REQUIRED_VARIABLES[$nRequiredVariables]=$var
}

# Array of variable names that must be specified in the configuration profile
DefineRequiredVariable DBHost
DefineRequiredVariable DBLogDir
DefineRequiredVariable DBUser
DefineRequiredVariable DBPassword
DefineRequiredVariable DBQueryTimeout
DefineRequiredVariable DBLoginTimeout
DefineRequiredVariable Input
DefineRequiredVariable Output
DefineRequiredVariable PAHost
DefineRequiredVariable MVSHost
DefineRequiredVariable MonitorHost
DefineRequiredVariable MonitoredServicesProfile

CheckRequiredVariable()
{
  if [ -z "$(eval echo \$$1)" ]; then
    print >&2 "Required variable $1 not set"
    exit 1
  fi
}

CheckRequiredVariables()
{
  typeset -i nVars=${#REQUIRED_VARIABLES[*]}
  typeset -i i=0
  while [ $i -lt $nVars ]; do
    varName=${REQUIRED_VARIABLES[$i]}
    CheckRequiredVariable $varName
    i=$i+1
  done
}

DumpVariables()
{
  for varName in ${REQUIRED_VARIABLES[*]}; do
    echo "$varName=$(eval echo \$$varName)"
  done

  echo "Required Hosts:"
  for host in ${RequiredHosts[*]}; do
    echo "    $host"
  done
}

LastSQLServerRestarts()
{
  print "#FileName|StartTime"
  print "#-40s|-30s"
  for f in "//$DBHost/$DBLogDir"/errorlog*; 
  do 
  {
    head -1 $f | awk -v f=$f '{printf("%s|%s %s\n", f, $1, $2)}'; 
  }
  done;
}

DiskUsage()
{
	host=$1
	ASIGetDiskSpace -S$host | awk '{gsub("%", " %");print;}'
}

DiskUsageAll()
{
  for host in ${RequiredHosts[*]}; do
  	DiskUsage $host
  done
}

CalcDiskUsageAll()
{
  print "#Host|Drive|Free(Mb)|Total(Mb)|Used(%)|UColor"
  print "#-15s |-6s |8s |8s |7s |-8s"
  DiskUsageAll | awk '{
    free_mb = $3 / 1024;
    size_mb = $7 / 1024;
    percent_used = 100 - ((free_mb * 100) / size_mb);

    if (percent_used >= 80 && percent_used < 90)
      color="YELLOW"
    else if (percent_used >= 90)
      color="RED"
    else
      color="GREEN"

    printf("%s|%s|%d|%d|%d|%s\n", $1, $2, free_mb, size_mb, percent_used, color);
  }' | sort -t'|' -r -k5 -n
}

# Gets the summary of space usage by all databases on the main database server. 
# List is sorted in order of decreasing severity and then by percent usage.
# Databases are considered YELLOW at 70% usage, RED at 90%.
GetAllDBSpaceUsed()
{
  #This is the new method created because of a transition to SQL Server 7.0

  ${ISQL} -Q"asisp_dbusage @delim='|'" | awk 'NR>2&&NF>0&&$4!="NULL" {
    sub("^[ ]", "", $0);
    sub("[ ]*$", "", $0);
    print $0;
  }'
}

QueryNTSchedulerInfo()
{
  at \\\\$DBHost | awk -v RS="[\012\015]" '{print}'
}

GetMonitorScriptsOutput()
{
  for f in //$MonitorHost/mgmt$/Logs/Monitor*`date +"%Y%m%d"`.txt;
  do
  {
    if [ -f $f ]; then
      line=`tail -30 $f | grep ProcessID | tail -1`
      echo $f: $line;
    fi
  }
  done;
}

QueryServiceStatus()
{
	Host=$1
	Service=$2
  svcStatFile="${Input}svcStat.txt"
	sc \\\\$Host query $Service | grep STATE | awk '{printf("%s|%s", $3,$4);}' > "$svcStatFile"
	if [ -s "$svcStatFile" ]; then
		print "$Host|$Service|"$(cat "$svcStatFile")
	else
		print "$Host|$Service|0|???"
	fi
	rm -f "$svcStatFile"
}

CheckForOpenTran()
{
	opentranfile="$Input"DBOpenTran.txt

	${ISQL} -Q"dbcc opentran (Object)" > "$opentranfile"
	grep -q "No active open transactions" "$opentranfile"
	status=$?
	if [ "$status" -eq "0" ]; then
		# no open transactions
		echo "No active open transactions"
	else
		spid=$(grep SPID "$opentranfile" | awk '{print $3}')
		startTime=`grep "Start Time" $opentranfile | awk '{print $4" "$5" "$6" "$7}'`
		echo "SPID="$spid"|BlockingSince="$startTime
		echo

		sql="set nocount on select spid, hostname, program_name, cmd from sysprocesses where spid="$spid" print '' dbcc inputbuffer ("$spid")"
		${ISQL} -Q"$sql"
	fi

  rm -f "$opentranfile"
}

PostProcessSQLResults()
{
  fmt=$1;
  awk -v fmt="$fmt" '
  NF>0&&NR==1 {printf("#"); for (i=1;i<NF;i++) printf("%s|", $i); printf("%s\n", $NF);}
  NR==2 {
    print fmt;
    nFields=NF;
    for (i=1; i<=nFields; i++) {
      flen[i] = length($i);
    }
  }
  NF>0&&NR>2  {
    offset=2;
    for (i=1;i<nFields;i++) {
      field=substr($0, offset, flen[i]);
      sub("^[ ]*", "", field);
      sub("[ ]*$", "", field);
      offset += flen[i] + 1;
      printf("%s|", field);
    }
    field=substr($0, offset, flen[nFields]);
    sub("^[ ]*", "", field);
    sub("[ ]*$", "", field);
    printf("%s\n", field);
  }'
}

GetDBLockSummary()
{
      # We are trying to fix the null problem...
	${ISQL} -Q"sp_locksummaryalldbs" | TR '\000' ' ' | PostProcessSQLResults "#5s |-13s|4s |-23s|-11s|12s |-17s|-19s"
}

GetDBLocks()
{
	# We are trying to fix the null problem...
	${ISQL} -dObject -Q"exec sp_showlocks" |	TR '\000' ' ' | PostProcessSQLResults "#6s |-31s|11s |-16s |-16s|-30s"
}

GetDBBlocks()
{
	${ISQL} -Q"sp_showblocks" | grep -v 'row.*affected' | PostProcessSQLResults "#-5s|-8s|-12s|-13s|-16s" | awk -v OutDir="$Input" -v LastDBBlocked=$LastDBBlocked 'NR==1 { printf("%s|COLOR\n", $0); }
    NR==2 { printf("%s|-8s\n", $0); }
    NR>2  { 
      if (NR==3)
        if (LastDBBlocked==1)
          LastDBBlocked = 2;
        else
          LastDBBlocked = 3;
      if (LastDBBlocked == 2) color = "YELLOW";
      else color = "RED";
      printf("%s|%s\n", $0, color);
    }
    END { 
      if (NR<3)
        LastDBBlocked = 1;
      statfile=sprintf("%sLastDBBlocked.txt", OutDir);
      print LastDBBlocked > statfile;
    }'
    LastDBBlocked=$(cat ${Input}LastDBBlocked.txt)
}

DumpDBQueues()
{
  ${ISQL} -Q"set nocount on exec asisp_dumpAllQueues '|'" | awk 'NR>2&&NF>0 {sub("^[ ]",""); sub("[ ][ ]*$",""); print $0;}'
}

GetSQLResponseTimes()
{  
  # If there is a problem with this, make sure that the server is updating the table via
  # an automatic procedure.

    ${ISQL} -Q"exec asisp_testsqlresponsetimes @exec=0, @delim='|'" | egrep '^[^|]*\|[^|]*(\|[^|]*)+$' | awk '// {
      sub("^[ ]", "", $0);
      sub("[ ]*$", "", $0);
      print $0;
    }'
}

GetTablesWithManyRows()
{
	${ISQL} -dObject -Q"asisp_printLargeTables" | awk '
  NF>0&&(NR==1||NR>2) {if (NR==1) printf("#"); for (i=1;i<NF;i++) printf("%s|", $i); printf("%s\n", $NF);}
  NR==2 {print "#-30s |9s |17s"}'
}

StagedEventStatus()
{
  ${ISQL} -Q"set nocount on exec asisp_stagedeventstatus @exec=1, @delim='|'" | awk 'NR>2&&NF>0 {sub("^[ ]",""); sub("[ ][ ]*$",""); print $0;}'
}

MonitorRequiredServices()
{
  print "#Name|Host|Service|CurrentState|SColor"
  print "#-34s|-11s |-21s |-11s|-8s"
  regdmp "$PROFILE_BASE_KEY$MonitoredServicesProfile" | awk '{
    while (1) {
      if ($0 ~ /^    [A-Za-z]/) { # start of service
        display_name=$0;
        setting["Host"]="";
        setting["Service"]="";
        setting["DesiredState"]="";
        getline;
        while ($0 ~ /^        [A-Za-z]/) {
          setting[$1] = $3;
          getline;
        }
        printf("display_name=\"%s\";host=%s;service=%s;desiredstate=%s\n", display_name, setting["Host"], setting["Service"], setting["DesiredState"]);
      }
      else
        if (getline != 1)
          break;
    }
  }' > MonitorRequiredServices.lst
  exec 4< MonitorRequiredServices.lst
  while read line <&4; do
    # reset variables
    display_name=
    host=
    service=
    currentstate=
    desiredstate=

    # evaluate variables for next service
    eval $line;

    # If host is the special value "UNUSED" then skip
    if [ "$host" = "UNUSED" ]; then
      continue
    fi

    # if any of the required settings are missing, assign a current state of ???
    if [ -z "$host" ]; then
      currentstate="BAD_HOST"
    fi
    if [ -z "$service" ]; then
      currentstate="BAD_SERVICE"
    fi
    if [ -z "$desiredstate" ]; then
      currentstate="BAD_DESIREDSTATE"
    fi

    # Query state of service
    if [ -z "$currentstate" ]; then
      currentstate=$(sc '\\'$host query $service | grep 'STATE.*:' | awk '{print $4}')
      if [ -z "$currentstate" ]; then
        currentstate="QUERY_FAILED"
      fi
    fi

    # Compute color by comparing current and desired states
    if [ $currentstate = $desiredstate ]; then
      color="GREEN";
    elif [ $currentstate = START_PENDING -a $desiredstate = RUNNING ]; then
      color="YELLOW";
    elif [ $currentstate = STOP_PENDING -a $desiredstate = STOPPED ]; then
      color="YELLOW";
    else
      color="RED";
    fi

    # Output service status
    print "$display_name|$host|$service|$currentstate|$color"
  done
}

GetMVSStatus()
{
  if [ $verbose = 1 ];then
    vopt=-v
  else
    vopt=
  fi

  "$BASE_DIR"MVSStatus -H $MVSHost -K $MonitorHost $vopt
}

GetPAgentStatus()
{
  "$BASE_DIR"PAgentStatus -H $PAHost
}

GetSQLExecTaskInfo()
{
  ${ISQL} -Q"sp_printScheduledTasks"
}

GetTECImportStatus()
{
  #We need to check if the TEC is set to a valid value before calling the
  #script to report it's status
  if [ $TECImportHost ] && [ $TECImportHost != "UNUSED" ]; then
    "$BASE_DIR"TECImportStatus -U$TECImportUser -P$TECImportPassword -S$TECImportHost
  fi
}

SectionBreak()
{
	title=$1
	echo
	echo
	echo "**************************************************************************"
	echo "*" $title
	echo "**************************************************************************"
}

typeset -i nSections=0
DefineSection()
{
  function=$1
  message=$2

  nSections=$[$nSections+1]
  Sections[$nSections]="$function|$message"
}

ExecSection()
{
  typeset -i sectionNo=$1
  SectionDef=${Sections[$sectionNo]}
  function=${SectionDef%|*}
  message=${SectionDef#*|}

  vprint "$message"
  $function > "$Input"$function.txt
}

SummarizeSection()
{
  typeset -i sectionNo=$1
  SectionDef=${Sections[$sectionNo]}
  function=${SectionDef%|*}
  message=${SectionDef#*|}

  SectionBreak "$message"
  "$BASE_DIR"pprint "$Input"$function.txt
}

SummarizeAll()
{
  typeset TimeStamp="$1"
	SectionBreak "Begin Time Stamp"
  echo "$TimeStamp"

  # Summarize all of the findings
  typeset -i i=1
  while [ $i -le $nSections ]; do
    SummarizeSection $i
    i=$[$i+1]
  done

	SectionBreak "End Time Stamp"
  date +"%Y/%m/%d %H:%M:%S"
}

DefineSection DumpVariables           "Outputting the configuration variables..."
DefineSection CheckForOpenTran        "Checking for open transaction..."
DefineSection GetDBBlocks             "Getting the database blocks..."
DefineSection GetDBLockSummary        "Getting the database lock summary..."
DefineSection GetDBLocks              "Getting the database locks..."
DefineSection GetSQLResponseTimes     "Getting SQL response times..."
DefineSection GetTablesWithManyRows   "Listing large tables..."
DefineSection QueryNTSchedulerInfo    "Getting NT Scheduler info..."
DefineSection GetMonitorScriptsOutput "Getting monitor script output..."
DefineSection CalcDiskUsageAll        "Getting disk usage statistics from hosts..."

#Commenting this out because it currently does not work correctly and we don't really
# care anyways.
#DefineSection GetAllDBSpaceUsed       "Getting database space usage statistics..."

DefineSection StagedEventStatus       "Getting database staged event statistics..."
DefineSection DumpDBQueues            "Getting database queue statistics..."
DefineSection MonitorRequiredServices "Obtaining status of all required services..."
DefineSection GetMVSStatus            "Getting MVS components status..."
DefineSection GetPAgentStatus         "Getting PAgent status..."
DefineSection GetSQLExecTaskInfo      "Getting SQLExecutive scheduled task information..."
DefineSection LastSQLServerRestarts   "Getting list of recent SQLServer Restarts..."

#Commenting this out because currently we don't care to see about this.
#DefineSection GetTECImportStatus      "Getting TEC Import Status..."

cycle()
{
  typeset StartTime;
  StartTime=$(date +"%Y/%d/%m %H:%M:%S")

  echo $StartTime > "$Input"HMSQueries.inprogress

  # Execute each defined section
  typeset -i i=1
  while [ $i -le $nSections ]; do
    ExecSection $i
    i=$[$i+1]
  done

  vprint "Summarizing findings into ${Input}Total_Summary.txt"
  SummarizeAll "$StartTime" > "$Input"Total_Summary.txt

  # Copying output file to contain timestamp
  vprint $(cp -v "$Input"Total_Summary.txt "$Output"Total_Summary`date +"%Y%m%d_%H%M%S"`".TXT")

  rm -f "$Input"HMSQueries.inprogress
}

# Usage: ConfigFromRegKey <key>
# Reads configuration assignments from the specified registry key. The full registry path used
# is $PROFILE_BASE_KEY\<key>.
ConfigFromRegKey()
{
  settingsFile="${Input}settings.$$.txt"
  profile_key=$PROFILE_BASE_KEY${*}
  vprint ConfigFromRegKey \"$profile_key\"
  rm -f ConfigFromRegKey.err
  regdmp "$profile_key" | awk 'function parse_setting() {
    if ($0 ~ "REG_DWORD") { # Convert hex value to decimal using dc
      sub("REG_DWORD 0x", "");
      val=toupper($3);
      cmd="echo 16i " val " Aop|dc";
      cmd | getline decimal_val;
      $3=decimal_val;
      close(cmd);
    }
    if (prefix != "") {
      repl=prefix "&";
      sub("[A-z]", repl, $1);
    }
    if (NF == 2)
      sub(" =", "=\"\"");
    env[$1] = $3;
    print $0;

    if (inhosts) {
      Host=toupper($3);
      # Add to unique list of hosts
      if (HaveHost[Host]==0) {
        RequiredHosts[++nHosts]=Host;
        HaveHost[Host]=1;
      }
    }
  }
  {
    #line below is necessary to remove trailing garbage appended to key names with security settings specified
    gsub(" \[[0-9 ]*\]", "", $0)

    if ($0 == "    DB" ||
        $0 == "    Notify" ||
        $0 == "    RemoteBackup" ||
        $0 == "    TECImport") {
      prefix=$1;
      inhosts=0;
    }
    else if ($0 == "    Hosts") {
      prefix="";
      inhosts=1;
    }
    else if ($0 ~ "^    [A-z]") {
      prefix="";
      inhosts=0;
    }
    if ($0 !~ /=/)
      continue;

    parse_setting();
  }
  END {
    for (i=1; i<=nHosts; i++) {
      Host=RequiredHosts[i];
      printf("RequiredHosts[%d] = %s\n", i-1, Host);
    }
  }' | sed 's/ = \(.*\)/='\''\1'\''/g' > $settingsFile

  if [ -f ConfigFromRegKey.err ]; then
    vprint $(cat ConfigFromRegKey.err)
    exit 1
  fi
  exec 4<$settingsFile
  while read setting<&4; do
    eval "$setting"
  done

  rm -f "$settingsFile"
  vprint "ConfigFromRegKey finished"
}

#
# main
#

# Process command line
while getopts hi:o:k:tv c; do
  case $c in
  i)  Input=$OPTARG;;
  o)  Output=$OPTARG;;
  k)  ConfigFromRegKey "$OPTARG";;
  t)  test=1;;
  v)  verbose=1;;
  h|?)  Usage; exit 0;;
  *)  Usage >&2; exit 1;;
  esac
done

# Make sure we have a good input directory
if [ -z "$Input" ]; then
  if [ ! -d "$ROOTDIR" ]; then
    print >&2 "Required directory \"$ROOTDIR\" does not exist"
    exit 1
  fi
  Input="$ROOTDIR/Input/"
fi
if [ ! -d "$Input" ]; then
  mkdir "$Input"
  if [ $? != 0 ]; then
    exit 1
  fi
fi
# Make sure $Input ends with a '/'
echo "$Input" | grep -q '/$'
if [ $? != 0 ]; then
  Input="$Input/"
fi

# Make sure we have a good output directory
if [ -z "$Output" ]; then
  if [ ! -d "$ROOTDIR" ]; then
    print >&2 "Required directory \"$ROOTDIR\" does not exist"
    exit 1
  fi
  Output="$ROOTDIR/Output/"
fi
if [ ! -d "$Output" ]; then
  mkdir "$Output"
  if [ $? != 0 ]; then
    exit 1
  fi
fi
# Make sure $Output ends with a '/'
echo "$Output" | grep -q '/$'
if [ $? != 0 ]; then
  Output="$Output/"
fi

# Set trap to cleanup process-specific files on exit
trap "rm -f HMSQueries.pid; exit" EXIT

echo $$ > HMSQueries.pid

# If in test mode then dump variables and exit
if [ $test = 1 ]; then
  DumpVariables
  CheckRequiredVariables
  exit
fi

# Make sure required variables are set
CheckRequiredVariables

# Set up standard command line for isql
export ISQL="isql -S$DBHost -U$DBUser -P$DBPassword -t$DBQueryTimeout -l$DBLoginTimeout -n -w3000"

# Loop forever
while true; do 
  cycle;
  vprint "Sleeping for $nCycleInterval sec."
  nNumCycles=$nNumCycles+1
  sleep $nCycleInterval;
done
