# $Header:   //archive/PVCSROOT/archive_nls/Mgmt/PAgentStatus.ksv   1.6   Jul 25 2001 16:39:16   seirlbec  $

# PAgentStatus.ksh

# Reports the status of all PAgents defined on a specified host to the standard output.
# Each line is the status of an agent, formatted as follows:
#   name|id|pid|pcolor|start_time|events/pid|entries|ecolor|max|enqueues|dequeues|spid|scolor
# where
#   name            is the name of the PAgent
#   id              is the unique id of the PAgent, represented as a 4 digit hex value
#   pid             is the last recorded process id of the PAgent
#   pcolor          is the alert color for the pid column
#   start_time      is the last recorded start time of the PAgent
#   events/pid      is the total number of events processed by the most current run
#   entries         is the current number of entries on the PAgent's queue
#   ecolor          is the alert color for the entries column
#   max             is the maximum number of entries the PAgent's queue can have
#   enqueue_age     is the amount of time in HH:MM:SS format since the last enqueue
#   dequeue_age     is the amount of time in HH:MM:SS format since the last dequeue
#   dcolor          is the alert color for the dequeue_age column
#   spid            is the database system process id for the agent
#   scolor          is the alert color for the spid column

# Base path for shell scripts
INSTALL_DIR=$(REG QUERY "SOFTWARE\Accessible Software, Inc.\Access1\1.0\InstallDirectory" | cut -f3 | sed 's/\\/\//g')
# Making sure there is a '\' on the end
INSTALL_DIR=$(echo $INSTALL_DIR | awk '/[^\/]$/ { $0 = $0 "/" ; print $0 ; next} // { print $0 }')
BASE_DIR="$INSTALL_DIR"Mgmt/HMS/ShellScripts/

Usage()
{
  print "Usage: PAgentStatus {-U<user>} {-P<password>} {-S<server>} {-H<host>}"
}

GetPAProcessIDs()
{
	${ISQL} -Q"set nocount on
    select convert(binary(2),id), spid
     from PropagationAgent_C,
       link l,
       sysprocesses s
     where id=dst_id
       and dst_cid='PA'
       and l.deleted=0
       and 'PA '+name*=s.program_name" | awk '/0x/ {sub("0x","",$1);printf("%s %s\n", toupper($1), $2);}'
}

#
# main
#

User=sa
QueryTimeout=60
LoginTimeout=30

while getopts H:U:S:P: c; do
  case $c in
  U)  User=$OPTARG;;
  S)  Server=$OPTARG;;
  P)  Password=$OPTARG;;
  H)  host=$OPTARG;;
  *)  Usage; exit 1;;
  esac
done

# If being called standalone then get the database parameters.
if [ -z "$ISQL" ]; then
  # Set up standard command line for isql
  ISQL="isql -S$Server -U$User -P$Password -t$QueryTimeout -l$LoginTimeout -n -w3000"
fi

if [ -z "$host" ]; then
  host=$(hostname)
fi

# Get the database process ids
GetPAProcessIDs > PAProcessIDs.txt

regdmp -m '\\'$host 'HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Propagation Agents' | awk 'function HexToDecimal(hex) {
  sub("0x", "", hex);
  val=toupper(hex);
  cmd="echo 16i " val " Aop|dc";
  cmd | getline val;
  close(cmd);
  return val;
}
function DisplaySeconds(secs)
{
  hours = int(secs / 3600);
  hfactor = hours*3600;
  minutes = int((secs - hfactor) / 60);
  seconds = secs - hfactor - minutes*60;
  r=sprintf("%02d:%02d:%02d", hours, minutes, seconds);
  return r;
}
/^    Instances/ {
  # We are in the block of instances defining the agents
  getline;
  while ($0 ~ /^        /) {
    name=$1;
    getline;
    while ($0 ~ /^            /) {
      if ($1 == "PropagationAgentID") {
        sub("0x0000", "", $4);
        id= $4;
      }
      else if ($1 == "ProcessID")
        pid= HexToDecimal($4);
      else if ($1 == "Time" && $2 == "Started")
        start_time  = $4 " " $5;
      else if ($1 == "NumEventsProcessed")
        num_events = HexToDecimal($4);
      getline;
    }
    id = toupper(id);
    AgentID[++nAgents] = id;
    AgentName[id] = name;
    AgentPID[id] = pid;
    AgentStartTime[id] = start_time;
    AgentNumEvents[id] = num_events;
  }
}
END {
  # Get the list of actual ASIPAgents running on the host
  cmd = "rkill /v " host " | grep -i asipagent";
  while (cmd | getline) {
    pids[++npids] = $2;
  }

  #full command for fqueuestat
  fqueuecomm = BASE_DIR "fqueuestat"

  # Get the queue stats
  cmd = sprintf("%s //%s/queues$/*.que| sed \"s/.*\([0-9A-F][0-9A-F][0-9A-F][0-9A-F]\)[.][Qq][Uu][Ee]/\1/\"", fqueuecomm, host);
  while (cmd | getline) {
    split($0, stats, "|");
    id = stats[1];
    MaxEntries[id] = stats[3];
    EnqueueCount[id] = stats[6];
    DequeueCount[id] = stats[7];
    entries = stats[9];
    FileEntries[id] = entries;
    EnqueueAge[id] = stats[13];
    dequeue_age = stats[14];
    DequeueAge[id] = dequeue_age;
    # If there are entries on the queue check dequeue threshold. 
    # Hardcoded to YELLOW at 1 minute, RED at three minutes
    if (entries > 0) {
      if (dequeue_age > 180)
        DequeueColor[id] = "RED";
      else if (dequeue_age > 60)
        DequeueColor[id] = "YELLOW";
      else
        DequeueColor[id] = "GREEN";
    }
    else
      DequeueColor[id] = "GREEN";
  }
  close(cmd);

  # Get the DB process status
  while (getline < "PAProcessIDs.txt") {
    id=$1;
    SPID[id]=$2;
  }

  print "#name|id|pid|pcolor|start_time|events/pid|entries|ecolor|max|enqueue_age|dequeue_age|dcolor|spid|scolor";
  print "#-9s|5s|5s |-6s |-17s|9s|6s |-6s |6s|12s|12s |-6s |4s |-4s";
  for (i = 1; i <= nAgents; i++) {
    id = AgentID[i];
    name = AgentName[id];

    pid = AgentPID[id];
    pid_color = "RED";
    for (j = 1; j <= npids; j++)
      if (pids[j] == pid) {
        pid_color = "GREEN";
        break;
      }

    start_time = AgentStartTime[id];
    num_events = AgentNumEvents[id];
    max_entries = MaxEntries[id];
    enqueue_age = DisplaySeconds(EnqueueAge[id]);
    dequeue_age = DisplaySeconds(DequeueAge[id]);
    dequeue_color = DequeueColor[id];
    entries = FileEntries[id];
    entries_color = "GREEN";

    spid = SPID[id];
    if (length(spid) == 0)
      spid_color = "RED";
    else
      spid_color = "GREEN";

    print name          \
      "|" id            \
      "|" pid           \
      "|" pid_color     \
      "|" start_time    \
      "|" num_events    \
      "|" entries       \
      "|" entries_color \
      "|" max_entries   \
      "|" enqueue_age   \
      "|" dequeue_age   \
      "|" dequeue_color \
      "|" spid          \
      "|" spid_color    ;
  }
}' BASE_DIR=$BASE_DIR host=$host
