#
# $Header:   //ARCHIVE/Archive_NLS/scripts/SendMVSRegisterObjects.ksv   1.16.1.3   Jul 24 2001 14:12:00   BRENT  $ 
#

delay_seconds=0;
bProcessBatchOnly=0;
bProcessMJOBsOnly=0;
os_id=NULL
reg_unreg_control_statments='';  # default to registering objects

# Name of program used to execute SQL commands
SQLCMD=${SQLCMD:-osql}

usage	()
{
  print ''
	print 'Usage: '$0' -c<cid> -i<id> [-b] [-m] [-u] [-o<os_id>] [-d<delay_seconds>]'
  print ''
  print '  Required Param(s):'
  print '  -c<cid>                 Object''s ClassID'
  print '  -i<id>                  Object''s ID'
  print ''
  print '  Optional Param(s):'
  print '  -b                      SysPlex batch jobs only'
  print '  -m                      MVS jobs only.  No SysPlex batch jobs.'
  print '  -u                      Unregister the object(s)'
  print '  -o<os_id>               OS Object''s ID to limit output to that OS'
  print '  -d<delay_seconds>       Number of seconds before command is executed (default='$delay_seconds')'
  print ''
  print '  Sample:'
  print '    '$0' -cCOMP -i2'
}


# Process input parameters
while getopts "c:i:bmuo:d:?" opt; do
	case $opt in
	c)  cid=$OPTARG;;
	i)  id=$OPTARG;;
	o)  os_id=$OPTARG;;
	b)  bProcessBatchOnly=1;;
  m)  bProcessMJOBsOnly=1;;
	u)  reg_unreg_control_statments='47OFF';;
	d)  delay_seconds=$OPTARG;;
	\?) usage
		  exit 16
		  ;;
	esac
done
shift $(($OPTIND - 1))

# Check for required params
typeset -i bMissingParam=0;
if [ "$cid" = "" ];         then echo 'ERROR: Missing required param -c<cid>'; bMissingParam=1; fi
if [ "$id" = "" ];          then echo 'ERROR: Missing required param -i<id>'; bMissingParam=1; fi
if  [ $bMissingParam -eq 1 ]; then
	usage
	exit 17
fi

# check for conflicting parameters
if [ $bProcessBatchOnly -eq 1 -a $bProcessMJOBsOnly -eq 1 ]; then;
  echo 'ERROR: -m and -b options are mutually exclusive.'
  usage
  exit 18
fi

generate_control_statements_BATC()
{
  # create output filename(s)
  bGenerateMasterOSList=$1;
  output_file="$2"
  osql_output_file=$output_path'/MVSRegisterObjects.'$timestamp'_'$$.BATCH.ISQL.OUT.TXT
  osql_temp_output_file=$output_path'/MVSRegisterObjects.'$timestamp'_'$$.BATCH.TEMP.ISQL.OUT.TXT 

  # generate control statements (BATC)
  osql_cmd='asisp_uploadGetKeyBatchJobs '$cid', '$id', '$os_id
  $SQLCMD -S$db_server -U$db_user -P$db_password -Q"$osql_cmd" -w5000 -o"$osql_output_file" -u

  # convert unicode output into UTF8
  utf2ucs -r < $osql_output_file > $osql_temp_output_file
  mv $osql_temp_output_file $osql_output_file

  awk -F"|" -v bGenerateMasterOSList=$bGenerateMasterOSList -v output_file="$output_file" -v master_control_statement=$master_control_statement '
    function getMsgDescriptions(cid, id)
    {
      s="";
      # special case(s) as DB does not have support
      #if (obj_cid == "IMS")
      #  return "45Z\\46IEF403I\\45Z\\46IEF404I";

      key=sprintf("%s-%d", obj_cid, obj_id);
      if (!g_msgdV[key])
        return ""
      n=split(g_msgdV[key], msgdV, /\|/);
      for (i=1; i<=n; i++) {
        if (i%2==1) {
          # first time through the loop and every other after
          if (i==1)
            s=sprintf("%s", s);   # first time through the loop
          else
            s=sprintf("%s\\", s); # line break every other time
          s=sprintf("%s45Z\\46%s", s, msgdV[i]);
        }
        else
          s=sprintf("%s\\45Z\\46%s", s, msgdV[i]);
      }
      return s;
    }
    /OS_ROW/ {
      # OS_ROW|OS|3|ZAZ0|Z8ACCES|4K2XILKJ|
      # OS_ROW|OS|24|ZBZ0|Z8ACCES|4K2XILKJ|
      #
      # generate the master_control_statement file
      # only if only processing BATCH jobs
      if (bGenerateMasterOSList==1) {
        split($4,os_name,/ /);
        print $3"|"os_name[1]"|"output_file >> master_control_statement;
      }
    }
    /MSGD_ROW/ {
      # MSGD_ROW|BATC|1813|IEF404I|                                                                                                                                                                                                                                     
      # MSGD_ROW|BATC|1814|IEF403I|                                                                                                                                                                                                                                     
      split($4,msgd,/ /);
      key=sprintf("%s-%d", $2, $3);
      if (g_msgdV[key])
        g_msgdV[key]=sprintf("%s|%s", g_msgdV[key], msgd[1]);
      else
        g_msgdV[key]=msgd[1]
    }
    /DATA_ROW/ {
      # DATA_ROW|7|BATC|6290|JUUPCJDP|                                                                                                                                                                                                                                  
      # DATA_ROW|7|BATC|6291|JUUPCJBJ|                                                                                                                                                                                                                                  
      split($5,job,/ /)
      cno=$2; obj_cid=$3; obj_id=$4; obj_name=job[1]; nativekey=sprintf("%6.6lX%4.4lX", obj_id, cno);

      if ( reg_unreg_control_statments == "" ) {
        MSGD=getMsgDescriptions(obj_cid, obj_id)
        printf("02\\05\\02%s\\03%s\\04%s\\%s~\n", obj_cid, obj_name, nativekey, MSGD) >> output_file;
      }
      else
        printf("02\\05\\02%s\\03%s\\04%s\\%s~\n", obj_cid, obj_name, nativekey, reg_unreg_control_statments) >> output_file;
      fi
    }
  ' "$osql_output_file";

  if [ -f "$osql_output_file" ]; then;
    rm "$osql_output_file";
  fi
}

process_MJOB()
{
  cid=${1:?'ERROR: output_line() - Missing Param <cid>'}
  id=${2:?'ERROR: output_line() - Missing Param <id>'}
  typeset -i last_os_id=0;

  # create output filename(s)
  osql_output_file=$output_path'/MVSRegisterObjects.'$timestamp'_'$$.ISQL.OUT.TXT
  osql_temp_output_file=$output_path'/MVSRegisterObjects.'$timestamp'_'$$.ISQL.TEMP.OUT.TXT

  osql_cmd='asisp_uploadGetOSJobs '$cid', '$id', '$os_id
  $SQLCMD -S$db_server -U$db_user -P$db_password -Q"$osql_cmd" -w5000 -o$osql_output_file -u

  # convert unicode output into UTF8
  utf2ucs -r < $osql_output_file > $osql_temp_output_file
  mv $osql_temp_output_file $osql_output_file

  awk -F"|" -v timestamp=$timestamp -v master_control_statement=$master_control_statement -v pid=$$ -v output_path=$output_path -v reg_unreg_control_statments=$reg_unreg_control_statments '
    BEGIN {
      last_os_id=0;
      last_obj_cid="";
    }
    function getMsgDescriptions(cid, id)
    {
      s="";
      # special case(s) as DB does not have support
      #if (obj_cid == "IMS")
      #  return "45Z\\46IEF403I\\45Z\\46IEF404I";

      key=sprintf("%s-%d", obj_cid, obj_id);
      if (!g_msgdV[key])
        return ""
      n=split(g_msgdV[key], msgdV, /\|/);
      for (i=1; i<=n; i++) {
        if (i%2==1) {
          # first time through the loop and every other after
          if (i==1)
            s=sprintf("%s", s);   # first time through the loop
          else
            s=sprintf("%s\\", s); # line break every other time
          s=sprintf("%s45Z\\46%s", s, msgdV[i]);
        }
        else
          s=sprintf("%s\\45Z\\46%s", s, msgdV[i]);
      }
      return s;
    }

    /MSGD_ROW/ {
      # MSGD_ROW|BATC|1813|IEF404I|                                                                                                                                                                                                                                     
      # MSGD_ROW|BATC|1814|IEF403I|                                                                                                                                                                                                                                     
      split($4,msgd,/ /);
      key=sprintf("%s-%d", $2, $3);
      if (g_msgdV[key])
        g_msgdV[key]=sprintf("%s|%s", g_msgdV[key], msgd[1]);
      else
        g_msgdV[key]=msgd[1]
    }
    /DATA_ROW/ {
      split($3,os,/ /); split($7,job,/ /)
      os_id=$2; os_name=os[1]; cno=$4; obj_cid=$5; obj_id=$6; obj_name=job[1]; nativekey=sprintf("%6.6lX%4.4lX", obj_id, cno);

      # check for new OS
      if (os_id != last_os_id) {
        print "*** NEW Operating System ["os_name"] ***";

        close(output_file);

        # create output filename(s)
        output_file=sprintf("%s/MVSRegisterObjects.%s_%d.CS.%s.%d.TXT", output_path, timestamp, pid, os_name, os_id);
        print os_id"|"os_name"|"output_file >> master_control_statement;

        last_os_id = os_id;
        last_obj_cid = "";
      }

      # check for new obj_cid
      if ( obj_cid != last_obj_cid ) {
        print "----- NEW ClassID ["obj_cid"] -----";
        last_obj_cid = obj_cid
      }

      if ( reg_unreg_control_statments == "" ) {
        MSGD=getMsgDescriptions(obj_cid, obj_id)
        if (obj_cid == "OMEG" || obj_cid == "IMS" || obj_cid == "IDMS")
          obj_cid = "STC";
        else if (obj_cid == "IMSS" || obj_cid == "IMTM" || obj_cid == "IMDB")
          obj_cid = "IMS";
        else if (obj_cid == "CMAS")
          obj_cid = "CICS";
        else if (obj_cid == "D2SS")
          obj_cid = "DB2";
        printf("02\\05\\02%s\\03%s\\04%s\\%s~\n", obj_cid, obj_name, nativekey, MSGD) >> output_file;
      }
      else {
        if (obj_cid == "OMEG" || obj_cid == "IMS" || obj_cid == "IDMS")
          obj_cid = "STC";
        else if (obj_cid == "IMSS" || obj_cid == "IMTM" || obj_cid == "IMDB")
          obj_cid = "IMS";
        else if (obj_cid == "CMAS")
          obj_cid = "CICS";
        else if (obj_cid == "D2SS")
          obj_cid = "DB2";
        printf("02\\05\\02%s\\03%s\\04%s\\%s~\n", obj_cid, obj_name, nativekey, reg_unreg_control_statments) >> output_file;
      }
      fi
    }

  ' $osql_output_file

  if [ -f "$osql_output_file" ]; then;
    rm "$osql_output_file";
  fi
}

main()
{
  timestamp=`date +"%Y%m%d%H%M%S"`

  # Get Database Server, User, Password from Registry
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

  # determine the output directory
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\Log" | awk '/Directory/ {gsub(/\\\\/, "/", $3); printf("output_path=%s\n",$3);}'`
  if [ "$output_path" = "" ]; then;
    if [ "$TEMP" != "" ]; then; output_path=`echo $TEMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMP" != "" ]; then; output_path=`echo $TMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMPDIR" != "" ]; then; output_path=`echo $TMPDIR | awk '{gsub(/\\\\/, "/"); print;}'`;
    fi;
  fi

  # Create Master list of control statement files
  master_control_statement=$output_path'/MVSRegisterObjects.'$timestamp'_'$$.MasterListOfControlStatements.TXT
  if [ -f "$master_control_statement" ]; then
    rm "$master_control_statement"
  fi

  # BATCH: create output filename(s) and generate control statements
  batc_output_file=$output_path'/MVSRegisterObjects.'$timestamp'_'$$.CS.BATC.TXT
  if [ $bProcessMJOBsOnly -eq 0 ]; then;
    generate_control_statements_BATC $bProcessBatchOnly "$batc_output_file"
  fi

  # MJOB: generate control statements (if not processing BATCH-only)
  if [ $bProcessBatchOnly -eq 0 ]; then;
    process_MJOB $cid $id
  fi

  # For each OS, wrap the control statements in JCL and send via FTP
  cat "$master_control_statement" | 
  while read line;
  do
  {
    # identify the OS id and OS control statement file
    eval `echo $line | awk -F"|" '{printf("os_id=%lu;os_name=%s;output_file=%s\n",$1,$2,$3);}'`

    # if only performing BATCH-only processing, create an OS-specific file
    if [ $bProcessBatchOnly -eq 1 ]; then;
      output_file=$output_path'/MVSRegisterObjects.'$timestamp'_'$$.CS.$os_name.$os_id.TXT
    fi;
    
    # concatinate the BATC control statements to the OS control statements
    if [ -f "$batc_output_file" ]; then;
      cat "$batc_output_file" >> "$output_file" 
    fi;

    # create audit_msg file
    audit_msg_file="$output_file".AUDIT.MSG
    echo $cid'|'$id'|ASI_02\\05|Object Registration (02\\05)' > "$audit_msg_file";
    
    # perform the upload
    if [ $delay_seconds -eq 0 ]; then;
      Upload_WrapAndSendAsFTP -cOS -i$os_id -f$output_file
    else
      SOON $delay_seconds sh Upload_WrapAndSendAsFTP -cOS -i$os_id -f$output_file
    fi
  }
  done;
  if [ -f "$master_control_statement" ]; then
    rm "$master_control_statement"
  fi
  if [ -f "$batc_output_file" ]; then;
    rm "$batc_output_file"
  fi;
}

main

