#!/bin/sh
# This script adds a component (product-version) into a LOB Instance (existing)

function usage {
  ASIShellMsg ASILobScripts 9 "$1" >&2
  ASIShellMsg ASILobScripts 10 >&2
  ASIShellMsg ASILobScripts 11 >&2
}

prod=""
ver=""
pprod=""
pver=""
pmatch=""
pname=""
plobid="0"
name=""
server=""
userid=""
password=""
option=""
pattern=""
expression=""
test=""
action="insert"

while getopts p:v:l:w:i:d:S:U:P:o:n:e:t:a:N:V: z
do
  if [ $z = p ]
  then
    prod=$OPTARG
  elif [ $z = v ]
  then
    ver=$OPTARG
  elif [ $z = l ]
  then
    pname=$OPTARG
  elif [ $z = i ]
  then
    plobid=$OPTARG
  elif [ $z = N ]
  then
    pprod=$OPTARG
  elif [ $z = V ]
  then
    pver=$OPTARG
  elif [ $z = o ]
  then
    option=$OPTARG
  elif [ $z = n ]
  then
    pattern=$OPTARG
  elif [ $z = e ]
  then
    expression=$OPTARG
  elif [ $z = t ]
  then
    test=$OPTARG
  elif [ $z = S ]
  then
    server=$OPTARG
  elif [ $z = U ]
  then
    userid=$OPTARG
  elif [ $z = P ]
  then
    password=$OPTARG
  elif [ $z = a ]
  then
    action=$OPTARG
  elif [ $z = w ]
  then
    pmatch=$OPTARG
  elif [ $z = d ]
  then
    name=$OPTARG    
  else
    usage $0
    exit 1
  fi
done

. gemargs.sh

# Invalid when BOTH product & version (as cname can be in one of them) are empty
if [ "x$prod" = x -a "x$ver" = x ] 
then
  usage $0
  exit 1
fi

if [ "x$pprod" = x -a "x$pver" = x ] 
then
  usage $0
  exit 1
fi

if [ "x$server" = x -o "x$userid" = x -o "x$password" = x ] 
then
  usage $0
  exit 1
fi

if [ "x$pname" = x -a "x$pmatch" = x -a "$plobid" = "0" ] 
then
  usage $0
  exit 1
fi

# Check if $plobid is a number
if ! expr "$plobid" : '[0-9][0-9]*$' > /dev/null 2>&1
then
  ASIShellMsg ASILobScripts 12 "$plobid" >&2
  usage $0
  exit 1
fi

if [ "$action" != "insert" -a "$action" != "update" -a "$action" != "delete" ]
then
  action="insert"
fi

memStr=""
if [ "x$option" != x -o "x$pattern" != x -o "x$expression" != x -o "x$test" != x ]
then
  memStr=", @memOptions=N'$option', @memPattern=N'$pattern', @memExpr=N'$expression', @memTest=N'$test'"
fi

# Escape single-quote in $pmatch 
pmatch=$(echo "$pmatch" | sed -e "s/'/''/g")

if ! tbsmsql.exe -n -MCONSOLE -X -w 500 -S "$server" -U "$userid" -P "$password" -d Object -Q "exec gemsp_addpvToLOBInst @product=N'$prod', @version=N'$ver', @pproduct=N'$pprod', @pversion=N'$pver', @action=N'$action', @pmatch=N'$pmatch', @plobname=N'$pname', @plobid=N'$plobid', @name=N'$name'$memStr" -o addpvToLOBInst.out
then
  ASIShellMsg ASILobScripts 2 "tbsmsql.exe - exec gemsp_addpvToLOBInst @product=N'$prod', @version=N'$ver', @pproduct=N'$pprod', @pversion=N'$pver', @action=N'$action', @pmatch=N'$pmatch', @plobname=N'$pname', @plobid=N'$plobid', @name=N'$name'$memStr"
  exit 1     
fi

if [ -f "addpvToLOBInst.out" ]
then
  grep "TBSM.*:" addpvToLOBInst.out
fi 





