#!/bin/ksh
# Adding tasks to a Software Component

function usage {
  ASIShellMsg ASIBldScripts 16 "$1" >&2
  ASIShellMsg ASIBldScripts 33 >&2
}

function inode {
  ls -id "$1" 2>&1 | awk '{ print $1 }'
}

function chkrc {
  if [ $? -ne 0 ]
  then
    ASIShellMsg ASIBldScripts 2 "$1" >&2
    exit 1
  fi
}

function cleanup {
  # Clean up as these inculde files are not needed
  if [ "x$(inode ${GEMDIR}/sql/GemObject.sqi)" != "x$(inode GemObject.sqi)" ]
  then
    rm -rf GemObject.sqi
  fi
  if [ "x$(inode ${GEMDIR}/sql/BusinessObject.sqi)" != "x$(inode BusinessObject.sqi)" ]
  then
    rm -rf BusinessObject.sqi
  fi
 
  # final clean up
  if [ -f "${tllsqi}.initial" ]
  then
    mv "${tllsqi}.initial" "$tllsqi"
    rm -rf "${tllsqi}.initial"
  fi
}

# register the trap to run cleanup at exit
trap cleanup 0

server=""
userid=""
password=""
product=""
version=""
tasklib=""
tllsqi=""
debug="no"

while getopts dS:U:P:p:v:t:f: o
do
  if [ $o = d ]
  then
    debug='yes'
  elif [ $o = S ]
  then
    server=$OPTARG
  elif [ $o = U ]
  then
    userid=$OPTARG
  elif [ $o = P ]
  then
    password=$OPTARG
  elif [ $o = p ]
  then
    product=$OPTARG
  elif [ $o = v ]
  then
    version=$OPTARG
  elif [ $o = t ]
  then
    tasklib=$OPTARG
  elif [ $o = f ]
  then
    tllsqi=$(basename "$OPTARG")
  else
    usage $0
    exit 1
  fi
done

. gemargs.sh

if [ "X$server" = X -o "X$userid" = X -o "X$password" = X -o "X$tllsqi" = X ] 
then
  usage $0
  exit 1
fi

# Check if the file exists and hang on to the inital version
if [ -f "$tllsqi" ]
then
  cp "$tllsqi" "${tllsqi}.initial"
else
  ASIShellMsg ASIBldScripts 17 "$tllsqi"
  exit 1
fi

# Synchronizing nls tables 
echo "
INCREMENT_METADATABASE_VERSION
" >> "$tllsqi"

if [ "X$product" = X -o "X$version" = X ]
then
  if [ "X$tasklib" = X ]
  then
    echo "sh get_cid_tlib.sh -S $server -U $userid -P $password -f $tllsqi"
    sh get_cid_tlib.sh -S "$server" -U "$userid" -P "$password" -f "$tllsqi"
    chkrc "sh get_cid_tlib.sh -S $server -U $userid -P $password -f $tllsqi" 
  else
    echo "sh get_cid_tlib.sh -S $server -U $userid -P $password -t $tasklib -f $tllsqi"
    sh get_cid_tlib.sh -S "$server" -U "$userid" -P "$password" -t "$tasklib" -f "$tllsqi"
    chkrc "sh get_cid_tlib.sh -S $server -U $userid -P $password -t $tasklib -f $tllsqi" 
  fi
else
  if [ "X$tasklib" = X ]
  then
    echo "sh get_cid_tlib.sh -S $server -U $userid -P $password -p $product -v $version -f $tllsqi"
    sh get_cid_tlib.sh -S "$server" -U "$userid" -P "$password" -p "$product" -v "$version" -f "$tllsqi"
    chkrc "sh get_cid_tlib.sh -S $server -U $userid -P $password -p $product -v $version -f $tllsqi" 
  else
    echo "sh get_cid_tlib.sh -S $server -U $userid -P $password -p $product -v $version -t $tasklib -f $tllsqi"
    sh get_cid_tlib.sh -S "$server" -U "$userid" -P "$password" -p "$product" -v "$version" -t "$tasklib" -f "$tllsqi"
    chkrc "sh get_cid_tlib.sh -S $server -U $userid -P $password -p $product -v $version -t $tasklib -f $tllsqi" 
  fi
fi

err1=$(cat $tllsqi | grep {gem_cid})
err2=$(cat $tllsqi | grep {task_lib})

if [ "X$err1" != X -o "X$err2" != X ]
then
  ASIShellMsg ASIBldScripts 18 "$tllsqi" "{gem_cid}" "{task_lib}" >&2 
  exit 1 
fi

if [ $debug = "yes" ]
then
  echo "GEMDIR = $GEMDIR"
  ASIShellMsg ASIBldScripts 3 "GEMDIR"
  ASIShellMsg ASIBldScripts 4
  read thisin
fi

if [ x"${GEMDIR}" = x ]
then 
  ASIShellMsg ASIBldScripts 5 "GEMDIR"
  exit 1
fi

echo "cp ${GEMDIR}/sql/GemObject.sqi .
cp ${GEMDIR}/sql/BusinessObject.sqi ."

cp ${GEMDIR}/sql/GemObject.sqi .
cp ${GEMDIR}/sql/BusinessObject.sqi .

clsql "$tllsqi"
chkrc "clsql $tllsqi" 

mv "${tllsqi}.initial" "$tllsqi"
rm -rf "${tllsqi}.initial"

# Run the sql file generated by clsql upon the Object DB
file=${tllsqi%.sqi}.sql
if [ -f "$file" ]
then
  echo "tbsmsql.exe -MCONSOLE -S $server -U $userid -P $password -d Object -i $file -o ${file}.out" 
  if [ $debug = "yes" ]
  then
    ASIShellMsg ASIBldScripts 4
    read thisin
    ASIShellMsg ASIBldScripts 6
  fi
  tbsmsql.exe -MCONSOLE -S "$server" -U "$userid" -P "$password" -d Object -i "$file" -o "${file}.out"
  chkrc "tbsmsql.exe -MCONSOLE -S $server -U $userid -P $password -d Object -i $file -o ${file}.out" 
fi

# Need to recycle Application Server (but doing all services, just in case) 
# echo "Recycle Tivoli Business System Services....."
# sh gemrecycle.sh recycle

ASIShellMsg ASIBldScripts 7

ASIShellMsg ASIBldScripts 8
