/* REXX */    

datain: arg GUID InFilePath
parse arg GUID InFilePath                    /* get input parameters */

/****  IMPORTANT:  Search for all references to TGMTask command, i.e. 'FIND tgmtask'     ****/ 
/****              and change the following parameters within this command:              ****/
/**** 'tgmtask -u <NVUID> -k <NVPSWD> -d 390 -t 30 -h <tgmtask host> -n <netview applid>'****/
/****                                                                                    ****/
/****  -u  --> replace <NVUID> with a valid UserID for the OS390 NetView host            ****/
/****  -k  --> replace <NVPSWD>with a valid Password assoicated with the NetView UserID  ****/
/****  -h  --> replace <tgmtask host> with the IP address of the TBSM NT Server that has ****/
/****          TGMTask configured to communicate with the OS390 NetView host             ****/
/****  -n  --> replace <netview applid> with the OS390 NetView host APPLID               ****/
                                         
yyyy = SUBSTR( DATE( "S" ), 1, 4 )         
mm   = SUBSTR( DATE( "S" ), 5, 2 )         
dd   = SUBSTR( DATE( "S" ), 7, 2 )         
TODAY   = yyyy||'/'||mm||'/'||dd           
NOWTIME = SUBSTR( TIME( "N" ), 1, 5 )      
fntime  = SUBSTR( TIME( "L" ), 10, 4 )
uidkey = ''
datekey = yyyy||'/'||mm||'/'||dd||NOWTIME
save_enum_value = ''

InFile = InFilePath||GUID||'.in'              /* set path to input file */

OutFile = InFilePath||GUID||'.out'            /* set path for output file */

TGMTaskConfig = '-u <NVUID> -k <NVPSWD> -d 390 -t 30 -h <tgmtask host> -n <netview applid>'

idx = 1
Do While Lines(InFile) > 0                    /* get all input lines */
  Line = Linein(InFile)
  datain.idx = Line
  If substr(datain.idx,1,15) = 'REQUEST_USER_ID' then 
   Do
    uidkey = substr(datain.idx,21,10)           /* save user id */
    uidkey = strip(uidkey)
   End
  If substr(datain.idx,1,17) = 'REQUEST_ENUM_NAME' then
   Do
    save_enum_value = substr(datain.idx,23,40)  /* save enum name value */
    save_enum_value = strip(save_enum_value)    /* remove trailing blanks */
   End
  If substr(datain.idx,1,12) = 'REQUEST_TYPE' then
   Do
    save_type_value = substr(datain.idx,18,40)  /* save type name value */
    save_type_value = strip(save_type_value)    /* remove trailing blanks */
   End
  If substr(datain.idx,1,14) = 'REQUEST_METHOD' then
   Do
    save_method_value = substr(datain.idx,20,40)  /* save method value */
    save_method_value = strip(save_method_value)    /* remove trailing blanks */
   End
  If substr(datain.idx,1,18) = 'REQUEST_OBJECT_CID' then
   Do
    save_cid_value = substr(datain.idx,24,40)  /* save cid value */
    save_cid_value = strip(save_cid_value)    /* remove trailing blanks */
   End 
  If substr(datain.idx,1,17) = 'REQUEST_OBJECT_ID' then
   Do
    save_id_value = substr(datain.idx,23,40)  /* save id value */
    save_id_value = strip(save_id_value)    /* remove trailing blanks */
   End
  If substr(datain.idx,1,19) = 'REQUEST_OBJECT_NAME' then
   Do
    save_module_value = substr(datain.idx,20,40)  /* save module name value */
    save_module_value = strip(save_module_value)    /* remove trailing blanks */
   End
  If substr(datain.idx,1,14) = 'REQUEST_SYSTEM' then
   Do
    save_system_value = substr(datain.idx,20,40)  /* save system name value */
    save_system_value = strip(save_system_value)    /* remove trailing blanks */
   End
  If substr(datain.idx,1,14) = 'REQUEST_QUERY_TYPE' then
   Do
    save_query_type_value = substr(datain.idx,20,40)  /* save query type name value */
    save_query_type_value = strip(save_query_type_value)  /* remove trailing blanks */
   End
  idx = idx + 1			/* get next line */
End    


totrecs = idx                                /* save total records read */
Rc = LineOut(InFile)                         /* close InFile */ 

If save_method_value = 'REQUEST_INITIALIZE' then
 Do
  retdata = 'REQUEST_RETURN_CODE|1|1|0'
  Line = Lineout(OutFile,retdata)            /* put line to file */
  retdata = 'REQUEST_ERROR_MSG|1|1|Request Initialize was successful'
  Line = Lineout(OutFile,retdata)            /* put line to file */
  retdata = 'REQUEST_INITIALIZE|1|1|Co-op Bank'
  Line = Lineout(OutFile,retdata)            /* put line to file */
  Rc = LineOut(OutFile)                      /* close OutFile */   
  exit
 End

If save_enum_value > '' then                 /* check for enumeration value */
 If save_enum_value \= 'REQUEST_GET_UIDS' then
  Do
   call setup_enum_values                     /* format enumumeration values */
   exit
  End

If uidkey = '' | uidkey = ' ' then            /* check for user id */
 uidkey = 'ADMIN'

tasklog = InFilePath||'TSD390TGMTTask'||yyyy||mm||dd||'.log'  

mvsdata = '<'                                   /* format partseq 0000 */ 
mvsdata = mvsdata||'UIDKEY='||uidkey||'|'       /* add uidkey for request part */
mvsdata = mvsdata||'DATEKEY='||datekey||'|'     /* add datekey for request part */
mvsdata = mvsdata||'PARTSEQ='||'0000|'          /* add partseq number */
mvsdata = mvsdata||'TOTPARTS='||totrecs -1||'|' /* add total parts to be sent */ 
mvsdata = mvsdata||'REQUEST_GUID='||InFilePath||GUID||'.out'||'|'   /*add guid*/ 
mvsdata = mvsdata||'>'                          /* end record char */ 

cmd = '"GTMIMAPI "'||' '||mvsdata      /* set up MVS Ingration Mgr Name with request data */ 

   
/* start TGMTask */ 
"tgmtask "||TGMTaskConfig||" -vv "||'"'||cmd'"'||">>"tasklog 

ridx = ''
If uidkey = 'ADMIN' then
 Do
  ridx = 1
  mvsdata = '<'			            /* begin record char */	
  mvsdata = mvsdata||'UIDKEY='||uidkey||'|'   /* add uidkey for request part */
  mvsdata = mvsdata||'DATEKEY='||datekey||'|' /* add datekey for request part */
  mvsdata = mvsdata||'PARTSEQ='||ridx||'|'     /* add partseq number */	
  mvsdata = mvsdata||'REQUEST_USER_ID='||uidkey||'|'  /*add user id of admin*/
  mvsdata = mvsdata||'REQUEST_GUID='||InFilePath||GUID||'.out'||'|'    /*add guid*/ 
  mvsdata = mvsdata||'>'                 /* end record char */ 
  "tgmtask "||TGMTaskConfig||" -vv "||'"'||cmd'"'||">>"tasklog 
  ridx = 2
  mvsdata = '<'			            /* begin record char */	
  mvsdata = mvsdata||'UIDKEY='||uidkey||'|'   /* add uidkey for request part */
  mvsdata = mvsdata||'DATEKEY='||datekey||'|' /* add datekey for request part */
  mvsdata = mvsdata||'PARTSEQ='||ridx||'|'     /* add partseq number */	
  mvsdata = mvsdata||'REQUEST_PASSWORD='||uidkey||'|'  /*add user id of admin*/
  mvsdata = mvsdata||'REQUEST_GUID='||InFilePath||GUID||'.out'||'|'    /*add guid*/ 
  mvsdata = mvsdata||'>'                 /* end record char */ 
  "tgmtask "||TGMTaskConfig||" -vv "||'"'||cmd'"'||">>"tasklog 
 ridx = 3 
 End

cmd = '"GTMIMAPI "'||' '||mvsdata      /* set up MVS Ingration Mgr Name with request data */ 

/* start TGMTask */           
"tgmtask "||TGMTaskConfig||" -vv "||'"'||cmd'"'||">>"tasklog 

if ridx = '' then
 ridx = 1
                                                  
savename = '' 
idx = 1                                    
Do While idx < totrecs                      /* format MVS data string */  
mvsdata = '<'			            /* begin record char */	
mvsdata = mvsdata||'UIDKEY='||uidkey||'|'   /* add uidkey for request part */
mvsdata = mvsdata||'DATEKEY='||datekey||'|' /* add datekey for request part */
mvsdata = mvsdata||'PARTSEQ='||ridx||'|'     /* add partseq number */


pidx = 1
lpos = length(datain.idx)
datain.idx = datain.idx||'|'

fpos = pos( '|', datain.idx, pidx )    /* get field name */
len = fpos - pidx
fname = substr(datain.idx,pidx,len)
pidx = pidx + len + 1                  /* point to next data item */ 

fpos = pos( '|', datain.idx, pidx )    /* get rowid */
len = fpos - pidx
rowid = substr(datain.idx,pidx,len)
pidx = pidx + len + 1                  /* point to next data item */ 

fpos = pos( '|', datain.idx, pidx )    /* get nbr of rows for rowid */
len = fpos - pidx
nbrrows = substr(datain.idx,pidx,len)
pidx = pidx + len + 1                  /* point to next data item */ 

savename = fname
  
mvsdata = mvsdata||fname||'='
 
fpos = pos( '|', datain.idx, pidx )      /* get data value */
len = fpos - pidx
If savename = 'REQUEST_SEVERITY' then
  value = substr(datain.idx,pidx,2)
else
 value = substr(datain.idx,pidx,len)
If savename = 'REQUEST_TYPE' then
  savereqtype = value
If savename = 'REQUEST_METHOD' then
  savemethod = value
If savename = 'REQUEST_GUID' then
 saveguid = value
savename = ''

mvsdata = mvsdata||value||'|'
mvsdata = mvsdata||'REQUEST_GUID='||InFilePath||GUID||'.out'||'|'    /*add guid*/ 
mvsdata = mvsdata||'>'                 /* end record char */ 

cmd = '"GTMIMAPI "'||' '||mvsdata      /* set up MVS Ingration Mgr Name with request data */ 

/* start TGMTask */           
"tgmtask "||TGMTaskConfig||" -vv "||'"'||cmd'"'||">>"tasklog 

idx = idx + 1
ridx = ridx + 1
End

mvsdata = '<'                                        /* format partseq 9999 */ 
mvsdata = mvsdata||'UIDKEY='||uidkey||'|'            /* add uidkey for request part */
mvsdata = mvsdata||'DATEKEY='||datekey||'|'          /* add datekey for request part */
mvsdata = mvsdata||'PARTSEQ='||'9999|'               /* add partseq number */
mvsdata = mvsdata||'REQUEST_TYPE='||save_type_value||'|' /*add request type */
mvsdata = mvsdata||'REQUEST_GUID='||InFilePath||GUID||'.out'||'|'   /*add guid*/ 
mvsdata = mvsdata||'ALL PARTS SENT|>' 
cmd = '"GTMIMAPI "'||' '||mvsdata      /* set up MVS Ingration Mgr Name with request data */ 

/* start TGMTask */
rexxoutcmd = 'REXX '||InFilePath||'TSD390RPout.rex'
"tgmtask "||TGMTaskConfig||" -vv "||'"'||cmd'"'||" | "rexxoutcmd

exit

setup_enum_values:
/* This routine will provide the enumeration values for the TSD390 GUI screens */
/* The enumeration values could be provided by using the TSD390 Data Model API function */
/* to retrieve them from the corresponding Data Attribute records */

retdata = 'REQUEST_RETURN_CODE'||'|'||'1'||'|'||'1'||'|'||'0'
Line = Lineout(OutFile,retdata)                      /* put line to file */

If save_enum_value = 'REQUEST_STATUS_CODE' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get Status Code enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'INITIAL'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'OPEN'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'CLOSED'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_PROBLEM_TYPE' then 
 Do
   retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get Problem Type enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'SOFTWARE'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'HARDWARE'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'NETWORK'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_PROBLEM_SEVERITY' then 
 Do
   retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get Problem Severity enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'01 - CRITICAL'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'02 - HIGH'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'03 - MEDIUM'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'04 - LOW'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_PROBLEM_ASSIGNEE_GROUP' then 
 Do
   retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get Problem Assignee Group enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP1'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP2'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP3'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP4'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP5'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP6'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP7'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP8'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP9'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'GROUP10'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_CHANGE_CATEGORY' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get change category enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Admin'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'App'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Facility'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Intranet'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Jobsched'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'JCL'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Info'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MSApp'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MSHardwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */      
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MSSoftwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MSVoice'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MFHardwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MFSoftwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MRHardwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'MRSoftwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Network'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Premises'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'PCHardwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'PCSoftwr'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Security'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Voice'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'3rdParty'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'UPSL'
  Line = Lineout(OutFile,retdata)      /* put line to file */    
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_CHANGE_COMPLEXITY' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get change complexity enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'A'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'B'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'C'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_CHANGE_IMPACT' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get change impact enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'High'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Low'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Medium'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_CHANGE_REASON' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get change reason enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'business'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'install'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'problem'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'product'
  Line = Lineout(OutFile,retdata)      /* put line to file */ 
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'remove'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'service'
  Line = Lineout(OutFile,retdata)      /* put line to file */ 
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_CHANGE_SITE' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get change site enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'dhouse'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'hdoffice'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'montford'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'olympic'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'pennylds'
  Line = Lineout(OutFile,retdata)      /* put line to file */ 
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'prescot'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'pyramid'
  Line = Lineout(OutFile,retdata)      /* put line to file */  
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_CHANGE_REQUESTOR_NAME' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get change requestor name enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'Blaine'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'John'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End

If save_enum_value = 'REQUEST_CHANGE_REQUESTOR_DEPT' then 
 Do
  retdata = 'REQUEST_ERROR_MSG'||'|'||'1'||'|'||'1'||'|'||'Get change requestor name enumerations request was successful'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'OPERATIONS'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  retdata = 'REQUEST_ENUM_VALUE'||'|'||'1'||'|'||'1'||'|'||'NETWORK'
  Line = Lineout(OutFile,retdata)      /* put line to file */
  Rc = LineOut(OutFile)                /* close OutFile */ 
  return
 End