-- $Header:   //Archive/Pvcsroot/Archive_NLS/Db/Sql/DB2Discovery.sqv   1.5.1.2   Jul 24 2001 15:33:14   Forthman  $
--
-- DB2Discovery.sql
-- 
-- Discovery procedures AND supporting tables FOR DB2

USE Object
GO
SET NOCOUNT ON
GO

-----------------------------------------------------------------------------------------------------------------------------------
-- DROP old version of DB2Discovery TABLE IF it EXISTS
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'DB2Discovery' AND type = 'U')
  AND NOT EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('DB2Discovery') AND name = 'dbvf_cid')
BEGIN
  PRINT 'Dropping old DB2Discovery TABLE'
  EXEC("
  DROP TABLE DB2Discovery
  ")
END


-- TABLE DB2Discovery
-- This TABLE receives bulk-loaded discovery events FOR DB2 resources
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'DB2Discovery' AND type = 'U')
BEGIN
  PRINT 'Creating TABLE DB2Discovery'
  EXEC("
  CREATE TABLE DB2Discovery
  (
    batchid           INT,
    SystemId          NVARCHAR(40)   NULL,
    obj_cid           ClassID           ,
    DB2_SUBSYSTEM     NVARCHAR(40)       ,
    DB2_VIEWFOLDER    NVARCHAR(40)   NULL,
    DB2_SUBCOMPONENT  NVARCHAR(40)   NULL,
    DB2_TABLESPACE    NVARCHAR(40)   NULL,
    DB2_TABLE         NVARCHAR(40)   NULL,
    DB2_INDEX         NVARCHAR(40)   NULL,
    os_id             ObjID         NULL   CONSTRAINT DB2Discovery_os_id_DF    DEFAULT NULL,
    dbsu_cid          ClassID       NULL   CONSTRAINT DB2Discovery_dbsu_cid_DF DEFAULT NULL,  
    dbsu_id           ObjID         NULL   CONSTRAINT DB2Discovery_dbsu_id_DF  DEFAULT NULL,
    dbvf_cid          ClassID       NULL   CONSTRAINT DB2Discovery_dbvf_cid_DF DEFAULT NULL,
    dbvf_id           ObjID         NULL   CONSTRAINT DB2Discovery_dbvf_id_DF  DEFAULT NULL,
    dbsc_cid          ClassID       NULL   CONSTRAINT DB2Discovery_dbsc_cid_DF DEFAULT NULL,
    dbsc_id           ObjID         NULL   CONSTRAINT DB2Discovery_dbsc_id_DF  DEFAULT NULL,
    dbts_cid          ClassID       NULL   CONSTRAINT DB2Discovery_dbts_cid_DF DEFAULT NULL,
    dbts_id           ObjID         NULL   CONSTRAINT DB2Discovery_dbts_id_DF  DEFAULT NULL,
    dbtb_cid          ClassID       NULL   CONSTRAINT DB2Discovery_dbtb_cid_DF DEFAULT NULL,
    dbtb_id           ObjID         NULL   CONSTRAINT DB2Discovery_dbtb_id_DF  DEFAULT NULL,
    dbin_cid          ClassID       NULL   CONSTRAINT DB2Discovery_dbin_cid_DF DEFAULT NULL,
    dbin_id           ObjID         NULL   CONSTRAINT DB2Discovery_dbin_id_DF  DEFAULT NULL,
    obj_name          ObjName       NULL   CONSTRAINT DB2Discovery_obj_name_DF DEFAULT NULL,
    obj_id            ObjID         NULL   CONSTRAINT DB2Discovery_obj_id_DF   DEFAULT NULL,
    obj_path          NVARCHAR(1000) NULL   CONSTRAINT DB2Discovery_obj_path_DF DEFAULT NULL,
    rediscovered      TINYINT              CONSTRAINT DB2Discovery_rediscoveredDF DEFAULT 0,
    DATE_TIME         DATETIME      NULL,
    SEQ_NO            NVARCHAR(40)   NULL,
    MVS               NVARCHAR(40)   NULL,
    STATE             NVARCHAR(40)   NULL,
    Event             NVARCHAR(40)   NULL,
    EventText         NVARCHAR(255)  NULL
  )
  ALTER TABLE DB2Discovery ADD CONSTRAINT DB2Discovery_batchid_FK FOREIGN KEY (batchid) REFERENCES Object.dbo.DiscoveryBatch (batchid)
  ALTER TABLE DB2Discovery ADD CONSTRAINT 
    DB2Discovery_instance_UQ UNIQUE (batchid, SystemId, obj_cid, DB2_SUBSYSTEM, DB2_SUBCOMPONENT, DB2_TABLESPACE, DB2_TABLE, DB2_INDEX)
  ")
END
GO

-- DROP old version of DB2Discovery TABLE IF it EXISTS
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ProcessedDB2Discovery' AND type = 'U')
  AND NOT EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('ProcessedDB2Discovery') AND name = 'dbvf_cid')
BEGIN
  PRINT 'Dropping old ProcessedDB2Discovery TABLE'
  EXEC("
  DROP TABLE ProcessedDB2Discovery
  ")
END

-- TABLE ProcessedDB2Discovery
-- TThis TABLE stores processed discovery records FOR ProcessedDB2 resources
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ProcessedDB2Discovery' AND type = 'U')
BEGIN
  PRINT 'Creating TABLE ProcessedDB2Discovery'
  EXEC("
  CREATE TABLE ProcessedDB2Discovery
  (
    ptime             DATETIME,
    batchid           INT,
    SystemId          NVARCHAR(40)  NULL,
    obj_cid           ClassID          ,
    DB2_SUBSYSTEM     NVARCHAR(40)      ,
    DB2_VIEWFOLDER    NVARCHAR(40)  NULL,
    DB2_SUBCOMPONENT  NVARCHAR(40)  NULL,
    DB2_TABLESPACE    NVARCHAR(40)  NULL,
    DB2_TABLE         NVARCHAR(40)  NULL,
    DB2_INDEX         NVARCHAR(40)  NULL,
    os_id             ObjID        NULL,
    dbsu_cid          ClassID      NULL,
    dbsu_id           ObjID        NULL,
    dbvf_cid          ClassID      NULL,
    dbvf_id           ObjID        NULL,
    dbsc_cid          ClassID      NULL,
    dbsc_id           ObjID        NULL,
    dbts_cid          ClassID      NULL,
    dbts_id           ObjID        NULL,
    dbtb_cid          ClassID      NULL,
    dbtb_id           ObjID        NULL,
    dbin_cid          ClassID      NULL,
    dbin_id           ObjID        NULL,
    obj_name          ObjName      NULL,
    obj_id            ObjID        NULL,
    obj_path          NVARCHAR(500) NULL,
    rediscovered      TINYINT          ,
    DATE_TIME         DATETIME     NULL,
    SEQ_NO            NVARCHAR(40)  NULL,
    MVS               NVARCHAR(40)  NULL,
    STATE             NVARCHAR(40)  NULL,
    Event             NVARCHAR(40)  NULL,
    EventText         NVARCHAR(255) NULL
  )
  ")
END
GO

---------------------------------------------------------------------------------------------------
-- DB2_VTRDIS TABLE

-- Pointer TABLE FOR VIEW-TABLE Relationship Discovery events received FROM DB2 
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'DB2_VTRDIS' AND type = 'U')
BEGIN
  PRINT 'Creating TABLE DB2_VTRDIS'
  EXEC("
  CREATE TABLE DB2_VTRDIS
  (
    batchid           INT,
    SystemId          NVARCHAR(40)  NULL, --  
    SubsysName        NVARCHAR(8)   NULL, --  
    VIEW_NAME         NVARCHAR(40)  NULL, --  
    DATABASE_NAME     NVARCHAR(40)  NULL, --  
    TABLESPACE_NAME   NVARCHAR(40)  NULL, --  
    TABLE_NAME        NVARCHAR(40)  NULL, --  
  )
  ")
END
GO

---------------------------------------------------------------------------------------------------
-- DB2Base_C
PRINT 'CREATE VIEW DB2Base_C'
GO

EXEC asisp_recreatePartitionedClassView @base_cid = 'D2BA', @additional_columns = '_Path,description,_HasOwnershipNote'
GO

---------------------------------------------------------------------------------------------------
-- DB2SubComponent_Rediscovery

PRINT 'CREATE VIEW DB2SubComponent_Rediscovery'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'DB2SubComponent_Rediscovery' AND type = 'V')
  DROP VIEW DB2SubComponent_Rediscovery
GO

CREATE VIEW DB2SubComponent_Rediscovery AS
  SELECT cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM DB2SubsystemComponent_C
  UNION
  SELECT cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM DB2DB_C
  UNION
  SELECT 'D2VW' AS cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM DB2View_C
GO

-----------------------------------------------------------------------------------
-- DB2Table_Rediscovery

PRINT 'CREATE VIEW DB2Table_Rediscovery'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'DB2Table_Rediscovery' AND type = 'V')
  DROP VIEW DB2Table_Rediscovery
GO

CREATE VIEW DB2Table_Rediscovery AS
  SELECT 'D2TB' AS cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM DB2Table_C
  UNION
  SELECT 'D2TP' AS cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM DB2TableSpacePartition_C
GO

-----------------------------------------------------------------------------------
-- DB2Index_Rediscovery

PRINT 'CREATE VIEW DB2Index_Rediscovery'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'DB2Index_Rediscovery' AND type = 'V')
  DROP VIEW DB2Index_Rediscovery
GO

CREATE VIEW DB2Index_Rediscovery AS
  SELECT 'D2IX' AS cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM DB2TableIndex_C
  UNION
  SELECT 'D2IP' AS cid, id, name, ctime, mtime, deleted, _AlertStateID, _PriorityID, _CurrentStateID, _DesiredStateID FROM DB2IndexPartition_C
GO

---------------------------------------------------------------------------------------------------
-- _LoadDB2DiscoveryBatch
PRINT 'Creating PROCEDURE _LoadDB2DiscoveryBatch'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadDB2DiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _LoadDB2DiscoveryBatch
GO

-- $Archive:   //Archive/Pvcsroot/Archive_NLS/Db/Sql/DB2Discovery.sqv  $
-- $Revision:   1.5.1.2  $
-- $Date:   Jul 24 2001 15:33:14  $
-- $Author:   Forthman  $
-- Loads a DB2Discovery batch FROM file path FOR the specified batch. 
-- The file IS IN native format -- see the definition of the #discovery temp TABLE IN the body of 
-- the PROCEDURE FOR the required columns.
CREATE PROCEDURE _LoadDB2DiscoveryBatch
  @cid          ClassID,
  @id           ObjID,
  @batchid      INT = NULL
AS
  IF @batchid IS NULL
  BEGIN
    SELECT @batchid = MIN(batchid)
    FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
    WHERE
      b.FormatType = 17
      AND ( b.ActionType = 1 OR b.ActionType = 8 )  -- Accept discovery AND VIEW TABLE relationship records.
      AND v.name = 'ENQUEUED'
    IF @batchid IS NULL
    BEGIN
      PRINT 'No DB2 DiscoveryBatch IS IN a ENQUEUED state, nothing to do!'
      RETURN 0
    END
  END
  ELSE
    UPDATE DiscoveryBatch SET pstat = v.pstat
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'ENQUEUED'

  DECLARE @ActionType INT
  SELECT @ActionType = ActionType FROM DiscoveryBatch WHERE batchid = @batchid

  -- CREATE temp TABLE FOR holding bulk-loaded results
  -- 1701 records are loaded directly to this TABLE
  -- 1708 records are loaded to #discovery1 first, THEN inserted to this TABLE 
  CREATE TABLE #discovery
  (
    FormatType        SMALLINT,
    ActionType        SMALLINT,
    TimeStamp         VARCHAR(40)  NULL, --  1
    SeqNo             VARCHAR(16)  NULL, --  2
    cid               VARCHAR(4)   NULL, --  3
    SystemId          VARCHAR(40)  NULL, --  4
    SubsysName        VARCHAR(8)   NULL, --  5
    MasterName        VARCHAR(40)  NULL, --  6
    LockMgrName       VARCHAR(40)  NULL, --  7
    DBSvcsName        VARCHAR(40)  NULL, --  8
    DDFName           VARCHAR(40)  NULL, --  9
    SPMgrName         VARCHAR(40)  NULL, -- 10
    BUFFER_POOLS      VARCHAR(40)  NULL, -- 11
    EDM_POOL          VARCHAR(40)  NULL, -- 12
    LOGS              VARCHAR(40)  NULL, -- 13
    DB2PM             VARCHAR(40)  NULL, -- 14
    DATABASE_NAME     VARCHAR(40)  NULL, -- 15
    TABLESPACE_NAME   VARCHAR(40)  NULL, -- 16
    TABLE_NAME        VARCHAR(40)  NULL, -- 17
    INDEX_NAME        VARCHAR(40)  NULL, -- 18
    TABLESPACE_PART   VARCHAR(40)  NULL, -- 19
    INDEX_PART        VARCHAR(40)  NULL, -- 20
    VIEW_NAME         VARCHAR(40)  NULL, -- 21
    STATE             VARCHAR(40)  NULL  -- 22
  )

  -- 1708 records are loaded directly to this TABLE first, THEN inserted to this TABLE 
  CREATE TABLE #discovery1
  (
    FormatType        SMALLINT,
    ActionType        SMALLINT,
    TimeStamp         VARCHAR(40)  NULL, --  1
    SeqNo             VARCHAR(16)  NULL, --  2
    cid               VARCHAR(4)   NULL, --  3
    SystemId          VARCHAR(40)  NULL, --  4
    SubsysName        VARCHAR(8)   NULL, --  5
    VIEW_NAME         VARCHAR(40)  NULL, --  6
    DATABASE_NAME     VARCHAR(40)  NULL, --  7
    TABLESPACE_NAME   VARCHAR(40)  NULL, --  8
    TABLE_NAME        VARCHAR(40)  NULL, --  9
  )

  -- Lookup file AND codepage of the batch
  DECLARE @FilePath NVARCHAR(500), @CodePage NVARCHAR(8)
  SELECT @FilePath = FilePath, @CodePage = CodePage FROM DiscoveryBatch WHERE batchid = @batchid
  IF @@ROWCOUNT = 0
  BEGIN
    RAISERROR('DiscoveryBatch id %d IS NOT valid', 3, 1, @batchid)
    RETURN -30000
  END

  DECLARE @err INT, @rc INT

  UPDATE DiscoveryBatch SET pstat = v.pstat
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
  -- Perform bulk load
  DECLARE @sql NVARCHAR(500)
  SELECT @sql ="
  BULK INSERT " + CASE @ActionType WHEN 1 THEN '#discovery' WHEN 8 THEN '#discovery1' END + " FROM '" + @FilePath + "'
  WITH
  (
    BATCHSIZE = 1000,
    CODEPAGE = '" + @CodePage + "',
    DATAFILETYPE = 'native'
  )"
  PRINT @sql
  EXEC(@sql)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('Error loading discovery batch file %s', 3, 1, @FilePath)
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @FilePath


  -- IF the records are FROM the VIEW TABLE Relationship THEN copy them INTO the big #discovery TABLE FOR single 
  --   coding below
  IF @ActionType = 8 
  BEGIN
    -- INSERT DISTINCT DBSSs
    INSERT #discovery ( FormatType, ActionType, cid, SystemId, SubsysName )
        SELECT DISTINCT FormatType, ActionType, 'DBSS', SystemId, SubsysName
        FROM #discovery1 
    -- INSERT DISTINCT DBVWs
    INSERT #discovery ( FormatType, ActionType, cid, SystemId, SubsysName, VIEW_NAME )
        SELECT DISTINCT FormatType, ActionType, cid, SystemId, SubsysName, VIEW_NAME 
        FROM #discovery1 
        WHERE cid IN ( 'DBVW', 'D2VW' )       
    -- INSERT DISTINCT DBDBs
    INSERT #discovery ( FormatType, ActionType, cid, SystemId, SubsysName, DATABASE_NAME )
        SELECT DISTINCT FormatType, ActionType, 'DBDB', SystemId, SubsysName, DATABASE_NAME
        FROM #discovery1
        WHERE cid IN ( 'DBTB', 'D2TB' )       
    -- INSERT DISTINCT DBTSs
    INSERT #discovery ( FormatType, ActionType, cid, SystemId, SubsysName, DATABASE_NAME, TABLESPACE_NAME )
        SELECT DISTINCT FormatType, ActionType, 'DBTS', SystemId, SubsysName, DATABASE_NAME, TABLESPACE_NAME
        FROM #discovery1
        WHERE cid IN ( 'DBTB', 'D2TB' )       
    -- INSERT DISTINCT DBTBs
    INSERT #discovery ( FormatType, ActionType, cid, SystemId, SubsysName, DATABASE_NAME, TABLESPACE_NAME, TABLE_NAME )
        SELECT DISTINCT FormatType, ActionType, cid, SystemId, SubsysName, DATABASE_NAME, TABLESPACE_NAME, TABLE_NAME 
        FROM #discovery1
        WHERE cid IN ( 'DBTB', 'D2TB' )       

    -- INSERT D2VWs that are known through the D2TB records avoid dups with DISTINCT SELECT ON INSERT to DB2Discovery
    INSERT #discovery ( FormatType, ActionType, cid, SystemId, SubsysName, VIEW_NAME )
        SELECT DISTINCT FormatType, ActionType, 'DBVW', SystemId, SubsysName, VIEW_NAME 
        FROM #discovery1 AS d1
        WHERE cid IN ( 'DBTB', 'D2TB' )       
          AND SubsysName IS NOT NULL 
          AND VIEW_NAME  IS NOT NULL

    -- UPDATE TimeStamp AND SeqNo
    UPDATE #discovery SET TimeStamp = (SELECT MAX(TimeStamp) FROM #discovery1),
                          SeqNo     = (SELECT MAX(SeqNo)     FROM #discovery1)
    -- INSERT D2TB records that point to D2VWs INTO the DB2_VTRDIS TABLE to establish USEC relationship 
    -- at the END of the Processing Step
    INSERT INTO DB2_VTRDIS ( batchid, SystemId, SubsysName, VIEW_NAME, DATABASE_NAME, TABLESPACE_NAME, TABLE_NAME )
            SELECT DISTINCT @batchid, SystemId, SubsysName, VIEW_NAME, DATABASE_NAME, TABLESPACE_NAME, TABLE_NAME 
        FROM #discovery1
        WHERE 
              cid = 'DBTB'
          AND SystemId        IS NOT NULL
          AND SubsysName      IS NOT NULL
          AND VIEW_NAME       IS NOT NULL
          AND DATABASE_NAME   IS NOT NULL
          AND TABLESPACE_NAME IS NOT NULL
          AND TABLE_NAME      IS NOT NULL

  END

  -- We may get rows which are duplicates ON everything but the DATE_TIME
  -- Workaround: UPDATE the DATE_TIME column FOR the given batch to the MAX(DATE_TIME) value
  -- FROM the batch - I have NOT double checked to see IF this IS happening IN DB2 data, but better to be safe 
  UPDATE #discovery SET TimeStamp = (SELECT MAX(TimeStamp) FROM #discovery)

  -- INSERT the records FROM temp TABLE INTO DB2Discovery
  INSERT INTO DB2Discovery
  (
    batchid,
    SystemId,
    obj_cid,
    DB2_SUBSYSTEM,
    DB2_SUBCOMPONENT,
    DB2_TABLESPACE,
    DB2_TABLE,
    DB2_INDEX,
    dbsu_cid,
    dbsc_cid,
    dbts_cid,
    dbtb_cid,
    dbin_cid,
    obj_name,
    DATE_TIME,
    SEQ_NO,
    STATE
  )
  SELECT DISTINCT
    @batchid,
    SystemId,
    cid AS obj_cid,
    CASE
      WHEN SubsysName    IS NOT NULL THEN SubsysName
    END AS DB2_SUBSYSTEM,
    CASE 
      WHEN MasterName    IS NOT NULL THEN MasterName
      WHEN LockMgrName   IS NOT NULL THEN LockMgrName
      WHEN DBSvcsName    IS NOT NULL THEN DBSvcsName
      WHEN DDFName       IS NOT NULL THEN DDFName
      WHEN SPMgrName     IS NOT NULL THEN SPMgrName
      WHEN BUFFER_POOLS  IS NOT NULL THEN BUFFER_POOLS
      WHEN EDM_POOL      IS NOT NULL THEN EDM_POOL
      WHEN LOGS          IS NOT NULL THEN LOGS
      WHEN DB2PM         IS NOT NULL THEN DB2PM
      WHEN DATABASE_NAME IS NOT NULL THEN DATABASE_NAME
      WHEN VIEW_NAME     IS NOT NULL THEN VIEW_NAME    
    END AS DB2_SUBCOMPONENT,
    CASE
      WHEN TABLESPACE_NAME IS NOT NULL THEN TABLESPACE_NAME
    END AS DB2_TABLESPACE,
    CASE
      WHEN TABLE_NAME      IS NOT NULL THEN TABLE_NAME
      WHEN TABLESPACE_PART IS NOT NULL THEN TABLESPACE_PART
    END AS DB2_TABLE,
    CASE
      WHEN INDEX_NAME IS NOT NULL THEN INDEX_NAME
      WHEN INDEX_PART IS NOT NULL THEN INDEX_PART
    END AS DB2_INDEX,
    CASE
      WHEN SubsysName    IS NOT NULL THEN 'D2SS'
    END AS dbsu_cid,
    CASE
      WHEN MasterName    IS NOT NULL THEN 'D2MS'
      WHEN LockMgrName   IS NOT NULL THEN 'D2LK'
      WHEN DBSvcsName    IS NOT NULL THEN 'D2DS'
      WHEN DDFName       IS NOT NULL THEN 'D2DD'
      WHEN SPMgrName     IS NOT NULL THEN 'D2SP'
      WHEN BUFFER_POOLS  IS NOT NULL THEN 'D2BP'
      WHEN EDM_POOL      IS NOT NULL THEN 'D2EP'
      WHEN LOGS          IS NOT NULL THEN 'D2LG'
      WHEN DB2PM         IS NOT NULL THEN 'D2PM'
      WHEN DATABASE_NAME IS NOT NULL THEN 'D2DB'
      WHEN VIEW_NAME     IS NOT NULL THEN 'D2VW'
    END AS dbsc_cid,
    CASE
      WHEN TABLESPACE_NAME IS NOT NULL THEN 'D2TS'
    END AS dbts_cid,
    CASE
      WHEN TABLE_NAME      IS NOT NULL THEN 'D2TB'
      WHEN TABLESPACE_PART IS NOT NULL THEN 'D2TP'
    END AS dbtb_cid,
    CASE
      WHEN INDEX_NAME IS NOT NULL THEN 'D2IX'
      WHEN INDEX_PART IS NOT NULL THEN 'D2IP'
    END AS dbin_cid,
    CASE cid 
      WHEN 'DBSS' THEN SubsysName
      WHEN 'DBMS' THEN MasterName
      WHEN 'DBLK' THEN LockMgrName
      WHEN 'DBDS' THEN DBSvcsName
      WHEN 'DBDD' THEN DDFName
      WHEN 'DBSP' THEN SPMgrName
      WHEN 'DBBP' THEN BUFFER_POOLS
      WHEN 'DBEP' THEN EDM_POOL
      WHEN 'DBLG' THEN LOGS
      WHEN 'DBPM' THEN DB2PM
      WHEN 'DBDB' THEN DATABASE_NAME
      WHEN 'DBTS' THEN TABLESPACE_NAME
      WHEN 'DBTB' THEN TABLE_NAME
      WHEN 'DBIX' THEN INDEX_NAME
      WHEN 'DBTP' THEN TABLESPACE_PART
      WHEN 'DBIP' THEN INDEX_PART
      WHEN 'DBVW' THEN VIEW_NAME
    END AS obj_name,
    SUBSTRING(TimeStamp,1,4)+'-'+SUBSTRING(TimeStamp,5,2)+'-'+SUBSTRING(TimeStamp,7,2)+' '+SUBSTRING(TimeStamp,9,12),
    SeqNo,
    STATE
  FROM #discovery
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @@ERROR <> 0
  BEGIN
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END

  UPDATE DiscoveryBatch SET pstat = v.pstat, numrows = @rc
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'LOADED'
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2SS
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2SS'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2SS' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2SS
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2VW
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2VW'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2VW' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2VW
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2TS
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2TS'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2TS' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2TS
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2TP
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2TP'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2TP' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2TP
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2TB
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2TB'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2TB' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2TB
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2IX
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2IX'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2IX' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2IX
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2IP
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2IP'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2IP' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2IP
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesViewTables
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesViewTables'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesViewTables' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesViewTables
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2ResourcesD2SC
PRINT 'Dropping PROCEDURE _DiscoverDB2ResourcesD2SC'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2ResourcesD2SC' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2ResourcesD2SC
GO

---------------------------------------------------------------------------------------------------
-- _DiscoverDB2Resources
PRINT 'Creating PROCEDURE _DiscoverDB2Resources'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverDB2Resources' AND type = 'P')
  DROP PROCEDURE _DiscoverDB2Resources
GO

-- $Archive:   //Archive/Pvcsroot/Archive_NLS/Db/Sql/DB2Discovery.sqv  $
-- $Revision:   1.5.1.2  $
-- $Date:   Jul 24 2001 15:33:14  $
-- $Author:   Forthman  $
CREATE PROCEDURE _DiscoverDB2Resources
  @cid          ClassID,
  @id           ObjID,
  @batchid      INT = NULL,
  @incremental  TINYINT = 0,
  @verbose      TINYINT = 0
AS
  DECLARE @rc INT

  SET NOCOUNT ON

  RAISERROR( '_DiscoverDB2Resources @cid = ''%s'', @id = %d, @batchid = %d, @incremental = %d , @verbose = %d', 0, 1, @cid, @id, @batchid, @incremental, @verbose)

  -- IF the batchid was NOT specified THEN pick the oldest one that IS IN a LOADED state
  IF @batchid IS NULL
  BEGIN
    SELECT @batchid = min(batchid) 
    FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
    WHERE
      FormatType = 17
      AND ActionType IN ( 1, 8, 12 )-- HACK ActionType 12, remove 12 after fixing data upstream
      AND v.name = 'LOADED'
    IF @batchid IS NULL
    BEGIN
      PRINT 'No DB2 DiscoveryBatch IS IN a LOADED state, nothing to do!'
      RETURN 0
    END
  END

-- Garbage collection eventually:IF NOT running IN incremental mode
--  IF @incremental = 0
--  BEGIN
    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Starting garbage collection'
    DECLARE linkc CURSOR FOR
      SELECT dol.link_type, RTRIM(dol.src_cid), RTRIM(dol.dst_cid), dst.cname
      FROM derived_obj_link AS dol
        JOIN obj_class AS src ON src.cid = dol.src_cid
        JOIN obj_class AS dst ON dst.cid = dol.dst_cid
        JOIN isa_chain AS dic ON dic.cid = dol.dst_cid
        JOIN link_type AS lt ON lt.link_type = dol.link_type
      WHERE
        src.flag_abstract = 0
        AND dst.flag_abstract = 0
        AND dic.base_cid IN ( 'D2SC', 'D2DB', 'D2VF', 'D2VW', 'D2TS', 'D2TB', 'D2TP', 'D2IX', 'D2IP'  )
        AND lt.flag_containment = 1
    OPEN linkc
    DECLARE @link_type LinkType, @src_cid ClassID, @dst_cid ClassID, @dst_cname NVARCHAR(24)
    FETCH NEXT FROM linkc INTO @link_type, @src_cid, @dst_cid, @dst_cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- DELETE link records
      DECLARE @table sysname
      SET @table = @src_cid + '_' + @dst_cid + '_' + @link_type
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Doing garbage collection ON ' + @table
      EXEC("
      DELETE FROM il FROM " + @table + " AS il JOIN link AS l ON
        l.link_type = '" + @link_type + "'
        AND l.src_cid = '" + @src_cid + "'
        AND l.src_id = il.src_id
        AND l.dst_cid = '" + @dst_cid + "'
        AND l.dst_id = il.dst_id
      WHERE
        l.deleted = 1
      ")
      SET @rc = @@ROWCOUNT
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS NVARCHAR)
    
      -- DELETE instance records
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Doing garbage collection ON ' + @dst_cname + '_C'
      EXEC("
      DELETE FROM " + @dst_cname + "_C WHERE id NOT IN (SELECT l.dst_id FROM link AS l WHERE l.dst_cid='" + @dst_cid + "' AND l.deleted=0)
      ")
      SET @rc = @@ROWCOUNT
      IF @verbose = 1
        PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + '  ROWCOUNT=' + CAST(@rc AS NVARCHAR)

      FETCH NEXT FROM linkc INTO @link_type, @src_cid, @dst_cid, @dst_cname
    END
    DEALLOCATE linkc
--  END -- END of garbage collection


  -- Take care of any mismatches between discovery data obj_cid AND obj_class.cid
  UPDATE DB2Discovery 
    SET obj_cid = 
      CASE obj_cid 
        WHEN 'DBBP' THEN 'D2BP'
        WHEN 'DBDB' THEN 'D2DB'
        WHEN 'DBDD' THEN 'D2DD'
        WHEN 'DBDS' THEN 'D2DS'
        WHEN 'DBEP' THEN 'D2EP'
        WHEN 'DBIP' THEN 'D2IP'
        WHEN 'DBIX' THEN 'D2IX'
        WHEN 'DBLG' THEN 'D2LG'
        WHEN 'DBLK' THEN 'D2LK'
        WHEN 'DBMS' THEN 'D2MS'
        WHEN 'DBPM' THEN 'D2PM'
        WHEN 'DBSP' THEN 'D2SP'
        WHEN 'DBSS' THEN 'D2SS'
        WHEN 'DBTB' THEN 'D2TB'
        WHEN 'DBTP' THEN 'D2TP'
        WHEN 'DBTS' THEN 'D2TS'
        WHEN 'DBVW' THEN 'D2VW'
        ELSE obj_cid
      END
    WHERE batchid = @batchid

  -- Map SystemId to os_id
  IF @verbose = 1
    PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Mapping SystemId to os_id'
  UPDATE child SET
    os_id = p.id
  FROM
    DB2Discovery AS child
    JOIN OS_V AS p ON p._SystemId = child.SystemId

  -- IF we could NOT map any of the records FROM the batch to an operating system THEN
  -- there's something wrong.
  IF NOT EXISTS
  (
    SELECT * FROM DB2Discovery WHERE batchid = @batchid AND os_id IS NOT NULL
  )
  BEGIN
    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: batch ' + CAST(@batchid AS NVARCHAR) + ' cannot be resolved to any operating system. The SystemId field of the discovery records does NOT match any known operating systems.'
    RETURN -30000
  END


  -- Compute the path of all objects. We can do this now because all operating systems have been looked up.
  -- Note: because the D2VFs are NOT known at this time, D2VFs AND their children D2VWs will be WRONG, fixed below 
  DECLARE osc CURSOR FOR
    SELECT DISTINCT os_id FROM DB2Discovery WHERE batchid = @batchid
  OPEN osc
  DECLARE @os_id ObjID, @Path NVARCHAR(255)
  FETCH NEXT FROM osc INTO @os_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC asisp_fullpath OS, @os_id, @Path OUTPUT
    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Updating paths FOR objects under ' + @Path
    UPDATE DB2Discovery SET obj_path = 
      @Path
      + '/' + DB2_SUBSYSTEM
      + CASE WHEN dbsc_cid IS NOT NULL THEN '/' + DB2_SUBCOMPONENT END
      + CASE WHEN dbts_cid IS NOT NULL THEN '/' + DB2_TABLESPACE   END
      + CASE WHEN dbtb_cid IS NOT NULL THEN '/' + DB2_TABLE        END
      + CASE WHEN dbin_cid IS NOT NULL THEN '/' + DB2_INDEX        END
    WHERE
      batchid = @batchid
      AND os_id = @os_id
    FETCH NEXT FROM osc INTO @os_id
  END
  DEALLOCATE osc




  -- Rediscover Subsystems: Step 1, UPDATE dbsu_id IN DB2Discovery records 
  UPDATE child SET
    dbsu_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2Subsystem_C AS obj ON          -- Find any existing DB2Subsystem objects by class,name
      obj.cid = child.dbsu_cid
      AND obj.name = child.DB2_SUBSYSTEM
    JOIN link AS pc ON                     -- Lookup FOR parent links of subsystems to make sure
      pc.src_cid = 'OS'                    -- we're under the right operating system
      AND pc.src_id = child.os_id 
      AND pc.dst_cid = obj.cid
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                   -- USE only Active objects
  WHERE
    child.batchid = @batchid               -- Reference the current batch

  -- Rediscover Subsystems: Step 2, UPDATE rediscovered, obj_id FOR D2SS objects that have dbsu_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbsu_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid            -- USE the current batch
    AND d.obj_cid = 'D2SS'          -- Only UPDATE DB2 Subsystem class with a dbsu_id value
    AND dbsu_id IS NOT NULL     


  -- CREATE any subsystems that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbsu_id IS NULL 
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''PHYC''',
      @ParentCIDColumn = '''OS''',
      @ParentIDColumn  = 'os_id',
      @ChildCIDColumn  = 'dbsu_cid',
      @ChildIDColumn   = 'dbsu_id',
      @ChildNameColumn = 'DB2_SUBSYSTEM',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL AND d.dbsu_id IS NULL AND d.obj_cid = ''D2SS''',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' subsystems'
    END
  END

  -- UPDATE subsystem obj_id FOR those subsystems that were just created
  UPDATE d SET
    obj_id      = dbsu_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid          -- USE the current batch 
    AND d.obj_cid = 'D2SS'          -- Only UPDATE DB2 Subsystems class with a dbsu_id value that were NOT rediscovered
    AND dbsu_id IS NOT NULL
    AND rediscovered = 0

  -- Display DB2Subsystems
  IF @verbose = 1
    SELECT 'DB2Subsystem', * FROM DB2Discovery AS d
    WHERE batchid = @batchid 
      AND d.obj_cid = 'D2SS'

  -- The NEXT several paragraphs refer to DB2 Subcomponents, this really means everything but D2VWs because D2VWs are
  --   bound to D2VFs
  -- Rediscover Subcomponents: Step 1, UPDATE dbsc_id IN DB2Discovery records 
  UPDATE child SET
    dbsc_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2SubComponent_Rediscovery AS obj ON   -- Find any existing DB2 Subcomponent objects by class,name
      obj.cid = child.dbsc_cid
      AND obj.name = child.DB2_SUBCOMPONENT
    JOIN link AS pc ON                           -- Restrict by parent subsystem
      pc.src_cid = child.dbsu_cid        
      AND pc.src_id = child.dbsu_id 
      AND pc.dst_cid = obj.cid
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                        -- USE only Active objects
    JOIN isa_chain AS pic ON                    -- Make parent classes known to WHERE clause
      pic.cid = child.dbsc_cid
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_SUBCOMPONENT IS NOT NULL        -- Only attempt to rediscovery DB2_SUBCOMPONENT with actual names
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS a instance OR a child of the these classes
    AND pc.dst_cid = child.dbsc_cid               -- Restrict match to the discovery record class

  -- Rediscover Subcomponents: Step 2, UPDATE rediscovered, obj_id FOR DB2 Subcomponent objects that have dbsc_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbsc_id
  FROM 
    DB2Discovery AS d
    JOIN isa_chain AS pic ON        -- Make parent classes known to WHERE clause
      pic.cid = d.dbsc_cid
  WHERE 
    d.batchid = @batchid                          -- USE the current batch
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS a instance OR a child of the these classes
    AND dbsc_id IS NOT NULL                       -- WHEN there IS an actual dbsc_id value
    AND dbts_cid IS NULL                          -- WHEN record describes a subcomponent AND NOT a child of a subcomponent

  -- CREATE any subcomponents that are children of the D2SC class that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    JOIN isa_chain AS pic ON pic.cid = d.obj_cid  -- Make parent classes known to WHERE clause
    WHERE d.batchid = @batchid 
      AND d.dbsc_cid IS NOT NULL                  -- Has a subcomponent
      AND pic.base_cid = 'D2SC'                   -- IS a child of the D2SC class
      AND d.dbsc_id IS NULL                       -- Subcomponent IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''PHYC''',
      @ParentCIDColumn = 'dbsu_cid',
      @ParentIDColumn  = 'dbsu_id',
      @ChildCIDColumn  = 'dbsc_cid',
      @ChildIDColumn   = 'dbsc_id',
      @ChildNameColumn = 'DB2_SUBCOMPONENT',
      @WhereClause     = 'JOIN isa_chain AS ic ON ic.cid = d.obj_cid 
                           WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.dbsc_id IS NULL 
                             AND ic.base_cid = ''D2SC''
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' subcomponents'
    END
  END



  -- CREATE any D2DB subcomponents that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbsc_cid = 'D2DB'     -- IS a D2DB subcomponent
      AND d.dbsc_id IS NULL        -- Subcomponent IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbsu_cid',
      @ParentIDColumn  = 'dbsu_id',
      @ChildCIDColumn  = 'dbsc_cid',
      @ChildIDColumn   = 'dbsc_id',
      @ChildNameColumn = 'DB2_SUBCOMPONENT',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.obj_cid = ''D2DB'' 
                             AND d.dbsc_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' subcomponents (databases)'
    END
  END

  -- D2VF rediscovery
  -- IF there IS a subsystem that has D2VWs 
  --   THEN CREATE a DB2Discovery record FOR each D2VF that EXISTS

  INSERT INTO
    DB2Discovery 
  (
    batchid,
    SystemId,
    obj_cid,
    DB2_SUBSYSTEM,
    DB2_VIEWFOLDER,
    os_id,
    dbsu_cid,
    dbsu_id,
    dbvf_cid,
    dbvf_id,
    obj_name,
    obj_id,
    rediscovered
  )
  SELECT DISTINCT
    @batchid,
    d.SystemId,
    l.dst_cid,
    d.DB2_SUBSYSTEM,
    obj.name,
    d.os_id,
    d.dbsu_cid,
    d.dbsu_id,
    l.dst_cid,
    obj.id,
    obj.name,
    obj.id,
    1
  FROM DB2Discovery AS d
    JOIN link AS l ON l.src_id = d.dbsu_id  -- JOIN link TABLE to provide list of D2VFs known to the DB2 Subsystem
      AND l.src_cid = d.dbsu_cid
      AND l.dst_cid = 'D2VF'
      AND l.deleted = 0                     -- USE only Active objects
    JOIN DB2ViewFolder_V AS obj             -- JOIN DB2ViewFolder_V to provide fields FOR SELECT
      ON obj.id = l.dst_id
  WHERE d.batchid = @batchid     -- Match ON batchid
    AND d.dbsu_id IS NOT NULL    -- Valid subsystem
    AND d.dbsc_cid = 'D2VW'      -- IS a D2VW record


  -- D2VW rediscovery
  -- IF subsystem -> D2VF -> D2VW 
  --  THEN mark D2VW AS rediscovered

  -- Rediscover DB2 Views: Step 1, UPDATE dbsc_id IN DB2Discovery records, AND its dbvf_cid AND dbvf_id since they are known
  UPDATE child SET
    dbsc_id        = obj.id,
    dbvf_cid       = pc.src_cid,
    dbvf_id        = pc.src_id,
    DB2_VIEWFOLDER = obj2.name
  FROM
    DB2Discovery AS child
    JOIN DB2View_V AS obj ON               -- Find any existing DB2 VIEW by name
          obj.name = child.DB2_SUBCOMPONENT
    JOIN link AS pc ON                     -- Lookup FOR parent links of db2 VIEW folder to make sure
          pc.src_cid = 'D2VF'              -- we're under the right folder, note the db2 VIEW folder's match floats (no id)
      AND pc.dst_cid = 'D2VW'
      AND pc.dst_id  = obj.id
      AND pc.deleted = 0                   -- USE only Active objects
    JOIN link AS pc2 ON                    -- Limit to D2VWs subsystem, by chaining with link TABLE
          pc2.src_cid = child.dbsu_cid
      AND pc2.src_id  = child.dbsu_id
      AND pc2.dst_cid = pc.src_cid
      AND pc2.dst_id  = pc.src_id
      AND pc2.deleted = 0                  -- USE only Active objects
    JOIN DB2ViewFolder_V AS obj2 ON        -- Hook IN attributes of VIEW Folder FOR SELECT statement
      obj2.id = pc.src_id
  WHERE
        child.batchid = @batchid           -- Reference the current batch
    AND child.obj_cid = 'D2VW'             -- record refers to a db2 VIEW
    AND obj.deleted   = 0                  -- Don't reference any deleted db2 VIEW folders


  -- Rediscover DB2 Views: Step 2, UPDATE rediscovered, obj_id FOR D2VW objects that have dbsc_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbsc_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid            -- USE the current batch
    AND d.obj_cid = 'D2VW'          -- Only UPDATE DB2 VIEW class with a dbsc_id value
    AND dbsc_id IS NOT NULL     


  -- IF there are D2VW that have NOT been rediscovered 
  --   AND there IS NOT a D2VF FOR the same subsystem 
  -- THEN
  --   CREATE a D2VF record FOR that subsystem

  INSERT INTO
    DB2Discovery 
  (
    batchid,
    SystemId,
    obj_cid,
    DB2_SUBSYSTEM,
    DB2_VIEWFOLDER,
    os_id,
    dbsu_cid,
    dbsu_id,
    dbvf_cid,
    dbvf_id,
    obj_name,
    obj_id,
    rediscovered
  )
  SELECT DISTINCT
    @batchid,
    d.SystemId,
    'D2VF',
    d.DB2_SUBSYSTEM,
    '',
    d.os_id,
    d.dbsu_cid,
    d.dbsu_id,
    'D2VF',
    NULL,
    '',
    NULL,
    0
  FROM DB2Discovery AS d
  WHERE 
        d.batchid  = @batchid      -- Match ON batchid
    AND d.dbsc_cid = 'D2VW'        -- IS a DB2 VIEW
    AND d.rediscovered = 0         -- that has NOT been rediscovered
    AND NOT EXISTS ( SELECT * FROM DB2Discovery AS d2 WHERE
        d2.batchid  = d.batchid    -- Match ON batchid
    AND d2.dbsu_cid = d.dbsu_cid   -- Match ON subsystem cid
    AND d2.dbsu_id  = d.dbsu_id    -- Match ON subsystem id
    AND d2.dbvf_cid = 'D2VF'       -- There IS a DB2 VIEW Folder
    )


  -- UPDATE undiscovered db2 views to have their db2 VIEW folder information match the a D2VF record
  SELECT dbsu_cid, dbsu_id, dbvf_cid, dbvf_id, DB2_VIEWFOLDER INTO #dbvf_detail 
  FROM DB2Discovery
  WHERE
        batchid = @batchid
    AND obj_cid = 'D2VF'

  UPDATE d SET
    d.dbvf_cid       = d2.dbvf_cid,
    d.dbvf_id        = d2.dbvf_id,
    d.DB2_VIEWFOLDER = d2.DB2_VIEWFOLDER
  FROM DB2Discovery AS d, #dbvf_detail AS d2
  WHERE 
        d.batchid      = @batchid
    AND d.dbsu_cid     = d2.dbsu_cid
    AND d.dbsu_id      = d2.dbsu_id
    AND d.obj_cid      = 'D2VW'
    AND d.rediscovered = 0

  DROP TABLE #dbvf_detail

 
  -- IF there are D2VF that have NOT been rediscovered
  -- THEN CREATE the D2VF objects

  -- CREATE any D2VF objects that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbvf_cid = 'D2VF'     -- IS a D2VF object
      AND d.dbvf_id IS NULL       -- This IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbsu_cid',
      @ParentIDColumn  = 'dbsu_id',
      @ChildCIDColumn  = 'dbvf_cid',
      @ChildIDColumn   = 'dbvf_id',
      @ChildNameColumn = 'DB2_VIEWFOLDER',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.obj_cid = ''D2VF'' 
                             AND d.dbvf_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' VIEW folder(s)'
    END
  END


  -- UPDATE discovered D2VF obj_id
  UPDATE d SET
    obj_id       = dbvf_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                          -- USE the current batch
    AND obj_cid = 'D2VF'                          -- Only D2VFs
    AND dbvf_id IS NOT NULL                       -- That have ids
    AND rediscovered = 0                          -- AND are newly discovered

  -- IF there IS a D2VW that has NOT been rediscovered
  -- THEN 
  --   assign the D2VF cid AND id FROM that subsystem (see D2VF records)
  -- This should be handled automatically by the _CreateDiscoveredObjects because it can match ON the name

  -- CREATE all D2VW that have NOT been rediscovered

  -- CREATE any D2VW objects that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbsc_cid = 'D2VW'     -- IS a D2VW object
      AND d.dbsc_id IS NULL       -- This IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbvf_cid',
      @ParentIDColumn  = 'dbvf_id',
      @ChildCIDColumn  = 'dbsc_cid',
      @ChildIDColumn   = 'dbsc_id',
      @ChildNameColumn = 'DB2_SUBCOMPONENT',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsu_id IS NOT NULL 
                             AND d.obj_cid = ''D2VW'' 
                             AND d.dbvf_id IS NOT NULL 
                             AND d.dbsc_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' VIEW(s)'
    END
  END

  -- Compute the path of all D2VF AND D2VW records, since everything IS now known.
  DECLARE osc CURSOR FOR
    SELECT DISTINCT os_id FROM DB2Discovery WHERE batchid = @batchid
  OPEN osc
  FETCH NEXT FROM osc INTO @os_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC asisp_fullpath OS, @os_id, @Path OUTPUT
    IF @verbose = 1
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Updating paths FOR objects under ' + @Path
    UPDATE DB2Discovery SET obj_path = 
      @Path
      + '/' + DB2_SUBSYSTEM
      + CASE WHEN dbvf_cid IS NOT NULL THEN '/' + DB2_VIEWFOLDER   END
      + CASE WHEN dbsc_cid IS NOT NULL THEN '/' + DB2_SUBCOMPONENT END
    WHERE
      batchid = @batchid
      AND os_id = @os_id
      AND dbvf_id IS NOT NULL
    FETCH NEXT FROM osc INTO @os_id
  END
  DEALLOCATE osc

  -- UPDATE subcomponent obj_id FOR those subcomponents that were just created
  UPDATE d SET
    d.obj_id      = dbsc_id
  FROM 
    DB2Discovery AS d
    JOIN isa_chain AS pic ON                      -- Make parent classes known to WHERE clause
      pic.cid = d.dbsc_cid
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS an instance OR a child of the these classes
    AND d.dbsc_id IS NOT NULL                     -- dbsc_id has actual value
    AND DB2_TABLESPACE IS NULL                    -- UPDATE only Subcomponent records 
    AND d.rediscovered = 0                        -- Was NOT previously rediscovered

  -- UPDATE the D2VF paths that have been discovered, doing it now AS a efficiency
  UPDATE DB2ViewFolder_C 
  SET _Path = d.obj_path
  FROM DB2ViewFolder_C AS c
    INNER JOIN DB2Discovery AS d ON c.id = d.obj_id
  WHERE 
        d.batchid      = @batchid
    AND d.obj_cid      = 'D2VF'
    AND d.rediscovered = 0

  -- UPDATE the D2VW paths that have been discovered, doing it now AS a efficiency
  UPDATE DB2View_C 
  SET _Path = d.obj_path
  FROM DB2View_C AS c
    INNER JOIN DB2Discovery AS d ON c.id = d.obj_id
  WHERE 
        d.batchid      = @batchid
    AND d.obj_cid      = 'D2VW'
    AND d.rediscovered = 0

  -- Display DB2SubComponents
  IF @verbose = 1
    SELECT 'DB2SubComponents', * FROM DB2Discovery AS d
    JOIN isa_chain AS pic ON                      -- Make parent classes known to WHERE clause
      pic.cid = d.dbsc_cid
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND pic.base_cid IN ( 'D2DB', 'D2VW', 'D2SC') -- IS an instance OR a child of the these classes

  -- Rediscover Tablespaces: Step 1, UPDATE dbts_id IN DB2Discovery records 
  UPDATE child SET
    dbts_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2TableSpace_C AS obj ON               -- Find any existing DB2 tablespace objects by class,name
      obj.name = child.DB2_TABLESPACE
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbsc_cid        
      AND pc.src_id = child.dbsc_id 
      AND pc.dst_cid = 'D2TS'
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_TABLESPACE IS NOT NULL          -- Only attempt to rediscovery DB2_TABLESPACE with actual names

  -- Rediscover Tablespaces: Step 2, UPDATE rediscovered, obj_id FOR DB2 Subcomponent objects that have dbts_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbts_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                      -- Reference the current batch
    AND d.dbts_id        IS NOT NULL          -- Only UPDATE records WHERE the tablespace IS known 
    AND d.DB2_TABLE      IS NULL              -- Only UPDATE tablespace records (IF it has a DB2_TABLE record it IS child)

  -- CREATE any tablespaces that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbts_cid IS NOT NULL   -- There IS a tablespace
      AND d.dbts_id  IS NULL       -- Tablespace IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbsc_cid',
      @ParentIDColumn  = 'dbsc_id',
      @ChildCIDColumn  = 'dbts_cid',
      @ChildIDColumn   = 'dbts_id',
      @ChildNameColumn = 'DB2_TABLESPACE',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbsc_id IS NOT NULL 
                             AND d.obj_cid = ''D2TS'' 
                             AND d.dbts_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' tablespaces'
    END
  END

  -- UPDATE tablespace obj_id FOR those tablespaces that were just created
  UPDATE d SET
    obj_id      = dbts_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND d.obj_cid = 'D2TS'                        -- IS a tablespace
    AND dbts_id IS NOT NULL                       -- dbts_id has actual value
    AND DB2_TABLE IS NULL                         -- UPDATE only tablespace records 
    AND rediscovered = 0                          -- Was NOT previously rediscovered

  -- Display DB2Tablespace
  IF @verbose = 1
    SELECT 'DB2Tablespace', * FROM DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                        -- USE the current batch 
    AND d.obj_cid = 'D2TS'                        -- IS an instance of D2TS

  -- Rediscover Tables: Step 1, UPDATE dbtb_id IN DB2Discovery records 
  UPDATE child SET
    dbtb_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2Table_Rediscovery AS obj ON          -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbtb_cid
      AND obj.name = child.DB2_TABLE
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbts_cid        
      AND pc.src_id = child.dbts_id 
      AND (pc.dst_cid = 'D2TB' OR pc.dst_cid = 'D2TP')
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_TABLE IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbtb_cid               -- Restrict match to the discovery record class

  -- Rediscover TABLE: Step 2, UPDATE rediscovered, obj_id FOR DB2 TABLE objects that have dbtb_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbtb_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                      -- Reference the current batch
    AND d.dbtb_id        IS NOT NULL          -- Only UPDATE records WHERE the TABLE IS known 
    AND d.DB2_INDEX      IS NULL              -- Only UPDATE TABLE records (IF it has a DB2_INDEX record it IS child)

  -- CREATE any tables that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbtb_cid IS NOT NULL   -- There IS a TABLE
      AND d.dbtb_id  IS NULL       -- TABLE IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbts_cid',
      @ParentIDColumn  = 'dbts_id',
      @ChildCIDColumn  = 'dbtb_cid',
      @ChildIDColumn   = 'dbtb_id',
      @ChildNameColumn = 'DB2_TABLE',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbts_id IS NOT NULL 
                             AND (d.obj_cid = ''D2TB'' OR d.obj_cid = ''D2TP'')
                             AND d.dbtb_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' tables'
    END
  END

  -- UPDATE TABLE obj_id FOR those tables that were just created
  UPDATE d SET
    obj_id      = dbtb_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2TB' OR d.obj_cid = 'D2TP') -- IS a TABLE (OR tablespace partition)
    AND dbtb_id IS NOT NULL                        -- dbtb_id has actual value
    AND DB2_INDEX IS NULL                          -- UPDATE only TABLE records 
    AND rediscovered = 0                           -- Was NOT previously rediscovered

  -- Display DB2 Tables (AND Partitions)
  IF @verbose = 1
    SELECT 'DB2Table', * FROM DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2TB' OR d.obj_cid = 'D2TP') -- IS a TABLE (OR tablespace partition)

  -- Rediscover Index: Step 1, UPDATE dbin_id IN DB2Discovery records 
  UPDATE child SET
    dbin_id = obj.id
  FROM
    DB2Discovery AS child
    JOIN DB2Index_Rediscovery AS obj ON          -- Find any existing DB2 TABLE objects by class,name
      obj.cid = child.dbin_cid
      AND obj.name = child.DB2_INDEX
    JOIN link AS pc ON                           -- Restrict by parent subcomponent
      pc.src_cid = child.dbtb_cid        
      AND pc.src_id = child.dbtb_id 
      AND (pc.dst_cid = 'D2IX' OR pc.dst_cid = 'D2IP')
      AND pc.dst_id = obj.id
      AND pc.deleted = 0                         -- USE only Active objects
  WHERE
    child.batchid = @batchid                      -- Reference the current batch
    AND child.DB2_INDEX IS NOT NULL               -- Only attempt to rediscovery DB2_TABLESPACE with actual names
    AND pc.dst_cid = child.dbin_cid               -- Restrict match to the discovery record class

  -- Rediscover Index: Step 2, UPDATE rediscovered, obj_id FOR DB2 Index objects that have dbin_id VALUES
  UPDATE d SET
    rediscovered = 1,
    obj_id       = dbin_id
  FROM 
    DB2Discovery AS d
  WHERE 
    d.batchid = @batchid                      -- Reference the current batch
    AND d.dbin_id        IS NOT NULL          -- Only UPDATE records WHERE the TABLE IS known 

  -- CREATE any indexes that were NOT rediscovered
  IF EXISTS (
  SELECT * FROM DB2Discovery AS d
    WHERE d.batchid = @batchid 
      AND d.dbin_cid IS NOT NULL   -- There IS an index
      AND d.dbin_id  IS NULL       -- Index IS unknown (NOT rediscovered)
  )
  BEGIN
    EXEC @rc = _CreateDiscoveredObjects
      @cid,
      @id,
      @batchid         = @batchid,
      @DiscoveryTable  = 'DB2Discovery',
      @LinkTypeColumn  = '''AGGC''',
      @ParentCIDColumn = 'dbtb_cid',
      @ParentIDColumn  = 'dbtb_id',
      @ChildCIDColumn  = 'dbin_cid',
      @ChildIDColumn   = 'dbin_id',
      @ChildNameColumn = 'DB2_INDEX',
      @WhereClause     = 'WHERE d.os_id IS NOT NULL 
                             AND d.dbtb_id IS NOT NULL 
                             AND ( d.obj_cid = ''D2IX'' OR d.obj_cid = ''D2IP'' )
                             AND d.dbin_id IS NULL 
                         ',
      @AdditionalDiscoveryColumns = ', obj_path',
      @AdditionalAttributeColumns = ', _Path',
      @verbose         = @verbose
    IF @rc > 0
    BEGIN
      PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ':Discovered ' + CAST(@rc AS NVARCHAR) + ' indexes'
    END
  END

  -- UPDATE index obj_id FOR those indexes that were just created
  UPDATE d SET
    obj_id      = dbin_id
  FROM 
    DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2IX' OR d.obj_cid = 'D2IP') -- IS an index (FOR TABLE OR partition)
    AND dbin_id IS NOT NULL                        -- dbin_id has actual value
    AND rediscovered = 0                           -- Was NOT previously rediscovered

  -- Display DB2 Indexes (FOR Tables AND Partitions)
  IF @verbose = 1
    SELECT 'DB2Index', * FROM DB2Discovery AS d
  WHERE
    d.batchid   = @batchid                         -- USE the current batch 
    AND (d.obj_cid = 'D2IX' OR d.obj_cid = 'D2IP') -- IS an index (FOR TABLE OR partition)

  -- STATE: NOT assigning state information during discovery.

  -- IF records batch IS a VIEW TABLE relationship 17 08 THEN 
  -- IF records are D2TB INSERT those records INTO D2VW_D2TB_USEC FOR those that have NOT been seen before.
  -- to make this work, need to have the D2VW id be available IN the record.
  --   to make this happen the record needs  to be inserted INTO the discovery batch, before it arrives IN this stored proc.

  -- Special handling FOR 1708 records:
  -- Establish USEC link FROM D2VF to D2TB 

  IF EXISTS ( SELECT * FROM DiscoveryBatch WHERE batchid = @batchid AND ActionType = 8 )
  BEGIN

    SELECT batchid, obj_cid, obj_id, SystemId, DB2_SUBSYSTEM, DB2_SUBCOMPONENT, DB2_TABLESPACE, DB2_TABLE INTO #d2vw_detail 
    FROM DB2Discovery 
    WHERE 
          batchid = @batchid
      AND obj_cid IN ( 'D2VW', 'D2TB' )

    INSERT INTO D2VW_D2TB_USEC                             -- Establish the USEC relationship
    ( src_id, dst_id )
    SELECT d1.obj_id, d2.obj_id 
    FROM 
      #d2vw_detail  AS d1,
      #d2vw_detail  AS d2,
      DiscoveryBatch AS b, 
      DB2_VTRDIS   AS p  
    WHERE 
          b.batchid         = @batchid       -- match ON current batch
      AND d2.batchid        = @batchid
      AND p.batchid         = @batchid
      AND b.ActionType      = 8              -- Insert only on Table View relationship discoveries (1708)
      AND d1.obj_cid        = 'D2VW'         -- src record is the view
      AND d2.obj_cid        = 'D2TB'         -- dst record is the table
      AND p.SystemId        = d2.SystemId                     -- matching one record (p) to two (d#)
      AND p.SubsysName      = d2.DB2_SUBSYSTEM
      AND p.DATABASE_NAME   = d2.DB2_SUBCOMPONENT
      AND p.TABLESPACE_NAME = d2.DB2_TABLESPACE
      AND p.TABLE_NAME      = d2.DB2_TABLE
      AND p.SystemId        = d1.SystemId
      AND p.SubsysName      = d1.DB2_SUBSYSTEM
      AND p.VIEW_NAME       = d1.DB2_SUBCOMPONENT
      AND NOT EXISTS (                  -- but only IF the record has NOT been created before
        SELECT * FROM D2VW_D2TB_USEC AS l 
        WHERE 
              l.src_id = d1.obj_id 
          AND l.dst_id = d2.obj_id
      )
    PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Created ' + CAST(@@ROWCOUNT AS NVARCHAR) + ' D2VW to D2TB relationship(s)' 
    DROP TABLE #d2vw_detail 
  END


  PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Moving records FROM DB2Discovery INTO ProcessedDB2Discovery FOR batch ' + CAST(@batchid AS NVARCHAR)
  -- Move records FROM IMSDiscovery INTO ProcessedIMSDiscovery
  INSERT ProcessedDB2Discovery SELECT GETDATE(), * 
  FROM DB2Discovery WHERE batchid = @batchid
  IF @@ERROR = 0
    DELETE DB2Discovery WHERE batchid = @batchid

  -- Mark discovery batch AS completed
  UPDATE DiscoveryBatch SET pend = GETDATE(), pstat = v.pstat, error = 0
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'COMPLETED'

  IF @verbose = 1
    PRINT CONVERT(NVARCHAR(40), GETDATE(), 121) + ': _DiscoverDB2Resources: Finished'
GO

---------------------------------------------------------------------------------------------------
-- asisp_discoverDB2Resource
PRINT 'Creating PROCEDURE asisp_discoverDB2Resource'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_discoverDB2Resource' AND type = 'P')
  DROP PROCEDURE asisp_discoverDB2Resource
GO

-- $Archive:   //Archive/Pvcsroot/Archive_NLS/Db/Sql/DB2Discovery.sqv  $/Archive/DB/SQL/IMSDiscovery.sqv  $
-- $Revision:   1.5.1.2  $
-- $Date:   Jul 24 2001 15:33:14  $
-- $Author:   Forthman  $
-- Creates AND processes a discovery batch FOR DB2 using the specified parameters FOR a single object discovery
CREATE PROCEDURE asisp_discoverDB2Resource
  @FormatType        SMALLINT,
  @ActionType        SMALLINT,
  @TimeStamp         NVARCHAR(40),
  @SeqNo             NVARCHAR(16),
  @cid               ClassID ,
  @SystemId          NVARCHAR(40),
  @SubsysName        NVARCHAR(8) ,
  @MasterName        NVARCHAR(40)=NULL,
  @LockMgrName       NVARCHAR(40)=NULL,
  @DBSvcsName        NVARCHAR(40)=NULL,
  @DDFName           NVARCHAR(40)=NULL,
  @SPMgrName         NVARCHAR(40)=NULL,
  @BUFFER_POOLS      NVARCHAR(40)=NULL,
  @EDM_POOL          NVARCHAR(40)=NULL,
  @LOGS              NVARCHAR(40)=NULL,
  @DB2PM             NVARCHAR(40)=NULL,
  @DATABASE_NAME     NVARCHAR(40)=NULL,
  @TABLESPACE_NAME   NVARCHAR(40)=NULL,
  @TABLE_NAME        NVARCHAR(40)=NULL,
  @INDEX_NAME        NVARCHAR(40)=NULL,
  @TABLESPACE_PART   NVARCHAR(40)=NULL,
  @INDEX_PART        NVARCHAR(40)=NULL,
  @VIEW_NAME         NVARCHAR(40)=NULL,
  @STATE             NVARCHAR(40)=NULL,
  @auto_create       TINYINT = 1,
  @obj_cid           ClassID OUTPUT,
  @obj_id            ObjID OUTPUT,
  @obj_name          ObjName OUTPUT,
  @verbose           TINYINT = 0
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_discoverDB2Resource
  ELSE
    SAVE TRANSACTION asisp_discoverDB2Resource

  -- CREATE a discovery batch record
  DECLARE @now DATETIME
  SELECT @now = GETDATE()
  INSERT DiscoveryBatch(ctime, FormatType, ActionType, numrows) 
    SELECT @now, @FormatType, @ActionType, 1
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END

  DECLARE @batchid INT
  SELECT @batchid = batchid FROM DiscoveryBatch WHERE ctime = @now
  IF @batchid IS NULL
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END

  -- SET object name based ON @cid
  SELECT @obj_name =
    CASE @cid
      WHEN 'DBSS' THEN @SubsysName
      WHEN 'DBMS' THEN @MasterName
      WHEN 'DBLK' THEN @LockMgrName
      WHEN 'DBDS' THEN @DBSvcsName
      WHEN 'DBDD' THEN @DDFName
      WHEN 'DBSP' THEN @SPMgrName
      WHEN 'DBBP' THEN @BUFFER_POOLS
      WHEN 'DBEP' THEN @EDM_POOL
      WHEN 'DBLG' THEN @LOGS
      WHEN 'DBPM' THEN @DB2PM
      WHEN 'DBDB' THEN @DATABASE_NAME
      WHEN 'DBTS' THEN @TABLESPACE_NAME
      WHEN 'DBTB' THEN @TABLE_NAME
      WHEN 'DBIX' THEN @INDEX_NAME
      WHEN 'DBTP' THEN @TABLESPACE_PART
      WHEN 'DBIP' THEN @INDEX_PART
      WHEN 'DBVW' THEN @VIEW_NAME
    END

  -- SET discovered object type to the type of object this will be after discovery
  -- This IS to handle that the give cid (like DBSS) gets converted during processing (to D2SS),
  -- Certain queries IN this PROCEDURE need to be aware of this conversion 
  DECLARE @cid_discovered NVARCHAR(40) 
  SELECT @cid_discovered  =
    CASE @cid
      WHEN 'DBSS' THEN 'D2SS'
      WHEN 'DBMS' THEN 'D2MS' 
      WHEN 'DBLK' THEN 'D2LK' 
      WHEN 'DBDS' THEN 'D2DS' 
      WHEN 'DBDD' THEN 'D2DD' 
      WHEN 'DBSP' THEN 'D2SP' 
      WHEN 'DBBP' THEN 'D2BP' 
      WHEN 'DBEP' THEN 'D2EP' 
      WHEN 'DBLG' THEN 'D2LG' 
      WHEN 'DBPM' THEN 'D2PM' 
      WHEN 'DBDB' THEN 'D2DB' 
      WHEN 'DBTS' THEN 'D2TS' 
      WHEN 'DBTB' THEN 'D2TB' 
      WHEN 'DBIX' THEN 'D2IX' 
      WHEN 'DBTP' THEN 'D2TP' 
      WHEN 'DBIP' THEN 'D2IP' 
      WHEN 'DBVW' THEN 'D2VW' 
    END

  -- CREATE temp TABLE that gets populated with the class AND name of
  -- each object that must be created
  CREATE TABLE #object
  (
    obj_cid   ClassID,
    obj_name  ObjName,
    is_object TINYINT
  )

  -- INSERT record to CREATE the specified object
  INSERT INTO #object VALUES ( @cid, @obj_name, 1 )

  -- INSERT parent records IF necessary

  -- DB2 TABLE Indexes need parent TABLE record 
  IF @cid = 'DBIX'
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBTB', @TABLE_NAME, 0 )
  END

  -- DB2 TABLE Partition Indexes need parent TABLE Partition record 
  IF @cid = 'DBIP'
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBTP', @TABLESPACE_PART, 0 )
  END

  -- DB2 TABLE AND TABLE Partition AND children need parent Tablespace record 
  IF @cid IN ( 'DBTB', 'DBTP', 'DBIX', 'DBIP' )
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBTS', @TABLESPACE_NAME, 0 )
  END

  -- DB2 Tablespace AND children need parent Database record 
  IF @cid IN ( 'DBTS', 'DBTB', 'DBTP', 'DBIX', 'DBIP' )
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBDB', @DATABASE_NAME, 0 )
  END

  -- DB2 Subcomponent AND children need parent Subsystem record 
  IF @cid <> 'DBSS' 
  BEGIN
    INSERT INTO #object
      VALUES ( 'DBSS', @SubsysName, 0 )
  END

  -- INSERT DB2Discovery records based ON the rows that have been entered INTO #object temp TABLE
  INSERT INTO DB2Discovery
  (
    batchid,
    SystemId,
    obj_cid,
    DB2_SUBSYSTEM,
    DB2_SUBCOMPONENT,
    DB2_TABLESPACE,
    DB2_TABLE,
    DB2_INDEX,
    dbsu_cid,
    dbsc_cid,
    dbts_cid,
    dbtb_cid,
    dbin_cid,
    obj_name,
    DATE_TIME,
    SEQ_NO,
    STATE
  )
  SELECT DISTINCT
    @batchid,
    @SystemId,
    o.obj_cid,
    CASE
      WHEN @SubsysName    IS NOT NULL THEN @SubsysName
    END AS DB2_SUBSYSTEM,
    CASE
      WHEN o.obj_cid = 'DBSS'         THEN NULL
      WHEN @MasterName    IS NOT NULL THEN @MasterName
      WHEN @LockMgrName   IS NOT NULL THEN @LockMgrName
      WHEN @DBSvcsName    IS NOT NULL THEN @DBSvcsName
      WHEN @DDFName       IS NOT NULL THEN @DDFName
      WHEN @SPMgrName     IS NOT NULL THEN @SPMgrName
      WHEN @BUFFER_POOLS  IS NOT NULL THEN @BUFFER_POOLS
      WHEN @EDM_POOL      IS NOT NULL THEN @EDM_POOL
      WHEN @LOGS          IS NOT NULL THEN @LOGS
      WHEN @DB2PM         IS NOT NULL THEN @DB2PM
      WHEN @DATABASE_NAME IS NOT NULL THEN @DATABASE_NAME
      WHEN @VIEW_NAME     IS NOT NULL THEN @VIEW_NAME
    END AS DB2_SUBCOMPONENT,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTS', 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLESPACE_NAME IS NOT NULL THEN @TABLESPACE_NAME
    END AS DB2_TABLESPACE,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLE_NAME      IS NOT NULL THEN @TABLE_NAME
      WHEN @TABLESPACE_PART IS NOT NULL THEN @TABLESPACE_PART
    END AS DB2_TABLE,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBIX', 'DBIP' ) THEN NULL
      WHEN @INDEX_NAME IS NOT NULL THEN @INDEX_NAME
      WHEN @INDEX_PART IS NOT NULL THEN @INDEX_PART
    END AS DB2_INDEX,
    CASE
      WHEN @SubsysName    IS NOT NULL THEN 'D2SS'
    END AS dbsu_cid,
    CASE
      WHEN o.obj_cid = 'DBSS'         THEN NULL
      WHEN @MasterName    IS NOT NULL THEN 'D2MS'
      WHEN @LockMgrName   IS NOT NULL THEN 'D2LK'
      WHEN @DBSvcsName    IS NOT NULL THEN 'D2DS'
      WHEN @DDFName       IS NOT NULL THEN 'D2DD'
      WHEN @SPMgrName     IS NOT NULL THEN 'D2SP'
      WHEN @BUFFER_POOLS  IS NOT NULL THEN 'D2BP'
      WHEN @EDM_POOL      IS NOT NULL THEN 'D2EP'
      WHEN @LOGS          IS NOT NULL THEN 'D2LG'
      WHEN @DB2PM         IS NOT NULL THEN 'D2PM'
      WHEN @DATABASE_NAME IS NOT NULL THEN 'D2DB'
      WHEN @VIEW_NAME     IS NOT NULL THEN 'D2VW'
    END AS dbsc_cid,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTS', 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLESPACE_NAME IS NOT NULL THEN 'D2TS'
    END AS dbts_cid,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBTB', 'DBTP', 'DBIX', 'DBIP' ) THEN NULL
      WHEN @TABLE_NAME      IS NOT NULL THEN 'D2TB'
      WHEN @TABLESPACE_PART IS NOT NULL THEN 'D2TP'
    END AS dbtb_cid,
    CASE
      WHEN o.obj_cid NOT IN ( 'DBIX', 'DBIP' ) THEN NULL
      WHEN @INDEX_NAME IS NOT NULL THEN 'D2IX'
      WHEN @INDEX_PART IS NOT NULL THEN 'D2IP'
    END AS dbin_cid,
    o.obj_name,

    SUBSTRING(@TimeStamp,1,4)+'-'+SUBSTRING(@TimeStamp,5,2)+'-'+SUBSTRING(@TimeStamp,7,2)+' '+SUBSTRING(@TimeStamp,9,12),
    @SeqNo,
    CASE WHEN o.is_object = 1 THEN @STATE ELSE NULL END
  FROM
    #object AS o
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END

  -- Special handling FOR 1708 records:
  -- 1708 records FOR the D2TB actually contain two objects the TABLE AND the VIEW that points to that TABLE.
  -- Need to have the VIEW be discovered with the TABLE so that an INSERT to the D2VW_D2TB_USEC TABLE can be made later.
  IF ( @FormatType = 17 AND @ActionType = 8 AND @cid = 'DBTB' AND @VIEW_NAME IS NOT NULL )
  BEGIN
    INSERT INTO DB2Discovery
    (
      batchid,
      SystemId,
      obj_cid,
      DB2_SUBSYSTEM,
      DB2_SUBCOMPONENT,
      dbsu_cid,
      dbsc_cid,
      obj_name,
      DATE_TIME,
      SEQ_NO
    )
    SELECT DISTINCT
      @batchid,
      @SystemId,
      'DBVW',
      CASE
        WHEN @SubsysName    IS NOT NULL THEN @SubsysName
      END AS DB2_SUBSYSTEM,
      @VIEW_NAME,
      CASE
        WHEN @SubsysName    IS NOT NULL THEN 'D2SS'
      END AS dbsu_cid,
      CASE
        WHEN @VIEW_NAME     IS NOT NULL THEN 'D2VW'
      END AS dbsc_cid,
      CASE
        WHEN @VIEW_NAME     IS NOT NULL THEN @VIEW_NAME
      END AS obj_name,
    SUBSTRING(@TimeStamp,1,4)+'-'+SUBSTRING(@TimeStamp,5,2)+'-'+SUBSTRING(@TimeStamp,7,2)+' '+SUBSTRING(@TimeStamp,9,12),
    @SeqNo
  END
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_discoverDB2Resource
    RETURN -30000
  END


  DECLARE @r INT
  EXEC @r = _DiscoverDB2Resources ROOT, 1, @batchid, @incremental=1, @verbose=@verbose

  -- Special handling FOR 1708 records:
  -- Establish USEC link FROM D2VF to D2TB 

  IF ( @FormatType = 17 AND @ActionType = 8 AND @cid = 'DBTB' AND @VIEW_NAME IS NOT NULL )
  BEGIN
    PRINT 'IN THE SPECIAL HANDLING SECTION'
    INSERT INTO D2VW_D2TB_USEC                             -- Establish the USEC relationship
    ( src_id, dst_id )
    SELECT d1.obj_id, d2.obj_id FROM ProcessedDB2Discovery AS d1
    INNER JOIN ProcessedDB2Discovery AS d2 ON d1.batchid = d2.batchid  -- one TABLE FOR each of side of the relationship
    WHERE 
          d1.batchid = @batchid         -- match ON current batch
      AND d1.obj_cid = 'D2VW'           -- src record IS the VIEW 
      AND d2.obj_cid = 'D2TB'           -- dst record IS the TABLE
      AND NOT EXISTS (                  -- but only IF the record has NOT been created before
        SELECT * FROM D2VW_D2TB_USEC AS l 
        WHERE 
              l.src_id = d1.obj_id 
          AND l.dst_id = d2.obj_id
      )
  END


  SELECT @obj_cid = obj_cid, @obj_id = obj_id, @obj_name = obj_name
  FROM ProcessedDB2Discovery
  WHERE batchid = @batchid AND obj_cid = @cid_discovered AND obj_name = @obj_name

  IF @tcount = 0
    COMMIT TRANSACTION asisp_discoverDB2Resource

  RETURN @r
GO

-- Scheduled Task: 'DB2 Discovery Process'

-----------------------------------------------------------------------------------------------------------------------------------
-- Scheduled Task: 'DB2 Discovery Load'
--
--EXEC msdb..sp_delete_job @job_name='DB2 Discovery Load'
DECLARE @job_name sysname, @job_id UNIQUEIDENTIFIER, @ret INT
SET @job_name = 'DB2 Discovery Load'
IF NOT EXISTS (SELECT * FROM msdb..sysjobs WHERE name = @job_name)
BEGIN
  PRINT 'Adding job ''' + @job_name + '''.'
  PRINT 'USE SQL Enterprise Manager to define a schedule FOR this job.'
  EXEC @ret = msdb..sp_add_job
    @job_name = @job_name,
    @description = 'Loading job FOR DB2 discovery batches.',
    @job_id = @job_id OUTPUT
  IF @ret = 0
  BEGIN
    EXEC msdb..sp_add_jobstep
      @job_id = @job_id,
      @subsystem = 'TSQL',
      @command = '_LoadDB2DiscoveryBatch ROOT, 0',
      @database_name = 'Object',
      @step_name = 'Load'
  END
END
GO

-----------------------------------------------------------------------------------------------------------------------------------
-- Scheduled Task: 'DB2 Discovery Process'
--
--EXEC msdb..sp_delete_job @job_name='DB2 Discovery Process'
DECLARE @job_name sysname, @job_id UNIQUEIDENTIFIER, @ret INT
SET @job_id = NULL
SET @job_name = 'DB2 Discovery Process'
IF NOT EXISTS (SELECT * FROM msdb..sysjobs WHERE name = @job_name)
BEGIN
  PRINT 'Adding job ''' + @job_name + '''.'
  EXEC @ret = msdb..sp_add_job
    @job_name = @job_name,
    @description = 'Processing job FOR DB2 discovery batches.',
    @job_id = @job_id OUTPUT
  IF @ret = 0
  BEGIN
    EXEC msdb..sp_add_jobstep
      @job_id = @job_id,
      @subsystem = 'TSQL',
      @command = '_DiscoverDB2Resources ROOT, 0',
      @database_name = 'Object',
      @step_name = 'Load'

    -- associate the job with a server
    EXEC msdb..sp_add_jobserver @job_name=@job_name, @server_name=@@SERVERNAME

    -- SET up the schedule FOR which to run the job
    EXEC msdb..sp_add_jobschedule 
      @name='DB2DiscoveryProcessSchedule',
      @job_name=@job_name,
      @enabled=1,
      @freq_type=4,              -- run daily
      @freq_interval=1,          -- run every day
      @freq_subday_type=4,       -- run by minute
      @freq_subday_interval=1    -- every minute
  END
END
GO


-----------------------------------------------------------------------------------------------------------------------------------
PRINT 'Done.'
GO


