USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = N'asisp_processEventViewerFilters' AND type = N'P')
  DROP PROCEDURE asisp_processEventViewerFilters
GO
PRINT 'Creating procedure asisp_processEventViewerFilters...'
GO
CREATE PROCEDURE asisp_processEventViewerFilters
  @Filter	NVARCHAR(1000),
  @MESGFilter	NVARCHAR(1000) = NULL OUTPUT,
  @EXCPFilter	NVARCHAR(1000) = NULL OUTPUT,
  @ObjFilter	NVARCHAR(1000) = NULL OUTPUT,
  @do_mesg	TINYINT = 1 OUTPUT,
  @do_excp	TINYINT = 1 OUTPUT
AS

  -- standard TRANSACTION logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_processEventViewerFilters
  ELSE
    SAVE TRAN asisp_processEventViewerFilters

  DECLARE @continue TINYINT
  DECLARE @cur_index INT, @esc_index INT, @prev_index INT, @delim NVARCHAR(100), @delim_escape NVARCHAR(100)
  SET @delim = '&'
  SET @delim_escape = '&&'
  SET @continue = 1
  SET @cur_index=0
  SET @prev_index=1

  WHILE @continue = 1
  BEGIN

    -- find the character indices
    DECLARE @ti INT
    SET @prev_index = @cur_index+1
    SET @cur_index = CHARINDEX(@delim, @Filter, @cur_index+1)
    SET @esc_index = CHARINDEX(@delim_escape, @Filter, @cur_index-1)
    WHILE (@esc_index = @cur_index OR (@cur_index-@esc_index < LEN(@delim_escape) AND @cur_index-@esc_index > 0)) AND @esc_index > 0 AND @cur_index > 0
    BEGIN
      SET @cur_index = CHARINDEX(@delim, @Filter, @cur_index+LEN(@delim_escape))
      SET @ti = CHARINDEX(@delim_escape, @Filter, @cur_index)
      IF @ti > 0
        SET @esc_index = @ti
    END
    IF @cur_index > 0
      SET @continue = 1
    ELSE
      SET @continue = 0
    -- process the record
    IF @cur_index > 0
    BEGIN
      DECLARE @value NVARCHAR(1000)
      SET @value = SUBSTRING(@Filter, @prev_index, @cur_index-@prev_index)
      -- replace the escape sequence with the delimeter
      SET @esc_index = CHARINDEX(@delim_escape, @value)
      WHILE @esc_index > 0
      BEGIN
        SET @value = SUBSTRING(@value, 1, @esc_index-1) + REPLACE(SUBSTRING(@value, @esc_index, LEN(@delim_escape)), @delim_escape, @delim) + SUBSTRING(@value, @esc_index+LEN(@delim_escape), LEN(@value)-(@esc_index+LEN(@delim_escape))+1)
        SET @esc_index = CHARINDEX(@delim_escape, @value, @esc_index+LEN(@delim))
      END

      -- translate some of the filter values
      IF CHARINDEX('evt_cid', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_cid')), 'evt_cid', 'oc.cid') + SUBSTRING(@value, LEN('evt_cid')+1, LEN(@value)-LEN('evt_cid'))
      ELSE IF CHARINDEX('obj_cid', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('obj_cid')), 'obj_cid', 'cid') + SUBSTRING(@value, LEN('obj_cid')+1, LEN(@value)-LEN('obj_cid'))
      ELSE IF CHARINDEX('evt_datetime', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_datetime')), 'evt_datetime', '_DATE') + SUBSTRING(@value, LEN('evt_datetime')+1, LEN(@value)-LEN('evt_datetime'))
      ELSE IF CHARINDEX('evt_alertstatestr', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_alertstatestr')), 'evt_alertstatestr', 'ev1.label') + SUBSTRING(@value, LEN('evt_alertstatestr')+1, LEN(@value)-LEN('evt_alertstatestr'))
      ELSE IF CHARINDEX('evt_alertstate', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_alertstate')), 'evt_alertstate', 'events._AlertState') + SUBSTRING(@value, LEN('evt_alertstate')+1, LEN(@value)-LEN('evt_alertstate'))
      ELSE IF CHARINDEX('evt_prioritystr', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_prioritystr')), 'evt_prioritystr', 'ev2.label') + SUBSTRING(@value, LEN('evt_prioritystr')+1, LEN(@value)-LEN('evt_prioritystr'))
      ELSE IF CHARINDEX('evt_priority', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_priority')), 'evt_priority', '_Priority') + SUBSTRING(@value, LEN('evt_priority')+1, LEN(@value)-LEN('evt_priority'))
      ELSE IF CHARINDEX('evt_seq_no', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_seq_no')), 'evt_seq_no', '_SEQ_NO') + SUBSTRING(@value, LEN('evt_seq_no')+1, LEN(@value)-LEN('evt_seq_no'))
      ELSE IF CHARINDEX('evt_name', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_name')), 'evt_name', 'events.name') + SUBSTRING(@value, LEN('evt_name')+1, LEN(@value)-LEN('evt_name'))
      ELSE IF CHARINDEX('evt_count', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_count')), 'evt_count', '_nEXCH') + SUBSTRING(@value, LEN('evt_count')+1, LEN(@value)-LEN('evt_count'))
      ELSE IF CHARINDEX('evt_statestr', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_statestr')), 'evt_statestr', 'st._State') + SUBSTRING(@value, LEN('evt_statestr')+1, LEN(@value)-LEN('evt_statestr'))
      ELSE IF CHARINDEX('evt_state', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_state')), 'evt_state', '_StateID') + SUBSTRING(@value, LEN('evt_state')+1, LEN(@value)-LEN('evt_state'))
      ELSE IF CHARINDEX('note_id', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('note_id')), 'note_id', 'ISNULL(noteid,0)') + SUBSTRING(@value, LEN('note_id')+1, LEN(@value)-LEN('note_id'))
      ELSE IF CHARINDEX('evt_id', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('evt_id')), 'evt_id', 'events.id') + SUBSTRING(@value, LEN('evt_id')+1, LEN(@value)-LEN('evt_id'))
      ELSE IF CHARINDEX('obj_id', @value) = 1
        SET @value = REPLACE(SUBSTRING(@value, 1, LEN('obj_id')), 'obj_id', '_ParentID') + SUBSTRING(@value, LEN('obj_id')+1, LEN(@value)-LEN('obj_id'))

      -- apppend the value to the proper filter
      IF CHARINDEX('resource', @value) = 1 OR CHARINDEX('cid', @value) = 1
      BEGIN
        IF @ObjFilter IS NULL
          SET @ObjFilter = @value
        ELSE
          SET @ObjFilter = @ObjFilter + N' AND ' + @value
      END
      ELSE
      BEGIN
        DECLARE @mesg_value NVARCHAR(1000), @excp_value NVARCHAR(1000)
        IF CHARINDEX('evt_detail', @value) = 1
        BEGIN
          SET @mesg_value = REPLACE(SUBSTRING(@value, 1, LEN('evt_detail')), 'evt_detail', '_REASON_CD') + SUBSTRING(@value, LEN('evt_detail')+1, LEN(@value)-LEN('evt_detail'))
          SET @excp_value = REPLACE(SUBSTRING(@value, 1, LEN('evt_detail')), 'evt_detail', '_EXCP_CD') + SUBSTRING(@value, LEN('evt_detail')+1, LEN(@value)-LEN('evt_detail'))
        END
        ELSE IF CHARINDEX('_StateID', @value) = 1 OR CHARINDEX('st._State', @value) = 1
        BEGIN
          SET @mesg_value = @value
          SET @do_excp = 0
        END
        ELSE IF CHARINDEX('_nEXCH', @value) = 1
        BEGIN
          SET @excp_value = @value
          SET @do_mesg = 0
        END
        ELSE
        BEGIN
          SET @excp_value = @value
          SET @mesg_value = @value
        END

        IF @do_mesg = 1 AND @MESGFilter IS NULL
          SET @MESGFilter = @mesg_value
        ELSE IF @do_mesg = 1
          SET @MESGFilter = @MESGFilter + N' AND ' + @mesg_value
        IF @do_excp = 1 AND @EXCPFilter IS NULL
          SET @EXCPFilter = @excp_value
        ELSE IF @do_excp = 1
          SET @EXCPFilter = @EXCPFilter + N' AND ' + @excp_value
      END
    END
  END

  -- finish transaction
  IF @tcount = 0
    COMMIT TRAN asisp_processEventViewerFilters
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = N'_GetEventViewerEvents' AND type = N'P')
  DROP PROCEDURE _GetEventViewerEvents
GO
PRINT 'Creating procedure _GetEventViewerEvents...'
GO
CREATE PROCEDURE _GetEventViewerEvents
  @cid			ClassID,
  @id			ObjID,
  @root_cid		ClassID = NULL,
  @root_id		ObjID = 0,
  @Filters		NVARCHAR(1000) = NULL,
  @OrderBy		NVARCHAR(1000) = NULL,
  @MaxRows		INT = 0,
  @GetChildren		TINYINT = 1,
  @Direction		TINYINT = 1,
  @mesg_id		ObjID = 0 OUTPUT,
  @excp_id		ObjID = 0 OUTPUT,
  @numevents		INT = 0 OUTPUT,
  @numfilteredevents	INT = 0 OUTPUT
AS

  -- GetChildren:
  -- 0: don't get child instances
  -- <> 0: get child instances

  -- Direction:
  -- 0: go backwards
  -- 1: go forwards

  SET NOCOUNT ON

  -- transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _GetEventViewerEvents
  ELSE
    SAVE TRAN _GetEventViewerEvents

  -- temp tables
  CREATE TABLE #descendent
  (
    cid		ClassID,
    id		ObjID,
    gen		INT
  )
  CREATE TABLE #obj
  (
    cid			ClassID,
    id			ObjID,
    src_cid		ClassID NULL,
    src_id		ObjID NULL,
    phys_cid		ClassID NULL,
    phys_id		ObjID NULL,
    resource		ObjName NULL,
    resourcepath	NVARCHAR(255) NULL
  )
  CREATE TABLE #obj_temp
  (
    cid			ClassID,
    id			ObjID,
    src_cid		ClassID NULL,
    src_id		ObjID NULL,
    phys_cid		ClassID NULL,
    phys_id		ObjID NULL,
    resource		ObjName NULL,
    resourcepath	NVARCHAR(255) NULL
  )
  CREATE TABLE #events
  (
    obj_cid		ClassID, -- not visible
    obj_id		ObjID, -- not visible
    evt_cid		ClassID,
    evt_id		ObjID, -- not visible
    evt_name		ObjName,
    evt_alertstate	INT,
    evt_alertstatestr	NVARCHAR(40),
    evt_priority	INT,
    evt_prioritystr	NVARCHAR(40),
    evt_state		INT,
    evt_statestr	NVARCHAR(40),
    evt_detail		NVARCHAR(255),
    evt_count		INT,
    evt_seq_no		NVARCHAR(50),
    evt_datetime	DATETIME,
    note_id		ObjID DEFAULT 0,
    resource		ObjName NULL,
    resourcepath	NVARCHAR(255) NULL
  )
  CREATE TABLE #final_events
  (
    id			INT IDENTITY(1,1),
    obj_cid		ClassID,
    obj_id		ObjID,
    evt_cid		ClassID,
    evt_id		ObjID,
    evt_name		ObjName,
    evt_alertstate	INT,
    evt_alertstatestr	NVARCHAR(40),
    evt_priority	INT,
    evt_prioritystr	NVARCHAR(40),
    evt_state		INT,
    evt_statestr	NVARCHAR(40),
    evt_detail		NVARCHAR(255),
    evt_count		INT,
    evt_seq_no		NVARCHAR(50),
    evt_datetime	DATETIME,
    note_id		ObjID DEFAULT 0,
    resource		ObjName NULL,
    resourcepath	NVARCHAR(255) NULL
  )
  create table #numevents
  (
    mesg_count	INT,
    excp_count	INT
  )

  -- populate the #numevents table
  INSERT INTO #numevents SELECT 0, 0

  -- see if the caller used the root_cid and root_id params
  IF @root_cid IS NOT NULL AND @root_id IS NOT NULL AND @root_id <> 0
    SELECT @cid = @root_cid, @id = @root_id

  -- get the starting place
  DECLARE @MaxRowsStr NVARCHAR(10), @DirectionStr NVARCHAR(10), @mesg_id_str NVARCHAR(10), @excp_id_str NVARCHAR(10)
  IF @excp_id IS NULL
    SET @excp_id = 0
  IF @mesg_id IS NULL
    SET @mesg_id = 0
  SET @MaxRowsStr = CAST(@MaxRows AS NVARCHAR(10))
  SET @DirectionStr = CAST(@Direction AS NVARCHAR(10))
  SET @mesg_id_str = CAST(@mesg_id AS NVARCHAR(10))
  SET @excp_id_str = CAST(@excp_id AS NVARCHAR(10))

  -- get the descendent objects
  IF (RTRIM(@cid) = 'LOB' OR RTRIM(@cid) = 'LOBC' OR RTRIM(@cid) = 'ROOT') AND @GetChildren <> 0
  BEGIN
    DECLARE @lobc_id ObjID
    IF RTRIM(@cid) = 'LOBC'
      SET @lobc_id = @id
    ELSE IF RTRIM(@cid) = 'ROOT'
    BEGIN
      EXEC asisp_descendents @cid, @id, "LIKE N'%'"
      SELECT @lobc_id = dst_id FROM containment WHERE src_cid = 'ROOT' AND src_id = 0 AND dst_cid = 'LOBC'
    END
    CREATE TABLE #lob_descendent
    (
      id  ObjID,
      gen INT
    )
    IF RTRIM(@cid) = 'LOBC' OR RTRIM(@cid) = 'ROOT'
    BEGIN
      IF RTRIM(@cid) = 'ROOT'
        INSERT INTO #descendent SELECT 'LOBC', @lobc_id, 1
      ELSE
        INSERT INTO #descendent SELECT 'LOBC', @lobc_id, 0
      DECLARE @dst_id ObjID
      DECLARE c_lobc INSENSITIVE CURSOR FOR SELECT dst_id FROM containment WHERE src_cid = 'LOBC' AND src_id = @lobc_id AND dst_cid = 'LOB' AND deleted = 0
      OPEN c_lobc
      FETCH NEXT FROM c_lobc INTO @dst_id
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        INSERT INTO #lob_descendent SELECT 0, @dst_id
        INSERT INTO #lob_descendent SELECT dst_id, gen FROM lob_link WHERE src_id = @dst_id ORDER BY gen
        FETCH NEXT FROM c_lobc INTO @dst_id
      END
      DEALLOCATE c_lobc
      INSERT INTO #descendent SELECT 'LOB', * FROM #lob_descendent
    END
    ELSE
    BEGIN
      INSERT INTO #lob_descendent SELECT 0, @id
      INSERT INTO #lob_descendent SELECT dst_id, gen FROM lob_link WHERE src_id = @id ORDER BY gen
    END
    INSERT INTO #descendent SELECT 'LOB', * FROM #lob_descendent
  END
  ELSE IF @GetChildren <> 0
    EXEC asisp_descendents @cid, @id, "LIKE N'%'"
  ELSE
    INSERT INTO #descendent SELECT @cid, @id, 0

  -- populate the #obj table
  INSERT INTO #obj (cid, id)
    SELECT cid, id from #descendent
  UPDATE o
    SET o.src_cid = l.src_cid, o.src_id = l.src_id
    FROM #obj AS o
      JOIN link AS l ON l.dst_cid = o.cid AND l.dst_id = o.id AND l.deleted = 0
      JOIN link_type AS lt ON lt.link_type = l.link_type
    WHERE (lt.flag_containment = 1 OR lt.flag_parent_child = 1)

  -- update the resource and resourcepath columns
  DECLARE @c_cid ClassID, @c_cname ObjName
  DECLARE resource_cursor INSENSITIVE CURSOR FOR SELECT DISTINCT o.cid, c.cname FROM #obj AS o JOIN obj_class AS c ON c.cid = o.cid
  OPEN resource_cursor
  FETCH NEXT FROM resource_cursor INTO @c_cid, @c_cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC("
      UPDATE obj
      SET obj.resource = o.name, obj.resourcepath = o.name
      FROM #obj AS obj JOIN " + @c_cname + "_V AS o ON o.id = obj.id
      WHERE obj.cid = N'" + @c_cid + "'
    ")
    FETCH NEXT FROM resource_cursor INTO @c_cid, @c_cname
  END
  DEALLOCATE resource_cursor
  DECLARE @src_cid ClassID, @rc INT, @loop INT
  SELECT @rc = COUNT(*) FROM #obj
  SET @c_cname = N''
  SET @loop = 0
  WHILE @rc > 0 AND @loop < 10
  BEGIN
    DECLARE resourcepath_cursor INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid, cname FROM #obj o JOIN obj_class oc ON oc.cid = o.src_cid
    OPEN resourcepath_cursor
    FETCH NEXT FROM resourcepath_cursor INTO @src_cid, @c_cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC("
      UPDATE obj SET src_cid = c.src_cid, src_id = c.src_id, resourcepath = parent.name + N'/' + resourcepath
      FROM #obj AS obj
        JOIN " + @c_cname + "_C AS parent ON parent.id = obj.src_id
        LEFT OUTER JOIN link AS c ON c.dst_cid = obj.src_cid AND c.dst_id = obj.src_id
        JOIN link_type AS lt ON lt.link_type = c.link_type
      WHERE obj.src_cid = N'" + @src_cid + "' AND obj.src_cid <> N'BUSC'
        AND (lt.flag_containment = 1 OR lt.flag_parent_child = 1)
      ")
      FETCH NEXT FROM resourcepath_cursor INTO @src_cid, @c_cname
    END
    DEALLOCATE resourcepath_cursor 
    SELECT @rc = COUNT(*) FROM #obj WHERE src_cid <> N'ENT' AND src_cid IS NOT NULL
    SET @loop = @loop + 1
  END

  -- special casing for LOBs
  UPDATE o
  SET o.phys_cid = l.dst_cid, o.phys_id = l.dst_id
  FROM #obj AS o JOIN link AS l ON o.cid = l.src_cid AND o.id = l.src_id AND l.link_type = 'LOBL' AND deleted = 0
  UPDATE #obj
  SET phys_cid = cid, phys_id = id
  WHERE phys_cid IS NULL AND phys_id IS NULL AND cid <> 'LOB'
  DELETE FROM #obj WHERE phys_cid IS NULL OR phys_id IS NULL

  -- before doing any filtering, get a count of the total number of events
  SELECT @numevents = COUNT(*) FROM Message_V AS events JOIN #obj AS d ON events._ParentCID = d.phys_cid AND events._ParentID = d.phys_id AND events.deleted = 0
  SELECT @numevents = @numevents + COUNT(*)
    FROM Exception_V AS events JOIN #obj AS d ON events._ParentCID = d.phys_cid AND events._ParentID = d.phys_id
      LEFT OUTER JOIN OwnedEvent oe ON oe.eventcid = 'EXCP' AND oe.eventid = events.id
      LEFT OUTER JOIN Note_V AS note ON oe.noteid = note.id
    WHERE events.deleted = 0 OR (oe.noteid <> 0 AND oe.noteid IS NOT NULL AND note._Status = 'Open')

  -- depending upon sorting and filtering, we may not be doing both messages and exceptions
  DECLARE @do_mesg TINYINT, @do_excp TINYINT
  SELECT @do_mesg = 1, @do_excp = 1

  -- set up the filters
  DECLARE @MESGFilter NVARCHAR(1000), @EXCPFilter NVARCHAR(1000), @ObjFilter NVARCHAR(1000)
  EXEC asisp_processEventViewerFilters @Filters, @MESGFilter OUTPUT, @EXCPFilter OUTPUT, @ObjFilter OUTPUT, @do_mesg OUTPUT, @do_excp OUTPUT

  -- apply the resource filter to the objects
  IF @ObjFilter IS NULL
    SET @ObjFilter = N''
  ELSE
    SET @ObjFilter = N' WHERE ' + @ObjFilter
  EXEC("
  INSERT INTO #obj_temp SELECT * FROM #obj " + @ObjFilter + "
  DELETE FROM #obj
  INSERT INTO #obj SELECT * FROM #obj_temp
  ")

  -- set up the order by clause
  DECLARE @orderby_clause NVARCHAR(1000), @orderby_clause_evt NVARCHAR(1000), @orderby_clause_excp NVARCHAR(1000), @orderby_clause_mesg NVARCHAR(1000)
  IF @OrderBy IS NOT NULL AND CHARINDEX('evt_datetime', @OrderBy) = 0
    SET @orderby_clause_evt = "ORDER BY " + @OrderBy + ", evt_datetime DESC"
  ELSE IF @OrderBy IS NULL
    SET @orderby_clause_evt = "ORDER BY evt_datetime DESC"
  ELSE
    SET @orderby_clause_evt = "ORDER BY " + @OrderBy

  SET @orderby_clause = REPLACE(@orderby_clause_evt, 'evt_datetime', '_DATE')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_alertstatestr', 'ev1.label')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_alertstate', 'events._AlertState')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_prioritystr', 'ev2.label')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_priority', '_Priority')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_seq_no', '_SEQ_NO')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_name', 'events.name')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_count', '_nEXCH')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_statestr', 'st._State')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_state', '_StateID')
  SET @orderby_clause = REPLACE(@orderby_clause, 'note_id', 'noteid')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_cid', 'oc.cid')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_id', 'events.id')
  SET @orderby_clause = REPLACE(@orderby_clause, 'obj_id', '_ParentID')
  -- object-specific order by clauses
  SET @orderby_clause_mesg = @orderby_clause
  SET @orderby_clause_excp = @orderby_clause
  SET @orderby_clause_excp = REPLACE(@orderby_clause_excp, 'evt_detail', '_EXCP_CD')
  SET @orderby_clause_mesg = REPLACE(@orderby_clause_mesg, 'evt_detail', '_REASON_CD')

  -- set up the message filter
  DECLARE @where_clause_mesg NVARCHAR(1000)
  SET @where_clause_mesg = N' WHERE events.deleted = 0'
  IF @MESGFilter IS NOT NULL
    SET @where_clause_mesg = @where_clause_mesg + N' AND (' + @MESGFilter + N')'
  IF @do_mesg = 1 AND CHARINDEX('_nEXCH', @orderby_clause_mesg) > 0
    SET @do_mesg = 0

  -- Populate the table with messages
  IF @do_mesg = 1
  BEGIN
    EXEC("
    DECLARE @rc INT, @foundid INT
    DECLARE @id INT, @ParentCID ClassID, @ParentID ObjID, @name ObjName, @AlertState INT, @Priority INT, @StateID INT, @REASON_CD NVARCHAR(255),
      @SEQ_NO NVARCHAR(50), @DATE DATETIME, @note_id ObjID, @alrs_str ObjName, @priority_str ObjName, @State NVARCHAR(40), @obj_cid ClassID, @obj_id ObjID
    DECLARE cursor__GetEventViewerEvents CURSOR FORWARD_ONLY DYNAMIC FOR
      SELECT _ParentCID, _ParentID, events.id, events.name, events._AlertState, _Priority, ISNULL(_StateID,0), st._State, _REASON_CD, _SEQ_NO, _DATE, ISNULL(oe.noteid, 0), ev1.label, ev2.label, d.cid, d.id
      FROM obj_class AS oc JOIN Message_V AS events ON oc.cid = 'MESG'
        JOIN #obj d ON events._ParentCID = d.phys_cid AND events._ParentID = d.phys_id
        LEFT OUTER JOIN OwnedEvent oe ON events.id = oe.eventid AND oe.eventcid = 'MESG'
        LEFT OUTER JOIN (enum_value AS ev1 JOIN enumeration AS enum1 ON enum1.eid = ev1.eid AND enum1.name = 'AlertState') ON ev1.value = events._AlertState
        LEFT OUTER JOIN (enum_value AS ev2 JOIN enumeration AS enum2 ON enum2.eid = ev2.eid AND enum2.name = 'EventPriority') ON ev2.value = events._Priority
        LEFT OUTER JOIN StateType_V st ON st.id = events._StateID
      " + @where_clause_mesg + "
      " + @orderby_clause_mesg + "
    OPEN cursor__GetEventViewerEvents
    UPDATE #numevents SET mesg_count = @@CURSOR_ROWS
    FETCH NEXT FROM cursor__GetEventViewerEvents INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @StateID, @State, @REASON_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str, @obj_cid, @obj_id
    IF " + @MaxRowsStr + " > 0 AND " + @DirectionStr + " = 1
      SELECT @rc = 1, @foundid = 0
    ELSE
      SELECT @rc = 0, @foundid = 0
    WHILE @@FETCH_STATUS <> -1 AND @rc <= " + @MaxRowsStr + "
    BEGIN
      IF (" + @DirectionStr + " = 1 AND (@foundid = 1 OR " + @mesg_id_str + " = 0)) OR (" + @DirectionStr + " = 0 AND @foundid = 0 AND " + @mesg_id_str + " > 0)
      BEGIN
        INSERT INTO #events
        SELECT @obj_cid, @obj_id, N'MESG', @id, @name, @AlertState, @alrs_str, @Priority, @priority_str, @StateID, @State, @REASON_CD, -1, @SEQ_NO, @DATE, ISNULL(@note_id, 0), NULL, NULL
        IF " + @MaxRowsStr + " > 0 AND " + @DirectionStr + " = 1
          SET @rc = @rc + 1
      END
      IF @foundid = 0 AND @id = " + @mesg_id_str + "
        SET @foundid = 1
      FETCH NEXT FROM cursor__GetEventViewerEvents INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @StateID, @State, @REASON_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str, @obj_cid, @obj_id
    END
    DEALLOCATE cursor__GetEventViewerEvents
    IF " + @DirectionStr + " = 0
    BEGIN
      DECLARE cursor_reverse CURSOR STATIC FOR
        SELECT obj_cid, obj_id, evt_id, evt_name, evt_alertstate, evt_priority, evt_state, evt_statestr, evt_detail, evt_seq_no, evt_datetime, note_id, evt_alertstatestr, evt_prioritystr
        FROM #events
        WHERE evt_cid = 'MESG'
        " + @orderby_clause_evt + "
      OPEN cursor_reverse
      DELETE FROM #events WHERE evt_cid = 'MESG'
      FETCH LAST FROM cursor_reverse INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @StateID, @State, @REASON_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str
      IF " + @MaxRowsStr + " > 0
        SELECT @rc = 1
      ELSE
        SELECT @rc = 0
      WHILE @@FETCH_STATUS <> -1 AND @rc <= " + @MaxRowsStr + "
      BEGIN
        INSERT INTO #events
        SELECT @ParentCID, @ParentID, N'MESG', @id, @name, @AlertState, @alrs_str, @Priority, @priority_str, @StateID, @State, @REASON_CD, -1, @SEQ_NO, @DATE, ISNULL(@note_id, 0), NULL, NULL
        IF " + @MaxRowsStr + " > 0
          SET @rc = @rc + 1
        FETCH PRIOR FROM cursor_reverse INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @StateID, @State, @REASON_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str
      END
      DEALLOCATE cursor_reverse
    END
    ")
  END

  -- set up the exception filters
  DECLARE @where_clause_excp NVARCHAR(1000)
  SET @where_clause_excp = N' WHERE (events.deleted = 0 OR (oe.noteid <> 0 AND oe.noteid IS NOT NULL AND note._Status = ''Open''))'
  IF @EXCPFilter IS NOT NULL
    SET @where_clause_excp = @where_clause_excp + N' AND (' + @EXCPFilter + N')'
  IF @do_excp = 1 AND (CHARINDEX('st._State', @orderby_clause_excp) > 0 OR CHARINDEX('_StateID', @orderby_clause_excp) > 0)
    SET @do_excp = 0

  -- Populate the table with exceptions
  IF @do_excp = 1
  BEGIN
    EXEC("
    DECLARE @rc INT, @foundid INT
    DECLARE @id INT, @ParentCID ClassID, @ParentID ObjID, @name ObjName, @AlertState INT, @Priority INT, @Count INT, @EXCP_CD NVARCHAR(255),
      @SEQ_NO NVARCHAR(50), @DATE DATETIME, @note_id ObjID, @alrs_str ObjName, @priority_str ObjName, @obj_cid ClassID, @obj_id ObjID
    DECLARE cursor__GetEventViewerEvents CURSOR FORWARD_ONLY DYNAMIC FOR
      SELECT _ParentCID, _ParentID, events.id, events.name, events._AlertState, _Priority, _nEXCH, _EXCP_CD, _SEQ_NO, _DATE, ISNULL(noteid, 0), ev1.label, ev2.label, d.cid, d.id
      FROM obj_class AS oc JOIN Exception_V AS events ON oc.cid = 'EXCP'
        JOIN #obj d ON events._ParentCID = d.phys_cid AND events._ParentID = d.phys_id
        LEFT OUTER JOIN OwnedEvent oe ON oe.eventid = events.id AND oe.eventcid = 'EXCP'
        LEFT OUTER JOIN Note_V AS note ON oe.noteid = note.id
        LEFT OUTER JOIN (enum_value AS ev1 JOIN enumeration AS enum1 ON enum1.eid = ev1.eid AND enum1.name = 'AlertState') ON ev1.value = events._AlertState
        LEFT OUTER JOIN (enum_value AS ev2 JOIN enumeration AS enum2 ON enum2.eid = ev2.eid AND enum2.name = 'EventPriority') ON ev2.value = events._Priority
      " + @where_clause_excp + "
      " + @orderby_clause_excp + "
    OPEN cursor__GetEventViewerEvents
    UPDATE #numevents SET excp_count = @@CURSOR_ROWS
    FETCH NEXT FROM cursor__GetEventViewerEvents INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @Count, @EXCP_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str, @obj_cid, @obj_id
    IF " + @MaxRowsStr + " > 0 AND " + @DirectionStr + " = 1
      SELECT @rc = 1, @foundid = 0
    ELSE
      SELECT @rc = 0, @foundid = 0
    WHILE @@FETCH_STATUS <> -1 AND @rc <= " + @MaxRowsStr + "
    BEGIN
      IF (" + @DirectionStr + " = 1 AND (@foundid = 1 OR " + @excp_id_str + " = 0)) OR (" + @DirectionStr + " = 0 AND @foundid = 0 AND " + @excp_id_str + " > 0)
      BEGIN
        INSERT INTO #events
        SELECT @obj_cid, @obj_id, N'EXCP', @id, @name, @AlertState, @alrs_str, @Priority, @priority_str, -1, NULL, @EXCP_CD, @Count, @SEQ_NO, @DATE, ISNULL(@note_id, 0), NULL, NULL
        IF " + @MaxRowsStr + " > 0 AND " + @DirectionStr + " = 1
          SET @rc = @rc + 1
      END
      IF @foundid = 0 AND @id = " + @excp_id_str + "
        SET @foundid = 1
      FETCH NEXT FROM cursor__GetEventViewerEvents INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @Count, @EXCP_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str, @obj_cid, @obj_id
    END
    DEALLOCATE cursor__GetEventViewerEvents
    IF " + @DirectionStr + " = 0
    BEGIN
      DECLARE cursor_reverse CURSOR STATIC FOR
        SELECT obj_cid, obj_id, evt_id, evt_name, evt_alertstate, evt_priority, evt_count, evt_detail, evt_seq_no, evt_datetime, note_id, evt_alertstatestr, evt_prioritystr
        FROM #events
        WHERE evt_cid = 'EXCP'
        " + @orderby_clause_evt + "
      OPEN cursor_reverse
      DELETE FROM #events WHERE evt_cid = 'EXCP'
      FETCH LAST FROM cursor_reverse INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @Count, @EXCP_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str
      IF " + @MaxRowsStr + " > 0
        SELECT @rc = 1
      ELSE
        SELECT @rc = 0
      WHILE @@FETCH_STATUS <> -1 AND @rc <= " + @MaxRowsStr + "
      BEGIN
        INSERT INTO #events
        SELECT @ParentCID, @ParentID, N'EXCP', @id, @name, @AlertState, @alrs_str, @Priority, @priority_str, -1, NULL, @EXCP_CD, @Count, @SEQ_NO, @DATE, ISNULL(@note_id, 0), NULL, NULL
        IF " + @MaxRowsStr + " > 0
          SET @rc = @rc + 1
        FETCH PRIOR FROM cursor_reverse INTO @ParentCID, @ParentID, @id, @name, @AlertState, @Priority, @Count, @EXCP_CD, @SEQ_NO, @DATE, @note_id, @alrs_str, @priority_str
      END
      DEALLOCATE cursor_reverse
    END
    ")
  END

  -- Populate the final table
  EXEC("
  IF " + @DirectionStr + " = 1
  BEGIN
    SET ROWCOUNT " + @MaxRowsStr + "
    INSERT INTO #final_events (obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr,
      evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath)
    SELECT *
    FROM #events
    " + @orderby_clause_evt + "
    SET ROWCOUNT 0
  END
  ELSE IF " + @DirectionStr + " = 0
  BEGIN
    DECLARE @obj_cid ClassID, @obj_id ObjID, @evt_cid ClassID, @evt_id ObjID, @evt_name ObjName, @evt_alertstate INT, @evt_alertstatestr NVARCHAR(40),
      @evt_priority INT, @evt_prioritystr NVARCHAR(40), @evt_state INT, @evt_statestr NVARCHAR(40), @evt_detail NVARCHAR(255), @evt_count INT, @evt_seq_no NVARCHAR(50),
      @evt_datetime DATETIME, @note_id ObjID, @resource NVARCHAR(255), @resourcepath NVARCHAR(255)
    DECLARE @rc INT
    IF " + @MaxRowsStr + " > 0
      SET @rc = 1
    ELSE
      SET @rc = 0
    DECLARE cursor_reverse_final CURSOR STATIC FOR
    SELECT *
    FROM #events
    " + @orderby_clause_evt + "
    OPEN cursor_reverse_final
    FETCH LAST FROM cursor_reverse_final INTO @obj_cid, @obj_id, @evt_cid, @evt_id, @evt_name, @evt_alertstate, @evt_alertstatestr, @evt_priority, @evt_prioritystr,
      @evt_state, @evt_statestr, @evt_detail, @evt_count, @evt_seq_no, @evt_datetime, @note_id, @resource, @resourcepath
    WHILE @@FETCH_STATUS <> -1 AND @rc <= " + @MaxRowsStr + "
    BEGIN
      INSERT INTO #final_events (obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr,
        evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath)
      SELECT @obj_cid, @obj_id, @evt_cid, @evt_id, @evt_name, @evt_alertstate, @evt_alertstatestr, @evt_priority, @evt_prioritystr,
        @evt_state, @evt_statestr, @evt_detail, @evt_count, @evt_seq_no, @evt_datetime, @note_id, @resource, @resourcepath
      FETCH PRIOR FROM cursor_reverse_final INTO @obj_cid, @obj_id, @evt_cid, @evt_id, @evt_name, @evt_alertstate, @evt_alertstatestr, @evt_priority, @evt_prioritystr,
        @evt_state, @evt_statestr, @evt_detail, @evt_count, @evt_seq_no, @evt_datetime, @note_id, @resource, @resourcepath
      IF " + @MaxRowsStr + " > 0
        SET @rc = @rc + 1
    END
    DEALLOCATE cursor_reverse_final
  END
  ")

  -- update the resource values
  UPDATE f
  SET f.resource = o.resource, f.resourcepath = o.resourcepath
  FROM #final_events f JOIN #obj o ON f.obj_cid = o.cid AND f.obj_id = o.id

  -- update the registration table
  DECLARE @last_mesg_id INT, @last_excp_id INT
  SELECT @last_mesg_id = evt_id FROM #final_events WHERE evt_cid = 'MESG' ORDER BY id
  SELECT @last_excp_id = evt_id FROM #final_events WHERE evt_cid = 'EXCP' ORDER BY id
  SET @mesg_id = ISNULL(@last_mesg_id, @mesg_id)
  SET @excp_id = ISNULL(@last_excp_id, @excp_id)

  EXEC("
  SELECT obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr, 
    evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath
  FROM #final_events
  " + @orderby_clause_evt + "
  ")

  -- populate the number of events
  SELECT @numfilteredevents = ISNULL(excp_count,0) + ISNULL(mesg_count,0) FROM #numevents

  -- finish transaction
  IF @tcount = 0
    COMMIT TRAN _GetEventViewerEvents
  SET NOCOUNT OFF
GO



SET NOCOUNT OFF
GO
PRINT 'Done.'
