USE Object
GO

PRINT 'Creating table GEM_LOBInstLookup'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name='GEM_LOBInstLookup' and type='U')  
  DROP TABLE GEM_LOBInstLookup

CREATE TABLE GEM_LOBInstLookup (
  LOBKey NVARCHAR(255),
  cid ClassID,
  oid_where NVARCHAR(255) DEFAULT '',
  LOBName NVARCHAR(255) DEFAULT '',
  lobid ObjID DEFAULT 0
)

/*CREATE TABLE GEM_LOBInstLookup (
  LOBKey NVARCHAR(255) PRIMARY KEY(LOBKey, pcid),
  pcid ClassID,
  poid ObjID,
  plobid ObjID,
  pmatch NVARCHAR(255)
)*/
GO

-- create a trigger on GEM_LOBInstLookup to clean GEM_InstFiltering
CREATE TRIGGER gemtr_cleanFltrTbl ON GEM_LOBInstLookup FOR DELETE
AS
  DELETE g FROM GEM_InstFiltering g JOIN deleted d ON g.LOBKey=d.LOBKey
  DELETE g FROM GEM_UpdateLOBView g JOIN deleted d ON g.LOBKey=d.LOBKey
GO  

----------------------------------------------------------------

PRINT 'Creating table GEM_UpdateLOBView'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name='GEM_UpdateLOBView' and type='U')  
  DROP TABLE GEM_UpdateLOBView

CREATE TABLE GEM_UpdateLOBView (
  cid ClassID UNIQUE(cid, id, LOBKey, action),
  id ObjID, 
  LOBKey NVARCHAR(255) DEFAULT '',
  action NVARCHAR(32) DEFAULT ''
)

/*
CREATE TABLE GEM_UpdateLOBView (
  cid ClassID PRIMARY KEY(cid, id),
  id ObjID
)
*/

----------------------------------------------------------------

PRINT 'Updating table GEM_InstFiltering'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name='GEM_InstFiltering' and type='U') BEGIN
  IF NOT EXISTS (SELECT * FROM syscolumns a, sysobjects b WHERE b.type='U' AND b.name='GEM_InstFiltering' AND b.id=a.id AND (a.name='enabled' OR a.name='exclusive' OR a.name='priority')) 
  BEGIN
    SELECT * INTO #tmpGEM_InstFiltering FROM GEM_InstFiltering
    DROP TABLE GEM_InstFiltering
    
    CREATE TABLE GEM_InstFiltering (
      cid ClassID UNIQUE (cid, LOBKey, memOptions, memPattern, memExpr, memTest),
      LOBKey NVARCHAR(225),
      memOptions NVARCHAR(32) NULL,
      memPattern NVARCHAR(64) NULL,
      memExpr NVARCHAR(64) NULL,
      memTest NVARCHAR(32) NULL,
      enabled TINYINT DEFAULT 1,
      exclusive TINYINT DEFAULT 0,
      priority TINYINT DEFAULT 0
    )

    INSERT GEM_InstFiltering SELECT *, 1, 0, 0 FROM #tmpGEM_InstFiltering
    DROP TABLE #tmpGEM_InstFiltering
  END
END
ELSE BEGIN
  CREATE TABLE GEM_InstFiltering (
    cid ClassID UNIQUE (cid, LOBKey, memOptions, memPattern, memExpr, memTest),
    LOBKey NVARCHAR(225),
    memOptions NVARCHAR(32) NULL,
    memPattern NVARCHAR(64) NULL,
    memExpr NVARCHAR(64) NULL,
    memTest NVARCHAR(32) NULL,
    enabled TINYINT DEFAULT 1,
    exclusive TINYINT DEFAULT 0,
    priority TINYINT DEFAULT 0
  )
END

--ALTER TABLE GEM_InstFiltering ADD enabled TINYINT NULL, exclusive TINYINT NULL, priority TINYINT NULL
--GO

--UPDATE GEM_InstFiltering SET enabled=1
--UPDATE GEM_InstFiltering SET exclusive=0
--UPDATE GEM_InstFiltering SET priority=0


----------------------------------------------------------------
