-- $Header:   //Archive/Archive_NLS/DB/sql/_CommitIntegrationRequest.sqv   1.3   Jul 19 2001 12:38:50   Blaine  
--
-- _CommitIntegrationRequest.sql
--
-- Module Description
--   Defines the _CommitIntegrationRequest stored procedure
--
USE Object
GO

---------------------------------------------------------------------------------------------------
-- _CommitIntegrationRequest
PRINT 'Creating procedure _CommitIntegrationRequest'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CommitIntegrationRequest' AND type = 'P')
  DROP PROCEDURE _CommitIntegrationRequest
GO

-- $Archive:   //Archive/Archive_NLS/DB/sql/_CommitIntegrationRequest.sqv  
-- $Revision:   1.3  
-- $Date:   Jul 19 2001 12:38:50  
-- $Author:   Blaine  
-- This proc invokes a request processor in order to process OpenAPI request. It is used to process Problem and Change requests.
--
CREATE PROCEDURE _CommitIntegrationRequest
  @cid  ClassID,
  @id   ObjID,
  @request_guid NVARCHAR(255),
  @request_error_no INT OUTPUT
AS
  
  DECLARE @OAerr_output NVARCHAR(255)
  DECLARE @OAerr_source NVARCHAR(255)
  DECLARE @OAerr_description NVARCHAR(255)
  DECLARE @OAreturn_code INT
  DECLARE @OIntegrationServer AS INT

  DECLARE @debug_mode AS NVARCHAR(3)
  DECLARE @timeout_value_string AS NVARCHAR(255)
  DECLARE @timeout_value AS INT

  DECLARE @request_field_row_count INT
  
  DECLARE @request_processor_language NVARCHAR(255)
  
  DECLARE @field_count INT
  DECLARE @current_field_name NVARCHAR(255)
  
  DECLARE @return_code INT
  
  DECLARE @request_command_processor AS NVARCHAR(255)
  DECLARE @request_type AS NVARCHAR(255)
  DECLARE @request_method AS NVARCHAR(255)
  
  DECLARE @request_field_name AS NVARCHAR(255)
  DECLARE @request_field_value AS NVARCHAR(255)
  
  DECLARE @err_description NVARCHAR(255)
  
  -- Need to CONVERT GUID prior to using it IN RAISEERROR.
  DECLARE @temp_guid NVARCHAR(255)
  SET @temp_guid = CONVERT(NVARCHAR(255),@request_guid)
    
  RAISERROR('_CommitIntegrationRequest: cid=%s, id=%d, request_guid=%s', 0, 1, @cid,@id,@temp_guid)
  
  SET NOCOUNT ON

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _CommitIntegrationRequest
  ELSE
    SAVE TRANSACTION _CommitIntegrationRequest

  -- Temporary TABLE used within this PROCEDURE.  
  CREATE TABLE #requestresults
    ( 
     request_results nvarchar (4000)
    )

  CREATE TABLE #reqoutput
    (
     outputdata nvarchar(4000)
    )

  CREATE TABLE #field_stats
    (
      field_name NVARCHAR(255),
      field_count INT
    )  


  -- Initialize variables.
  SET @OAreturn_code = 0
  SET @request_error_no = 0

  WHILE @request_error_no = 0
  BEGIN

    -- Get and set Integration Options.
    -- See IF we are IN debug mode.
    SELECT @debug_mode = Upper(Rtrim(propertyvalue)) FROM SystemConfiguration WHERE groupname = 'API_INTEGRATION_OPTIONS' and propertyname = 'DEBUG_MODE'
    SET @debug_mode = isnull(@debug_mode,'ON')

    -- See IF there is a timeout value.
    SELECT @timeout_value_string = Rtrim(propertyvalue) FROM SystemConfiguration WHERE groupname = 'API_INTEGRATION_OPTIONS' and propertyname = 'TIMEOUT_VALUE'
    SET @timeout_value_string = isnull(@timeout_value_string,'45')

    -- Make sure Timeout value is numeric. If not set it to the default value.
    IF ISNUMERIC(@timeout_value_string) <> 1 
       SET @timeout_value = 45
    ELSE
    BEGIN
      IF Convert(INT,@timeout_value_string) = 0
        SET @timeout_value = 45
      ELSE
        SET @timeout_value = Convert(INT,@timeout_value_string)
    END
         
    -- Debug PRINT statement.
    PRINT 'Debug Mode = ' + @debug_mode
    PRINT 'Timeout Value = ' + Convert(NVARCHAR(20),@timeout_value)

    -- CREATE an instanace of the CIntegrationManger class.
    EXEC @OAreturn_code = sp_OACreate 'IntegrationManager.CIntegrationManager', @OIntegrationServer OUT
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

       -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT CREATE instance of Integration Manager Class. Probable cause class IS NOT registered.')

      SET @request_error_no = -30000
      BREAK

    END
  
    SET @request_type = ''
    SET @request_command_processor = ''
    SET @request_processor_language = ''
  
    -- Get the request type FROM the original request.
    SELECT @request_type = request_type, @request_method = method FROM IntegrationRequest WHERE request_guid = @request_guid

    -- Debug PRINT statement.
    PRINT 'Request Type := ' + @request_type
    PRINT 'Request Method := ' + @request_method

    -- Get request processor's name AND language type FROM SystemConfiguration.
    -- Debug PRINT statement.
    PRINT 'Getting request processor AND language.'

    SELECT @request_command_processor = propertyvalue FROM SystemConfiguration WHERE groupname = RTRIM(@request_type) AND propertyname = 'REQUEST_PROCESSOR_NAME'
    SELECT @request_processor_language = propertyvalue FROM SystemConfiguration WHERE groupname = RTRIM(@request_type) AND propertyname = 'REQUEST_PROCESSOR_LANGUAGE'
 
    -- Make sure we have all of the information to invoke the request processor.
    IF (@request_type = '') OR (@request_method = '') OR (@request_command_processor = '') 
    BEGIN
         
      INSERT INTO #requestresults VALUES('returncode|1|1|-30000')
      SET @err_description = 'Request Type := ' + @request_type + ' OR Request Method := ' + @request_method + ' OR Request Processor Name := ' + @request_command_processor + ' could NOT be determined.'
      INSERT INTO #requestresults VALUES('errormsg|1|1|' + @err_description)

      SET @request_error_no = -30000
      BREAK
   
    END
   
    -- Debug PRINT statement.
    PRINT 'Request GUID = ' + @request_guid
    PRINT 'Request Type = ' + @request_type
    PRINT 'Request Method = ' + @request_method
    PRINT 'Request Processor Name = ' + @request_command_processor
    PRINT 'Request Processor Language = ' + @request_processor_language

    -- Default request procesror language IS a WIN32 exe.
    IF  @request_processor_language IS NULL
      SET @request_processor_language = NULL

    -- SET IntegrationManager Timeout properties.
    PRINT 'Setting timeout value = ' + Convert(NVARCHAR(8),@timeout_value)
    EXEC @OAreturn_code = sp_OASetProperty @OIntegrationServer, 'TimeOutSeconds', @timeout_value
    IF @OAreturn_code <> 0
    BEGIN

       -- Send back original ActiveX error code.
       INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

       -- Try to get the error information FOR the failed CREATE.
       EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
       INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT SET TimeOutSeconds property.')

       SET @request_error_no = -30000
       BREAK

    END
  
    -- SET IntegrationManager properties.
    EXEC @OAreturn_code = sp_OASetProperty @OIntegrationServer, 'GUID', @temp_guid
    IF @OAreturn_code <> 0
    BEGIN

       -- Send back original ActiveX error code.
       INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

       -- Try to get the error information FOR the failed CREATE.
       EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
       INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT SET GUID property.')

       SET @request_error_no = -30000
       BREAK

    END

    -- SET request processor language prefix.
    EXEC @OAreturn_code = sp_OASetProperty @OIntegrationServer, 'Processorlanguage', @request_processor_language
    IF @OAreturn_code <> 0
    BEGIN

       -- Send back original ActiveX error code.
       INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

       -- Try to get the error information FOR the failed CREATE.
       EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
       INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT SET Processorlanguage property.')

       SET @request_error_no = -30000
       BREAK

    END
      
    EXEC @OAreturn_code = sp_OASetProperty @OIntegrationServer, 'RequestProcessor', @request_command_processor
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT SET RequestProcessor property.')

      SET @request_error_no = -30000
      BREAK

    END

    EXEC @OAreturn_code = sp_OASetProperty @OIntegrationServer, 'Processorlanguage', @request_processor_language
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))  

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT SET ProcessorLanguage property.')

      SET @request_error_no = -30000
      BREAK

    END
  
    EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'CreateInputFile', @return_code out
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT EXEC OpenInputFile method.')

      SET @request_error_no = -30000
      BREAK

    END

    -- Debug PRINT statement.
    PRINT 'RETURN code FROM CreateInputFile = ' + CONVERT(NVARCHAR(16),@return_code)
    IF @return_code <> 0
    BEGIN

      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := CreateInputFile method failed. Source := CIntegrationManager.CreateInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

      SET @request_error_no = -30000
      BREAK

    END
  
    -- SET object properties.
    EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'AppendInputFile', @return_code out, @sFieldName = 'REQUEST_TYPE', @sFieldValue = @request_type, @lRowId = 1, @lRowCount = 1
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT EXEC AppendInputFile method WHEN appending REQUEST_TYPE.')

      SET @request_error_no = -30000
      BREAK

    END

    IF @return_code <> 0
    BEGIN

      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @return_code))
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := AppendInputFile method failed WHEN appending REQUEST_TYPE. Source := CIntegrationManager.AppendInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

      SET @request_error_no = -30000
      BREAK

    END

    EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'AppendInputFile', @return_code out, @sFieldName = 'REQUEST_METHOD', @sFieldValue = @request_method, @lRowId = 1, @lRowCount = 1 
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT EXEC AppendInputFile method WHEN appending REQUEST_METHOD.')

      SET @request_error_no = -30000
      BREAK

    END

    IF @return_code <> 0
    BEGIN

      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @return_code))
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := AppendInputFile method failed WHEN appending REQUEST_METHOD. Source := CIntegrationManager.AppendInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

      SET @request_error_no = -30000
      BREAK

    END
    
    -- Debug PRINT statement.
    PRINT 'Appending Object CID to input file.'
    EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'AppendInputFile', @return_code out, @sFieldName = 'REQUEST_OBJECT_CID', @sFieldValue = @cid, @lRowId = 1, @lRowCount = 1 
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT EXEC AppendInputFile method WHEN appending REQUEST_OBJECT_CID.')

      SET @request_error_no = -30000
      BREAK

    END

    IF @return_code <> 0
    BEGIN

      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @return_code))
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := AppendInputFile method failed WHEN appending REQUEST_OBJECT_CID. Source := CIntegrationManager.AppendInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

      SET @request_error_no = -30000
      BREAK

    END

    -- Debug PRINT statement.
    PRINT 'Appending Object ID to input file.'

    -- Can't do inline conversion WHEN calling this method, so CONVERT to temp variable first.
    DECLARE @temp_id NVARCHAR(30)
    SET @temp_id = CONVERT(NVARCHAR(30),@id)
    EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'AppendInputFile', @return_code out, @sFieldName = 'REQUEST_OBJECT_ID', @sFieldValue = @temp_id, @lRowId = 1, @lRowCount = 1 
    IF @OAreturn_code <> 0
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT EXEC AppendInputFile method WHEN appending REQUEST_OBJECT_ID.')

      SET @request_error_no = -30000
      BREAK

    END

    IF @return_code <> 0
    BEGIN

      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @return_code))
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := AppendInputFile method failed WHEN appending REQUEST_OBJECT_ID. Source := CIntegrationManager.AppendInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

      SET @request_error_no = -30000
      BREAK

    END

    -- ADD all of the integration fields to the CIntegrationManager object.
    -- Populate temp TABLE with the field record counts.
    INSERT INTO #field_stats SELECT field_name,COUNT(*) FROM IntegrationFields WHERE request_guid = @request_guid GROUP BY field_name

    -- Curor to traverse all of the request fields FOR the request_guid passed IN.
    DECLARE integration_fields_cursor CURSOR FOR SELECT field_name, field_value FROM IntegrationFields WHERE request_guid = @request_guid ORDER BY row_id FOR READ ONLY

    OPEN integration_fields_cursor
    FETCH NEXT FROM integration_fields_cursor INTO @request_field_name, @request_field_value

    SET @field_count = 1
    SET @current_field_name = @request_field_name

    PRINT 'Processing input fields.'
    -- Loop AND INSERT all of the request name value pairs INTO the input file.
    WHILE @@FETCH_STATUS = 0
    BEGIN

      -- Get record COUNT FOR current name value pair.         
      SELECT @request_field_row_count = field_count FROM #field_stats WHERE RTRIM(field_name) = RTRIM(@request_field_name)

      -- Debug PRINT statement.
      PRINT '@request_field_row_count := ' + CONVERT(NVARCHAR(16),@request_field_row_count)
      PRINT 'Attempting to inserted field := ' + @request_field_name + ' INTO input file.'

      -- ADD each name value pair to the input file.
      EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'AppendInputFile', @return_code out, @sFieldName = @request_field_name, @sFieldValue = @request_field_value, @lRowId = @field_count, @lRowCount = @request_field_row_count
      IF @OAreturn_code <> 0
      BEGIN

        -- Send back original ActiveX error code.
        INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

        -- Try to get the error information FOR the failed CREATE.
        EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
        INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + ' Could NOT EXEC AppendInputFile method WHEN appending ' + @request_field_name)

        SET @request_error_no = -30000
        BREAK

      END

      IF @return_code <> 0
      BEGIN

        INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @return_code))
        INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := AppendInputFile method failed WHEN appending REQUEST_METHOD. Source := CIntegrationManager.AppendInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

        SET @request_error_no = -30000
        BREAK

      END

      FETCH NEXT FROM integration_fields_cursor INTO @request_field_name, @request_field_value
      IF @current_field_name <> @request_field_name
      BEGIN

        SET @field_count = 1
        SET @current_field_name = @request_field_name

      END
      ELSE
        SET @field_count = @field_count + 1

    END -- END of FETCH loop.

    PRINT 'Input field processing complete.'
    CLOSE integration_fields_cursor
    DEALLOCATE integration_fields_cursor

    -- Get out of farthest outer loop IF an error was encountered IN the FETCH loop.  	              
    IF @request_error_no = -30000 
      BREAK

    -- Attempt to close the input file.
    PRINT 'Attempting to close input file.'
    EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'CloseInputFile', @return_code out 
    IF @OAreturn_code <> 0 
    BEGIN

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + 'CloseInputFile method failed.')

      SET @request_error_no = -30000
      BREAK

    END

    IF @return_code <> 0
    BEGIN

      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @return_code))
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := CloseInputFile method failed. Source := CIntegrationManager.CloseInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

      SET @request_error_no = -30000
      BREAK

    END

    -- Debug PRINT statement.  
    PRINT 'Input file closed'

    -- Debug PRINT statement.  
    PRINT 'Attempting to call ProcessRequest.'

    EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'ProcessRequest', @return_code out

    IF @OAreturn_code <> 0 
    BEGIN

      -- Debug PRINT statement.    
      PRINT 'ProcessRequest Method failed'

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := ' + @OAerr_description + ' Source := ' + @OAerr_source + 'ProcessRequest method failed.')

      SET @request_error_no = -30000
      BREAK

    END

    -- Debug PRINT statement.  
    PRINT 'Attempting to get processor output.'

    INSERT #reqoutput EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer ,'GetRequestOutput', NULL 
    PRINT 'Processor output returned.'

    IF @OAreturn_code <> 0
    BEGIN

      PRINT 'GetRequestOutput Method call failed.'
      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := GetRequestOutput method failed. Source := CIntegrationManager.RequestOutput.')

      SET @request_error_no = -30000
      BREAK

    END

    -- Debug PRINT statement.  
    PRINT 'Attempting to DELETE input file.'

    -- Call the method that deletes the <guid>.in file FROM the data directory.
    IF @debug_mode = 'OFF'
    BEGIN
      EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'DeleteDataFiles', @return_code out     
      SET @OAreturn_code = 0
      SET @return_code = 0

      IF @OAreturn_code <> 0

      BEGIN

        -- Debug PRINT statement.  
        PRINT 'DeleteDataFiles failed.'

        -- Send back original ActiveX error code.
        INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

        -- Try to get the error information FOR the failed CREATE.
        EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
        INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := DeleteDataFiles method failed. Source := CIntegrationManager.DeleteDataFiles.')

        SET @request_error_no = -30000
        BREAK

      END

      IF @return_code <> 0
      BEGIN

        INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @return_code))
        INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := DeleteInputFile method failed. Source := CIntegrationManager.DeleteInputFile. RETURN Code := ' + CONVERT(NVARCHAR(10),@return_code))

        SET @request_error_no = -30000
        BREAK

      END

    END  -- END of debug mode test.

    -- IF the request completed successfully THEN UPDATE the request status.
    -- Call the method that deleted the <guid>.in file FROM the data directory.
    DECLARE @RequestReturnCode INT

    -- Debug PRINT statement.  
    PRINT 'Attempting to get request process RETURN code.'

    EXEC @OAreturn_code = sp_OAGetProperty @OIntegrationServer, 'RequestReturnCode', @RequestReturnCode OUT
    IF @OAreturn_code <> 0
    BEGIN

      -- Debug PRINT statement.  
      PRINT 'RequestReturnCode Property get failed.'

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := RequestReturnCode method failed. Source := CIntegrationManager.RequestReturnCode.')

      SET @request_error_no = -30000
      BREAK

    END

    -- Debug PRINT statement.  
    PRINT 'Attempting to UPDATE committed request.'

    IF @RequestReturnCode = 0
    BEGIN

      PRINT 'Updating IntegrationRequest'
      -- UPDATE IntegrationRequest SET status = 1 WHERE request_guid = @request_guid
      IF @@ERROR = 0
        PRINT 'Calling _CleanUpIntegrationRequest'
      -- EXEC _CleanUpIntegrationRequest @cid, @id , @request_guid, @request_error_no OUTPUT
      ELSE
      BEGIN

        -- Debug PRINT statement.  
        PRINT 'Call to _CleanUpIntegrationRequest failed.'

        INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @@ERROR))
        INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := Could NOT UPDATE request AS committed.')

        SET @request_error_no = -30000
        BREAK

      END
    END

    -- Debug PRINT statement.  
    PRINT 'Attempting to destroy object instance.'

    EXEC @OAreturn_code = sp_OADestroy @OIntegrationServer
    IF @OAreturn_code <> 0 
    BEGIN

      -- Debug PRINT statement.  
      PRINT 'Attempting to destroy object instance failed.'

      -- Send back original ActiveX error code.
      INSERT INTO #requestresults VALUES('PROC_RETURN_CODE|1|1|' + CONVERT(NVARCHAR(16), @OAreturn_code))

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
      INSERT INTO #requestresults VALUES('PROC_ERROR_MSG|1|1|Description := sp_OADestroy failed. Source := sp_OADestroy.')

      SET @request_error_no = -30000
      BREAK

    END

    -- IF we get here THEN BREAK out of farthest outer loop.
    BREAK

  END

  -- Debug PRINT statement.  
  PRINT 'Attempting to RETURN final result SET.'

  -- Combine request processor output to any other output.
  INSERT INTO #reqoutput SELECT * FROM #requestresults

  -- RETURN final result SET.
  SELECT * FROM #reqoutput

  -- Debug PRINT statement.  
  PRINT 'Attempting to DROP temp tables.'

  -- Explicitly drp temp tables.  
  EXEC('DROP TABLE #requestresults')
  EXEC('DROP TABLE #reqoutput')
  EXEC('DROP TABLE #field_stats')

  -- Debug PRINT statement.  
  PRINT 'Attempting to finish TRANSACTION _CommitIntegrationRequest'
  -- Finish up TRANSACTION
  IF (@@ERROR <> 0) 
  BEGIN
    ROLLBACK TRANSACTION  _CommitIntegrationRequest
  END 
  ELSE
    IF @tcount = 0
      COMMIT TRANSACTION  _CommitIntegrationRequest
GO
