-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/_DiscoverCPSMResourcesCICS.sqv   1.3   Jul 26 2001 12:46:34   BRENT  $
--
-- _DiscoverCPSMResourcesCICS.sql
--
-- Module Description
--   Defines the _DiscoverCPSMResourcesCICS stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _DiscoverCPSMResourcesCICS
PRINT 'Creating procedure _DiscoverCPSMResourcesCICS'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverCPSMResourcesCICS' AND type = 'P')
  DROP PROCEDURE _DiscoverCPSMResourcesCICS
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/_DiscoverCPSMResourcesCICS.sqv  $
-- $Revision:   1.3  $
-- $Date:   Jul 26 2001 12:46:34  $
-- $Author:   BRENT  $
-- Procedure used by _DiscoverCPSMResources to discover CICS objects from the CPSMDiscovery table
CREATE PROCEDURE _DiscoverCPSMResourcesCICS
  @cid          ClassID,
  @id           ObjID,
  @batchid      int,
  @incremental  TINYINT = 0,
  @verbose      TINYINT = 0
AS
  PRINT '_DiscoverCPSMResourcesCICS'
    + ' @cid = ' + CASE WHEN @cid IS NULL THEN 'NULL' ELSE '''' + @cid + '''' END
    + ', @id = ' + CASE WHEN @id IS NULL THEN 'NULL' ELSE CAST(@id AS VARCHAR) END
    + ', @batchid = ' + CASE WHEN @batchid IS NULL THEN 'NULL' ELSE CAST(@batchid AS VARCHAR) END
    + ', @incremental = ' + CASE WHEN @incremental IS NULL THEN 'NULL' ELSE CAST(@incremental AS VARCHAR) END
    + ', @verbose = ' + CASE WHEN @verbose IS NULL THEN 'NULL' ELSE CAST(@verbose AS VARCHAR) END

  -- If there is no outer transaction then begin a transaction, otherwise create
  -- a transaction savepoint
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _DiscoverCPSMResourcesCICS
  ELSE
    SAVE TRANSACTION _DiscoverCPSMResourcesCICS

  -- Perform garbage-collection on tables we reference here so that we don't
  -- have to pay attention to deleted flags

  DELETE CICSPlexAggrRule_C WHERE deleted = 1
  
  DELETE i FROM CIPX_CICF_PHYC as i
    JOIN containment AS c ON c.src_cid = 'CIPX' AND c.src_id = i.src_id AND c.dst_cid = 'CICF' AND c.dst_id = i.dst_id
  WHERE c.deleted = 1

  DELETE i FROM CICF_CICS_MAGC AS i 
    JOIN link AS c ON c.src_cid = 'CICF' AND c.src_id = i.src_id AND c.dst_cid = 'CICS' AND c.dst_id = i.dst_id AND c.link_type = 'MAGC'
  WHERE
    c.deleted = 1

  DELETE i FROM OS_CICS_RUNC AS i 
    JOIN link AS c ON c.src_cid = 'OS' AND c.src_id = i.src_id AND c.dst_cid = 'CICS' AND c.dst_id = i.dst_id AND c.link_type = 'RUNC'
  WHERE
    c.deleted = 1

  -- temp table for holding discovered CICS records
  CREATE TABLE #CICS
  (
    cipx_id     ObjID NULL,
    cics_id     ObjID NULL,
    CICSName    ObjName,
    cics_desc   NVARCHAR(255) NULL,
    MASType     INT,
    JOBNAME     NVARCHAR(40) NULL,
    APPLID      NVARCHAR(40) NULL,
    SystemId    NVARCHAR(40) NULL,
    MVSID       NVARCHAR(40) NULL,
    STATE1      NVARCHAR(40) NULL,
    FolderName  ObjName NULL,
    available   bit, -- available for aggregation
    old_os_link NVARCHAR(4) NULL,
    old_os_id   ObjID NULL,
    new_os_id   ObjID NULL
  )

  CREATE TABLE #CIPX (
    id     ObjID NOT NULL,
    name   ObjName
  )

  INSERT INTO #CIPX
    SELECT DISTINCT id, name 
      FROM CICSPlex_V v
      JOIN containment c ON c.dst_cid = 'CIPX' AND c.dst_id = v.id AND c.deleted = 0
      JOIN CPSMDiscovery d ON v.name = d.CICSPLEX

  INSERT #CICS
    SELECT
      cipx.id,
      NULL,
      CICSName,
      NULL,
      CASE
        WHEN MASTYPE IS NULL THEN 1
        WHEN MASTYPE = 'N/A' THEN 1
        WHEN MASTYPE = 'LOCAL' THEN 4
        WHEN MASTYPE = 'REMOTE' THEN 8
        ELSE 1
      END,
      JOBNAME,
      APPLID,
      SystemId,
      MVSID,
      STATE1,
      NULL,
      1,
      NULL,
      NULL,
      NULL
    FROM
      CPSMDiscovery AS d
        LEFT OUTER JOIN #CIPX AS cipx ON cipx.name = d.CICSPLEX
    WHERE
      batchid = @batchid
      AND d.cid = 'CIMA'

  -- Update CICS ids based on previously existing relationships to aggregation folders
  UPDATE c SET cics_id = cics.id, cics_desc = cics._Desc
  FROM
    #CICS AS c
      JOIN CIPX_CICF_PHYC AS cipx_cicf ON cipx_cicf.src_id = c.cipx_id
      JOIN CICF_CICS_MAGC AS cicf_cics ON cicf_cics.src_id = cipx_cicf.dst_id
      JOIN CICS_V AS cics ON cics.id = cicf_cics.dst_id AND cics.name = c.CICSName
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
    RETURN -30200
  END

  -- Lookup previously existing relationships between OS's and discovered regions
  UPDATE c SET old_os_link = os_cics.link_type, old_os_id = os_cics.src_id
  FROM
    #CICS AS c
      JOIN link AS os_cics ON os_cics.src_cid = 'OS' AND os_cics.dst_cid = 'CICS' AND os_cics.dst_id = c.cics_id

  -- If there are any regions in the set that have not been found which have a non-null SystemId
  -- or MVSID then try to find any previously existing physical links between OS's and regions that
  -- match by name
  IF EXISTS (SELECT * FROM #CICS WHERE cics_id IS NULL AND SystemId IS NOT NULL OR MVSID IS NOT NULL)
  BEGIN
    IF @verbose = 1
    BEGIN
      PRINT 'Trying to match regions to previous OS physical links'
      SELECT 'BYNAME' AS 'BYNAME', * FROM #CICS WHERE cics_id IS NULL AND MVSID IS NOT NULL
    END
    -- Only those records with a distinct match of SystemId/MVSID to OS are candidates,
    -- so we first create a count table
    CREATE TABLE #OS_CICS_BY_NAME
    (
      link_type NVARCHAR(4) NULL,
      CICSName  NVARCHAR(8),
      SystemId  NVARCHAR(40) NULL,      
      MVSID     NVARCHAR(40) NULL, 
      nmatches  INT
    )
    INSERT INTO #OS_CICS_BY_NAME 
      SELECT DISTINCT link.link_type, cics.name, c.SystemId, c.MVSID, COUNT(*) 
      FROM #CICS AS c 
        JOIN OS_V AS os ON os._SystemId = c.SystemId
        JOIN CICS_C AS cics ON cics.name = c.CICSName
        JOIN link ON link.src_cid = 'OS' AND link.src_id = os.id AND link.dst_cid = 'CICS' AND link.dst_id = cics.id
      WHERE c.SystemId IS NOT NULL
      GROUP BY link.link_type, cics.name, c.SystemId, c.MVSID
    INSERT INTO #OS_CICS_BY_NAME 
      SELECT DISTINCT link.link_type, cics.name, c.SystemId, c.MVSID, COUNT(*) 
      FROM #CICS AS c 
        JOIN OS_V AS os ON os._SMFID = c.MVSID
        JOIN CICS_C AS cics ON cics.name = c.CICSName
        JOIN link ON link.src_cid = 'OS' AND link.src_id = os.id AND link.dst_cid = 'CICS' AND link.dst_id = cics.id
      WHERE c.SystemId IS NULL AND c.MVSID IS NOT NULL
      GROUP BY link.link_type, cics.name, c.SystemId, c.MVSID
    IF @verbose = 1
      SELECT '#OS_CICS_BY_NAME' AS '#OS_CICS_BY_NAME', * FROM #OS_CICS_BY_NAME

    UPDATE c SET cics_id = cics.id, old_os_id = os.id, old_os_link = link.link_type
      FROM #CICS AS c
        JOIN #OS_CICS_BY_NAME AS byname ON byname.CICSName = c.CICSName AND 
           (byname.MVSID = c.MVSID OR byname.SystemId = c.SystemId)
        JOIN OS_V AS os ON 
           (os._SystemId = byname.SystemId OR (byname.SystemId IS NULL AND os._SMFID = byname.MVSID))
        JOIN CICS_C AS cics ON cics.name = byname.CICSName
        JOIN link ON link.src_cid = 'OS' AND link.src_id = os.id AND link.dst_cid = 'CICS' AND link.dst_id = cics.id
      WHERE byname.nmatches = 1    
  END

  IF @verbose = 1  
    SELECT '#CICS' AS '#CICS', * FROM #CICS

  -- Allocate ids for regions that have to be created
  DECLARE @base_cics_id ObjID, @num_new_cics INT
  SELECT @num_new_cics = COUNT(*) FROM #CICS WHERE cics_id IS NULL
  IF @num_new_cics > 0
  BEGIN
    EXEC @base_cics_id = alloc_CICS @num_new_cics
    CREATE TABLE #NEW_CICS
    (
      cipx_id   ObjID NULL,
      CICSName  ObjName,
      rowno     INT IDENTITY(0,1)
    )
    INSERT #NEW_CICS SELECT DISTINCT cipx_id, CICSName FROM #CICS WHERE cics_id IS NULL
    UPDATE c SET cics_id = @base_cics_id + rowno 
    FROM #CICS AS c JOIN #NEW_CICS AS n ON
      (n.cipx_id = c.cipx_id OR n.cipx_id IS NULL)
      AND n.CICSName = c.CICSName
    IF @verbose = 1
      SELECT '#NEW_CICS' AS '#NEW_CICS', * FROM #NEW_CICS
  END

  -- Now loop through the CICSPlexAggrRules for each CICSPlex and figure out how
  -- the regions should be aggregated
  CREATE TABLE #CICF_CICS
  (
    cipx_id     ObjID,
    FolderName  ObjName,
    cicf_id     ObjID NULL,
    cics_id     ObjID
  )
  DECLARE cipxc INSENSITIVE CURSOR FOR SELECT DISTINCT cipx_id FROM #CICS WHERE cipx_id IS NOT NULL
  OPEN cipxc
  DECLARE @cipx_id ObjID
  FETCH NEXT FROM cipxc INTO @cipx_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    DECLARE ciarc INSENSITIVE CURSOR FOR
      SELECT name, _DescPattern, _Exclusive, _FolderName, _RegionNamePattern, _SystemNamePattern, _TypeMask
      FROM CICSPlexAggrRule_V
      WHERE _ParentCID = 'CIPX' AND _ParentID = @cipx_id AND _Enabled = 1
      ORDER BY _Priority DESC
    OPEN ciarc
    DECLARE @RuleName ObjName, @DescPattern NVARCHAR(255), @Exclusive INT, @FolderName ObjName,
            @RegionNamePattern NVARCHAR(255), @SystemNamePattern NVARCHAR(255), @TypeMask INT
    FETCH NEXT FROM ciarc INTO @RuleName, @DescPattern, @Exclusive, @FolderName, @RegionNamePattern, @SystemNamePattern, @TypeMask
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      IF @verbose = 1
      BEGIN
        PRINT 'Processing Rule ''' + @RuleName + ''''
        PRINT '  DescPattern       = ''' + @DescPattern + ''''
        PRINT '  RegionNamePattern = ''' + @RegionNamePattern + ''''
        PRINT '  SystemNamePattern = ''' + @SystemNamePattern + ''''
        PRINT '  TypeMask          = ' + CAST(@TypeMask AS NVARCHAR(9))
        SELECT @RuleName AS RuleName, @cipx_id AS cipx_id, @FolderName AS FolderName, NULL AS cicf_id, cics_id
        FROM
          #CICS AS c
        WHERE
          available = 1
          AND (@DescPattern IS NULL OR @DescPattern = '' OR cics_desc LIKE @DescPattern)
          AND (@RegionNamePattern IS NULL OR @RegionNamePattern = '' OR CICSName LIKE @RegionNamePattern)
          AND (@SystemNamePattern IS NULL OR @SystemNamePattern = '' OR MVSID LIKE @SystemNamePattern)
          AND (@TypeMask = NULL OR @TypeMask = 0 OR MASType & @TypeMask <> 0)
      END
      INSERT #CICF_CICS(cipx_id, FolderName, cicf_id, cics_id)
        SELECT @cipx_id, @FolderName, NULL, cics_id
      FROM
        #CICS AS c
      WHERE
        available = 1
        AND (@DescPattern IS NULL OR @DescPattern = '' OR cics_desc LIKE @DescPattern)
        AND (@RegionNamePattern IS NULL OR @RegionNamePattern = '' OR CICSName LIKE @RegionNamePattern)
        AND (@SystemNamePattern IS NULL OR @SystemNamePattern = '' OR MVSID LIKE @SystemNamePattern)
        AND (@TypeMask = NULL OR @TypeMask = 0 OR MASType & @TypeMask <> 0)
      -- If this was an 'Exclusive' rule then mark the matched regions as unavailable for further matching
      IF @@ROWCOUNT > 0 AND @Exclusive = 1
        UPDATE c SET available = 0 
        FROM #CICS AS c 
          JOIN #CICF_CICS AS cicf_cics ON c.cics_id = cicf_cics.cics_id
        WHERE
          cicf_cics.cipx_id = @cipx_id AND cicf_cics.FolderName = @FolderName
      FETCH NEXT FROM ciarc INTO @RuleName, @DescPattern, @Exclusive, @FolderName, @RegionNamePattern, @SystemNamePattern, @TypeMask
    END
    DEALLOCATE ciarc
    FETCH NEXT FROM cipxc INTO @cipx_id
  END
  DEALLOCATE cipxc

/* MAINVIEW: No longer a valid assumption
  -- Now remove any newly discovered regions that did not get aggregated as these are not monitored
  DELETE c FROM #CICS AS c
  WHERE c.cics_id >= @base_cics_id AND c.cics_id NOT IN (SELECT DISTINCT cics_id FROM #CICF_CICS)
*/

  -- Create new regions
  IF @verbose = 1
    SELECT 'NEW_REGION_CHECK' AS 'NEW_REGION_CHECK', @base_cics_id AS '@base_cics_id', * FROM #CICS
  IF EXISTS (SELECT * FROM #CICS WHERE cics_id >= @base_cics_id)
  BEGIN
    IF @verbose = 1
      SELECT 'REGION_TO_CREATE' AS 'REGION_TO_CREATE', * FROM #CICS WHERE cics_id >= @base_cics_id
    INSERT CICS_C(id, name) SELECT cics_id, CICSName FROM #CICS WHERE cics_id >= @base_cics_id
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
      RETURN -30201
    END
    INSERT CICS_A(id, _ChildEventMatrixID, _ExceptionMatrixID, _SysplexResource, _APPLID, _JOBNAME, _MASTYPE, _SystemName)
      SELECT cics_id, 0, 0, 0, APPLID, JOBNAME, MASType, MVSID
      FROM #CICS WHERE cics_id >= @base_cics_id
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
      RETURN -30202
    END
  END

  -- Create any required CICSFolder objects
  CREATE TABLE #CICF
  (
    cipx_id     ObjID,
    FolderName  ObjName,
    cicf_id     ObjID NULL
  )
  INSERT #CICF
    SELECT DISTINCT cipx_id, FolderName, cicf_id
    FROM #CICF_CICS
  UPDATE c SET cicf_id = cicf.id
    FROM #CICF_CICS AS c
      JOIN CIPX_CICF_PHYC AS cipx_cicf ON cipx_cicf.src_id = c.cipx_id
      JOIN CICSFolder_C AS cicf ON cicf.id = cipx_cicf.dst_id AND cicf.name = c.FolderName
  UPDATE c SET cicf_id = cicf.id
    FROM #CICF AS c
      JOIN CIPX_CICF_PHYC AS cipx_cicf ON cipx_cicf.src_id = c.cipx_id
      JOIN CICSFolder_C AS cicf ON cicf.id = cipx_cicf.dst_id AND cicf.name = c.FolderName

  IF @verbose = 1
    SELECT '#CICF' AS '#CICF', * FROM #CICF

  DECLARE @nCICF INT
  SELECT @nCICF = COUNT(*) FROM #CICF WHERE cicf_id IS NULL
  IF @nCICF > 0
  BEGIN
    CREATE TABLE #NEW_CICF
    (
      cipx_id     ObjID,
      FolderName  ObjName,
      rowno       INT IDENTITY(0,1)
    )
    INSERT #NEW_CICF(cipx_id, FolderName) SELECT cipx_id, FolderName FROM #CICF WHERE cicf_id IS NULL

    -- Allocate ids for new instances
    DECLARE @base_cicf_id INT
    EXEC @base_cicf_id = alloc_CICF @nCICF

    -- Insert instance records
    INSERT CICSFolder_C(id, name, _ChildEventMatrixID, _ExceptionMatrixID)
      SELECT @base_cicf_id+rowno, FolderName, 0, 0
      FROM #NEW_CICF
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
      RETURN -30203
    END

    -- Insert containment link records
    INSERT CIPX_CICF_PHYC(src_id, dst_id)
      SELECT cipx_id, @base_cicf_id+rowno
      FROM #NEW_CICF
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
      RETURN -30204
    END

    -- Insert object creation notification records
    INSERT PendingNotification(type, fields, detail)
      SELECT
        EventType.type,
        CONVERT(NVARCHAR(5), CICF.cno) + '|' + CONVERT(NVARCHAR(9), @base_cicf_id + rowno) + '|CICF|PHYC|' +
        CONVERT(NVARCHAR(5), CIPX.cno) + '|' + CONVERT(NVARCHAR(9), cipx_id) AS fields,
        FolderName AS detail
      FROM
        #NEW_CICF,
        EventType,
        obj_class AS CICF,
        obj_class AS CIPX
      WHERE
        EventType.name = 'ObjectCreated'
        AND CICF.cid = 'CICF'
        AND CIPX.cid = 'CIPX'

      -- Update temp table to track new folders
      UPDATE c SET cicf_id = @base_cicf_id+rowno
        FROM #CICF_CICS AS c
          JOIN #NEW_CICF AS n ON n.cipx_id = c.cipx_id AND n.FolderName = c.FolderName
  END

  -- Create links establishing aggregation of regions into folders
  IF @verbose = 1
    SELECT 'NEW CICF_CICS_MAGC' AS 'NEW CICF_CICS_MAGC', new.cicf_id, new.cics_id
    FROM #CICF_CICS AS new
    WHERE NOT EXISTS
    (
      SELECT *
      FROM CICF_CICS_MAGC AS old
      WHERE old.src_id = new.cicf_id AND old.dst_id = new.cics_id
    )
  INSERT CICF_CICS_MAGC(src_id, dst_id)
    SELECT new.cicf_id, new.cics_id
    FROM #CICF_CICS AS new
    WHERE NOT EXISTS
    (
      SELECT *
      FROM CICF_CICS_MAGC AS old
      WHERE old.src_id = new.cicf_id AND old.dst_id = new.cics_id
    )
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
    RETURN -30205
  END

  IF @verbose = 1
    SELECT '#CICF_CICS' AS '#CICF_CICS', * FROM #CICF_CICS

  IF @incremental = 0
  BEGIN
    -- Remove any CICF_CICS_MAGC links for regions that no longer match the aggregation rules.
    UPDATE old SET deleted = 1
      FROM CICF_CICS_MAGC AS old
        JOIN CIPX_CICF_PHYC AS cipx_cicf ON cipx_cicf.dst_id = old.src_id
      WHERE
        cipx_cicf.src_id IN (SELECT DISTINCT cipx_id FROM #CICF_CICS)
        AND NOT EXISTS
        (
          SELECT *
          FROM #CICF_CICS AS new
          WHERE
            new.cipx_id = cipx_cicf.src_id
            AND new.cicf_id = cipx_cicf.dst_id
            AND new.cics_id = old.dst_id
        )
  END

  -- Establish OS->CICS relationships, which indicate that a region is running on a given host.
  IF EXISTS (SELECT * FROM #CICS WHERE MVSID IS NOT NULL OR SystemId IS NOT NULL)
  BEGIN
    -- Only those records with a distinct match of MVSID/SystemId to OS get updated,
    -- so we first create a count table
    CREATE TABLE #CICS_OS
    (
      cics_id ObjID,
      MVSID NVARCHAR(40) NULL, 
      SystemId NVARCHAR(40) NULL, 
      nmatches INT
    )

    INSERT #CICS_OS 
      SELECT c.cics_id, c.MVSID, c.SystemId, COUNT(*) 
      FROM #CICS AS c 
      JOIN OS_V AS os ON os._SystemId = c.SystemId
      JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
      WHERE c.SystemId IS NOT NULL
      GROUP BY c.cics_id, c.MVSID, c.SystemId
    INSERT #CICS_OS 
      SELECT c.cics_id, c.MVSID, c.SystemId, COUNT(*) 
      FROM #CICS AS c 
      JOIN OS_V AS os ON os._SMFID = c.MVSID
      JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
      WHERE c.SystemId IS NULL AND c.MVSID IS NOT NULL
      GROUP BY c.cics_id, c.MVSID, c.SystemId

    UPDATE c SET new_os_id = os.id
      FROM #CICS AS c
        JOIN #CICS_OS AS cics_os ON cics_os.cics_id = c.cics_id
        JOIN OS_V AS os ON os._SystemId = cics_os.SystemId
        JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
      WHERE cics_os.nmatches = 1 AND c.SystemId IS NOT NULL
    UPDATE c SET new_os_id = os.id
      FROM #CICS AS c
        JOIN #CICS_OS AS cics_os ON cics_os.cics_id = c.cics_id
        JOIN OS_V AS os ON os._SMFID = cics_os.MVSID
        JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
      WHERE cics_os.nmatches = 1 AND c.SystemId IS NULL AND c.MVSID IS NOT NULL

    IF @verbose = 1
      SELECT '#CICS_OS' AS '#CICS_OS', * FROM #CICS_OS

    -- We now have several cases to account for:
    -- 1. old_os_id = new_os_id  AND old_os_link = 'RUNC'  -> do nothing
    -- 2. old_os_id <> new_os_id AND old_os_link = 'RUNC'  -> update src_id in OS_CICS_RUNC table
    -- 3. new_os_id IS NOT NULL  AND old_os_link = 'PHYC'  -> replace PHYC link with RUNC link
    -- 4. old_os_id IS NOT NULL AND new_os_id IS NULL      -> delete old link
    -- 5. old_os_id IS NULL AND new_os_id IS NOT NULL      -> create new OS_CICS_RUNC link

    -- 2. old_os_id <> new_os_id AND old_os_link = 'RUNC'  -> update src_id in OS_CICS_RUNC table
    IF EXISTS (SELECT * FROM #CICS WHERE old_os_id <> new_os_id AND old_os_link = 'RUNC')
    BEGIN
      UPDATE i SET src_id = c.new_os_id
        FROM OS_CICS_RUNC AS i JOIN #CICS AS c ON c.old_os_id = i.src_id AND c.cics_id = i.dst_id
      WHERE
        c.old_os_id <> c.new_os_id AND c.old_os_link = 'RUNC'
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
        RETURN -30206
      END
    END

    -- 3. new_os_id IS NOT NULL  AND old_os_link = 'PHYC'  -> replace PHYC link with RUNC link
    IF EXISTS (SELECT * FROM #CICS WHERE new_os_id IS NOT NULL AND old_os_link = 'PHYC')
    BEGIN
      UPDATE i SET deleted = 1
        FROM OS_CICS_PHYC AS i JOIN #CICS AS c ON c.old_os_id = i.src_id AND c.cics_id = i.dst_id
      WHERE
        c.new_os_id IS NOT NULL AND c.old_os_link = 'PHYC'
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
        RETURN -30207
      END
      INSERT OS_CICS_RUNC(src_id, dst_id)
        SELECT new_os_id, cics_id
        FROM #CICS AS c
        WHERE c.new_os_id IS NOT NULL AND c.old_os_link = 'PHYC'
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
        RETURN -30208
      END
    END

    -- 4. old_os_id IS NOT NULL AND new_os_id IS NULL      -> delete old link
    IF EXISTS (SELECT * FROM #CICS WHERE old_os_id IS NOT NULL AND new_os_id IS NULL)
    BEGIN
      UPDATE i SET deleted = 1
        FROM OS_CICS_PHYC AS i JOIN #CICS AS c ON c.old_os_id = i.src_id AND c.cics_id = i.dst_id
      WHERE
        c.new_os_id IS NOT NULL AND c.old_os_link = 'PHYC'
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
        RETURN -30209
      END
      UPDATE i SET deleted = 1
        FROM OS_CICS_RUNC AS i JOIN #CICS AS c ON c.old_os_id = i.src_id AND c.cics_id = i.dst_id
      WHERE
        c.new_os_id IS NOT NULL AND c.old_os_link = 'RUNC'
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
        RETURN -30210
      END
    END

    -- 5. old_os_id IS NULL AND new_os_id IS NOT NULL      -> create new OS_CICS_RUNC link
    IF EXISTS (SELECT * FROM #CICS WHERE old_os_id IS NULL AND new_os_id IS NOT NULL)
    BEGIN
      INSERT OS_CICS_RUNC(src_id, dst_id)
        SELECT new_os_id, cics_id
        FROM #CICS AS c
        WHERE c.old_os_id IS NULL AND c.new_os_id IS NOT NULL
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
        RETURN -30211
      END
    END
  END

  -- Update the CPSMDiscovery table with the ids of the discovered objects
  UPDATE d SET
    obj_cid = 'CICS',
    obj_id = c.cics_id
  FROM
    CPSMDiscovery AS d
    JOIN #CICS AS c ON c.CICSName = d.CICSName AND (c.SystemId = d.SystemId OR (c.SystemId IS NULL AND c.MVSID = d.MVSID))
  WHERE
    batchid = @batchid AND d.cid = 'CIMA'
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCICS
    RETURN -30212
  END

  IF @verbose = 1
    SELECT 'FINAL' AS 'FINAL', * FROM #CICS

  -- If we're the outer transaction then commit
  IF @tcount = 0
    COMMIT TRANSACTION _DiscoverCPSMResourcesCICS
GO