-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/_DiscoverCPSMResourcesCMAS.sqv   1.3   Jul 26 2001 12:47:10   BRENT  $
--
-- _DiscoverCPSMResourcesCMAS.sql
--
-- Module Description
--   Defines the _DiscoverCPSMResourcesCMAS stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _DiscoverCPSMResourcesCMAS
PRINT 'Creating procedure _DiscoverCPSMResourcesCMAS'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverCPSMResourcesCMAS' AND type = 'P')
  DROP PROCEDURE _DiscoverCPSMResourcesCMAS
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/_DiscoverCPSMResourcesCMAS.sqv  $
-- $Revision:   1.3  $
-- $Date:   Jul 26 2001 12:47:10  $
-- $Author:   BRENT  $
CREATE PROCEDURE _DiscoverCPSMResourcesCMAS
  @cid          ClassID,
  @id           ObjID,
  @batchid      int,
  @incremental  TINYINT = 0,
  @verbose      TINYINT = 0
AS
  IF @verbose = 1
    PRINT '_DiscoverCPSMResourcesCMAS'
      + ' @cid=''' + @cid + ''''
      + ', @id=' + CAST(@id AS VARCHAR)
      + ', @batchid=' + CAST(@batchid AS VARCHAR)
      + ', @incremental=' + CAST(@incremental AS VARCHAR)
      + ', @verbose=' + CAST(@verbose AS VARCHAR)

  -- If there is no outer transaction then begin a transaction, otherwise create
  -- a transaction savepoint
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _DiscoverCPSMResourcesCMAS
  ELSE
    SAVE TRANSACTION _DiscoverCPSMResourcesCMAS

  -- Perform garbage-collection on tables we reference here so that we don't
  -- have to pay attention to deleted flags
  DELETE i FROM CIPX_CMAF_PHYC AS i
    JOIN containment AS c ON c.src_cid = 'CIPX' AND c.src_id = i.src_id AND c.dst_cid = 'CMAF' AND c.dst_id = i.dst_id
    WHERE c.deleted = 1

  DELETE i FROM CMAF_CMAS_USEC AS i
    JOIN link AS c ON c.src_cid = 'CMAF' AND c.src_id = i.src_id AND c.dst_cid = 'CMAS' AND c.dst_id = i.dst_id AND c.link_type = 'USEC'
    WHERE c.deleted = 1

  DELETE i FROM OS_CMAS_PHYC AS i
    JOIN link AS c ON c.src_cid = 'OS' AND c.src_id = i.src_id AND c.dst_cid = 'CMAS' AND c.dst_id = i.dst_id AND c.link_type = 'PHYC'
    WHERE c.deleted = 1

  -- temp table for holding discovered CMAS records
  CREATE TABLE #CMAS
  (
    cipx_id   ObjID NULL,
    cmaf_id   ObjID NULL,
    cmas_id   ObjID NULL,
    CICSPLEX  NVARCHAR(40) NULL,
    CMAS      ObjName,
    JOBNAME   NVARCHAR(40) NULL,
    APPLID    NVARCHAR(40) NULL,
    SystemId  NVARCHAR(40) NULL,
    MVSID     NVARCHAR(40) NULL,
    STATE1    NVARCHAR(40) NULL,
    os_id     ObjID NULL,
    cmas_row  INT IDENTITY(0,1)
  )

  CREATE TABLE #CIPX (
    id     ObjID NOT NULL,
    name   ObjName
  )

  INSERT INTO #CIPX
    SELECT DISTINCT id, name 
      FROM CICSPlex_V v
      JOIN containment c ON c.dst_cid = 'CIPX' AND c.dst_id = v.id AND c.deleted = 0
      JOIN CPSMDiscovery d ON v.name = d.CICSPLEX

  INSERT #CMAS
    SELECT cipx.id, NULL, NULL, CICSPLEX, CMAS, JOBNAME, APPLID, SystemId, MVSID, STATE1, NULL
    FROM
      CPSMDiscovery AS d
        LEFT OUTER JOIN #CIPX AS cipx ON cipx.name = d.CICSPLEX
    WHERE
      batchid = @batchid
      AND d.cid = 'CICM'

  -- Update CMAF ids
  UPDATE c SET cmaf_id = cipx_cmaf.dst_id
  FROM
    #CMAS AS c
      JOIN CIPX_CMAF_PHYC AS cipx_cmaf ON cipx_cmaf.src_id = c.cipx_id
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
    RETURN -30100
  END

  -- Update CMAS ids
  UPDATE c SET cmas_id = cmas.id
  FROM
    #CMAS AS c
      JOIN CIPX_CMAF_PHYC AS cipx_cmaf ON cipx_cmaf.src_id = c.cipx_id
      JOIN CMAF_CMAS_USEC AS cmaf_cmas ON cmaf_cmas.src_id = cipx_cmaf.dst_id
      JOIN CMAS_C AS cmas ON cmas.id = cmaf_cmas.dst_id AND cmas.name = c.CMAS
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
    RETURN -30101
  END

  IF @verbose = 1
    SELECT '#CMAS' AS '#CMAS', * FROM #CMAS

  -- Create any required CMAS folders underneath the CICSPlexes
  CREATE TABLE #CMAF
  (
    cipx_id ObjID,
    cmaf_id ObjID NULL
  )
  INSERT #CMAF 
    SELECT DISTINCT cipx_id, cipx_cmaf.dst_id 
    FROM #CMAS 
      LEFT OUTER JOIN CIPX_CMAF_PHYC AS cipx_cmaf ON cipx_cmaf.src_id = #CMAS.cipx_id
    WHERE
      #CMAS.cipx_id IS NOT NULL
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
    RETURN -30102
  END

  IF EXISTS (SELECT * FROM #CMAF WHERE cmaf_id IS NULL)
  BEGIN
    PRINT 'Creating new CMASFolder objects'
    -- Create a temp table with an identity column and one row per each CMASFolder that needs to be created
    CREATE TABLE #NEW_CMAF
    (
      cipx_id  ObjID,
      rowno    INT IDENTITY(0,1)
    )
    DECLARE @nCMAF INT
    INSERT #NEW_CMAF(cipx_id) SELECT DISTINCT cipx_id FROM #CMAF WHERE cmaf_id IS NULL
    SELECT @nCMAF = @@ROWCOUNT
    DECLARE @base_id ObjID
    EXEC @base_id = alloc_CMAF @nCMAF

    IF @verbose = 1
      SELECT 'NEW_CMAF' AS 'NEW_CMAF', * FROM #NEW_CMAF

    -- Insert the instance records for the CMAS folders
    INSERT CMASFolder_C(id, name, _ChildEventMatrixID, _ExceptionMatrixID) SELECT @base_id + rowno, '', 0, 0 FROM #NEW_CMAF

    -- Insert the link records specifying the parent links for the CMAS folders
    INSERT CIPX_CMAF_PHYC(src_id, dst_id) SELECT cipx_id, @base_id + rowno FROM #NEW_CMAF

    -- Insert object creation notification records
    INSERT PendingNotification(type, fields, detail)
      SELECT
        EventType.type,
        CONVERT(NVARCHAR(5), CMAF.cno) + '|' + CONVERT(NVARCHAR(9), @base_id + rowno) + '|CMAF|PHYC|' +
        CONVERT(NVARCHAR(5), CIPX.cno) + '|' + CONVERT(NVARCHAR(9), cipx_id) AS fields,
        '' AS detail
      FROM
        #NEW_CMAF,
        EventType,
        obj_class AS CMAF,
        obj_class AS CIPX
      WHERE
        EventType.name = 'ObjectCreated'
        AND CMAF.cid = 'CMAF'
        AND CIPX.cid = 'CIPX'

    -- Update the #CMAS table with the new CMAF ids
    UPDATE c SET cmaf_id = @base_id + rowno FROM #CMAS AS c JOIN #NEW_CMAF AS n ON n.cipx_id = c.cipx_id WHERE c.cmaf_id IS NULL
  END

  -- It is possible that there are CMAS objects that have already been discovered and which reside under
  -- an OS but which were not previously known to be used by a given CICSPlex. We need to look for these
  -- by matching the CICSPlex, CMAS name, and OS name from the discovery data to the OS_CMAS_PHYC and
  -- CMAS_C tables. We do this only for the discovery records that were not matched up based on previously
  -- existing CMAF_CMAS_USEC relationships. We first have to create a temp table that counts the number
  -- of matches for a given CMAS/MVSID pair since we can only associate a CMAS to an OS when there are
  -- distinct matches.
  CREATE TABLE #os_cmas_by_name
  (
    CMAS      NVARCHAR(40),
    SystemId  NVARCHAR(40) NULL,
    MVSID     NVARCHAR(40) NULL,
    cmas_id   ObjID,
    nmatches  INT
  )
  INSERT #os_cmas_by_name
    SELECT c.CMAS, c.SystemId, c.MVSID, cmas.id, COUNT(*)
    FROM #CMAS AS c
      JOIN OS_V AS os ON (os._SystemId = c.SystemId OR (c.SystemId IS NULL AND os._SMFID = c.MVSID))
      JOIN OS_CMAS_PHYC AS os_cmas ON os_cmas.src_id = os.id
      JOIN CMAS_C AS cmas ON cmas.id = os_cmas.dst_id AND cmas.name = c.CMAS
    WHERE
      (c.SystemId IS NOT NULL OR c.MVSID IS NOT NULL)
      AND c.cmas_id IS NULL
    GROUP BY c.CMAS, c.SystemId, c.MVSID, cmas.id
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
    RETURN -30103
  END

  -- Update the temp table used for tracking the progress of the discovery process with
  -- the ids of the OS/CMAS pairs that match distinctly.
  UPDATE c SET cmas_id = byname.cmas_id, os_id = os.id
  FROM #CMAS AS c
    JOIN OS_V AS os ON (os._SystemId = c.SystemId OR (c.SystemId IS NULL AND os._SMFID = c.MVSID))
    JOIN OS_CMAS_PHYC AS os_cmas ON os_cmas.src_id = os.id
    JOIN CMAS_C AS cmas ON cmas.id = os_cmas.dst_id AND cmas.name = c.CMAS
    JOIN #os_cmas_by_name AS byname ON byname.cmas_id = os_cmas.dst_id
  WHERE
    byname.nmatches = 1
    AND (c.SystemId IS NOT NULL OR c.MVSID IS NOT NULL)
    AND c.cmas_id IS NULL
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
    RETURN -30104
  END

  IF @verbose = 1
    SELECT '#os_cmas_by_name' AS '#os_cmas_by_name', * FROM #os_cmas_by_name

  -- Create the links that establish the usage of the CMAS's discovered by name
  IF EXISTS
  (
    SELECT *
    FROM
      #CMAS AS c
        JOIN #os_cmas_by_name AS byname ON byname.cmas_id = c.cmas_id AND byname.CMAS = c.CMAS
    WHERE
      byname.nmatches = 1
      AND c.cmaf_id IS NOT NULL
      AND NOT EXISTS (SELECT * FROM CMAF_CMAS_USEC WHERE src_id = c.cmaf_id AND dst_id = c.cmas_id)
  )
  BEGIN
    IF @verbose = 1
      SELECT 'NEW CMAF_CMAS_USEC' AS 'NEW CMAF_CMAS_USEC', c.cmaf_id, c.cmas_id, *
      FROM
        #CMAS AS c
          JOIN #os_cmas_by_name AS byname ON byname.cmas_id = c.cmas_id AND byname.CMAS = c.CMAS
      WHERE
        byname.nmatches = 1
        AND NOT EXISTS (SELECT * FROM CMAF_CMAS_USEC WHERE src_id = c.cmaf_id AND dst_id = c.cmas_id)
    INSERT PendingNotification(type, fields, detail)
      SELECT
        et.type,
        'USEC' + '|' +
        CONVERT(NVARCHAR(9), ISNULL(src.cno, 0))     + '|' +
        CONVERT(NVARCHAR(9), ISNULL(c.cmaf_id, 0))   + '|' +
        CONVERT(NVARCHAR(9), ISNULL(dst.cno, 0))     + '|' +
        CONVERT(NVARCHAR(9), ISNULL(c.cmas_id, 0))   + '|' +
        CONVERT(NVARCHAR(9), ISNULL(lt.flags, 0)),
        ''
      FROM
        EventType AS et,
        obj_class AS src,
        obj_class AS dst,
        link_type AS lt,
        #CMAS AS c
          JOIN #os_cmas_by_name AS byname ON byname.cmas_id = c.cmas_id AND byname.CMAS = c.CMAS
      WHERE
        byname.nmatches = 1
        AND NOT EXISTS (SELECT * FROM CMAF_CMAS_USEC WHERE src_id = c.cmaf_id AND dst_id = c.cmas_id)
        AND et.name = 'LinkCreated'
        AND src.cid = 'CMAF'
        AND dst.cid = 'CMAS'
        AND lt.link_type = 'USEC'
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
      RETURN -30105
    END
    INSERT CMAF_CMAS_USEC(src_id, dst_id)
      SELECT c.cmaf_id, c.cmas_id
      FROM
        #CMAS AS c
          JOIN #os_cmas_by_name AS byname ON byname.cmas_id = c.cmas_id AND byname.CMAS = c.CMAS
      WHERE
        byname.nmatches = 1
        AND NOT EXISTS (SELECT * FROM CMAF_CMAS_USEC WHERE src_id = c.cmaf_id AND dst_id = c.cmas_id)
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
      RETURN -30106
    END
  END

  -- Update attributes of existing CMAS objects
  UPDATE cmas SET _APPLID = d.APPLID, _JOBNAME = d.JOBNAME, _SystemName = d.MVSID
  FROM CMAS_C AS cmas JOIN #CMAS AS d ON d.cmas_id = cmas.id
  WHERE
    cmas._APPLID <> d.APPLID
    OR cmas._JOBNAME <> d.JOBNAME
    OR cmas._SystemName <> d.MVSID
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesCMAS
    RETURN -30107
  END

  -- Create required CMAS objects
  IF EXISTS (SELECT * FROM #CMAS WHERE cmas_id IS NULL)
  BEGIN
    IF @verbose = 1
      SELECT 'NEW_CMAS' AS 'NEW_CMAS', *
      FROM #CMAS
      WHERE cmas_id IS NULL
    CREATE TABLE #NEW_CMAS 
    (
      cipx_id  ObjID NULL, 
      cmaf_id  ObjID NULL,
      CMAS     NVARCHAR(40),
      cmas_id  ObjID NULL,
      APPLID   NVARCHAR(40) NULL,
      JOBNAME  NVARCHAR(40) NULL,
      SystemId NVARCHAR(40) NULL,
      MVSID    NVARCHAR(40) NULL,
      STATE1   NVARCHAR(40) NULL,
      cmas_row INT,
      rowno    INT IDENTITY(0, 1)
    )
    INSERT #NEW_CMAS
      SELECT cipx_id, cmaf_id, CMAS, cmas_id, APPLID, JOBNAME, SystemId, MVSID, STATE1, cmas_row
      FROM #CMAS
      WHERE cmas_id IS NULL
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN _DiscoverCPSMResourcesCMAS
      RETURN -30108
    END
    DECLARE @nCMAS INT
    SELECT @nCMAS = COUNT(*) FROM #CMAS
    EXEC @base_id = alloc_CMAS @nCMAS

    -- Insert the instance records for the CMAS objects
    INSERT INTO CMAS_C(id, name, _APPLID, _JOBNAME, _SystemName, _SysplexResource, _ChildEventMatrixID, _ExceptionMatrixID, _CurrentStateID) 
      SELECT @base_id + n.rowno, n.CMAS, n.APPLID, n.JOBNAME, n.MVSID, 0, 0, 0, ISNULL(s.id, 0)
      FROM #NEW_CMAS AS n
      LEFT OUTER JOIN StateType_V AS s ON s.name LIKE 'CMAS%' AND UPPER(s._State) = UPPER(n.STATE1)
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN _DiscoverCPSMResourcesCMAS
      RETURN -30109
    END

    -- Insert the link records specifying the usage links for the CMAS objects by CICSPlex
    INSERT INTO CMAF_CMAS_USEC(src_id, dst_id)
      SELECT n.cmaf_id, @base_id + n.rowno
      FROM #NEW_CMAS AS n 
      WHERE n.cmaf_id IS NOT NULL
IF @@ROWCOUNT > 0
      -- Insert object creation notification records
      INSERT PendingNotification(type, fields, detail)
        SELECT
          et.type,
          CONVERT(NVARCHAR(5), CMAS.cno) + '|' + CONVERT(NVARCHAR(9), @base_id + n.rowno) + '|CMAS|USEC|' +
          CONVERT(NVARCHAR(5), CMAF.cno) + '|' + CONVERT(NVARCHAR(9), n.cmaf_id) AS fields,
          n.CMAS AS detail
        FROM
          #NEW_CMAS AS n,
          EventType AS et,
          obj_class AS CMAS,
          obj_class AS CMAF
        WHERE
          et.name = 'ObjectCreated'
          AND CMAS.cid = 'CMAS'
          AND CMAF.cid = 'CMAF'
          AND n.cmaf_id IS NOT NULL

    -- Update the #CMAS table with the new CMAS ids
    UPDATE c SET cmas_id = @base_id + rowno 
      FROM #CMAS AS c JOIN #NEW_CMAS AS n ON c.cmas_row = n.cmas_row
  END

  IF @incremental = 0
  BEGIN
    -- Remove references to CMAS objects that were not rediscovered.
    IF @verbose = 1
      SELECT 'UNREFERENCED CMAS' AS 'UNREFERENCED CMAS', * 
      FROM CMAF_CMAS_USEC as usec
        JOIN CIPX_CMAF_PHYC AS c ON c.dst_id = usec.src_id
        WHERE EXISTS (SELECT * FROM #CMAS WHERE c.src_id = cipx_id AND c.dst_id = cmaf_id)
          AND NOT EXISTS (SELECT * FROM #CMAS WHERE cipx_id = c.src_id AND cmaf_id = c.dst_id AND cmas_id = usec.dst_id)
    UPDATE usec SET deleted = 1
    FROM CMAF_CMAS_USEC as usec
      JOIN CIPX_CMAF_PHYC AS c ON c.dst_id = usec.src_id
      WHERE EXISTS (SELECT * FROM #CMAS WHERE c.src_id = cipx_id AND c.dst_id = cmaf_id)
        AND NOT EXISTS (SELECT * FROM #CMAS WHERE cipx_id = c.src_id AND cmaf_id = c.dst_id AND cmas_id = usec.dst_id)
  END

  -- Try to match all CMAS regions that have a MVSID specified to an OS
  UPDATE c SET os_id = os.id
  FROM #CMAS AS c 
    JOIN OS_CMAS_PHYC AS os_cmas ON c.cmas_id = os_cmas.dst_id
    JOIN OS_C AS os ON os.id = os_cmas.src_id

  IF @verbose = 1
    SELECT '#CMAS_AFTER_CREATE' AS '#CMAS_AFTER_CREATE', * FROM #CMAS

  IF EXISTS (SELECT * FROM #CMAS WHERE MVSID IS NOT NULL AND os_id IS NULL)
  BEGIN
    -- Only those records with a distinct match of MVSID to OS get updated,
    -- so we first create a count table
    CREATE TABLE #CMAS_OS
    (
      cmas_id  ObjID,
      SystemId NVARCHAR(40) NULL,
      MVSID    NVARCHAR(40) NULL, 
      nmatches INT
    )
    IF @verbose = 1
    BEGIN
      SELECT '#CMAS_OS_by_SystemId' AS '#CMAS_OS_by_SystemId', c.cmas_id, c.SystemId, MVSID, COUNT(*) 
        FROM #CMAS AS c 
        JOIN OS_V AS os ON os._SystemId = c.SystemId
        JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
        WHERE c.SystemId IS NOT NULL AND c.os_id IS NULL
        GROUP BY c.cmas_id, c.SystemId, c.MVSID
      SELECT '#CMAS_OS_by_MVSID' AS '#CMAS_OS_by_MVSID', c.cmas_id, c.SystemId, c.MVSID, COUNT(*) 
        FROM #CMAS AS c 
        JOIN OS_V AS os ON os._SMFID = c.MVSID
        JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
        WHERE c.SystemId IS NULL AND c.MVSID IS NOT NULL AND c.os_id IS NULL
        GROUP BY c.cmas_id, c.SystemId, c.MVSID
    END
    INSERT INTO #CMAS_OS 
      SELECT c.cmas_id, c.SystemId, MVSID, COUNT(*) 
      FROM #CMAS AS c 
      JOIN OS_V AS os ON os._SystemId = c.SystemId
      JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
      WHERE c.SystemId IS NOT NULL AND c.os_id IS NULL
      GROUP BY c.cmas_id, c.SystemId, c.MVSID
    INSERT INTO #CMAS_OS 
      SELECT c.cmas_id, c.SystemId, c.MVSID, COUNT(*) 
      FROM #CMAS AS c 
      JOIN OS_V AS os ON os._SMFID = c.MVSID
      JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
      WHERE c.SystemId IS NULL AND c.MVSID IS NOT NULL AND c.os_id IS NULL
      GROUP BY c.cmas_id, c.SystemId, c.MVSID

    INSERT INTO OS_CMAS_PHYC(src_id, dst_id) 
      SELECT os.id, c.cmas_id
      FROM #CMAS_OS AS c 
      JOIN OS_V AS os ON (os._SystemId = c.SystemId OR (c.SystemId IS NULL AND os._SMFID = c.MVSID))
      JOIN containment AS con ON con.dst_cid = 'OS' AND con.dst_id = os.id AND con.deleted = 0
      WHERE c.nmatches = 1
    INSERT INTO PendingNotification(type, fields, detail)
      SELECT
        et.type,
        'PHYC' + '|' +
        CONVERT(NVARCHAR(9), ISNULL(src.cno, 0))   + '|' +
        CONVERT(NVARCHAR(9), ISNULL(l.src_id, 0))  + '|' +
        CONVERT(NVARCHAR(9), ISNULL(dst.cno, 0))   + '|' +
        CONVERT(NVARCHAR(9), ISNULL(c.cmas_id, 0)) + '|' +
        CONVERT(NVARCHAR(9), ISNULL(lt.flags, 0)),
        ''
      FROM
        EventType AS et,
        obj_class AS src,
        obj_class AS dst,
        link_type AS lt,
        #CMAS_OS AS c JOIN OS_CMAS_PHYC AS l ON l.dst_id = c.cmas_id
      WHERE
        c.nmatches = 1
        AND et.name = 'LinkCreated'
        AND src.cid = 'OS'
        AND dst.cid = 'CMAS'
        AND lt.link_type = 'PHYC'
    UPDATE c SET os_id = cmas_os.src_id
    FROM #CMAS AS c JOIN OS_CMAS_PHYC AS cmas_os ON cmas_os.dst_id = c.cmas_id
  END

  -- Update the CPSMDiscovery table with the ids of the discovered objects
  UPDATE d SET
    obj_cid = 'CMAS',
    obj_id = c.cmas_id
  FROM
    CPSMDiscovery AS d
      JOIN #CMAS AS c ON
        d.cid = 'CICM'
        AND d.CMAS = c.CMAS
        AND (d.CICSPLEX = c.CICSPLEX OR c.CICSPLEX IS NULL)
        AND (d.SystemId = c.SystemId OR c.SystemId IS NULL)
        AND (d.MVSID = c.MVSID OR c.MVSID IS NULL)
  WHERE
    d.batchid = @batchid

  -- If we're the outer transaction then commit
  IF @tcount = 0
    COMMIT TRANSACTION _DiscoverCPSMResourcesCMAS
GO
