-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/_FindParentLinks.sqv   1.0   Jul 19 2001 16:50:46   BRENT  $
--
-- _FindParentLinks.sql
--
-- Module Description
--   Defines the _FindParentLinks stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _FindParentLinks
PRINT 'Creating procedure _FindParentLinks'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_FindParentLinks' AND type = 'P')
  DROP PROCEDURE _FindParentLinks
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/_FindParentLinks.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 19 2001 16:50:46  $
-- $Author:   BRENT  $
-- Implements the FindParentLinks method.
-- Returns the list of managed objects under the calling object that match the specified criteria.
CREATE PROCEDURE _FindParentLinks
  @cid              ClassID,
  @id               ObjID,
  @MaxGenerations   INT = 0
AS
  SET NOCOUNT ON
  
  DECLARE @cname NVARCHAR(24), @id_str NVARCHAR(9)
  DECLARE @src_cid ClassID, @src_id ObjID, @src_name ObjName

  -- Get the list of ancestor objects of the calling object
  CREATE TABLE #ancestor
  (
    link_type LinkType,
    src_cid   ClassID,
    src_id    ObjID,
    dst_cid   ClassID,
    dst_id    ObjID,
    src_name  ObjName NULL,
    gen       INT
  )    

  INSERT #ancestor SELECT '', @cid, @id, '', 0, NULL, 0

  -- IF object IS directly linked THEN get its parent info dynammically
  IF (SELECT flag_direct_containment FROM obj_class WHERE cid = @cid) = 1
  BEGIN
    SELECT @cname = cname, @id_str = CONVERT(NVARCHAR(9), @id) FROM obj_class WHERE cid = @cid
    EXEC("
    DECLARE parentc INSENSITIVE CURSOR FOR SELECT _ParentCID, _ParentID FROM " + @cname + "_V WHERE id = " + @id_str
    )
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_FindParentLinks @cid=''%s'', @id=%d: Error looking up ParentCID, ParentID', 3, 1, @cid, @id)
      RETURN -30000
    END
    OPEN parentc
    FETCH NEXT FROM parentc INTO @src_cid, @src_id
    DEALLOCATE parentc
    INSERT #ancestor SELECT '', @src_cid, @src_id, @cid, @id, NULL, 1
  END

  -- Start iterating through the link generations, up the specified @MaxGenerations
  DECLARE @gen INT
  SET @gen = 1
  IF @MaxGenerations = 0 SELECT @MaxGenerations = max(generation) FROM link_chain
  WHILE @gen <= @MaxGenerations
  BEGIN
    INSERT #ancestor 
      SELECT src.link_type, src.src_cid, src.src_id, src.dst_cid, src.dst_id, NULL, @gen
      FROM #ancestor AS prev JOIN link AS src ON src.dst_cid = prev.src_cid AND src.dst_id = prev.src_id
      WHERE prev.gen = @gen - 1
    SET @gen = @gen + 1
  END

  -- We don't need the seed row anymore
  DELETE #ancestor WHERE gen = 0

  -- Lookup parent object names
  DECLARE srcc INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid FROM #ancestor
  OPEN srcc
  FETCH NEXT FROM srcc INTO @src_cid
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @cname = cname FROM obj_class WHERE cid = @src_cid
    EXEC("
    UPDATE #ancestor SET src_name = name FROM " + @cname + "_V WHERE src_cid = '" + @src_cid + "' AND id = src_id
    ")
    FETCH NEXT FROM srcc INTO @src_cid
  END
  DEALLOCATE srcc

  SELECT
    link_type,
    src_cid,
    src.cno AS src_cno,
    src_id,
    dst_cid,
    dst.cno AS dst_cno,
    dst_id,
    src_name,
    gen
  FROM #ancestor AS a
    JOIN obj_class AS src ON src.cid = a.src_cid
    JOIN obj_class AS dst ON dst.cid = a.dst_cid
GO
