-- $Header:   //ARCHIVE/Archive_NLS/db/sql/_GetDependencies.sql   1.0   Jun 29 2001 11:37:08   Marc  $

USE Object
GO

PRINT 'Creating procedure _GetDependencies'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetDependencies' AND type = 'P')
  DROP PROCEDURE _GetDependencies
GO

-- $Archive:   //ARCHIVE/Archive_NLS/db/sql/_GetDependencies.sql  $
-- $Revision:   1.0  $
-- $Author:   Marc  $
-- $Date:   Jun 29 2001 11:37:08  $
-- Procedure used by 'Business Impact' function to return an inverted hierarchy
-- where calling object is on the top, and all objects that are linked to it from
-- source links are shown underneath it.
CREATE PROCEDURE _GetDependencies
  @cid ClassID,
  @id ObjID,
  @securityLevel INT = 0
AS
  SET NOCOUNT ON

  -- Temp table holding all generations of objects being collected
  CREATE TABLE #dependencies
  (
    src_cid ClassID,
    src_id  ObjID,
    dst_cid ClassID,

    dst_id  ObjID,
    phy_cid ClassID,
    phy_id  ObjID,
    gen     INT,
    node    NVARCHAR(255)
  )

  -- Temp table for holding the objects collected in one generation. Needed to generate the IDENTITY
  -- values for the node ids.
  CREATE TABLE #gen
  (
    src_cid ClassID,
    src_id  ObjID,
    dst_cid ClassID,
    dst_id  ObjID,
    gen     INT,
    nodeid  INT IDENTITY(1,1)
  )

  -- Temp table for holding the objects collected in one generation. Needed to generate the IDENTITY
  -- values for the node ids.
  CREATE TABLE #gen1
  (
    src_cid ClassID,
    src_id  ObjID,
    dst_cid ClassID,
    dst_id  ObjID,
    gen     INT,
    nodeid  INT IDENTITY(1,1),
    link_type ClassID
  )

  -- If the calling object is an LOB then we resolve the LOB to the physical object before starting
  IF @cid = 'LOB'
    EXEC asisp_followphysicallink @id, @cid out, @id out

  -- Seed table with row for object and its immediate links
  DECLARE @gen INT, @rc INT, @rc2 INT
  SELECT @gen = 0, @rc = 1
  INSERT #dependencies SELECT @cid, @id, '', 0, @cid, @id, @gen, '00001'
  SELECT @gen = @gen + 1

  -- if the user does not have permission to view the physical tree, then skip the physical objects
  IF (@securityLevel IS NOT NULL AND @securityLevel <> 0) OR @cid = 'LOB'
  BEGIN
    INSERT #gen SELECT src_cid, src_id, dst_cid, dst_id, @gen FROM containment c where dst_cid = @cid AND dst_id = @id AND deleted = 0
      AND src_cid NOT IN ('LOBC', 'BUSC')
    SELECT @rc = @@ROWCOUNT
  END

  -- Seed LOB links. We skip over the immediately linked LOBs in order to avoid the awkwardness of displaying the a root object
  -- surrounded by LOB objects that look just like it.
  INSERT #gen 
    SELECT l2.src_cid, l2.src_id, l2.dst_cid, l2.dst_id, @gen 
    FROM link l1, link l2
    WHERE 
      l1.link_type = 'LOBL' AND l1.dst_cid = @cid AND l1.dst_id = @id AND l1.deleted = 0
      AND l2.link_type LIKE 'LOB[CL]' AND l2.dst_cid = l1.src_cid AND l2.dst_id = l1.src_id AND l2.deleted = 0
    AND l2.src_cid NOT IN ('LOBC', 'BUSC')
  SELECT @rc = @rc + @@ROWCOUNT
  INSERT #dependencies
    SELECT g.src_cid, g.src_id, g.dst_cid, g.dst_id, g.src_cid, g.src_id, g.gen, 
      '00001.' + REPLICATE('0', 5-LEN(CONVERT(NVARCHAR(9), nodeid))) + CONVERT(NVARCHAR(9), nodeid)
    FROM #gen g

  -- Now traverse up the physical and logical trees
  WHILE @rc > 0 AND @gen < 6
  BEGIN
--SELECT @gen AS GEN
    -- Get the containing objects for the previous generation
    SELECT @gen = @gen + 1
    TRUNCATE TABLE #gen
    -- Follow containment links
    INSERT #gen
      SELECT DISTINCT c.src_cid, c.src_id, c.dst_cid, c.dst_id, @gen
      FROM containment c, #dependencies prev
      WHERE prev.gen = @gen - 1 AND c.dst_cid = prev.src_cid AND c.dst_id = prev.src_id AND c.src_cid NOT IN ('BUSC', 'LOBC') AND c.deleted = 0
    SELECT @rc = @@ROWCOUNT

--SELECT 'containment' as 'containment', @gen AS CurrentGen, * FROM #gen

    -- Follow logical links for LOBs
    INSERT #gen
      SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen
      FROM link l, #dependencies prev
      WHERE prev.gen = @gen - 1 AND prev.src_cid='LOB' AND l.link_type = 'LOBL'
        AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.deleted = 0 AND l.src_cid <> 'LOB'
    SELECT @rc = @rc + @@ROWCOUNT

--SELECT 'LOBL' as 'LOBL', @gen AS CurrentGen, * FROM #gen

    TRUNCATE TABLE #gen1
    INSERT #gen1
      SELECT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen, l.link_type
      FROM link l, #dependencies prev
      WHERE prev.gen = @gen - 1 AND prev.src_cid='LOB' AND l.link_type = 'LOBL'
        AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.src_cid = 'LOB' AND l.deleted = 0
    SELECT @rc2 = @@ROWCOUNT
    SELECT @rc = @rc + @rc2
    WHILE @rc2 > 0
    BEGIN
      INSERT #gen1
        SELECT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen, l.link_type
        FROM link l, #gen1 prev
        WHERE prev.src_cid = 'LOB' AND l.link_type = 'LOBL'
          AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.deleted = 0
      SELECT @rc2 = @@ROWCOUNT
      DELETE FROM #gen1 WHERE src_cid = dst_cid AND src_id = dst_id
    END
    -- Resolve LOBL links FROM LOB to LOB into LOBC links
    INSERT INTO #gen1 SELECT l.src_cid, l.src_id, prev.dst_cid, prev.dst_id, @gen, l.link_type
      FROM #gen1 prev, link l
      WHERE prev.src_cid = 'LOB' AND l.link_type = 'LOBC' AND prev.link_type = 'LOBL' AND gen = @gen
        AND l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id AND l.deleted = 0
        INSERT #gen SELECT src_cid, src_id, dst_cid, dst_id, gen
      FROM #gen1 WHERE @gen = gen AND link_type <> 'LOBL'

    INSERT #dependencies
      SELECT g.src_cid, g.src_id, g.dst_cid, g.dst_id, g.src_cid, g.src_id, g.gen,
        prev.node + '.' + REPLICATE('0', 5-LEN(CONVERT(NVARCHAR(9), nodeid))) + CONVERT(NVARCHAR(9), nodeid)
      FROM #dependencies prev, #gen g
      WHERE g.dst_cid = prev.src_cid AND g.dst_id = prev.src_id
  END

--SELECT 'AFTER' AS 'AFTER', * FROM #dependencies

  -- Now we need to resolve all LOBs to their linked physical objects (if they have any). This drives the display of the object type
  UPDATE d SET phy_cid = ll.phy_cid, phy_id = ll.phy_id 
    FROM #dependencies AS d JOIN lob_link AS ll ON d.phy_cid = 'LOB' AND d.phy_id = ll.dst_id
  WHERE
    ll.phy_cid IS NOT NULL

  -- Create the temp table used to display results to the caller. We could collapse this and #dependencies
  -- if there is a performance problem.
  CREATE TABLE #nodes
  (
    nodeid            NVARCHAR(255),
    generation        INT,
    cid               ClassID,
    id                ObjID,
    label             NVARCHAR(255),
    name              ObjName,
    alertstateid      INT,
    resid             NVARCHAR(8),
    hasownershipnote  TINYINT,
    priorityid        INT,
    numchildren       INT,
    phy_cid           ClassID
  )
  INSERT #nodes SELECT node, gen, src_cid, src_id, label, '', 1, '', 0, 0, 0, phy_cid FROM #dependencies d, obj_class c WHERE c.cid = d.phy_cid

  -- Update the name and resid columns by cursoring over the distinct set of classes represented
  -- in #dependencies
  DECLARE dc CURSOR FOR SELECT DISTINCT d.cid, c.cname, pc.cid, pc.cname FROM #nodes d, obj_class c, obj_class pc WHERE c.cid = d.cid AND pc.cid = phy_cid
  OPEN dc
  DECLARE @cname NVARCHAR(24), @phy_cid ClassID, @phy_cname NVARCHAR(24)
  FETCH NEXT FROM dc INTO @cid, @cname, @phy_cid, @phy_cname
  WHILE @@fetch_status <> -1
  BEGIN
    EXEC("
      UPDATE n SET name = v.name, alertstateid=_AlertStateID, priorityid=_PriorityID, hasownershipnote=_HasOwnershipNote 
      FROM #nodes AS n, " + @cname + "_V v
      WHERE n.cid='" + @cid + "' AND v.id = n.id
    ")
    EXEC("
      UPDATE #nodes SET resid = _LargeIconName FROM " + @phy_cname + "_S WHERE phy_cid = '" + @phy_cid + "'
    ")
    FETCH NEXT FROM dc INTO @cid, @cname, @phy_cid, @phy_cname
  END
  DEALLOCATE dc

  SELECT nodeid, generation, cid, id, label, name, alertstateid, resid, hasownershipnote, priorityid, numchildren, phy_cid, 'deleted' = 0 FROM #nodes ORDER BY nodeid
GO

PRINT 'Done.'
GO