-- $Header:   //Archive/Archive_NLS/DB/sql/_GetMenuItems.sqv   1.1   Jun 20 2001 17:24:42   Blaine  $

PRINT 'Creating procedure _GetMenuItems...'

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetMenuItems' AND type = 'P')
  DROP PROCEDURE _GetMenuItems
GO

-- $Archive:   //Archive/Archive_NLS/DB/sql/_GetMenuItems.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jun 20 2001 17:24:42  $
-- $Author:   Blaine  $
CREATE PROCEDURE _GetMenuItems
  @cid          ClassID,
  @id           ObjID,
  @PrevVerNo    INT OUTPUT,       -- Input: Previous version retrieved by caller (0 FOR none).
                                  -- Output: The current version number being retrieved
  @test         TINYINT=0,        -- IF nonzero THEN the output IS formatted to look like the
                                  -- menu tree would appear to a user. The output will contain
                                  -- only the menu item labels, properly indented to indicate
                                  -- the menu hierarchy.
  @ClientVerNo  NVARCHAR(16)='002.001.003.000'
                                  -- Optional client version. IN the form 'nnn.nnn.nnn.nnn'.
                                  -- IF specified THEN only menu items with a min/max version
                                  -- range that contains the client''s version are returned.'
AS
  -- See IF caller IS up-to-date
  DECLARE @CurrVerNo INT
  SELECT @CurrVerNo = verno FROM MenuItemVersion
  IF @PrevVerNo > 0 AND @PrevVerNo = @CurrVerNo
    RETURN

  -- SET parameter so caller knows current version being retrieved
  SELECT @PrevVerNo = @CurrVerNo

  CREATE TABLE #mi
  (
    obj_cid     ClassID,
    obj_id      ObjID,
    name        sysname,
    id          INT,
    item_order  NCHAR(30),
    io_00       TINYINT,
    io_01       TINYINT,
    io_02       TINYINT,
    io_03       TINYINT,
    label       NVARCHAR(80),
    parent_id   INT,
    parent_name sysname,
    control_id  NVARCHAR(255),
    security    NVARCHAR(40),
    override_id INT,
    flags       INT,
    min_version NVARCHAR(16),
    max_version NVARCHAR(16)
  )

  DECLARE @mi_visibleself INT, @mi_visibleaggregate INT, @mi_eliminateoverride INT, @mi_separator INT
  SELECT 
    @mi_visibleself = 0x10000000, 
    @mi_visibleaggregate = 0x20000000,
    @mi_eliminateoverride = 0x00000080,
    @mi_separator = 0x00000001

  -- Get all 'self' menu items FOR the object. This IS based ON its class AND id.
  INSERT #mi 
    SELECT DISTINCT obj_cid, obj_id, name, id, item_order, io_00, io_01, io_02, io_03, label, parent_id, '', control_id, security, override_id, mi.flags, min_version, max_version
    FROM MenuItem mi, isa_chain i
    WHERE 
      i.cid = @cid 
      AND mi.obj_cid = i.base_cid 
      AND (mi.obj_id = 0 OR (mi.obj_cid = @cid AND mi.obj_id = @id))
      AND mi.flags & @mi_visibleself <> 0

  -- Get all aggregate menu items FOR the object. This IS based ON the classes that can be
  -- contained either directly OR indirectly underneath the class of the calling object.
  INSERT #mi 
    SELECT DISTINCT obj_cid, obj_id, name, id, item_order, io_00, io_01, io_02, io_03, label, parent_id, '', control_id, security, override_id, mi.flags, min_version, max_version
    FROM MenuItem mi, isa_chain i, link_chain l
    WHERE 
      l.src_cid = @cid
      AND i.base_cid = l.cid
      AND mi.obj_cid = i.cid 
      AND mi.obj_id = 0
      AND mi.flags & @mi_visibleaggregate <> 0
      AND NOT EXISTS (SELECT * FROM #mi WHERE #mi.id = mi.id)
/*
  --hack FOR direct containment
  INSERT #mi 
    SELECT DISTINCT obj_cid, obj_id, name, id, item_order, io_00, io_01, io_02, io_03, label, parent_id, '', control_id, security, override_id, mi.flags, min_version, max_version
    FROM
      MenuItem AS mi
        JOIN direct_containment AS d ON d.dst_cid = mi.obj_cid
        JOIN isa_chain AS i ON i.base_cid = d.src_cid
        JOIN link_chain l ON l.cid = i.base_cid AND l.dst_cid = @cid
    WHERE 
      mi.obj_id = 0
      AND mi.flags & @mi_visibleaggregate <> 0
      AND NOT EXISTS (SELECT * FROM #mi WHERE #mi.id = mi.id)
*/
  IF @cid = 'LOB'
  BEGIN
    DECLARE @mi_visiblelink INT
    SELECT @mi_visiblelink = 0x40000000

    -- Get all 'link/self' menu items FOR the ultimate physical object that this IS LOB IS linked to.
    INSERT #mi 
      SELECT DISTINCT obj_cid, obj_id, name, id, item_order, io_00, io_01, io_02, io_03, label, parent_id, '', control_id, security, override_id, mi.flags, min_version, max_version
      FROM MenuItem mi, isa_chain i, lob_link l
      WHERE 
        l.dst_id = @id
        AND i.cid = phy_cid 
        AND mi.obj_cid = i.base_cid 
        AND (mi.obj_id = 0 OR (mi.obj_cid = phy_cid AND mi.obj_id = phy_id))
        AND 
        mi.flags & (@mi_visiblelink + @mi_visibleself) = (@mi_visiblelink + @mi_visibleself)
        AND NOT EXISTS (SELECT * FROM #mi WHERE #mi.id = mi.id)

    -- Get all aggregate menu items FOR the object that would be callable FROM the object that
    -- this LOB IS linked to (IF any). These items must also have the 0x40000000 BIT SET.
    DECLARE @phy_cid ClassID
    SELECT DISTINCT @phy_cid = phy_cid FROM lob_link WHERE dst_id = @id
    IF @phy_cid IS NOT NULL
    BEGIN
      INSERT #mi 
        SELECT DISTINCT obj_cid, obj_id, name, id, item_order, io_00, io_01, io_02, io_03, label, parent_id, '', control_id, security, override_id, mi.flags, min_version, max_version
        FROM MenuItem mi, isa_chain i, link_chain l
        WHERE 
          l.src_cid = @phy_cid
          AND i.base_cid = l.cid
          AND mi.obj_cid = i.cid 
          AND mi.obj_id = 0
          AND mi.flags & (@mi_visiblelink + @mi_visibleaggregate) = (@mi_visiblelink + @mi_visibleaggregate)
          AND NOT EXISTS (SELECT * FROM #mi WHERE #mi.id = mi.id)
    END

    -- Get all aggregate menu items FOR the object that are based ON the DISTINCT SET of classes of
    -- the objects contained either directly OR indirectly underneath the calling LOB
    INSERT #mi 
      SELECT DISTINCT obj_cid, obj_id, name, id, item_order, io_00, io_01, io_02, io_03, label, parent_id, '', control_id, security, override_id, mi.flags, min_version, max_version
      FROM MenuItem mi, isa_chain i, link_chain lc, lob_link ll
      WHERE
        ll.src_id = @id
        AND lc.dst_cid = phy_cid
        AND i.base_cid = lc.cid
        AND mi.obj_cid = i.cid
        AND mi.obj_id = 0
        AND mi.flags & (@mi_visiblelink + @mi_visibleaggregate) = (@mi_visiblelink + @mi_visibleaggregate)
        AND NOT EXISTS (SELECT * FROM #mi WHERE #mi.id = mi.id)
/*
--hack
    INSERT #mi 
      SELECT DISTINCT obj_cid, obj_id, name, id, item_order, io_00, io_01, io_02, io_03, label, parent_id, '', control_id, security, override_id, mi.flags, min_version, max_version
      FROM
        MenuItem AS mi
          JOIN direct_containment AS d ON d.dst_cid = mi.obj_cid
          JOIN isa_chain AS i ON i.base_cid = d.src_cid
          JOIN lob_link AS ll ON ll.phy_cid = i.cid AND ll.dst_id = @id
      WHERE
        mi.obj_id = 0
        AND mi.flags & (@mi_visiblelink + @mi_visibleaggregate) = (@mi_visiblelink + @mi_visibleaggregate)
        AND NOT EXISTS (SELECT * FROM #mi WHERE #mi.id = mi.id)
*/
  END

  DELETE #mi WHERE id IN (SELECT override_id FROM #mi)  -- remove overridden entries

  DELETE #mi WHERE flags & @mi_eliminateoverride = @mi_eliminateoverride  -- remove eliminated entries

  -- Remove any item named 'DELETE' that IS ON an object of a non-deletable class
  DELETE #mi 
    FROM obj_class c, isa_chain i
    WHERE 
      #mi.name = 'DELETE' 
      AND #mi.obj_cid = i.base_cid 
      AND i.cid = @cid 
      AND c.cid = @cid 
      AND c.flag_nondeletable = 1

  -- Remove any items that are outside of the specified client version number
  IF @ClientVerNo IS NOT NULL
    DELETE #mi WHERE @ClientVerNo NOT between min_version AND max_version

  -- Remove any items that have a parent id that IS NOT also IN the result SET
  DELETE #mi WHERE parent_id <> 0 AND NOT EXISTS (SELECT * FROM #mi parent WHERE parent.id = #mi.parent_id)
  WHILE @@ROWCOUNT > 0
    DELETE #mi WHERE parent_id <> 0 AND NOT EXISTS (SELECT * FROM #mi parent WHERE parent.id = #mi.parent_id)

  -- Remove duplicate separators
  CREATE TABLE #mi_filtered
  (
    id          INT PRIMARY KEY,
    parent_id   INT,
    flags       INT,
    deleted     TINYINT,
    row_id      INT identity(0,1)
  )
  INSERT #mi_filtered(id, parent_id, flags, deleted) SELECT id, parent_id, flags, 0 FROM #mi
  UPDATE #mi_filtered SET deleted = 1 
  WHERE 
    flags & @mi_separator = @mi_separator 
    AND EXISTS 
    (
      SELECT * FROM #mi_filtered prev 
      WHERE 
        prev.row_id = #mi_filtered.row_id-1 
        AND prev.parent_id = #mi_filtered.parent_id
        AND prev.flags & @mi_separator = @mi_separator
    )
  DELETE #mi WHERE id NOT IN (SELECT id FROM #mi_filtered WHERE deleted = 0)

  -- remove TSD menu items IF Problem System NOT enabled
  IF NOT EXISTS (SELECT * FROM SystemConfiguration WHERE groupname = 'PROBLEM_SYSTEM' AND propertyname = 'SYSTEM_NAME' AND propertyvalue = 'TSD')
  BEGIN
    RAISERROR('_GetMenuItems: Removing TSD Distributed AND TSD/390 INSERT AND VIEW menu items...', 0, 1)
    DELETE FROM #mi WHERE name LIKE 'TSD[_]%' AND obj_cid='MNO2'
  END
  IF NOT EXISTS (SELECT * FROM SystemConfiguration WHERE groupname = 'PROBLEM_SYSTEM' AND propertyname = 'SYSTEM_NAME' AND propertyvalue = 'TSD390')
  BEGIN
    RAISERROR('_GetMenuItems: Removing TSD Distributed AND TSD/390 INSERT AND VIEW menu items...', 0, 1)
    DELETE FROM #mi WHERE name LIKE 'TSD390[_]%' AND obj_cid='MNO2'
  END

  -- remove TBSMChange menu items IF Change System is NOT integrated.
  -- If there is no request processor to process chnage requests then there isn't a Change System available
  -- so remove the menu items for Change.
  IF NOT EXISTS (SELECT * FROM SystemConfiguration WHERE groupname = 'REQUEST_CHANGE' AND propertyname = 'REQUEST_PROCESSOR_NAME')
  BEGIN
    RAISERROR('_GetMenuItems: Removing Change INSERT AND VIEW menu items...', 0, 1)
    DELETE FROM #mi WHERE name LIKE 'CHANGE[_]%' AND obj_cid='MNO2'
  END

  -- Execute any methods specified in the control_id field
  DECLARE @control_id NVARCHAR(255), @mid INT, @method MethodName
  DECLARE c_methods INSENSITIVE CURSOR FOR SELECT id, control_id FROM #mi WHERE control_id like '%[%]this.%[%]%'
  OPEN c_methods
  FETCH NEXT FROM c_methods INTO @mid, @control_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC asisp_processcontrolid @cid, @id, @control_id=@control_id OUTPUT
    UPDATE #mi SET control_id = @control_id WHERE id = @mid
    FETCH NEXT FROM c_methods INTO @mid, @control_id
  END
  CLOSE c_methods
  DEALLOCATE c_methods

  IF @test = 0
  BEGIN
    UPDATE #mi SET parent_name = parent.name FROM #mi, #mi parent WHERE parent.id = #mi.parent_id
    SELECT
      obj_cid,
      obj_id,
      name,
      id,
      item_order=
        CONVERT(INT,io_00)*1000000+
        CONVERT(INT,io_01)*10000+
        CONVERT(INT,io_02)*100+
        CONVERT(INT,io_03),
      label,
      parent_name,
      control_id,
      security,
      override_id,
      flags
    FROM #mi
    ORDER BY item_order
  END
  ELSE
  BEGIN
    CREATE TABLE #mitest ( name sysname, label NVARCHAR(80), mi_id INT, item_order NCHAR(30), flags INT, indent INT )
    INSERT #mitest SELECT name, label, id, item_order, flags, 0 FROM #mi
    UPDATE #mitest SET indent = 2 FROM MenuItem WHERE id = mi_id AND io_00 > 0
    UPDATE #mitest SET indent = 4 FROM MenuItem WHERE id = mi_id AND io_01 > 0
    UPDATE #mitest SET indent = 6 FROM MenuItem WHERE id = mi_id AND io_02 > 0
    UPDATE #mitest SET indent = 8 FROM MenuItem WHERE id = mi_id AND io_03 > 0
    UPDATE #mitest SET indent = 10 FROM MenuItem WHERE id = mi_id AND io_04 > 0
    UPDATE #mitest SET indent = 12 FROM MenuItem WHERE id = mi_id AND io_05 > 0
    UPDATE #mitest SET indent = 14 FROM MenuItem WHERE id = mi_id AND io_06 > 0
    UPDATE #mitest SET indent = 16 FROM MenuItem WHERE id = mi_id AND io_07 > 0
    UPDATE #mitest SET indent = 18 FROM MenuItem WHERE id = mi_id AND io_08 > 0
    UPDATE #mitest SET indent = 20 FROM MenuItem WHERE id = mi_id AND io_09 > 0
    UPDATE #mitest SET label = replicate('-', 30) WHERE flags & @mi_separator = @mi_separator
    UPDATE #mitest SET label = '<' + name + '>' WHERE label = ''
    SELECT replicate(' ', indent) + label, item_order FROM #mitest ORDER BY item_order
  END
  PRINT 'DONE.'
GO