-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/_LoadOPCPlan_CreateBCJs.sqv   1.0   Jul 17 2001 14:01:56   BRENT  $
--
-- _LoadOPCPlan_CreateBCJs.sql
--
-- Module Description
--   Defines the _LoadOPCPlan_CreateBCJs stored procedure


USE Object
GO

---------------------------------------------------------------------------------------------------
-- _LoadOPCPlan_CreateBCJs
PRINT 'Creating procedure _LoadOPCPlan_CreateBCJs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadOPCPlan_CreateBCJs' AND type = 'P')
  DROP PROCEDURE _LoadOPCPlan_CreateBCJs
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/_LoadOPCPlan_CreateBCJs.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 17 2001 14:01:56  $
-- $Author:   BRENT  $
-- Supporting procedure for _LoadOPCPlan that updates/creates the required batch job objects
CREATE PROCEDURE _LoadOPCPlan_CreateBCJs
  @plan_id    INT,
  @chkpt      INT OUTPUT
AS
  -- Transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _LoadOPCPlan_CreateBCJs
  ELSE
    SAVE TRAN _LoadOPCPlan_CreateBCJs

  DECLARE @msg NVARCHAR(255)

  -- Reset the _SchedStatusFlags and deleted columns for all existing jobs in the applications for this plan
  UPDATE b SET
    _SchedStatusFlags = 0,
    deleted = 0
  FROM BATCH_V AS b
  WHERE b._ParentCID = 'BTCY'
    AND b._ParentID IN (SELECT DISTINCT btcy_id FROM opc_operation WHERE plan_id = @plan_id)

  -- Replace NULL job_name values in opc_operation with empty strings so joins to BATCH_V are simpler
  EXEC asisp_checkpointmessage 'Updating NULL opc_operation.job_name values', 1, @chkpt
  SET @chkpt = @chkpt + 1
  UPDATE opc_operation SET job_name = '' WHERE plan_id = @plan_id AND job_name IS NULL

  EXEC asisp_checkpointmessage 'Updating opc_operation.job_id from existing BATCH records', 1, @chkpt
  SET @chkpt = @chkpt + 1
  UPDATE o SET job_id = b.id
    FROM opc_operation AS o JOIN BATCH_V AS b ON 
      b._ParentCID = 'BTCY'
      AND o.btcy_id = b._ParentID
      AND o.job_name = b.name
      AND o.op_name = b._OPCOpNum
      AND CONVERT(NVARCHAR(5), o.est_start, 108) = CONVERT(NVARCHAR(5), b._SchedStartTime, 108)
      AND o.instanceno = b._OPCOpInstanceNo
    WHERE o.plan_id = @plan_id
  IF @@ROWCOUNT > 0
  BEGIN
/*
    SELECT
      'update BATCH',
      COUNT(*) AS n,
      b._ParentID,
      b.name,
      b._OPCOpNum,
      o.est_start,
      o.duration,
      o.est_stop
    FROM
      BATCH_V AS b JOIN opc_operation AS o ON
        b._ParentCID = 'BTCY'
        AND o.btcy_id = b._ParentID 
        AND o.job_name = b.name
        AND o.op_name = b._OPCOpNum
    WHERE o.plan_id = @plan_id
    GROUP BY
      b._ParentID,
      b.name,
      b._OPCOpNum,
      o.est_start,
      o.duration,
      o.est_stop
    ORDER BY
      COUNT(*) DESC
*/
    -- Update BATCH schedule attributes
    UPDATE b SET
      _SchedStartTime = o.est_start,
      _SchedStatusFlags = 1,
      _SchedDuration = o.duration,
      _SchedStopTime = o.est_stop,
      _InitEstComplete = o.est_stop,
      _ImmedOverrideDuration = NULL,
      _ImmedOverrideStartTime = NULL
    FROM
      BATCH_V AS b JOIN opc_operation AS o ON
        b._ParentCID = 'BTCY'
        AND o.btcy_id = b._ParentID
        AND o.job_name = b.name
        AND o.op_name = b._OPCOpNum
        AND CONVERT(NVARCHAR(5), o.est_start, 108) = CONVERT(NVARCHAR(5), b._SchedStartTime, 108)
        AND o.instanceno = b._OPCOpInstanceNo
      WHERE o.plan_id = @plan_id
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('Aborting due to error updating BATCH_V from opc_operation', 3, 1)
      ROLLBACK TRAN _LoadOPCPlan_CreateBCJs
      RETURN -30000
    END
  END

  -- Any opc_operation rows for the plan with job_id = null must have BATCH entries created for them
  EXEC asisp_checkpointmessage 'Creating BATCH instances', 1, @chkpt
  DECLARE @nbcj INT
  SELECT @nbcj = COUNT(*) FROM opc_operation WHERE plan_id = @plan_id AND job_id IS NULL
  IF @nbcj > 0
  BEGIN
/*
SELECT 'new BATCH', COUNT(*), btcy_id, ISNULL(job_name,''), op_name, est_start
FROM opc_operation
WHERE plan_id = @plan_id AND job_id IS NULL
GROUP BY btcy_id, job_name, est_start, op_name
ORDER BY COUNT(*) DESC, btcy_id, job_name, est_start, op_name
*/
    CREATE TABLE #new_job
    (
      btcy_id           ObjID,
      name              NVARCHAR(8),
      status            NCHAR(1) NULL,
      description       NVARCHAR(255) NULL,
      _OPCOpNum         NVARCHAR(8),
      _OPCOpInstanceNo  INT,
      _SchedStartTime   DATETIME NULL,
      _SchedDuration    INT NULL,
      _SchedStopTime    DATETIME NULL,
      rowno             INT IDENTITY(0, 1)
    )

    -- Create the new records
    INSERT #new_job
    SELECT DISTINCT btcy_id, ISNULL(job_name,''), status, op_desc, op_name, instanceno, est_start, duration, est_stop
    FROM opc_operation
    WHERE plan_id = @plan_id AND job_id IS NULL
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('Error computing new BATCH records, aborting', 3, 1)
      ROLLBACK TRAN _LoadOPCPlan_CreateBCJs
      RETURN -30000
    END

    SELECT @nbcj = COUNT(*) FROM #new_job
    DECLARE @base_id ObjID
    EXEC @base_id = alloc_BATC @nbcj
    
    INSERT BATCH_C(id, name, description, _OPCOpNum, _OPCOpInstanceNo, _SchedStartTime, _SchedStatusFlags, _SchedDuration, _SchedStopTime, _InitEstComplete, _ParentCID, _ParentID, _CurrentStateID, _DesiredStateID)
      SELECT @base_id+j.rowno, j.name, j.description, j._OPCOpNum, j._OPCOpInstanceNo, j._SchedStartTime, 1, j._SchedDuration, j._SchedStopTime, j._SchedStopTime, 'BTCY', j.btcy_id, sttp.id, sttp.id
      FROM #new_job AS j, StateType_V AS sttp
      WHERE sttp.name = CASE j.status WHEN 'C' THEN 'BATC_Completed' ELSE 'BATC_Pending' END
        AND (j.status IS NULL OR j.status <> 'C') -- do not look at completed jobs
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('Error creating new BATCH records, aborting', 3, 1)
      ROLLBACK TRAN _LoadOPCPlan_CreateBCJs
      RETURN -30000
    END

    -- Update opc_operation again for the new jobs.
    EXEC asisp_checkpointmessage 'Updating opc_operation.job_id from newly created BATCH records for operations with names', 1, @chkpt
    SET @chkpt = @chkpt + 1
    UPDATE o SET job_id = b.id
      FROM opc_operation AS o JOIN BATCH_V AS b ON 
        b._ParentCID = 'BTCY'
        AND o.btcy_id = b._ParentID
        AND o.job_name = b.name
        AND o.op_name = b._OPCOpNum
        AND CONVERT(NVARCHAR(5), o.est_start, 108) = CONVERT(NVARCHAR(5), b._SchedStartTime, 108)
        AND o.instanceno = b._OPCOpInstanceNo
      WHERE o.plan_id = @plan_id AND o.job_id IS NULL
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN _LoadOPCPlan_CreateBCJs
      RETURN -30000
    END
  END

  -- Now make sure that there are LOB analogs to all BATCH objects in the plan based upon
  -- the opc_app_link table
  DECLARE @r INT
  EXEC @r = _LoadOPCPlan_CreateBCJLOBs @plan_id, @chkpt OUTPUT
  IF @r < 0
  BEGIN
    ROLLBACK TRAN _LoadOPCPlan_CreateBCJs
    RETURN @r
  END

  IF @tcount = 0
    COMMIT TRAN _LoadOPCPlan_CreateBCJs
GO
