-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_createEXCT.sqv   1.0   Jul 18 2001 13:13:40   BRENT  $
--
-- asisp_createEXCT.sql
--
-- Module Description
--   Defines the asisp_createEXCT stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_createEXCT
PRINT 'Creating procedure asisp_createEXCT'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_createEXCT' AND type = 'P')
  DROP PROCEDURE asisp_createEXCT
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_createEXCT.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 18 2001 13:13:40  $
-- $Author:   BRENT  $
-- Custom version of create procedure for ExceptionCategory that checks the existence of the
-- link table. This is helpful because this procedure is most often called when classes are 
-- getting created and the link table may not already exist.
CREATE PROCEDURE asisp_createEXCT
  @name                          ObjName,
  @parent_cid                    ClassID = '',
  @parent_id                     ObjID = 0,
  @Desc                          NVARCHAR(255) = "",
  @Order                         SMALLINT = 0,
  @link_type                     LinkType = NULL
AS
-- RAISERROR('asisp_createEXCT ''%s'', ''%s'', %d', 0, 1, @name, @parent_cid, @parent_id)
  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_createEXCT
  ELSE
    SAVE TRANSACTION asisp_createEXCT
  
  -- Allocate an id for the new object
  DECLARE @id ObjID
  EXEC @id = alloc_EXCT
  IF @id = 0
  BEGIN
    ROLLBACK TRANSACTION asisp_createEXCT
    RETURN @id
  END

  DECLARE @r INT
  
  -- Insert class record
  INSERT ExceptionCategory_C_BASE(id, name) SELECT  @id, @name
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRANSACTION asisp_createEXCT
    RETURN -30000
  END
  INSERT ExceptionCategory_C_LOCAL(id, langid, description) SELECT  @id, @@LANGID, NULL
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRANSACTION asisp_createEXCT
    RETURN -30000
  END
 
  -- Insert attribute vector
  DECLARE @id_str NVARCHAR(9)
  SELECT @id_str = CONVERT(NVARCHAR(9), @id)
  INSERT ExceptionCategory_A_BASE
  (
    id,
    _ExceptionTypes,
    _Order
  )
  SELECT
    @id,
    'ATRC/EXCT-' + @id_str + '/EXTP',
    @Order
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRANSACTION asisp_createEXCT
    RETURN -30000
  END
  INSERT ExceptionCategory_A_LOCAL
  (
    id,
    langid,
    _Desc
  )
  SELECT
    @id,
    @@LANGID,
    @Desc
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRANSACTION asisp_createEXCT
    RETURN -30000
  END

  -- Insert parent link, if specified
  IF @parent_cid <> ''
  BEGIN
    -- Make sure link table exists
    IF ISNULL(@link_type, '') <> ''
    BEGIN
      EXEC @r = asisp_create_link_table 'STCR', 'EXCT', @link_type, @recreate_triggers=0
      IF @r < 0
      BEGIN
        RAISERROR('asisp_createEXCT: aborting due to error creating link table', 11, 1)
        ROLLBACK TRANSACTION asisp_createEXCT
        RETURN @r
      END
    END

    EXEC @r = asisp_insertparentlink @parent_cid, @parent_id, 'EXCT', @id, @link_type
    IF @r < 0
    BEGIN
      ROLLBACK TRANSACTION asisp_createEXCT
      RETURN @r
    END

    -- Send create notification
    DECLARE @parent_cno SMALLINT
    SELECT @link_type = link_type, @parent_cno = cno FROM link, obj_class
      WHERE src_cid = @parent_cid AND src_id = @parent_id AND dst_cid = 'EXCT' AND dst_id = @id
        AND cid = src_cid
    EXEC asisp_notifyObjectCreated 22, 55, @id, @name, @parent_cno, @parent_id, @link_type
  END
  
  -- Any automatically created children are created here
  DECLARE @child_id ObjID

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION asisp_createEXCT
  RETURN @id
GO
