-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/sql/asisp_createObjPumpMsg.sqv   1.1   Jul 12 2001 12:01:26   Forthman  $
--
-- asisp_createObjPumpMsg
--
-- Module Description
--   Defines the asisp_createObjPumpMsg stored procedure

Use Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_createObjPumpMsg
PRINT 'Create procedure asisp_createObjPumpMsg'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name LIKE 'asisp_createObjPumpMsg' AND type = 'P' )
  DROP PROCEDURE asisp_createObjPumpMsg
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/sql/asisp_createObjPumpMsg.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jul 12 2001 12:01:26  $
-- $Author:   Forthman  $
-- Last modification:
--   Added @SourceCID, @SourceID parameters, which are passed IN by the event handler since it
--   knows the id of the OS which it IS processing. This was done so that the OS id IS NOT looked up
--   FROM the SEQ_NO field.
CREATE PROCEDURE asisp_createObjPumpMsg
  @SourceCID      ClassID,
  @SourceID       ObjID,
  @DATETIME       DATETIME,      -- 00
  @SEQ_NO         NVARCHAR(50),   -- 01
  @OBJECT_NAME    NVARCHAR(30),   -- 03
  @MESSAGE        NVARCHAR(255),  -- 46
  @PROGRAM_NAME   NVARCHAR(255) = '',  -- 80
  @MESSAGE_STATUS NVARCHAR(255) = '',  -- 81
  @PARM_1         NVARCHAR(255) = '',  -- 73
  @PARM_2         NVARCHAR(255) = '',  -- 74
  @PARM_3         NVARCHAR(255) = '',  -- 75
  @PARM_4         NVARCHAR(255) = '',  -- 76
  @PARM_5         NVARCHAR(255) = '',  -- 77
  @PARM_6         NVARCHAR(255) = '',  -- 78
  @PARM_7         NVARCHAR(255) = ''   -- 79
AS 
  DECLARE @dt NVARCHAR(255)
  SELECT @dt=CONVERT(NVARCHAR(255),@DATETIME)
  RAISERROR('asisp_createObjPumpMsg: @MESSAGE=%s, @OBJECT_NAME=%s, @SEQ_NO=''%s'', @DATETIME=''%s''', 0, 1, @MESSAGE, @OBJECT_NAME, @SEQ_NO, @dt)
  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_createObjPumpMsg
  ELSE
    SAVE TRAN asisp_createObjPumpMsg

  IF @PROGRAM_NAME IS NULL SELECT @PROGRAM_NAME = ''
  IF @MESSAGE_STATUS IS NULL SELECT @MESSAGE_STATUS = ''
  IF @PARM_1       IS NULL SELECT @PARM_1 = ''
  IF @PARM_2       IS NULL SELECT @PARM_2 = ''
  IF @PARM_3       IS NULL SELECT @PARM_3 = ''
  IF @PARM_4       IS NULL SELECT @PARM_4 = ''
  IF @PARM_5       IS NULL SELECT @PARM_5 = ''
  IF @PARM_6       IS NULL SELECT @PARM_6 = ''
  IF @PARM_7       IS NULL SELECT @PARM_7 = ''

  -- Lookup OS name
  DECLARE @os_name ObjName
  SELECT @os_name = name FROM OS_V WHERE id = @SourceID

  -- CREATE A1MI message
  RAISERROR('asisp_createObjPumpMsg: creating A1M1 message', 0, 1)
  EXEC asisp_createA1MI '', @SourceCID, @SourceID, @DATETIME, @DATETIME, 0,
                        @SEQ_NO, @OBJECT_NAME, @MESSAGE, @PROGRAM_NAME,
                        @PARM_1, @PARM_2, @PARM_3, @PARM_4, @PARM_5, @PARM_6
                  
  -- Custom Handling below
  DECLARE @rc INT
  IF @MESSAGE='GTM0246'
    EXEC @rc=asisp_createObjPumpMsg_ACC0246 @SourceID, @os_name, @DATETIME, @SEQ_NO, @OBJECT_NAME, @MESSAGE, @PROGRAM_NAME,
                                            @MESSAGE_STATUS, @PARM_1, @PARM_2, @PARM_3, @PARM_4, @PARM_5, @PARM_6, @PARM_7
  ELSE IF @MESSAGE='GTM0225'
    EXEC @rc=asisp_createObjPumpMsg_ACC0225 @SourceID, @os_name, @DATETIME, @SEQ_NO, @OBJECT_NAME, @MESSAGE, @PROGRAM_NAME,
                                            @MESSAGE_STATUS, @PARM_1, @PARM_2, @PARM_3, @PARM_4, @PARM_5, @PARM_6, @PARM_7
  ELSE
  BEGIN
    -- NOT an error
    RAISERROR('asisp_createObjPumpMsg: has no logic to process message [%s]', 0, 1, @MESSAGE)
    ROLLBACK TRAN asisp_createObjPumpMsg
    RETURN 0
  END
  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN asisp_createObjPumpMsg
GO