-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_create_link_table.sqv   1.0   Jul 18 2001 13:13:30   BRENT  $
--
-- asisp_create_link_table.sql
--
-- Module Description
--   Defines the asisp_create_link_table stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_create_link_table
PRINT 'Creating procedure asisp_create_link_table'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_create_link_table' AND type = 'P')
  DROP PROCEDURE asisp_create_link_table
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_create_link_table.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 18 2001 13:13:30  $
-- $Author:   BRENT  $
-- Creates the link table and associated triggers for the specified src_cid/dst_cid/link_type
CREATE PROCEDURE asisp_create_link_table
  @src_cid            ClassID,
  @dst_cid            ClassID,
  @link_type          NVARCHAR(4),
  @recreate_triggers  TINYINT=1  -- Optionally disable trigger recreation by specifying @recreate_triggers=0
AS
  DECLARE @table sysname
  SET @table = RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + RTRIM(@link_type)

  -- If the specified link is not in the derived_obj_link table this is most likely due to the
  -- table being out-of-date, so we call asisp_make_isa_chain, which has as one side-effect the
  -- rebuilding derived_obj_link.
  IF NOT EXISTS (SELECT * FROM derived_obj_link AS dol JOIN link_type AS lt ON lt.link_type = dol.link_type WHERE dol.src_cid = @src_cid AND dol.dst_cid = @dst_cid AND dol.link_type = @link_type)
  BEGIN
    PRINT 'Link with src_cid=[' + @src_cid + '], dst_cid=[' + @dst_cid + '], link_type=[' + @link_type + '] not in derived_obj_link. Rebuilding isa_chain'
    EXEC Meta..asisp_make_isa_chain
  END
  -- Now if the link is not there we've got trouble
  IF NOT EXISTS (SELECT * FROM derived_obj_link AS dol JOIN link_type AS lt ON lt.link_type = dol.link_type WHERE dol.src_cid = @src_cid AND dol.dst_cid = @dst_cid AND dol.link_type = @link_type)
  BEGIN
    RAISERROR('Invalid link: src_cid=[%s], dst_cid=[%s], link_type=[%s]', 11, 1, @src_cid, @dst_cid, @link_type)
  END
  ELSE
  BEGIN
    IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = @table AND type = 'U')
    BEGIN
      PRINT 'Creating table ' + @table
      EXEC("
      CREATE TABLE " + @table + "
      (
        src_id       ObjID    CONSTRAINT " + @table + "_PK PRIMARY KEY CLUSTERED(src_id, dst_id),
        dst_id       ObjID,
        ctime        DATETIME CONSTRAINT " + @table + "_ctime_DF DEFAULT GETDATE(),
        deleted      TINYINT  CONSTRAINT " + @table + "_deleted_DF DEFAULT 0
      )
      ")
      EXEC("
      GRANT INSERT ON " + @table + " to public
      GRANT DELETE ON " + @table + " to public
      GRANT SELECT ON " + @table + " to public
      GRANT UPDATE ON " + @table + " to public
      ")
    END

    DECLARE @trigger sysname
    SET @trigger = 'tI_' + @table
    IF @recreate_triggers = 1 OR NOT EXISTS (SELECT * FROM sysobjects WHERE name = @trigger AND type = 'TR')
    BEGIN
      IF EXISTS (SELECT * FROM derived_obj_link AS dol JOIN link_type AS lt ON lt.link_type = dol.link_type WHERE dol.src_cid = @src_cid AND dol.dst_cid = @dst_cid AND dol.link_type = @link_type)
      BEGIN
        PRINT 'Creating trigger ' + @trigger
        IF EXISTS (SELECT * FROM sysobjects WHERE name = '" + @trigger + "' AND type = 'TR')
          EXEC("DROP TRIGGER " + @trigger)

        IF EXISTS (SELECT * FROM derived_obj_link AS dol JOIN link_type AS lt ON lt.link_type = dol.link_type WHERE lt.flag_containment = 1 AND dol.src_cid = @src_cid AND dol.dst_cid = @dst_cid AND dol.link_type = @link_type)

          EXEC("-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_create_link_table.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 18 2001 13:13:30  $
-- $Author:   BRENT  $
-- Insert trigger created by asisp_create_link_table
CREATE TRIGGER " + @trigger + " ON " + @table + " FOR INSERT AS
  DECLARE @rc INT, @src_id ObjID, @dst_id ObjID
  SELECT @rc=@@ROWCOUNT
  INSERT containment(dst_cid, dst_id, src_cid, src_id)  SELECT '" + @dst_cid + "', dst_id, '" + @src_cid + "', src_id FROM inserted
  INSERT link(link_type, src_cid, src_id, dst_cid, dst_id)
    SELECT '" + @link_type + "', '" + @src_cid + "', src_id, '" + @dst_cid + "', dst_id FROM inserted")

        ELSE -- Not a containment link, so just insert into the link table

        EXEC("-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_create_link_table.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 18 2001 13:13:30  $
-- $Author:   BRENT  $
-- Insert trigger created by asisp_create_link_table
CREATE TRIGGER " + @trigger + " ON " + @table + " FOR INSERT AS
  DECLARE @rc INT, @src_id ObjID, @dst_id ObjID
  SELECT @rc=@@ROWCOUNT
  INSERT link(link_type, src_cid, src_id, dst_cid, dst_id)
    SELECT '" + @link_type + "', '" + @src_cid + "', src_id, '" + @dst_cid + "', dst_id FROM inserted")

      END
    END
  END
GO
