-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Db/Sql/asisp_dumpAllQueues.sqv   1.1   Jul 23 2001 16:16:54   Forthman  $
-- asisp_dumpAllQueues.sql
-- 
-- Module Definition
--   Defines the asisp_dumpAllQueues stored procedure

Use Object
GO
------------------------------------------------------------------------------------
-- asisp_dumpAllQueues
PRINT 'Create procedure asisp_dumpAllQueues'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_dumpAllQueues' AND type = 'P' )
  DROP PROCEDURE asisp_dumpAllQueues
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Db/Sql/asisp_dumpAllQueues.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jul 23 2001 16:16:54  $
-- $Author:   Forthman  $
-- Outputs statistics on queue-oriented tables by calling all procedures of the form 'asisp_dump%Queue'
--  It gets these names from the MonitoredQueueTable which should be prepopulated.
CREATE PROCEDURE asisp_dumpAllQueues
  @delim        NCHAR(1) = NULL,
  @output_table sysname = NULL
AS
  SET NOCOUNT ON

  CREATE TABLE #queuename
  (
    queuename    sysname,
    id           INT identity
  )
  INSERT #queuename SELECT SUBSTRING(command, 11, len(command)-15) FROM MonitoredQueueTable

  CREATE TABLE #dump_output
  (
    entries            INT NULL,
    pending_no         INT NULL,
    pending_ctime      DATETIME NULL,
    processed_no       INT NULL,
    processed_ptime    DATETIME NULL,
    id                 INT identity   
  )

  DECLARE pc INSENSITIVE CURSOR FOR SELECT command FROM MonitoredQueueTable
  OPEN pc
  DECLARE @proc sysname
  FETCH NEXT FROM pc INTO @proc
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    INSERT #dump_output EXEC(@proc)
    FETCH NEXT FROM pc INTO @proc
  END
  DEALLOCATE pc

  CREATE TABLE #dump_age
  (
    id                     INT,
    pending_age_days       INT,
    pending_age_hours      INT,
    pending_age_minutes    INT,
    processed_age_days     INT,
    processed_age_hours    INT,
    processed_age_minutes  INT,
    entries_alert          NVARCHAR(8),
    pending_age_alert      NVARCHAR(8),
    processed_age_alert    NVARCHAR(8),
    num_yellows            INT,
    num_reds               INT
  )
  DECLARE @now DATETIME
  SELECT @now = GETDATE() 
  INSERT #dump_age 
    SELECT 
      id,
      0,
      0,
      isnull(DATEDIFF(minute, pending_ctime, @now), 0),
      0,
      0,
      isnull(DATEDIFF(minute, processed_ptime, @now), 0),
      "GREEN",
      "GREEN",
      "GREEN",
      0,
      0
    FROM
      #dump_output

  -- Check thresholds
  UPDATE #dump_age SET entries_alert = "YELLOW" 
    FROM QueueThresholds t, #dump_output d, #queuename q
    WHERE d.id = #dump_age.id AND q.id = d.id AND t.queuename = q.queuename 
      AND t.entries_yellow > 0 AND d.entries > t.entries_yellow AND d.entries <= t.entries_red
  UPDATE #dump_age SET entries_alert = "RED" 
    FROM QueueThresholds t, #dump_output d, #queuename q
    WHERE d.id = #dump_age.id AND q.id = d.id AND t.queuename = q.queuename 
      AND t.entries_red > 0 AND d.entries > t.entries_red

  UPDATE #dump_age SET pending_age_alert = "YELLOW" 
    FROM QueueThresholds t, #queuename q
    WHERE q.id = #dump_age.id AND t.queuename = q.queuename 
      AND t.last_enqueue_yellow > 0 AND pending_age_minutes > t.last_enqueue_yellow AND pending_age_minutes <= t.last_enqueue_red
  UPDATE #dump_age SET pending_age_alert = "RED" 
    FROM QueueThresholds t, #queuename q
    WHERE q.id = #dump_age.id AND t.queuename = q.queuename 
      AND t.last_enqueue_red > 0
      AND
      (
        (pending_age_minutes > t.last_enqueue_red)
        OR
        (pending_age_minutes IS NULL)
      )

  UPDATE age SET processed_age_alert = "YELLOW" 
    FROM #dump_age AS age
      JOIN #dump_output AS d ON d.id = age.id
      JOIN #queuename AS q ON q.id = age.id
      JOIN QueueThresholds AS t ON t.queuename = q.queuename
    WHERE 
      d.entries > 0
      AND t.last_dequeue_yellow > 0 
      AND
      (
        (processed_age_minutes > t.last_dequeue_yellow AND processed_age_minutes <= t.last_dequeue_red)
        OR
        (d.processed_ptime IS NULL 
         AND pending_age_minutes > t.last_dequeue_yellow AND pending_age_minutes <= t.last_dequeue_red)
      )
  UPDATE age SET processed_age_alert = "RED" 
    FROM #dump_age AS age
      JOIN #dump_output AS d ON d.id = age.id
      JOIN #queuename AS q ON q.id = age.id
      JOIN QueueThresholds AS t ON t.queuename = q.queuename
    WHERE
      d.entries > 0 -- entries on queue
      AND t.last_dequeue_red > 0 -- red threshold is non-zero
      AND
      (
        (processed_age_minutes > t.last_dequeue_red) -- processed age exceed threshold
        OR
        (d.processed_ptime IS NULL AND pending_age_minutes > t.last_dequeue_red)
      )

  -- Count alerts for sorting
  UPDATE #dump_age SET 
    num_yellows = CASE entries_alert WHEN "YELLOW" THEN 1 ELSE 0 END +
                  CASE pending_age_alert WHEN "YELLOW" THEN 1 ELSE 0 END +
                  CASE processed_age_alert WHEN "YELLOW" THEN 1 ELSE 0 END,
    num_reds = CASE entries_alert WHEN "RED" THEN 1 ELSE 0 END +
          CASE pending_age_alert WHEN "RED" THEN 1 ELSE 0 END +
               CASE pending_age_alert WHEN "RED" THEN 1 ELSE 0 END

  UPDATE #dump_age SET 
    pending_age_days = pending_age_minutes / (24 * 60),
    processed_age_days = processed_age_minutes / (24 * 60)
  UPDATE #dump_age SET 
    pending_age_hours = (pending_age_minutes / 60) - (pending_age_days * 24), 
    processed_age_hours = (processed_age_minutes / 60) - (processed_age_days * 24)
  UPDATE #dump_age SET
    pending_age_minutes = pending_age_minutes % 60,
    processed_age_minutes = processed_age_minutes % 60

  -- Output the data
  IF @delim IS NULL
  BEGIN
    IF @output_table IS NULL
    BEGIN
      SELECT 
        CAST(queuename AS NVARCHAR(30)) AS queuename,
        entries,
        entries_alert,
        pending_no,
        pending_ctime,
        pending_age=
          CONVERT(NVARCHAR(16),
            CONVERT(NVARCHAR(3), pending_age_days) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), pending_age_hours))) + CONVERT(NVARCHAR(2), pending_age_hours) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), pending_age_minutes))) + CONVERT(NVARCHAR(2), pending_age_minutes) + ':00'
          )
        ,
        pending_age_alert,
        processed_no,
        processed_ptime,
        processed_age=
          CONVERT(NVARCHAR(16),
            CONVERT(NVARCHAR(3), processed_age_days) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), processed_age_hours))) + CONVERT(NVARCHAR(2), processed_age_hours) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), processed_age_minutes))) + CONVERT(NVARCHAR(2), processed_age_minutes) + ':00'
          )
        ,
        processed_age_alert
      FROM #queuename q, #dump_output o, #dump_age a
      WHERE q.id = o.id AND o.id = a.id
      ORDER BY num_reds desc, num_yellows desc, entries desc
    END
    ELSE
    BEGIN
      EXEC("
      INSERT INTO " + @output_table + "
      SELECT 
        CAST(queuename AS NVARCHAR(30)) AS queuename,
        entries,
        entries_alert,
        pending_no,
        pending_ctime,
        pending_age=
          CONVERT(NVARCHAR(16),
            CONVERT(NVARCHAR(3), pending_age_days) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), pending_age_hours))) + CONVERT(NVARCHAR(2), pending_age_hours) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), pending_age_minutes))) + CONVERT(NVARCHAR(2), pending_age_minutes) + ':00'
          )
        ,
        pending_age_alert,
        processed_no,
        processed_ptime,
        processed_age=
          CONVERT(NVARCHAR(16),
            CONVERT(NVARCHAR(3), processed_age_days) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), processed_age_hours))) + CONVERT(NVARCHAR(2), processed_age_hours) + ':' +
            replicate('0', 2 - len(CONVERT(NVARCHAR(2), processed_age_minutes))) + CONVERT(NVARCHAR(2), processed_age_minutes) + ':00'
          )
        ,
        processed_age_alert
      FROM #queuename q, #dump_output o, #dump_age a
      WHERE q.id = o.id AND o.id = a.id
      ORDER BY num_reds desc, num_yellows desc, entries desc
      ")
    END
  END
  ELSE
  BEGIN
    CREATE TABLE #output ( line NVARCHAR(255), id INT identity )
    INSERT #output
     SELECT
        '#Queue' + @delim + 
        'Entries' + @delim + 
        'EColor' + @delim + 
        'Total Enq.' + @delim + 
        'Last Enq.' + @delim + 
        'Enq. Age' + @delim + 
        'EnqColor' + @delim + 
        'Total Deq.' + @delim + 
        'Last Deq.' + @delim + 
        'Deq. Age' + @delim + 
        'DeqColor'

    INSERT #output
      SELECT
        '#-14s' + @delim + 
        '8s ' + @delim + 
        '-6s ' + @delim + 
        '10s ' + @delim + 
        '-18s' + @delim + 
        '11s ' + @delim + 
        '-8s ' + @delim + 
        '10s ' + @delim + 
        '-18s' + @delim + 
        '11s ' + @delim + 
        '-6s '

    INSERT #output(line)
      SELECT
        queuename + @delim +
        CONVERT(NVARCHAR(9), entries) + @delim +
        entries_alert + @delim +
        CONVERT(NVARCHAR(9), pending_no) + @delim +
        CONVERT(NVARCHAR(40), pending_ctime) + @delim +
          CONVERT(NVARCHAR(3), pending_age_days) + ':' +
          replicate('0', 2 - len(CONVERT(NVARCHAR(2), pending_age_hours))) + CONVERT(NVARCHAR(2), pending_age_hours) + ':' +
          replicate('0', 2 - len(CONVERT(NVARCHAR(2), pending_age_minutes))) + CONVERT(NVARCHAR(2), pending_age_minutes) + ':00' + @delim +
        pending_age_alert + @delim +
        CONVERT(NVARCHAR(9), processed_no) + @delim +
        CONVERT(NVARCHAR(40), processed_ptime) + @delim +
          CONVERT(NVARCHAR(3), processed_age_days) + ':' +
          replicate('0', 2 - len(CONVERT(NVARCHAR(2), processed_age_hours))) + CONVERT(NVARCHAR(2), processed_age_hours) + ':' +
          replicate('0', 2 - len(CONVERT(NVARCHAR(2), processed_age_minutes))) + CONVERT(NVARCHAR(2), processed_age_minutes) + ':00' + @delim +
        processed_age_alert
      FROM #queuename q, #dump_output o, #dump_age a
      WHERE q.id = o.id AND o.id = a.id
      ORDER BY num_reds desc, num_yellows desc, entries desc

    SELECT line FROM #output
  END
GO
