--
-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_getExceptionCategory.sqv   1.1   Jul 12 2001 18:53:42   DAVID  $
--

USE Object
GO
SET NOCOUNT ON
GO

--------------------------------------------------------------------------------
-- asisp_getExceptionCategory
PRINT 'Creating PROCEDURE asisp_getExceptionCategory...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='asisp_getExceptionCategory')
  DROP PROCEDURE asisp_getExceptionCategory
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_getExceptionCategory.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jul 12 2001 18:53:42  $
-- $Author:   DAVID  $
-- Gets OR creates an ExceptionType that matches the specified cid, category, AND type.
CREATE PROCEDURE asisp_getExceptionCategory
  @cid             ClassID,
  @category_name   NVARCHAR(255),
  @category_desc   NVARCHAR(255)=''
AS
  RAISERROR('asisp_getExceptionCategory: cid=%s, name=[%s], desc=[%s]', 0, 1, @cid, @category_name, @category_desc)

  -- form the name of the object
  DECLARE @name NVARCHAR(255)
  SELECT @name=RTRIM(@cid)+'_'+@category_name

  -- attempt to find the object (first by name, then by description)
  -- NOTE: We are accomodating two naming conventions
  --  1. Legacy Exception Categories and asisp_getExceptionCategory use <cid>_<category_name>
  --  2. TBSM 1.5 BusinessObject.sqi uses <category_name>
  DECLARE @id ObjID
  SELECT @id = id 
    FROM ExceptionCategory_C AS c
      JOIN link AS l ON id=dst_id AND dst_cid='EXCT' AND src_cid='STCR' AND src_id=1 AND link_type=@cid AND l.deleted=0
    WHERE name=@name
  IF @id IS NULL
    SELECT @id = v.id
      FROM ExceptionCategory_V AS v
        JOIN link AS l ON link_type=@cid AND dst_cid='EXCT' AND dst_id = v.id AND src_cid='STCR' AND src_id=1 AND l.deleted = 0
      WHERE _Desc = @category_desc
  IF @id IS NULL
    SELECT @id = v.id
      FROM ExceptionCategory_V AS v
        JOIN link AS l ON link_type=@cid AND dst_cid='EXCT' AND dst_id = v.id AND src_cid='STCR' AND src_id=1 AND l.deleted = 0
      WHERE _Desc = @category_name
  IF @id IS NOT NULL
  BEGIN
    RAISERROR('asisp_getExceptionCategory: already created.  id=%ld', 0, 1, @id)
    RETURN @id
  END
  ELSE
  BEGIN
    IF @category_desc=''
      -- backward compatibility code only (FOR PerfMon Counters)
      SELECT @category_desc=''''+@category_name+''' PerfMon Object'
    EXEC @id=asisp_createEXCT @name, STCR, 1, @category_desc, 0, @cid
    IF @id<0
    BEGIN
      RAISERROR('asisp_getExceptionCategory: FAILED to CREATE EXCT ''%s'' ON cid=%s', 11, 1, @category_name, @cid)
      RETURN -30000
    END
  END
  RETURN @id
GO

---------------------------------------------------------------------------------
PRINT 'Done.'
GO

