--
-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_getExceptionType.sqv   1.0   Jul 12 2001 18:36:20   DAVID  $
--

USE Object
GO
SET NOCOUNT ON
GO

--------------------------------------------------------------------------------
-- asisp_getExceptionType
PRINT 'Creating PROCEDURE asisp_getExceptionType...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='asisp_getExceptionType')
  DROP PROCEDURE asisp_getExceptionType
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_getExceptionType.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 12 2001 18:36:20  $
-- $Author:   DAVID  $
-- Gets OR creates an ExceptionType that matches the specified cid, category, AND type.
CREATE PROCEDURE asisp_getExceptionType
  @cid             ClassID,
  @category_name   NVARCHAR(255),
  @type_name       NVARCHAR(255),
  @alertstate_id   INT=2,
  @priority_id     INT=5,
  @category_id     ObjID=NULL,
  @category_desc   NVARCHAR(255)='',
  @type_desc       NVARCHAR(255)=''
AS
  RAISERROR('asisp_getExceptionType: cid=%s, category_name=%s, type_name=%s, alertstate_id=%d, priority_id=%d, category_id=%d, category_desc=[%s], @type_desc=[%s]', 0, 1,
        @cid, @category_name, @type_name, @alertstate_id, @priority_id, @category_id, @category_desc, @type_desc)
  --
  -- Sample Usage
  --
  -- EXEC asisp_getExceptionType NTSR, LogicalDisk, '% Free SPACE'
  -- EXEC asisp_getExceptionType NTSR, Processor, '% Processor Time'
  -- EXEC asisp_getExceptionType NTSR, Process, '% Processor Time'
  -- EXEC asisp_getExceptionType NTSR, Memory, 'Available Bytes'
  --
  DECLARE @name NVARCHAR(255)
  DECLARE @id   ObjID
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_getExceptionType
  ELSE
    SAVE TRAN asisp_getExceptionType

  -- Lookup AND/OR CREATE the Exception Category
  IF ISNULL(@category_id,0) = 0
  BEGIN
    EXEC @category_id=asisp_getExceptionCategory @cid, @category_name, @category_desc
    IF @category_id<0
    BEGIN
      RAISERROR('asisp_getExceptionType: ERROR - could NOT CREATE exception category ''%s'' ON cid=%s', 11, 1, @category_name, @cid)
      ROLLBACK TRAN asisp_getExceptionType
      RETURN -30000
    END
  END

  -- Lookup AND/OR CREATE the Exception Type (first by name, then by exception)
  -- NOTE: We are accomodating two naming conventions
  --  1. Legacy Exception Types and asisp_getExceptionType use <cid>_<category>_<extp_name>
  --  2. TBSM 1.5 BusinessObject.sqi uses <cid>_<extp_name>
  SELECT @name=SUBSTRING(RTRIM(@cid)+'_'+@category_name+'_'+@type_name, 1, 54)
  SELECT @id = id
    FROM ExceptionType_V AS et
      JOIN link l ON l.dst_cid = 'EXTP' AND l.dst_id = et.id AND l.src_cid = 'EXCT' AND l.src_id = @category_id
    WHERE l.deleted = 0 AND name = @name AND et._Exception = @type_name
  IF @id IS NULL
    SELECT @id=v.id
      FROM ExceptionType_V AS v
        JOIN link AS l ON src_cid='EXCT' AND src_id=@category_id AND dst_cid='EXTP' AND dst_id = v.id AND l.deleted = 0
      WHERE _Exception = @type_name
  IF @id IS NOT NULL
  BEGIN
    RAISERROR('asisp_getExceptionType: already created.  id=%ld', 0, 1, @id)
    ROLLBACK TRAN asisp_getExceptionType
    RETURN @id
  END
  ELSE
  BEGIN
    DECLARE @excp_name  NVARCHAR(255)
    DECLARE @priority   NVARCHAR(255)
    DECLARE @alertstate NVARCHAR(255)
    IF @type_desc=''
      -- backward compatibility code only (FOR PerfMon Counters)
      SELECT @type_desc=''''+@type_name+''' PerfMon Counter FOR '''+@category_name+''' Object',
             @excp_name=@category_name+'\'+@type_name
    ELSE
      SELECT @excp_name=@type_name
    SELECT @alertstate='GENC/STCR-1/ALRS-'+CONVERT(NVARCHAR(255),@alertstate_id)+'/T/Desc',
           @priority='GENC/STCR-1/PRI6-'+CONVERT(NVARCHAR(255),@priority_id)+'/T/Priority'
    EXEC @id=asisp_createEXTP @name, EXCT, @category_id, @Desc=@type_desc, @Exception=@excp_name, @Priority=@priority, @AlertState=@alertstate
    IF @id<0
    BEGIN
      RAISERROR('asisp_getExceptionType: FAILED to CREATE EXTP ''%s'' ON cid=%s', 11, 1, @category_name, @cid)
      ROLLBACK TRAN asisp_getExceptionType
      RETURN -30000
    END
  END
  IF @tcount = 0
    COMMIT TRAN asisp_getExceptionType
  RETURN @id
GO

---------------------------------------------------------------------------------
PRINT 'Done.'
GO

