-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/asisp_getchildlist.sqv   1.1   Jul 23 2001 17:19:10   BRENT  $
--
-- asisp_getchildlist.sql

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_getchildlist
PRINT 'Creating procedure asisp_getchildlist'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_getchildlist' AND type = 'P')
  DROP PROCEDURE asisp_getchildlist
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/asisp_getchildlist.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jul 23 2001 17:19:10  $
-- $Author:   BRENT  $
-- Gets the children of the specified object. 'Children' for the purposes of this procedure
-- are objects that are on the destination side of links where the link_type has flag_parent_child
-- set to 1.
CREATE PROCEDURE asisp_getchildlist
  @cid  ClassID,
  @id   ObjID
AS
  DECLARE @var_set NVARCHAR(80)
  SELECT @var_set = 'SELECT @cid = ''' + RTRIM(@cid) + ''', @id = ' + CONVERT(NVARCHAR(9), @id)
  EXEC("
  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_getchildlist
  ELSE
    SAVE TRANSACTION asisp_getchildlist
  DECLARE @cid ClassID, @id ObjID " + @var_set + "
  CREATE TABLE #childlist
  (
    link_type   LinkType,
    src_cid     ClassID,
    src_cno     SMALLINT,
    src_id      ObjID,
    dst_cid     ClassID,
    dst_cno     SMALLINT,
    dst_id      ObjID PRIMARY KEY (link_type, dst_cid, dst_id, src_cid, src_id),
    flags       INT,
    generation  INT
  )
  
  DECLARE @generation INT, @rc INT, @err INT
  SELECT @generation = 1
  
  -- Insert the children, as determined by link_type.flag_parent_child
  INSERT #childlist
    SELECT 
      l.link_type,
      l.src_cid,
      src.cno,
      l.src_id,
      l.dst_cid,
      dst.cno,
      l.dst_id,
      t.flags,
      @generation
    FROM
      link l,
      link_type t,
      obj_class src,
      obj_class dst
    WHERE
      -- the specified object is on the source side of a nondeleted link
      l.src_cid=@cid AND src_id=@id AND deleted=0
      -- join the link to the link_type table
      AND l.link_type=t.link_type
      -- look for only the parent-child link types
      AND t.flag_parent_child=1
      -- lookup class numbers
      AND src.cid = l.src_cid
      AND dst.cid = l.dst_cid
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('asisp_getchildlist: insert #childlist failed: error=%d', 4, 1, @err)
    ROLLBACK TRANSACTION asisp_getchildlist
  END
  -- For LOB children we also get their linked objects.  This loop traverses all the way from the
  -- LOB to its ultimate physical object
  IF EXISTS (SELECT * FROM #childlist WHERE dst_cid = 'LOB')

  BEGIN
    WHILE @rc > 0
    BEGIN
      RAISERROR('Getting generation %d for %s %d', 0, 1, @generation, @cid, @id)
      SELECT @generation = @generation + 1
      INSERT #childlist 
        SELECT 
          l.link_type,
          l.src_cid,
          src.cno,
          l.src_id,
          l.dst_cid,
          dst.cno,
          l.dst_id,
          t.flags,
          @generation
        FROM 
          link l /*(INDEX=src_link)*/,
          #childlist o,
          link_type t,
          obj_class src,
          obj_class dst
        WHERE 
          -- Look for objects that are linked to objects in #childlist from the previous generation
          l.src_cid=o.dst_cid AND l.src_id=o.dst_id
          AND NOT (l.src_cid=o.src_cid AND l.src_id=o.src_id)
          AND deleted=0 
          AND generation=@generation-1
          -- join link to link_type
          AND l.link_type=t.link_type 
          -- look only for the logical links
          AND flag_logical=1
          -- lookup class numbers
          AND src.cid = l.src_cid
          AND dst.cid = l.dst_cid
      SELECT @rc = @@ROWCOUNT, @err = @@ERROR
      IF @err <> 0
      BEGIN
        RAISERROR('asisp_getchildlist: insert #childlist failed at generation %d: error=%d', 4, 1, @generation, @err)
        ROLLBACK TRANSACTION asisp_getchildlist
      END
    END
  END
  -- Select the grandchildrens source links.  These are needed by the current model implementation
  SELECT 
    l.link_type,
    l.src_cid,
    src_cno=src.cno,
    l.src_id,
    l.dst_cid,
    dst_cno=dst.cno,
    dst_id=min(l.dst_id),
    flags=t.flags,
    generation=-1  -- this generation value denotes the results as link info only
  FROM
    link l /*(INDEX=src_link)*/,
    #childlist o,
    link_type t,
    obj_class src,
    obj_class dst
  WHERE
    -- the specified object is on the source side of a nondeleted link
    l.src_cid=o.dst_cid AND l.src_id=o.dst_id AND deleted=0 AND generation=1
    -- join the link to the link_type table
    AND l.link_type=t.link_type
    -- look for only the parent-child link types
    AND t.flag_parent_child=1
    -- lookup class numbers
    AND src.cid = l.src_cid
    AND dst.cid = l.dst_cid
  GROUP BY
    l.link_type,
    l.src_cid,
    src.cno,
    l.src_id,
    l.dst_cid,
    dst.cno,
    t.flags
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('asisp_getchildlist: failed getting grandchildren links: error=%d', 4, 1, @err)
    ROLLBACK TRANSACTION asisp_getchildlist
  END
  -- Special LOB logic: if the object has LOB children that are linked to LOBs we need for each one of these

  -- children to select one of the parent-child links where the linked LOB is on the source side.  Again, this
  -- is a requirement for the current model implementation to display the object correctly.
  SELECT
    l.link_type,
    l.src_cid,
    src_cno=src.cno,
    l.src_id,
    l.dst_cid,
    dst_cno=dst.cno,
    dst_id=min(l.dst_id),
    flags=t.flags,
    generation=-1
  FROM
    #childlist o,
    link l /*(INDEX=src_link)*/,
    link_type t,
    obj_class src,
    obj_class dst
  WHERE
    -- looking for only LOB->LOB links
    o.src_cid = 'LOB' AND o.dst_cid = 'LOB'
    -- the object on the target side of the link is the object we want to get a parent-child link for
    AND l.src_cid = 'LOB'
    AND l.src_id = o.dst_id
    AND l.deleted = 0
    AND l.link_type = t.link_type
    AND flag_parent_child = 1
    -- lookup class numbers
    AND src.cid = l.src_cid
    AND dst.cid = l.dst_cid
  GROUP BY
    l.link_type,
    l.src_cid,
    src.cno,
    l.src_id,
    l.dst_cid,
    dst.cno,
    t.flags
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('asisp_getchildlist: LOB logic failed: error=%d', 4, 1, @err)
    ROLLBACK TRANSACTION asisp_getchildlist
  END
  -- For each distinct class selected, we join #childlist to the appropriate class view
  DECLARE childlistc CURSOR FOR SELECT DISTINCT dst_cid FROM #childlist
  OPEN childlistc
  DECLARE @dst_cid ClassID, @select_links TINYINT
  DECLARE @proc NVARCHAR(80), @initial_columns NVARCHAR(255), @joined_tables NVARCHAR(255), @where_clause NVARCHAR(255)
  SELECT @initial_columns = 'cl.link_type, cl.src_cid, cl.src_cno, cl.src_id, cl.dst_cid, cl.dst_cno, cl.dst_id, cl.flags, cl.generation,'
  SELECT @err = 0, @select_links = 1
--  RAISERROR('@initial_columns=''%s''', 0, 1, @initial_columns)
  FETCH NEXT FROM childlistc INTO @dst_cid
  WHILE @@FETCH_STATUS <> -1 AND @err = 0
  BEGIN
    SELECT @proc = 'EXEC asisp_joinedview' + @dst_cid + ' '
    SELECT @joined_tables = ',#childlist cl'
--    SELECT @where_clause = 'cl.dst_cid = ''''' + RTRIM(@dst_cid) + ''''' AND v.id=cl.dst_id AND generation<>-1'
    SELECT @where_clause = 'cl.dst_cid = ''''' + RTRIM(@dst_cid) + ''''' AND v.id=cl.dst_id'
--    RAISERROR('@proc=''%s''', 0, 1, @proc)
--    RAISERROR('@joined_tables=''%s''', 0, 1, @joined_tables)
--    RAISERROR('@where_clause=''%s''', 0, 1, @where_clause)
    EXEC(@proc
         + 
         ' ''' + @initial_columns + ''','
         +
         ' ''' + @joined_tables + ''','
         +
         ' ''' + @where_clause + ''''
        )
    SELECT @err = @@ERROR
    IF @err <> 0
    BEGIN
      SELECT @select_links = 0
      close childlistc
      BREAK
    END
    FETCH NEXT FROM childlistc INTO @dst_cid
  END
  DEALLOCATE childlistc
  IF @err <> 0
  BEGIN
    RAISERROR('asisp_getchildlist: failed making class view: error=%d', 4, 1, @err)
    ROLLBACK TRANSACTION asisp_getchildlist
  END
  IF @tcount = 0
    COMMIT TRANSACTION asisp_getchildlist
--  RAISERROR('#childlist:', 0, 1)
--  SELECT * FROM #childlist
  ")
GO
