-- $Header:   //ARCHIVE/Archive_NLS/db/sql/asisp_process_HSM_Event.sqv   1.0   Jul 20 2001 13:19:26   BRENT  $
--
-- asisp_process_HSM_Event.sql
--
-- Module Description
--   Defines the asisp_process_HSM_Event stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_process_HSM_Event 
PRINT 'Creating procedure asisp_process_HSM_Event '
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_process_HSM_Event ' AND type = 'P')
  DROP PROCEDURE asisp_process_HSM_Event 
GO

-- $Archive:   //ARCHIVE/Archive_NLS/db/sql/asisp_process_HSM_Event.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 20 2001 13:19:26  $
-- $Author:   BRENT  $
CREATE PROCEDURE asisp_process_HSM_Event 
  @_msgid NVARCHAR(255), 
  @_msgtext NVARCHAR(255), 
  @_SourceCID ClassID, 
  @_SourceID ObjID, 
  @_hostdate DATETIME,
  @_seq_no NVARCHAR(50),
  @_action TINYINT = -1, 
  @_obj_type ClassID = NULL, 
  @_hsm_name ObjName = NULL, 
  @_event_cid ClassID = NULL, 
  @_event_name NVARCHAR(255) = NULL, 
  @_AlertStateID INT = NULL, 
  @_PriorityID INT = NULL, 
  @verbose TINYINT=1
AS
  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_process_HSM_Event
  ELSE
    SAVE TRAN asisp_process_HSM_Event

  DECLARE @obj_id ObjID
  DECLARE @time DATETIME
  DECLARE @event_text NVARCHAR(4000)

  IF @verbose = 1 BEGIN
    PRINT 'asisp_process_HSM_event called with action=' + CAST(@_action AS NVARCHAR) + ', ' +
          'obj_type=''' + CAST(@_obj_type AS NVARCHAR) + ''', ' + 
          'hsm_name=''' + CAST(@_hsm_name AS NVARCHAR) + ''', ' +
          'event_cid=''' + CAST(@_event_cid AS NVARCHAR) + ''', ' +
          'event_name=''' + CAST(@_event_name AS NVARCHAR) + ''', ' +
          'AlertStateID=' + CAST(@_AlertStateID AS NVARCHAR) + ', ' +
          'PriorityID=' + CAST(@_PriorityID AS NVARCHAR) + ', ' +
          'msgid=''' + CAST(@_msgid AS NVARCHAR) + ''', ' +
          'msgtext=''' + CAST(@_msgtext AS NVARCHAR) + ''', ' +
          'SourceCID=''' + CAST(@_SourceCID AS NVARCHAR) + ''', ' +
          'SourceID=''' + CAST(@_SourceID AS NVARCHAR) + ''', ' +
          'hostdate=''' + CAST(@_hostdate AS NVARCHAR) + ''''
--   PRINT 'len(msgtext)=
  END

  -- find the specific object FOR which the event will be created

  -- start out by finding the HSM object
  SELECT @obj_id = v.id
  FROM HSM_V v
  JOIN containment c on c.src_cid = 'OS' AND c.src_id = @_SourceID AND c.dst_cid = 'HSM' AND c.dst_id = v.id AND c.deleted = 0
  WHERE v.name = @_hsm_name

  -- if the event doesn't apply to an HSM object, find the child of the HSM object
  IF @obj_id IS NOT NULL AND @_obj_type <> 'HSM'
    SELECT @obj_id = dst_id FROM containment
    WHERE src_cid = 'HSM' AND src_id = @obj_id AND dst_cid = @_obj_type AND deleted = 0

  -- generate/clear the alert
  IF @obj_id IS NOT NULL BEGIN
    SET @event_text = @_msgid + ' ' + @_msgtext
    SET @time = CURRENT_TIMESTAMP
    IF @_event_cid = 'EXCP' BEGIN /* exception processing */
      IF @_action = 1 BEGIN /* CREATE an exception */
        IF @verbose = 1
          PRINT 'CREATE a ' +  @_event_name + ' exception FOR ' + @_obj_type + ' ' + @_hsm_name + ' (' + CAST(@obj_id AS NVARCHAR) + ')' +
                ', AlertState=' + CAST(@_AlertStateID AS NVARCHAR) +
                ', Priority=' + CAST(@_PriorityID AS NVARCHAR)
/**/
        EXEC asisp_createEXCP @name=@_event_name, @parent_cid=@_obj_type, @parent_id=@obj_id, 
                              @DATE=@time, @TIME=@time, 
                              @SEQ_NO=@_seq_no,
                              @EXCP_NAME=@_event_name, @EXCP_CD=@event_text 
--                              @AlertState=@_AlertStateID, @Priority=@_PriorityID
/**/
      END /* CREATE an exception */
      ELSE BEGIN -- clear an exception
        IF @verbose = 1
          PRINT 'Clear a ' +  @_event_name + ' exception FOR ' + @_obj_type + ' ' + @_hsm_name + ' (' + CAST(@obj_id AS NVARCHAR) + ')' +
              ', AlertState=' + CAST(@_AlertStateID AS NVARCHAR) +
              ', Priority=' + CAST(@_PriorityID AS NVARCHAR)
/**/
        EXEC asisp_createEXCP @name=@_event_name, @parent_cid=@_obj_type, @parent_id=@obj_id, 
                              @DATE=@time, @TIME=@time, 
                              @SEQ_NO=@_seq_no,
                              @EXCP_NAME=@_event_name, @EXCP_CD='OK', 
                              @AlertState=-1, @Priority=-1
/**/
      END /* clear an exception */
    END /* exception processing */
    ELSE BEGIN /* message processing */
      IF @_action = 1 BEGIN /* CREATE a message */
        IF @verbose = 1
          PRINT 'CREATE a ' +  @_event_name + ' message FOR ' + @_obj_type + ' ' + @_hsm_name + ' (' + CAST(@obj_id AS NVARCHAR) + ')' +
              ', AlertState=' + CAST(@_AlertStateID AS NVARCHAR) +
              ', Priority=' + CAST(@_PriorityID AS NVARCHAR)
/**/
        EXEC asisp_createMESG @name=@_event_name, @parent_cid=@_obj_type, @parent_id=@obj_id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@_seq_no,
                              @MESSAGE=@_event_name, @REASON_CD=@event_text
--                              @AlertState=@_AlertStateID, @Priority=@_PriorityID
/**/
      END /* CREATE a message */
      ELSE BEGIN /* clear a message */
        IF @verbose = 1
          PRINT '"clearing" a message'
/**/
        EXEC asisp_createMESG @name=@_event_name, @parent_cid=@_obj_type, @parent_id=@obj_id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@_seq_no,
                              @MESSAGE=@_event_name, @REASON_CD=@event_text
--                              @AlertState=@_AlertStateID, @Priority=@_PriorityID
/**/
      END /* ELSE, clear a message */
    END /* message processing */
  END

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN asisp_process_HSM_Event
GO

