IF EXISTS(SELECT * FROM sysobjects WHERE name = 'asisp_setinitiallobstate' AND type = 'P')
  DROP PROCEDURE asisp_setinitiallobstate
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_setinitiallobstate.SQv  $
-- $Revision:   1.0  $
-- $Date:   Jul 20 2001 14:22:16  $
-- $Author:   Xiaoyan  $
-- Called WHEN a LOB IS created to initialize the propagation arrays AND generate the right
-- alerts based ON what object the LOB IS linked to.
CREATE PROCEDURE asisp_setinitiallobstate
  @lob_id     ObjID,
  @dst_cid    ClassID,
  @dst_id     ObjID,
  @flag       TINYINT = 0   -- triggers pass flag=0, app-server passes flag=1.  Only one should be enabled
AS
  RAISERROR('asisp_setinitiallobstate %d, %s, %d, %d', 0, 1, @lob_id, @dst_cid, @dst_id, @flag)

  DECLARE @cno SMALLINT, @r INT

  -- SET to 1 to avoid hitting any IntArray related code, OR to 0 to enable the normal complete proc
  DECLARE @IntArrayStub TINYINT
  SELECT @IntArrayStub = 0

  IF @flag = 1
  BEGIN
    RAISERROR('asisp_setinitiallobstate %d, %s, %d, %d - NOT enabled', 0, 1, @lob_id, @dst_cid, @dst_id, @flag)
    RETURN
  END

  -- TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_setinitiallobstate
  ELSE
    SAVE TRAN asisp_setinitiallobstate

  -- Lookup parent cid/id
  DECLARE @parent_cid ClassID, @parent_id ObjID
  SELECT @parent_cid = src_cid, @parent_id = src_id 
  FROM containment 
  WHERE dst_cid = 'LOB' AND dst_id = @lob_id

  -- Make sure we have good propagation matrices ON this LOB
  EXEC @r = asisp_fixobjcountmatrices 'LOB', @lob_id
  IF @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_setinitiallobstate
    RETURN -30000
  END

  DECLARE @table sysname, @sql NVARCHAR(255)

  -- IF we're NOT linked directly to a non-LOB object, THEN traverse the link chain
  IF @dst_cid = 'LOB'
    EXEC asisp_traverseloblink @dst_cid OUTPUT, @dst_id OUTPUT
  IF @dst_cid IS NULL
  BEGIN
    RAISERROR('asisp_setinitiallobstate %d, %s, %d - NULL dst_cid', 0, 1, @lob_id, @dst_cid, @dst_id)
    -- This IS NOT treated AS a ROLLBACK inducing error
    GOTO EndOfProc
  END

  -- Lookup the scalar attributes of the object that we need
  IF EXISTS (SELECT * FROM obj_attr a, isa_chain i WHERE a.cid = i.base_cid AND i.cid = @dst_cid AND name = 'CurrentStateID')
  BEGIN
    SELECT @table = cname + '_V' FROM obj_class WHERE cid = @dst_cid
    SELECT @sql = 'SELECT _CurrentStateID ' +
                  'FROM ' + @table + 
                  ' WHERE id = ' + CONVERT(NVARCHAR(9), @dst_id)
    DECLARE @State INT
    EXEC @r = asisp_declarecursor lobstate_c, @sql, 1
    IF @r = 0
    BEGIN
      OPEN lobstate_c
      FETCH NEXT FROM lobstate_c INTO @State
      DEALLOCATE lobstate_c
    END
  END
  IF @State IS NULL
    SELECT @State = 0

  -- Lookup the ownership status
  IF EXISTS (SELECT * FROM obj_attr a, isa_chain i WHERE a.cid = i.base_cid AND i.cid = @dst_cid AND name = 'HasOwnershipNote')
  BEGIN
    SELECT @table = cname + '_V' FROM obj_class WHERE cid = @dst_cid
    SELECT @sql = 'SELECT _HasOwnershipNote ' +
                  'FROM ' + @table + 
                  ' WHERE id = ' + CONVERT(NVARCHAR(9), @dst_id)
    DECLARE @HasOwnershipNote INT
    EXEC @r = asisp_declarecursor lobstate_c, @sql, 1
    IF @r = 0
    BEGIN
      OPEN lobstate_c
      FETCH NEXT FROM lobstate_c INTO @HasOwnershipNote
      DEALLOCATE lobstate_c
    END
  END
  IF @HasOwnershipNote IS NULL
    SELECT @HasOwnershipNote = 0

  DECLARE @ExcpAlertState INT
  SELECT @ExcpAlertState = 1

  -- Lookup the AlertState corresponding to the message
  DECLARE @MesgAlertState INT, @MesgPriority INT, @MESSAGE NVARCHAR(20), @msgd_id ObjID
  SELECT @MesgAlertState = _AlertState, @MesgPriority = _Priority, @MESSAGE = CONVERT(NVARCHAR(20),_MESSAGE)
  FROM Message_V
  WHERE id = 
    (
    SELECT max(id) FROM Message_V
    WHERE
      _ParentCID = @dst_cid
      AND _ParentID = @dst_id
      AND _Priority <> 5
      AND deleted = 0
    )
  IF @MesgAlertState IS NULL
    SELECT @MesgAlertState = 1
  SELECT @msgd_id = msgd_id FROM MessageDescription_T WHERE obj_cid = @dst_cid AND MsgID = @MESSAGE
  RAISERROR('asisp_setinitiallobstate @lob_id=%d: @MesgAlertState=%d', 0, 1, @lob_id, @MesgAlertState)
  IF @MesgAlertState > 1 AND @MesgPriority IS NOT NULL
    EXEC asisp_setmesgalertstatepri 'LOB', @lob_id, @msgd_id, @MesgAlertState, @MesgPriority

  -- Accumulate counts FOR all exceptions attached to the linked object
  DECLARE @hCountArray INT, @AlertState INT, @Priority INT, @nEXCH INT, @hr INT

  IF @IntArrayStub = 0
  BEGIN
    DECLARE @cell_val INT
    EXEC @hr = sp_OACreate 'IntArray.IntArray.1', @hCountArray OUTPUT
    IF @hr <> 0
    BEGIN
      RAISERROR('asisp_setinitiallobstate %d, %s, %d: Error creating COUNT matrix', 11, 1, @lob_id, @dst_cid, @dst_id)
      ROLLBACK TRAN asisp_setinitiallobstate
    RETURN -30000
    END    
    EXEC @hr = sp_OAMethod @hCountArray, 'CREATE', NULL, 3, 5
    IF @hr <> 0
    BEGIN
      RAISERROR('asisp_setinitiallobstate %d, %s, %d: Error initializing COUNT matrix', 11, 1, @lob_id, @dst_cid, @dst_id)
      ROLLBACK TRAN asisp_setinitiallobstate
      RETURN -30000
    END    

    SELECT @sql = 'SELECT _AlertState, _Priority, _nEXCH FROM Exception_V e ' + 
      'WHERE _ParentCID=''' + RTRIM(@dst_cid) + ''' AND _ParentID = ' + CONVERT(NVARCHAR(9), @dst_id) + 
      ' AND deleted = 0'
    EXEC asisp_declarecursor excp_c, @sql, 1
    OPEN excp_c
    FETCH NEXT FROM excp_c INTO @AlertState, @Priority, @nEXCH
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC @hr = sp_OAMethod @hCountArray, 'IncrementCell', @cell_val OUTPUT, @AlertState, @Priority, @nEXCH
      FETCH NEXT FROM excp_c INTO @AlertState, @Priority, @nEXCH
    END
    DEALLOCATE excp_c

    -- Lookup the linked object's exception thresholds
    DECLARE @hThresholdArray INT
    EXEC @hr = sp_OACreate 'IntArray.IntArray.1', @hThresholdArray OUTPUT
    IF @hr <> 0
    BEGIN
      RAISERROR('asisp_setinitiallobstate %d, %s, %d: Error creating threshold matrix', 11, 1, @lob_id, @dst_cid, @dst_id)
      ROLLBACK TRAN asisp_setinitiallobstate
      RETURN -30000
    END    
    EXEC @hr = sp_OAMethod @hThresholdArray, 'CREATE', NULL, 3, 5
    IF @hr <> 0
    BEGIN
      RAISERROR('asisp_setinitiallobstate %d, %s, %d: Error initializing threshold matrix', 11, 1, @lob_id, @dst_cid, @dst_id)
      ROLLBACK TRAN asisp_setinitiallobstate
      RETURN -30000
    END    
    EXEC asisp_getobjectthresholds @hThresholdArray, @dst_cid, @dst_id, 'ExceptionMatrix'
    IF @hThresholdArray IS NULL
    BEGIN
      RAISERROR('asisp_setinitiallobstate %d, %s, %d: Error getting threshold matrix', 11, 1, @lob_id, @dst_cid, @dst_id)
      ROLLBACK TRAN asisp_setinitiallobstate
      RETURN -30000
    END

    DECLARE @YellowAlertState INT, @RedAlertState INT
    SELECT @YellowAlertState = 1, @RedAlertState = 1
    EXEC @YellowAlertState = asisp_processlobexcparrays @lob_id, @hCountArray, @hThresholdArray, 2, @parent_cid, @parent_id
    EXEC @RedAlertState = asisp_processlobexcparrays @lob_id, @hCountArray, @hThresholdArray, 3, @parent_cid, @parent_id
    IF @RedAlertState > @YellowAlertState
      SELECT @ExcpAlertState = @RedAlertState
    ELSE
      SELECT @ExcpAlertState = @YellowAlertState
  END

  -- Generate child event FOR message IF the MesgAlertState IS NOT green
  IF @MesgAlertState > 1
  BEGIN
    DECLARE @chev_name ObjName, @chev_id ObjID
    SELECT @chev_name = 'LOB-' + CONVERT(NVARCHAR(9), @lob_id)
    EXEC @chev_id = asisp_createCHEV
      @chev_name,
      @parent_cid,
      @parent_id,
      @ChildCID='LOB',
      @ChildID=@lob_id,
      @EventCID='MESG',
      @EventDetail=@MESSAGE,
      @EventCount=0,
      @Direction=1,
      @AlertState=@MesgAlertState,
      @Priority=@MesgPriority
  END

  -- UPDATE scalar attributes of the LOB
  -- AlertState comes FROM the most serious of the message AND exception alert states
  IF @ExcpAlertState > @MesgAlertState
    SELECT @AlertState = @ExcpAlertState
  ELSE
    SELECT @AlertState = @MesgAlertState
--  RAISERROR('AlertState=%d', 0, 1, @AlertState)
--  RAISERROR('CurrentState=%d', 0, 1, @State)
  UPDATE LineOfBusiness_A SET 
    _AlertStateID   = @AlertState,
    _CurrentStateID = @State, -- CurrentState comes directly FROM the linked object
    _LinkedCID      = @dst_cid,
    _HasOwnershipNote = @HasOwnershipNote
  WHERE 
    id = @lob_id

  -- Inform dispatcher of new lob by sending it an LOBLinked event (#13)
  SELECT @cno = cno FROM obj_class WHERE cid = @dst_cid
  RAISERROR('asisp_setinitiallobstate %d, %s, %d: sending LOBLinked event', 0, 1, @lob_id, @dst_cid, @dst_id)
  EXEC asisp_notifyLOBLinked
      13 /* ASIXpEvent::LOBLinked */,
      @cno,
      @dst_id

EndOfProc:
  -- TRANSACTION logic
  IF @tcount = 0
    COMMIT TRAN asisp_setinitiallobstate
