-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Db/Sql/asisp_stagedeventstatus.sqv   1.0   Jul 23 2001 16:31:14   Forthman  $
-- asisp_stagedeventstatus.sql
-- 
-- Module Definition
--   Defines the asisp_stagedeventstatus stored procedure

Use Object
GO
------------------------------------------------------------------------------------
-- asisp_stagedeventstatus
PRINT 'Create procedure asisp_stagedeventstatus'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_stagedeventstatus' AND type = 'P' )
  DROP PROCEDURE asisp_stagedeventstatus
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Db/Sql/asisp_stagedeventstatus.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 23 2001 16:31:14  $
-- $Author:   Forthman  $
-- This procedure allows you to check on the status of the Staging Tables.
CREATE PROCEDURE asisp_stagedeventstatus
  @exec TINYINT=0, -- 0=output last sample, 1=refresh
  @delim NCHAR(1)=NULL
AS
  IF @exec=1
  BEGIN
    SET NOCOUNT ON
    DECLARE tc CURSOR FOR 
      SELECT name, rows_yellow, rows_red, idle_yellow, idle_red, sample_rows, sample_time, sample_timestamp, idle_time, idle_alert, rows_alert
      FROM MonitoredStagingTable
    OPEN tc
    DECLARE @name sysname, @rows_yellow INT, @rows_red INT, @idle_yellow INT, @idle_red INT, @sample_rows INT, @sample_time DATETIME, @sample_timestamp BINARY(8), @idle_time DATETIME, @idle_alert INT, @rows_alert INT
    FETCH NEXT FROM tc INTO @name, @rows_yellow, @rows_red, @idle_yellow, @idle_red, @sample_rows, @sample_time, @sample_timestamp, @idle_time, @idle_alert, @rows_alert
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- Get current TABLE statistics
      EXEC('DECLARE stc CURSOR FOR SELECT min(timestamp), min(ctime), COUNT(*) FROM ' + @name)
      OPEN stc
      DECLARE @this_timestamp BINARY(8), @this_idle_time DATETIME, @this_rows INT, @this_idle_alert INT, @this_rows_alert INT, @minutes INT
      FETCH NEXT FROM stc INTO @this_timestamp, @this_idle_time, @this_rows
      DEALLOCATE stc
      SELECT @this_idle_alert = 1, @this_rows_alert = 1
      IF @this_rows > 0
      BEGIN
        -- See IF the timestamp has NOT changed. IF so the TABLE IS idle AND we USE the idle thresholds
        IF @this_timestamp = @sample_timestamp
        BEGIN
          SELECT @minutes = DATEDIFF(minute, @idle_time, GETDATE())
          IF @minutes > @idle_red SELECT @this_idle_alert = 3
          ELSE IF @minutes > @idle_yellow SELECT @this_idle_alert = 2
        END
        -- Check the row thresholds
        IF @this_rows > @rows_red SELECT @this_rows_alert = 3
        ELSE IF @this_rows > @rows_yellow SELECT @this_rows_alert = 2
      END
      ELSE
        SELECT @idle_time = NULL
      UPDATE MonitoredStagingTable SET 
        sample_time = GETDATE(),
        sample_rows = @this_rows,
        sample_timestamp = @this_timestamp,
        idle_time = @this_idle_time,
        idle_alert = @this_idle_alert,
        rows_alert = @this_rows_alert
      WHERE name = @name
      FETCH NEXT FROM tc INTO @name, @rows_yellow, @rows_red, @idle_yellow, @idle_red, @sample_rows, @sample_time, @sample_timestamp, @idle_time, @idle_alert, @rows_alert
    END
    DEALLOCATE tc
    SET NOCOUNT OFF
  END
  IF @delim IS NULL
    SELECT
      'TABLE Name'  = name,
      'Sample Time' = sample_time,
      'Rows'        = sample_rows,
      'RCOLOR'      = CASE rows_alert WHEN 1 THEN 'GREEN' WHEN 2 THEN 'YELLOW' ELSE 'RED' END,
      'Idle Since'  = idle_time,
      'ICOLOR'      = CASE idle_alert WHEN 1 THEN 'GREEN' WHEN 2 THEN 'YELLOW' ELSE 'RED' END
    FROM
      MonitoredStagingTable
    ORDER BY idle_alert * 10 + rows_alert desc, idle_time, sample_rows desc
  ELSE
  BEGIN
    CREATE TABLE #output ( line NVARCHAR(255), alert INT, id INT identity )
  
    INSERT #output(line, alert)
      SELECT
        '#TABLE' + @delim + 
        'Sample Time' + @delim + 
        'Rows' + @delim + 
        'RCOLOR' + @delim + 
        'Idle Since' + @delim + 
        'ICOLOR',
        5
    INSERT #output(line, alert)
      SELECT
        '#-27s' + @delim + 
        '-17s ' + @delim + 
        '8s ' + @delim + 
        '-6s ' + @delim + 
        '-17s' + @delim + 
        '-6s ',
        4
    INSERT #output(line, alert)
      SELECT
        name + @delim +
        CONVERT(NVARCHAR(30), sample_time) + @delim +
        CONVERT(NVARCHAR(9), sample_rows) + @delim +
        CASE rows_alert WHEN 1 THEN 'GREEN' WHEN 2 THEN 'YELLOW' ELSE 'RED' END + @delim +
        CONVERT(NVARCHAR(30), idle_time) + @delim +
        CASE idle_alert WHEN 1 THEN 'GREEN' WHEN 2 THEN 'YELLOW' ELSE 'RED' END,
        CASE WHEN rows_alert > idle_alert THEN rows_alert ELSE idle_alert END
      FROM MonitoredStagingTable
    SELECT line FROM #output
    ORDER BY alert desc, id
  END
GO