SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_addLinkRule]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_addLinkRule]
GO




/* Add a link rule using the specifications given by the parameters. Return the new rule_id or 0 is something went
   wrong.
 */

CREATE PROCEDURE cl_addLinkRule           
		@linkname        NVARCHAR(128)='',
		@src_direction   TINYINT = 0,
                @cid             ClassID,
                @iid             ObjID,
		@dest_direction  TINYINT = 0,
                @dest_cid        ClassID,
                @dest_keyname    NVARCHAR(32),
                @dest_condition  NVARCHAR(511),
                @enabled         BIT,
                @excl            BIT,
                @priority        INT
AS

RAISERROR( 'cl_addLinkRule: @linkname=%s, @src_direction=%d, @cid=%s, @iid=%d, 
           @dest_direction=%d, @dest_cid=%s, @dest_keyname=%s, @dest_condition=%s,
           @priority=%d',
           0, 1,
           @linkname, @src_direction, @cid, @iid, @dest_direction, @dest_cid, @dest_keyname,
           @dest_condition, @priority )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_addLinkRule                        
ELSE
  SAVE TRAN cl_addLinkRule                        


DECLARE @rule_id    INT

INSERT INTO CL_LinkRules VALUES( @linkname,                   
                                 @src_direction,              
                                 @cid,                      
                                 @iid, 
                                 @dest_direction,
                                 @dest_cid,               
                                 @dest_keyname,
                                 @dest_condition,
                                 @enabled,
                                 @excl,
                                 @priority )
                      
IF @@ROWCOUNT = 1 BEGIN    
  SET @rule_id = @@IDENTITY
  RAISERROR( 'Rule added with rule id=%d', 0, 1, @rule_id )
  -- commit transaction
  IF @tcount = 0                           
    COMMIT TRAN cl_addLinkRule 
END
ELSE BEGIN
  SET @rule_id = 0
  RAISERROR( 'Error occurred attempting to insert into CL_LinkRules', 0, 1 )
  -- rollback transaction
  ROLLBACK TRAN cl_addLinkRule 
END

RETURN @rule_id




GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

