PRINT 'Creating procedure cl_cleanStageTbls'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_cleanStageTbls')
  DROP PROCEDURE cl_cleanStageTbls
GO

CREATE PROCEDURE cl_cleanStageTbls
AS
  -- Clean-up list:
  -- This proc cleans CL_StageBulkData, CL_StageDeltaData & CL_StageEvData

  RAISERROR('cl_cleanStageTbls: ', 0, 1)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN cl_cleanStageTbls
  ELSE
    SAVE TRAN cl_cleanStageTbls

  DECLARE @trid INT, @evtid INT, @action TINYINT, @trtype TINYINT
  CREATE TABLE #transid (id INT, type INT)
  INSERT #transid SELECT DISTINCT trId, trType FROM CL_Stage WHERE trState=5

  DECLARE clean_c CURSOR FOR SELECT a.trId, a.evDataId, b.type FROM CL_Status a, #transid b
    WHERE a.trId=b.id AND a.status LIKE '%ERROR%'
  OPEN clean_c
  FETCH NEXT FROM clean_c INTO @trid, @evtid, @trtype

  WHILE @@FETCH_STATUS <> -1 BEGIN 
    -- If ERROR is on one event, store just that one, else store all in BULK
    IF @trtype = 1 BEGIN
      IF @evtid <> 0
        INSERT CL_Store SELECT @trid, @evtid, action, root, keyword, value FROM CL_StageBulkData 
          WHERE trId=@trid AND evDataId=@evtid
      ELSE
        INSERT CL_Store SELECT @trid, @evtid, action, root, keyword, value FROM CL_StageBulkData 
          WHERE trId=@trid 
    END
    ELSE IF @trtype = 2 
      INSERT CL_Store SELECT @trid, @evtid, action, root, keyword, value FROM CL_StageDeltaData 
        WHERE trId=@trid AND evDataId=@evtid
    ELSE IF @trtype = 3
      INSERT CL_Store SELECT @trid, @evtid, 0, root, keyword, value FROM CL_StageEvData 
        WHERE trId=@trid AND evDataId=@evtid

    FETCH NEXT FROM clean_c INTO @trid, @evtid, @trtype
  END
  DEALLOCATE clean_c

  DELETE a FROM CL_StageBulkData a, #transid b WHERE a.trId=b.id
  DELETE a FROM CL_StageDeltaData a, #transid b WHERE a.trId=b.id
  DELETE a FROM CL_StageEvData a, #transid b WHERE a.trId=b.id

  IF @tcount = 0 
    COMMIT TRAN cl_cleanStageTbls
