PRINT 'Creating procedure cl_cleanUp'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_cleanUp')
  DROP PROCEDURE cl_cleanUp
GO

CREATE PROCEDURE cl_cleanUp
AS
  -- Clean-up list:
  -- Finally have to create job to clean-up CL_Stage and CL_Status every 24/? hours
  -- Have to clean CL_IDCache that have CL_WHERE as keyword when the table gets big

  RAISERROR('cl_cleanUp: ', 0, 1)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN cl_cleanUp
  ELSE
    SAVE TRAN cl_cleanUp

  DECLARE @cleanup_interval INT, @ret INT
  SELECT @cleanup_interval=CAST(_Value AS INT) FROM CL_Options 
    WHERE _Option='cleanup interval in hours'

  -- Store (ERROR) and clean processed transactions from CL_StageBulkData, CL_StageDeltaData 
  -- and CL_StageEvData
  EXEC cl_cleanStageTbls

  -- Clean-up CL_Scheduled_SPCalls when no error
  DELETE FROM CL_Scheduled_SPCalls WHERE state<>0

  -- Store transactions that are rolled-back
  INSERT CL_Store SELECT a.trId, a.evDataId, a.action, a.root, a.keyword, a.value
    FROM CL_StageBulkData a JOIN CL_Stage b 
    ON (a.trId=b.trId) WHERE b.trState=3
  INSERT CL_Store SELECT a.trId, a.evDataId, a.action, a.root, a.keyword, a.value
    FROM CL_StageDeltaData a JOIN CL_Stage b 
    ON (a.trId=b.trId) WHERE b.trState=3
  INSERT CL_Store SELECT a.trId, a.evDataId, 0, a.root, a.keyword, a.value
    FROM CL_StageEvData a JOIN CL_Stage b 
    ON (a.trId=b.trId) WHERE b.trState=3

  -- Clean CL_Stage, CL_Status & CL_Store
  CREATE TABLE #transid (id INT)
  INSERT #transid SELECT DISTINCT trId FROM CL_Stage 
    WHERE DATEDIFF(HOUR, trTimeCommitted, GETDATE()) > @cleanup_interval
  
  DELETE a FROM CL_Store a, #transid b WHERE a.trId=b.id
  DELETE a FROM CL_Status a, #transid b WHERE a.trId=b.id
  DELETE a FROM CL_Stage a, #transid b WHERE a.trId=b.id
  DELETE a FROM CL_StageBulkData a, #transid b WHERE a.trId=b.id
  DELETE a FROM CL_StageDeltaData a, #transid b WHERE a.trId=b.id
  DELETE a FROM CL_StageEvData a, #transid b WHERE a.trId=b.id

  -- Clean non-instanceID entries in CL_IDCache only when the table gets too big
  SELECT * FROM CL_IDCache
  IF @@ROWCOUNT > 10000
    DELETE FROM CL_IDCache WHERE keyword <> 'CL_InsID' AND 
				 DATEDIFF(HOUR, time, GETDATE()) > @cleanup_interval
  
  -- Purge Job history's of CommonListener Jobs
  -- Not sure if this really has to be done
--  exec msdb..sp_purge_jobhistory @job_name='CLProcessNextTransactionJob'
  EXEC msdb..sp_purge_jobhistory @job_name='CLCleanUpJob'
  EXEC msdb..sp_purge_jobhistory @job_name='CLScheduled_SPCallsJob'  
  EXEC msdb..sp_purge_jobhistory @job_name='CL_ProcessRulesJob'

  IF @tcount = 0 
    COMMIT TRAN cl_cleanUp
