SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_deleteLinksForRule]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_deleteLinksForRule]
GO




/* Given a rule ID, delete links derived from the rule ( if not also derived from another rule ), and remove the  
   entries that link the rule IDs to link IDs.                                                                     
 */

CREATE PROCEDURE cl_deleteLinksForRule      
		@rule_id INT                  
AS

RAISERROR( 'cl_deleteLinksForRule: @rule_id=%d', 0, 1, @rule_id )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_deleteLinksForRule                 
ELSE
  SAVE TRAN cl_deleteLinksForRule                 


DECLARE @link_id            INT
DECLARE @return             INT

-- cursor through link instances created by the rule and delete the links derived from this rule, but only if
-- not also derived from another rule 
DECLARE ruleToLinkCursor CURSOR FOR 
  SELECT link_id FROM CL_LinkRulesToLinks a where a.rule_id = @rule_id
         AND NOT EXISTS ( SELECT * FROM CL_LinkRulesToLinks b WHERE b.link_id = a.link_id AND b.rule_id <> @rule_id )

OPEN ruleToLinkCursor

SET @return = 0

FETCH NEXT FROM ruleToLinkCursor INTO @link_id

WHILE @@FETCH_STATUS = 0 AND @return !< 0 BEGIN

  EXEC @return = delete_LNKX @link_id

  IF @return < 0         
    RAISERROR( 'error returned from delete_LNKX for link id=%d', 0, 1, @link_id )
  ELSE BEGIN
    RAISERROR( 'deleted link object (LNKX) with instance id=%d', 0, 1, @link_id )
    FETCH NEXT FROM ruleToLinkCursor INTO @link_id
  END

END 

CLOSE ruleToLinkCursor
DEALLOCATE ruleToLinkCursor

IF @return = 0 BEGIN
  -- delete the "link rules to links" entries
  DELETE FROM CL_LinkRulesToLinks WHERE rule_id = @rule_id

  IF @@ERROR <> 0 BEGIN
    RAISERROR( 'error deleting from CL_LinkRulesToLinks for rule id=%d', 0, 1, @rule_id )
    SET @return = -30000 
  END
  ELSE
    RAISERROR( 'deleted from CL_LinkRulesToLinks for rule id=%d', 0, 1, @rule_id )
END

IF @return < 0          
  ROLLBACK TRAN cl_deleteLinksForRule
ELSE 
  -- commit transaction
  IF @tcount = 0                         
    COMMIT TRAN cl_deleteLinksForRule

RETURN @return



GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

