PRINT 'Creating procedure cl_getLinkDsts'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_getLinkDsts')
  DROP PROCEDURE cl_getLinkDsts
GO

CREATE PROCEDURE cl_getLinkDsts
@cid ClassID,
@kn NVARCHAR(64),
@kv NVARCHAR(255),
@action TINYINT = 0,
@src_cid ClassID = ''
AS
  -- output in #dst_oids
  -- Action can be 1 (or) anything
  -- If action is 1, ONLY then CL_AutoPlacement keywords are processed 
  -- src_cid is needed when processing keyword CL_AutoPlacement
  RAISERROR('cl_getLinkDsts: @cid=%s, @kn=%s, @kv=%s, @action=%d, @src_cid=%s', 0, 1, 
			    @cid, @kn, @kv, @action, @src_cid)

  DECLARE @cname NVARCHAR(24), @sql NVARCHAR(1024), @idx INT
  DECLARE @autoname NVARCHAR(128), @autoarg NVARCHAR(255), @autocid ClassID, @autoid ObjID
  SELECT @cname=cname FROM obj_class WHERE cid=@cid      

  TRUNCATE TABLE #dst_oids
  
  INSERT #dst_oids SELECT oid FROM CL_IDCache 
    WHERE cid=@cid AND keyword=@kn AND value=@kv AND deleted=0	-- when @kn is CL_InstID
  IF @kn = 'CL_InstID_LIKE'
    INSERT #dst_oids SELECT oid FROM CL_IDCache 
      WHERE cid=@cid AND keyword='CL_InstID' AND value LIKE @kv AND deleted=0
  ELSE IF @kn = 'CL_WHERE' BEGIN
    SET @sql = 'INSERT #dst_oids SELECT id FROM ' + @cname + '_V ' + @kv + ' AND deleted=0'
    EXEC (@sql)
  END
  ELSE IF @kn = 'CL_AutoPlacement_name' AND @action = 1 BEGIN -- For now, only ENT is supported 
    SET @idx = CHARINDEX(',', @kv)

    IF @idx > 0 BEGIN
      SET @autoname = LEFT(@kv, (@idx-1))
      SET @autoarg = RIGHT(@kv, (LEN(@kv)-@idx))
      EXEC cl_autoPlacement @autoname, @autoarg, @autocid OUTPUT, @autoid OUTPUT

      -- There might be a case when dst_cid is not same as @autocid. For now we take
      -- @autocid to have precedence over @dst_cid if they are not same      
      IF @autoid > 0 AND @autocid <> '' INSERT #dst_oids SELECT @autoid      
    END
  END
  ELSE IF @kn = 'CL_AutoPlacement' AND @action = 1 BEGIN 
    DECLARE @src_cname NVARCHAR(24)
    SELECT @src_cname=cname FROM obj_class WHERE cid=@src_cid
    SET @autoname = @src_cname + '_to_' + @cname

    -- Currently cl_autoPlacement does ONLY PHYC links
    SELECT @autoarg=condition FROM CL_LinkRules WHERE enabled=1 AND src_cid=@src_cid AND 
      src_id=0 AND dst_cid=@cid AND key_name=@kn AND name='PHYC'

    EXEC cl_autoPlacement @autoname, @autoarg, @autocid OUTPUT, @autoid OUTPUT
    IF @autoid > 0 AND @autocid <> '' INSERT #dst_oids SELECT @autoid
  END

