SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_getLinkRule]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_getLinkRule]
GO




/* Retrieve a link rule that matches the specifications given by the parameters. Return 0 for the rule_id if not found.
 */

CREATE PROCEDURE cl_getLinkRule           
		@linkname        NVARCHAR(128),
		@src_direction   TINYINT = 0,
                @cid             ClassID,
                @iid             ObjID,
		@dest_direction  TINYINT = 0,
                @dest_cid        ClassID,
                @dest_keyname    NVARCHAR(32),
                @dest_condition  NVARCHAR(511)
AS

RAISERROR( 'cl_getLinkRule: @linkname=%s, @src_direction=%d, @cid=%s, @iid=%d, 
           @dest_direction=%d, @dest_cid=%s, @dest_keyname=%s, @dest_condition=%s',
           0, 1,
           @linkname, @src_direction, @cid, @iid, @dest_direction, @dest_cid, @dest_keyname,
           @dest_condition )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_getLinkRule                        
ELSE
  SAVE TRAN cl_getLinkRule                        


DECLARE @rule_id    INT

SELECT @rule_id = IDENTITYCOL FROM CL_LinkRules 
                  WHERE name = @linkname                            AND   
                        src_direction = @src_direction              AND
                        src_cid = @cid                              AND
                        src_id = @iid                               AND
                        dst_direction = @dest_direction             AND
                        ( dst_cid = @dest_cid OR dst_cid IS NULL )  AND
                        key_name = @dest_keyname                    AND
                        condition = @dest_condition
                      
IF @rule_id IS NULL
  SET @rule_id = 0

-- commit transaction
IF @tcount = 0                           
  COMMIT TRAN cl_getLinkRule 

RETURN @rule_id




GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

