PRINT 'Creating procedure cl_parseLinks'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_parseLinks')
  DROP PROCEDURE cl_parseLinks
GO

CREATE PROCEDURE cl_parseLinks
@type NVARCHAR(255) OUTPUT,
@dir NVARCHAR(255) OUTPUT,
@src_cid ClassID OUTPUT,
@disp VARCHAR(8000) OUTPUT
AS
  -- All the attributes are in #attrs table
  -- Output will be in #contd_links
  RAISERROR('cl_parseLinks: @type=%s OUT, @dir=%s OUT, @src_cid=%s OUT, @disp OUT', 0, 1, 
			    @type, @dir, @src_cid, @disp)
  
  -- attribute input table
  CREATE TABLE #attrs_in (
    keyword NVARCHAR(64),
    value NVARCHAR(255),
    cmptype NVARCHAR(16)
  )
  -- matching oids to the above input attributes
  CREATE TABLE #oids (
    oid ObjID
  )
  CREATE TABLE #dst_lnks (
    cid ClassID UNIQUE(cid, keyword, value),
    keyword NVARCHAR(64),
    value NVARCHAR(255)
  )

  -- get all event processing info
  SET @dir = 0
  SELECT @type=value FROM #attrs WHERE root='link' AND keyword='type'
  SELECT @dir=value FROM #attrs WHERE root='link' AND keyword='dir'

  -- Validate & Get Link type & direction
  IF @type IS NULL OR @type = '' BEGIN
    SET @disp = @disp + ', Link type is NULL/empty'
    RETURN -1
  END

  -- Validate & Get Source CID
  SELECT @src_cid=value FROM #attrs 
    WHERE (root='link:src1:id1' OR root='link:src1:likeid1') AND (keyword='class' OR keyword='prdver')
  IF @src_cid IS NULL OR @src_cid = '' BEGIN
    SET @disp = @disp + ', Invalid source class (' + @src_cid + ')'
    RETURN -1
  END
  
  -- Get Source OIDs
  INSERT #attrs_in SELECT keyword, value, '=' FROM #attrs WHERE root LIKE 'link:src1:id1%'
  IF @@ROWCOUNT <= 0
    INSERT #attrs_in SELECT keyword, value, ' LIKE ' FROM #attrs WHERE root LIKE 'link:src1:likeid1%'
  EXEC cl_findOid @src_cid

  IF NOT EXISTS (SELECT * FROM #oids) BEGIN
    SET @disp = @disp + ', Could not locate source resource(s)'
    RETURN -1
  END

  -- Get Destination CID & respective WHERE clauses
  DECLARE @i NVARCHAR(32), @instrid NVARCHAR(255), @kv NVARCHAR(255), @dst_cid ClassID
  SELECT @instrid=value FROM #attrs WHERE root='CommonListener' AND keyword='instrId'
  SET @i = 1

  WHILE EXISTS (SELECT * FROM #attrs WHERE root LIKE 'link:tgt'+@i+'%') BEGIN
    -- Validate & Get Destination CID
    SELECT @dst_cid=value FROM #attrs 
      WHERE (root='link:tgt'+@i+':id1' OR root='link:tgt'+@i+':likeid1') AND (keyword='class' OR keyword='prdver')
    IF @dst_cid IS NULL OR @dst_cid = '' BEGIN
      SET @disp = @disp + ', Invalid destination class (' + @dst_cid + ')'
      SET @i = @i + 1
      CONTINUE
    END

    -- Get Destination OIDs
    IF @dst_cid = 'ENT' BEGIN
      INSERT #dst_lnks SELECT @dst_cid, 'CL_AutoPlacement_name', 'CLInstrumentation_to_ENT,'+@instrid
      SET @i = @i + 1
      CONTINUE
    END
   
    DECLARE @dst_cname NVARCHAR(24), @strout NVARCHAR(255), @cmp NVARCHAR(16)
    SELECT @dst_cname=cname FROM obj_class WHERE cid=@dst_cid
    TRUNCATE TABLE #attrs_in

    INSERT #attrs_in SELECT keyword, value, '=' FROM #attrs 
      WHERE root LIKE 'link:tgt'+@i+':id1%'
    IF @@ROWCOUNT <= 0
      INSERT #attrs_in SELECT keyword, value, ' LIKE ' FROM #attrs 
	WHERE root LIKE 'link:tgt'+@i+':likeid1%'

    SELECT @cmp=cmptype, @kv=value FROM #attrs_in WHERE keyword='instid'
    IF @@ROWCOUNT > 0 BEGIN
      IF @cmp = ' LIKE ' 
	INSERT #dst_lnks SELECT @dst_cid, 'CL_InstID_LIKE', @kv       
      ELSE
        INSERT #dst_lnks SELECT @dst_cid, 'CL_InstID', @kv
    END
    ELSE BEGIN
      EXEC cl_attrToStr @dst_cname, @strout OUTPUT
      IF @strout <> '' 
        INSERT #dst_lnks SELECT @dst_cid, 'CL_WHERE', @strout   
      ELSE
        INSERT #dst_lnks SELECT @dst_cid, 'CL_AutoPlacement', '' 
    END    
    SET @i = @i + 1
  END

  INSERT #contd_links SELECT a.oid, b.cid, b.keyword, b.value
    FROM #oids a CROSS JOIN #dst_lnks b


