PRINT 'Creating procedure cl_processBulk'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processBulk')
  DROP PROCEDURE cl_processBulk
GO

CREATE PROCEDURE cl_processBulk
@trid INT
AS
  -- input in #events table
  RAISERROR('cl_processBulk: @trid=%d', 0, 1, @trid)

  CREATE TABLE #attrs_in (
    keyword NVARCHAR(64),
    value NVARCHAR(255),
    cmptype NVARCHAR(16)
  )
  CREATE TABLE #oids (
    oid ObjID
  )
  DECLARE @cid ClassID, @evtid INT, @oid ObjID, @bulkdisp NVARCHAR(1024), @cstr NVARCHAR(32)
  DECLARE @disp VARCHAR(8000) 
  SET @disp = ''

  -- temporary table to store cid-instid pair(s) in the event
  CREATE TABLE #bulk (evtid INT, class NVARCHAR(255), instid NVARCHAR(255))
  INSERT #bulk SELECT DISTINCT evtid, value, '' FROM #events 
    WHERE root='inst' AND keyword='class'
  UPDATE #bulk SET b.instid=e.value FROM #bulk b 
    JOIN #events e ON (e.evtid=b.evtid)
    WHERE e.root='inst' AND e.keyword='instid'
  DELETE FROM #bulk WHERE (instid='' OR class='')	-- delete invalid message(s)

  -- get all cids in this BULK
  SELECT DISTINCT class INTO #allcids FROM #bulk

  -- get Instrumentation ID
  DECLARE @instrid NVARCHAR(255)
  SELECT @instrid=instrId FROM CL_Stage a, CL_Registration b
    WHERE a.trId=@trid AND a.registrationId=b.registrationId 


  --  ** CREATING new resources **
  DECLARE @cstr2 NVARCHAR(32), @createproc NVARCHAR(128), @time DATETIME, @name NVARCHAR(255)
  SET @cstr = 0
  SET @cstr2 = 0

  DECLARE add_c CURSOR FOR SELECT a.evtid, a.class FROM #bulk a LEFT OUTER JOIN CL_IDCache b
    ON (a.class=b.cid AND b.keyword='CL_InstID' AND a.instid=b.value AND b.deleted=0) 
    WHERE b.oid IS NULL AND a.class IN (SELECT cid FROM obj_class)
  OPEN add_c
  FETCH NEXT FROM add_c INTO @evtid, @cid

  SET @time = GETDATE()
  WHILE @@FETCH_STATUS <> -1 BEGIN
    SET @name = ''
    SET @cstr = @cstr + 1

    SET @createproc = 'asisp_create' + @cid
    EXEC @oid = @createproc 'InProgress'

    IF @oid > 0 BEGIN
      -- Update CL_IDCache
      INSERT CL_IDCache (cid, oid, keyword, value, instrID)
        SELECT @cid, @oid, 'CL_InstID', instid, @instrid FROM #bulk
        WHERE class=@cid AND evtid=@evtid
   
      -- When name is not specified, use instid instead
      SELECT @name=value FROM #events WHERE evtid=@evtid AND root='inst' AND keyword='name'
      IF @name = '' OR @name IS NULL BEGIN
        DELETE FROM #events WHERE evtid=@evtid AND root='inst' AND keyword='name'
        INSERT #events SELECT @evtid, 'inst', 'name', instid FROM #bulk 
          WHERE class=@cid AND evtid=@evtid
      END

      SET @cstr2 = @cstr2 + 1
    END

    FETCH NEXT FROM add_c INTO @evtid, @cid
  END
  DEALLOCATE add_c
  SET @bulkdisp = 'BULK Discovery: Total Creates = ' + @cstr 
  SET @bulkdisp = @bulkdisp + ', Successful Creates = ' + @cstr2


  -- ** UPDATING new and existing resources **

  CREATE TABLE #bulk_update (evtid INT, cid ClassID, oid ObjID)
  CREATE TABLE #inst_attrs (oid ObjID, keyword NVARCHAR(64), value NVARCHAR(255))
  DECLARE @updateproc NVARCHAR(128)

  INSERT #bulk_update SELECT a.evtid, a.class, b.oid FROM #bulk a LEFT OUTER JOIN CL_IDCache b
    ON (a.class=b.cid AND b.keyword='CL_InstID' AND a.instid=b.value AND b.deleted=0) 
    WHERE b.oid IS NOT NULL
  DECLARE upd_c CURSOR FOR SELECT DISTINCT cid FROM #bulk_update
  OPEN upd_c
  FETCH NEXT FROM upd_c INTO @cid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    TRUNCATE TABLE #inst_attrs
    INSERT #inst_attrs SELECT a.oid, b.keyword, b.value FROM #bulk_update a
      JOIN #events b ON (a.evtid=b.evtid)
      WHERE a.cid=@cid AND (b.root='inst' OR b.root='inst:attr1')

    SET @updateproc = 'cl_process' + @cid 
    EXEC @updateproc @action='ADD_UPDATE'
    SET @disp = @disp + ', Updated ' + @cid

    FETCH NEXT FROM upd_c INTO @cid 
  END
  DEALLOCATE upd_c
  EXEC cl_logStatus @trid, 0, @disp, 'OK'

  -- Post-Create processing
  -- Link rules processing
  INSERT CL_ModifiedClasses SELECT DISTINCT cid, @time FROM CL_IDCache 
    WHERE keyword='CL_InstID' AND DATEDIFF(ms, @time, time)>0 AND
    cid NOT IN (SELECT cid FROM CL_ModifiedClasses)

  -- LOB processing
  INSERT GEM_UpdateLOBView (cid, id) SELECT idc.cid, idc.oid FROM CL_IDCache idc
    JOIN GEM_InstFiltering gif ON (idc.cid=gif.cid)
    WHERE idc.keyword='CL_InstID' AND DATEDIFF(ms, @time, time)>0

  SELECT @cstr=COUNT(*) FROM #bulk_update 
  SET @bulkdisp = @bulkdisp + ', Creates+Updates = ' + @cstr 


  -- ** DELETING resources not in event (from the same Instrumentation ID)**

  DECLARE @sql NVARCHAR(512)
  CREATE TABLE #bulk_delete(cid ClassID , oid ObjID)
  INSERT #bulk_delete SELECT b.cid, b.oid FROM #bulk a RIGHT OUTER JOIN CL_IDCache b
    ON (a.class=b.cid AND b.keyword='CL_InstID' AND a.instid=b.value AND b.deleted=0) 
    WHERE a.instid IS NULL AND b.instrID=@instrid AND b.keyword='CL_InstID' AND
	  b.cid IN (SELECT DISTINCT class FROM #bulk) 

  -- Computing resources of cids (created by the instrID) not in BULK but were in Registration list
  -- Same functionality as <empty/> tag

  INSERT #bulk_delete SELECT b.cid, c.oid
    FROM CL_Registration a JOIN CL_RegistrationList b ON (a.registrationId=b.registrationId)
    JOIN CL_IDCache c ON (b.cid=c.cid) 
    WHERE a.instrId=@instrid AND c.instrID=@instrid AND c.keyword='CL_InstID' AND c.deleted=0
	  AND b.cid NOT IN (SELECT class FROM #allcids)

  -- Schedule deletes and mark then deleted in CL_IDCache    
  INSERT CL_Scheduled_SPCalls (sp_call)
    SELECT 'delete_' + cid + ' ' + CAST(oid AS NVARCHAR) FROM #bulk_delete
  UPDATE c SET c.deleted=1 FROM CL_IDCache c JOIN #bulk_delete d
    ON (c.cid=d.cid AND c.oid=d.oid)
			 
  -- Log Delete count
  SELECT @cstr=COUNT(*) FROM #bulk_delete
  SET @bulkdisp = @bulkdisp + ', Scheduled Deletes = ' + @cstr 
  EXEC cl_logStatus @trid, 0, @bulkdisp, 'OK'

 


