PRINT 'Creating procedure cl_processBulkEvents'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processBulkEvents')
  DROP PROCEDURE cl_processBulkEvents
GO

CREATE PROCEDURE cl_processBulkEvents
AS
  RAISERROR('cl_processBulkEvents: ', 0, 1)
  
  CREATE TABLE #bulk_events (
    eid INT,
    cid ClassID DEFAULT '',
    instid NVARCHAR(255) DEFAULT NULL,
    cmp NVARCHAR(16) DEFAULT '',
    oid ObjID DEFAULT 0,
    type NVARCHAR(16) DEFAULT 'EXCP',	
    name NVARCHAR(255) DEFAULT '',
    descp NVARCHAR(255) DEFAULT '',
    time NVARCHAR(255) DEFAULT CURRENT_TIMESTAMP,
    state NVARCHAR(255) DEFAULT NULL,
    stateid INT DEFAULT NULL,
    alert NVARCHAR(255) DEFAULT NULL,
    priority NVARCHAR(255) DEFAULT NULL,
    severity NVARCHAR(255) DEFAULT NULL,
    sev_type NVARCHAR(64) DEFAULT NULL,
    toparent NVARCHAR(255) DEFAULT NULL,    
    action NVARCHAR(255) DEFAULT ''
  )

  INSERT #bulk_events (eid) SELECT DISTINCT evtid FROM #events
  UPDATE be SET be.name=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event' AND e.keyword='type'
  UPDATE be SET be.name=RTRIM(LTRIM(be.name+' '+e.value)) FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event' AND e.keyword='context'
  UPDATE be SET be.cid=e.value, be.cmp=ISNULL(SUBSTRING(e.root, 7, CHARINDEX('id1', e.root)-7), '')
    FROM #events e JOIN #bulk_events be ON (e.evtid=be.eid) 
    WHERE (e.keyword='class' OR e.keyword='prdver') AND (e.root='event:id1' OR e.root='event:likeid1')
  UPDATE be SET be.instid=e.value FROM #events e JOIN #bulk_events be ON (e.evtid=be.eid) 
    WHERE e.keyword='instid' AND (e.root='event:id1' OR e.root='event:likeid1')
  UPDATE be SET be.toparent=e.value FROM #events e JOIN #bulk_events be ON (e.evtid=be.eid) 
    WHERE e.keyword='toparent' AND (e.root='event:id1' OR e.root='event:likeid1')
  UPDATE be SET be.time=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event' AND e.keyword='time' AND ISNUMERIC(e.value)=1
  UPDATE be SET be.action=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event' AND e.keyword='action'
  UPDATE be SET be.type=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event:tbsm1' AND e.keyword='type'
  UPDATE be SET be.descp=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event:tbsm1' AND e.keyword='desc'
  UPDATE be SET be.state=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event:tbsm1' AND e.keyword='state'
  UPDATE be SET be.alert=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event:tbsmstatus1' AND e.keyword='alert'
  UPDATE be SET be.priority=e.value FROM #events e JOIN #bulk_events be 
    ON (e.evtid=be.eid) WHERE e.root='event:tbsmstatus1' AND e.keyword='priority'
  UPDATE be SET be.severity=e.value, be.sev_type=SUBSTRING(e.root, 7, CHARINDEX('status', e.root)-7) 
    FROM #events e JOIN #bulk_events be ON (e.evtid=be.eid) 
    WHERE e.keyword='severity' AND (e.root='event:tecstatus1' OR e.root='event:tngstatus1' OR e.root='event:nvdstatus1')

  -- Delete when cid is invalid
  INSERT #event_log SELECT eid, 'ERROR', 'Invalid ClassID' FROM #bulk_events
    WHERE cid NOT IN (SELECT cid FROM obj_class)
  DELETE FROM #bulk_events WHERE cid NOT IN (SELECT cid FROM obj_class)

  -- Delete when MSG and name is empty
  INSERT #event_log SELECT eid, 'ERROR', 'Empty event name for MSG event' FROM #bulk_events 
    WHERE type='MSG' AND name=''
  DELETE FROM #bulk_events WHERE type='MSG' AND name=''

  -- get object id if instid is used and cmp not like
  UPDATE be SET be.oid=clc.oid FROM CL_IDCache clc 
    JOIN #bulk_events be ON (be.cid=clc.cid AND be.instid=clc.value)
    WHERE clc.keyword='CL_InstID' AND clc.deleted=0 AND be.cmp<>'like'

  -- get/map to TBSM AlertState and Priority 
  UPDATE be SET be.alert=cls.alertState, be.priority=cls.priority FROM #bulk_events be
    JOIN CL_Severities cls ON (cls.name=be.sev_type AND cls.severity=LOWER(be.severity))

  -- Set AlertState to Green when reset
  UPDATE #bulk_events SET alert=1, priority=5, descp='UNKNOWN' WHERE action='reset' 

  -- Check if this event has a pre-defined MessageDescription
  UPDATE be SET alert=-1, priority=-1 FROM MessageDescription_T mdt 
    JOIN #bulk_events be ON (mdt.obj_cid=be.cid AND mdt.MsgID=be.name)
    WHERE be.type='MSG' AND be.alert IS NULL AND be.priority IS NULL

  -- Default alertState and priority for invalid values
  -- use ISNUMERIC if this gives problems with string that are not numbers
  UPDATE #bulk_events SET alert='1' WHERE alert <> -1 AND alert NOT BETWEEN 1 AND 3
  UPDATE #bulk_events SET priority='5' WHERE priority <> -1 AND priority NOT BETWEEN 1 AND 5

  UPDATE #bulk_events SET alert=NULL WHERE alert=-1
  UPDATE #bulk_events SET priority=NULL WHERE priority=-1

  -- Update StateID
  UPDATE be SET be.stateid=s.id FROM StateType_V s 
    JOIN link l ON (l.dst_id=s.id)
    JOIN #bulk_events be ON (l.link_type=be.cid AND s._State=be.state)
    WHERE l.deleted=0 AND l.src_cid='STCR' AND l.dst_cid='STTP' AND be.type='MSG'

  CREATE TABLE #staged_evt (
    eid INT,
    type NVARCHAR(16) DEFAULT 'EXCP',
    action NVARCHAR(255) DEFAULT '',
    parent_cid ClassID,
    parent_id ObjID,
    evt_name NVARCHAR(255),
    evt_cd NVARCHAR(255) DEFAULT '',
    state NVARCHAR(40) DEFAULT '',
    stateid INT DEFAULT NULL,
    alertstate INT,
    priority INT,
    time DATETIME,
  )

  -- Load events where object id is known
  INSERT #staged_evt (eid, type, action, parent_cid, parent_id, evt_name, evt_cd, stateid, alertstate, priority) 
    SELECT eid, type, action, cid, oid, name, descp, stateid, alert, priority FROM #bulk_events
    WHERE oid > 0 AND toparent IS NULL

  -- Get oids for events that dont have instance ids or that have toparent set
  CREATE TABLE #attrs_in (
    keyword NVARCHAR(64),
    value NVARCHAR(255),
    cmptype NVARCHAR(16)
  )
  CREATE TABLE #oids (
    oid ObjID
  )
  CREATE TABLE #coids (
    cid ClassID,
    oid ObjID
  )
  CREATE TABLE #tmp_coids (
    cid ClassID,
    oid ObjID
  )

  DECLARE @eid INT
  DECLARE getid_c CURSOR FOR SELECT eid FROM #bulk_events 
    WHERE eid NOT IN (SELECT eid FROM #staged_evt)
  OPEN getid_c
  FETCH NEXT FROM getid_c INTO @eid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    DECLARE @cid ClassID, @instid NVARCHAR(255), @toparent NVARCHAR(255)
    SELECT @cid=cid, @instid=instid, @toparent=toparent FROM #bulk_events WHERE eid=@eid
    TRUNCATE TABLE #attrs_in
    TRUNCATE TABLE #coids
    TRUNCATE TABLE #tmp_coids

    INSERT #attrs_in SELECT keyword, value, '=' FROM #events
      WHERE root LIKE 'event:id1%' AND evtid=@eid
    IF @@ROWCOUNT <= 0
      INSERT #attrs_in SELECT keyword, value, ' LIKE ' FROM #events
        WHERE root LIKE 'event:likeid1%' AND evtid=@eid
 
    IF @cid = '%' 
      INSERT #coids SELECT cid, oid FROM CL_IDCache 
        WHERE keyword='CL_InstID' AND value LIKE @instid AND deleted=0
    ELSE BEGIN
      EXEC cl_findOid @cid
      INSERT #coids SELECT @cid, oid FROM #oids
    END

    IF NOT EXISTS (SELECT * FROM #coids) BEGIN
      INSERT #event_log SELECT @eid, 'ERROR', 'Could not locate resource(s)'
      CONTINUE
    END

    -- Attach to parent that is connected through toparent value (typically PHYC)
    IF @toparent IS NOT NULL BEGIN
      INSERT #tmp_coids SELECT * FROM #coids
      TRUNCATE TABLE #coids
      INSERT #coids SELECT l.src_cid, l.src_id FROM link l JOIN #tmp_coids t
        ON (t.cid=l.dst_cid AND t.oid=l.dst_id)
        WHERE l.link_type=@toparent AND l.deleted=0
    END

    INSERT #staged_evt (eid, type, action, parent_cid, parent_id, evt_name, evt_cd, stateid, alertstate, priority) 
      SELECT be.eid, be.type, be.action, c.cid, c.oid, be.name, be.descp, be.stateid, be.alert, be.priority
      FROM #bulk_events be, #coids c WHERE be.eid=@eid

    FETCH NEXT FROM getid_c INTO @eid
  END
  DEALLOCATE getid_c

  -- Convert if time in sec from epoch
  DECLARE @deltaGMT INT
  EXEC master.dbo.xp_regread 'HKEY_LOCAL_MACHINE',
  			     'SYSTEM\CurrentControlSet\Control\TimeZoneInformation',
			     'ActiveTimeBias', 
			     @deltaGMT OUTPUT
  SET @deltaGMT = @deltaGMT * 60

  UPDATE #staged_evt SET time=DATEADD(ss, CAST(be.time AS INT)+@deltaGMT, 'Jan 01 1970')
    FROM #bulk_events be WHERE ISNUMERIC(be.time)=1 

    -- Exceptions and Messages can only be associated with classes that derive from MN02
  INSERT #event_log SELECT eid, 'ERROR', 'Cannot put EXCP/MESG on ' + parent_cid + ' as its not derived from MNO2'
    FROM #staged_evt WHERE parent_cid NOT IN (SELECT cid FROM isa_chain WHERE base_cid='MNO2')
  DELETE FROM #staged_evt WHERE parent_cid NOT IN (SELECT cid FROM isa_chain WHERE base_cid='MNO2')

  -- Set events and messages in staged tables
  EXEC cl_setBulkEvents
