PRINT 'Creating procedure cl_processContd'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processContd')
  DROP PROCEDURE cl_processContd
GO

CREATE PROCEDURE cl_processContd
@disp VARCHAR(8000) OUTPUT
AS
  RAISERROR('cl_processContd: @disp OUT', 0, 1, @disp)
  /* Input in #linkCandidates
     CREATE TABLE #linkCandidates ( 
	name               NVARCHAR(128),
	src_direction      TINYINT,    
	src_cid            ClassID,
	src_id             ObjID,
	dst_direction      TINYINT,    
	dst_cid            ClassID,
	dst_id             ObjID,
	rule_id            INT 
     )
  */
  -- Delete non-containment links
  DELETE FROM #linkCandidates WHERE name NOT IN 
	(SELECT link_type FROM link_type WHERE flag_containment=1 OR flag_parent_child=1)

  -- Delete links that are not valid to have parent-child relationship
  -- Cannot do this with obj_link_type as its view with data coming from three different tables
  -- and not sure if thats the reason, but outer joins dont work properly in call below
  DELETE a FROM #linkCandidates a LEFT OUTER JOIN derived_obj_link b 
    ON (a.name=b.link_type AND a.src_cid=b.dst_cid AND a.dst_cid=b.src_cid)
    WHERE b.link_type IS NULL
  IF @@ROWCOUNT > 0 SET @disp = @disp + ', Links deleted due to invalid parent-child relation'

  -- Check if a parent already exists for containment links
  -- Cannot make a PHYC link between a child that already has one deleted in link table
  DELETE a FROM #linkCandidates a 
    JOIN containment b ON (a.src_cid=b.dst_cid AND a.src_id=b.dst_id)
    JOIN link_type lt ON (a.name=lt.link_type)
    WHERE lt.flag_containment=1 

  -- Check if link already exists
  DELETE a FROM #linkCandidates a, link b 
    WHERE a.name=b.link_type AND a.src_cid=b.dst_cid AND a.src_id=b.dst_id AND
          a.dst_cid=b.src_cid AND a.dst_id=b.src_id AND b.deleted=0

  -- Create links between (src_cid, src_id) & (dst_cid, dst_id)
  DECLARE @name NVARCHAR(255), @src_cid ClassID, @src_id ObjID, @dst_cid ClassID, @dst_id ObjID, @rid INT, @ret INT
  DECLARE lnk_c CURSOR FOR SELECT DISTINCT name, src_cid, src_id, dst_cid, dst_id, rule_id FROM #linkCandidates
  OPEN lnk_c
  FETCH NEXT FROM lnk_c INTO @name, @src_cid, @src_id, @dst_cid, @dst_id, @rid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    EXEC @ret = asisp_createlink @type=@name, @src_cid=@dst_cid, @src_id=@dst_id, 
	    		         @dst_cid=@src_cid, @dst_id=@src_id
  
    -- Delete the containment rule after its first match
    IF @rid > 0 DELETE FROM CL_LinkRules WHERE rule_id=@rid
    SET @disp = @disp + ', Created ' + @name + ' link(' + @src_cid + ', ' + CAST(@src_id AS NVARCHAR) + 
   	        ', ' + @dst_cid + ', ' + CAST(@dst_id AS NVARCHAR) + ')'

    FETCH NEXT FROM lnk_c INTO @name, @src_cid, @src_id, @dst_cid, @dst_id, @rid
  END
  DEALLOCATE lnk_c

  -- Update _ParentCID and _ParentID for classes that have flag_directly_contained set
  -- Have to do this when name is a containment link and src_cid (child) is directly contained
  DELETE a FROM #linkCandidates a JOIN link_type lt ON (lt.link_type=a.name)
    WHERE lt.flag_containment=0

  DELETE a FROM #linkCandidates a JOIN obj_class oc ON (oc.cid=a.src_cid)
    WHERE oc.flag_direct_containment=0

  DECLARE @sql NVARCHAR(1024), @src_cname NVARCHAR(24)
  DECLARE lnk2_c CURSOR FOR SELECT DISTINCT src_cid FROM #linkCandidates
  OPEN lnk2_c
  FETCH NEXT FROM lnk2_c INTO @src_cid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    SELECT @src_cname=cname FROM obj_class WHERE cid=@src_cid

    SET @sql = 'UPDATE v SET v._ParentCID=lc.dst_cid, v._ParentID=lc.dst_id ' + 
	       'FROM ' + @src_cname + '_V v JOIN #linkCandidates lc ON(lc.src_id=v.id) ' + 
	       'WHERE lc.src_cid=''' + @src_cid + ''''
    EXEC (@sql)

    FETCH NEXT FROM lnk2_c INTO @src_cid
  END
  DEALLOCATE lnk2_c
