PRINT 'Creating procedure cl_processContdRules'
GO

IF EXISTS(SELECT * FROM sysobjects WHERE type='P' AND name='cl_processContdRules')
  DROP PROCEDURE cl_processContdRules
GO

CREATE PROCEDURE cl_processContdRules
@dst_cid ClassID,
@time DATETIME
AS  
  -- input in #attrs table
  RAISERROR('cl_processContdRules: @dst_cid=%s, @time TIME', 0, 1, @dst_cid)
  CREATE TABLE #linkCandidates( 
    name               NVARCHAR(128),
    src_direction      TINYINT,      
    src_cid            ClassID,      
    src_id             ObjID,        
    dst_direction      TINYINT,     
    dst_cid            ClassID,      
    dst_id             ObjID,        
    rule_id            INT 
  )         

  -- process instanceIDs
  INSERT #linkCandidates SELECT lr.name, 0, lr.src_cid, lr.src_id, 
    0, lr.dst_cid, idc.oid, lr.rule_id FROM CL_LinkRules lr 
    JOIN CL_IDCache idc ON (idc.cid=lr.dst_cid AND lr.key_name=idc.keyword AND idc.value=lr.condition)
    WHERE lr.dst_cid=@dst_cid AND idc.deleted=0 AND lr.enabled=1

  -- process non-instanceIDs
  DECLARE @sql NVARCHAR(1024), @dst_cname NVARCHAR(24), @kn NVARCHAR(255), @kv NVARCHAR(255), @dst_id ObjID
  CREATE TABLE #dstids (id ObjID)
  DECLARE lnk_c CURSOR FOR SELECT DISTINCT key_name, condition FROM CL_LinkRules
    WHERE enabled=1 AND key_name <> 'CL_InstID' AND dst_cid=@dst_cid
  OPEN lnk_c
  FETCH NEXT FROM lnk_c INTO @kn, @kv

  SELECT @dst_cname=cname FROM obj_class WHERE cid=@dst_cid
  WHILE @@FETCH_STATUS <> -1 BEGIN
    TRUNCATE TABLE #dstids

    IF @kn = 'CL_InstID_LIKE' 
      INSERT #dstids SELECT oid FROM CL_IDCache 
        WHERE cid=@dst_cid AND keyword='CL_InstID' AND value LIKE @kv AND deleted=0
    ELSE IF @kn = 'CL_WHERE' BEGIN
      SET @sql = 'INSERT #dstids SELECT id FROM ' + @dst_cname + '_V ' + @kv + 
		 ' AND deleted=0 AND DATEDIFF(ms, ''' + CAST(@time AS NVARCHAR) + ''', mtime)>0'
      EXEC (@sql)
    END
    ELSE IF @kn = 'CL_AutoPlacement_name' BEGIN	-- this case never happens now as a rule is not added when child is not known
      DECLARE @idx INT, @aname NVARCHAR(128), @aarg NVARCHAR(255)
      DECLARE @acid ClassID, @aid ObjID

      SET @idx = CHARINDEX(',', @kv)
      IF @idx > 0 BEGIN
        SET @aname = LEFT(@kv, (@idx-1))
        SET @aarg = RIGHT(@kv, (LEN(@kv)-@idx))
        EXEC cl_autoPlacement @aname, @aarg, @acid OUTPUT, @aid OUTPUT
        IF @acid = @dst_cid INSERT #dstids SELECT @aid
      END
    END

    SELECT @dst_id = MIN(id) FROM #dstids
    IF @dst_id > 0 
      INSERT #linkCandidates SELECT name, 0, src_cid, src_id, 0, dst_cid, @dst_id, rule_id
        FROM CL_LinkRules WHERE dst_cid=@dst_cid AND enabled=1 AND key_name<>'CL_InstID' 
 	AND key_name=@kn AND condition=@kv

    FETCH NEXT FROM lnk_c INTO @kn, @kv
  END
  DEALLOCATE lnk_c

  -- Process the link list
  IF EXISTS (SELECT * FROM #linkCandidates) BEGIN
    DECLARE @disp VARCHAR(8000)
    SET @disp = ''
    EXEC cl_processContd @disp OUT
    EXEC cl_logStatus 0, 0, @disp, 'OK'
  END
  


