PRINT 'Creating procedure cl_processEvent'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processEvent')
  DROP PROCEDURE cl_processEvent
GO

CREATE PROCEDURE cl_processEvent
@cid ClassID,
@disp VARCHAR(8000) OUTPUT
AS
  -- All the attributes are in #attrs table
  RAISERROR('cl_processEvent: @cid=%s, @disp OUT', 0, 1, @cid)
  
  -- attribute input table
  CREATE TABLE #attrs_in (
    keyword NVARCHAR(64),
    value NVARCHAR(255),
    cmptype NVARCHAR(16)
  )
  -- matching oids to the above input attributes
  CREATE TABLE #oids (
    oid ObjID
  )

  CREATE TABLE #coids (
    cid ClassID,
    oid ObjID
  )

  INSERT #attrs_in SELECT keyword, value, '=' FROM #attrs WHERE root LIKE 'event:id1%'
  IF @@ROWCOUNT <= 0
    INSERT #attrs_in SELECT keyword, value, ' LIKE ' FROM #attrs WHERE root LIKE 'event:likeid1%'

  IF @cid = '%' BEGIN
    DECLARE @instid NVARCHAR(255), @cmp NVARCHAR(16)
    SELECT @instid=value, @cmp=cmptype FROM #attrs_in WHERE keyword='instid'

    IF @cmp = '='
      INSERT #coids SELECT cid, oid FROM CL_IDCache 
        WHERE keyword='CL_InstID' AND value=@instid AND deleted=0
    ELSE
      INSERT #coids SELECT cid, oid FROM CL_IDCache 
        WHERE keyword='CL_InstID' AND value LIKE @instid AND deleted=0
  END
  ELSE BEGIN
    EXEC cl_findOid @cid
    INSERT #coids SELECT @cid, oid FROM #oids
  END

  -- when toparent tag is set, the event is attached to its parent(s)
  DECLARE @toparent NVARCHAR(255)
  SELECT @toparent=value FROM #attrs_in WHERE keyword='toparent'
  IF @toparent IS NOT NULL BEGIN
    SELECT * INTO #tmp_coids FROM #coids
    TRUNCATE TABLE #coids
    INSERT #coids SELECT l.src_cid, l.src_id FROM link l JOIN #tmp_coids t
      ON (t.cid=l.dst_cid AND t.oid=l.dst_id)
      WHERE l.link_type=@toparent AND l.deleted=0
  END

  -- Could not locate resource(s) with the info in id/likeid tag
  IF NOT EXISTS (SELECT * FROM #coids) BEGIN
    SET @disp = @disp + ', Could not locate resource(s)'
    RETURN -1
  END

  DECLARE @evttype NVARCHAR(8), @evtname ObjName, @evtdesc NVARCHAR(255), @oid ObjID
  DECLARE @currState INT, @currPriority INT, @evtcontext NVARCHAR(255), @sev NVARCHAR(16)
  DECLARE @act NVARCHAR(16), @alert NVARCHAR(8), @priority NVARCHAR(8), @sevroot NVARCHAR(128)
  DECLARE @state NVARCHAR(255), @stateid INT
  SET @evtname = ''
  SET @evtcontext = ''
  SET @evtdesc = ''
  SET @act = 'Created/updated'
  SET @currState = 1
  SET @currPriority = 5

  -- get all event processing info
  SELECT @evtname=value FROM #attrs WHERE root='event' AND keyword='type'
  SELECT @evtcontext=value FROM #attrs WHERE root='event' AND keyword='context'
  SELECT @act=value FROM #attrs WHERE root='event' AND keyword='action'
  SELECT @evttype=value FROM #attrs WHERE root='event:tbsm1' AND keyword='type'
  SELECT @evtdesc=value FROM #attrs WHERE root='event:tbsm1' AND keyword='desc'
  SELECT @state=value FROM #attrs WHERE root='event:tbsm1' AND keyword='state'
  SELECT @priority=value FROM #attrs WHERE root='event:tbsmstatus1' AND keyword='priority'
  SELECT @alert=value FROM #attrs WHERE root='event:tbsmstatus1' AND keyword='alert'
  SELECT @sev=value, @sevroot=root FROM #attrs WHERE keyword='severity' AND
    (root='event:tecstatus1' OR root='event:tngstatus1' OR root='event:nvdstatus1')
  SET @evtname = RTRIM(LTRIM(@evtname + ' ' + @evtcontext))

  IF @evtname = '' AND @evttype = 'MSG' BEGIN
    SET @disp = @disp + ', Empty event name for MSG event'
    RETURN -1
  END

  -- get/map to TBSM AlertState and Priority 
  IF @sev IS NOT NULL
    SELECT @currState=alertState, @currPriority=priority FROM CL_Severities 
      WHERE severity=LOWER(@sev) AND name=SUBSTRING(@sevroot, 7, 3)

  -- AlertState valid values 
  IF (ISNUMERIC(@alert)=1) AND @alert BETWEEN 1 AND 3 
    SET @currState = @alert

  IF (ISNUMERIC(@priority)=1) AND @priority BETWEEN 1 AND 5 
    SET @currPriority = @priority
  
  -- Set AlertState to Green when reset
  IF @act = 'reset' BEGIN 
    SET @currState = 1        
    SET @currPriority = 5
    SET @evtdesc = 'UNKNOWN'
  END

  -- get Timestamp
  DECLARE @timestamp NVARCHAR(255), @time DATETIME
  SELECT @timestamp=value FROM #attrs WHERE root='event' AND keyword='time'  
  EXEC cl_getLocalTime @timestamp, @time OUTPUT

  -- alert-priority log string
  DECLARE @apstr NVARCHAR(32)
  SET @apstr = CAST(@currState AS NVARCHAR) + '-' + CAST(@currPriority AS NVARCHAR)

  DECLARE id_c CURSOR FOR SELECT cid, oid FROM #coids
  OPEN id_c
  FETCH NEXT FROM id_c INTO @cid, @oid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    -- Exceptions and Messages can only be associated with classes that derive from MN02
    IF NOT EXISTS (SELECT * FROM isa_chain WHERE cid=@cid AND base_cid='MNO2') BEGIN
      SET @disp = @disp + ', Cannot put EXCP/MESG on ' + @cid + ' as its not derived from MNO2'
      CONTINUE
    END

    -- process all Exceptions of the resource. Default is [reset]
    IF @evtname = '' OR @evtname IS NULL BEGIN
      EXEC cl_clearEXCP @cid, @oid, @act

      IF @act <> 'remove'
        SET @disp = @disp + ', Reseted all EXCPs for (' + @cid + ', ' + CAST(@oid AS NVARCHAR) + ')'
      ELSE
	SET @disp = @disp + ',Removed all EXCPs for (' + @cid + ', ' + CAST(@oid AS NVARCHAR) + ')'
    END
    ELSE BEGIN	-- process specific Exception or Message
      IF @evttype = 'MSG' BEGIN
	SET @stateid = NULL

        -- get StateID
        IF @state IS NOT NULL
	  SELECT @stateid=s.id FROM StateType_V s JOIN link l
 	    ON (l.link_type=@cid AND l.src_cid='STCR' and l.dst_cid='STTP' AND l.dst_id=s.id)
	    WHERE l.deleted=0 AND s._State=@state

        -- Check if this event has a pre-defined MessageDescription
        IF @alert IS NULL AND @priority IS NULL AND EXISTS
 	   (SELECT * FROM MessageDescription_T WHERE obj_cid=@cid AND MsgID=@evtname) BEGIN
	  SET @currState = NULL
      	  SET @currPriority = NULL
        END

        EXEC asisp_createMESG @name=@evtname, @parent_cid=@cid, @parent_id=@oid, @DATE=@time,
                	      @TIME=@time, @MESSAGE=@evtname, @REASON_CD=@evtdesc,
	                      @AlertState=@currState, @Priority=@currPriority, @StateID=@stateid
        SET @disp = @disp + ', Created MESG with ' + @apstr + ' for (' + @cid + ', ' + CAST(@oid as NVARCHAR) + ')'
      END
      ELSE BEGIN
        -- getExceptionType will create the exception object for cid if necessary
        EXEC asisp_getExceptionType @cid, @cid, @evtname, 1, 5, NULL, 
				    'CommonListener', 'Dynamically defined' 
        -- clear previous exception if exists
        EXEC asisp_createEXCP @name=@evtname, @parent_cid=@cid, @parent_id=@oid, @DATE=@time,
                              @TIME=@time, @EXCP_NAME=@evtname, @EXCP_CD='OK', 
	    		      @AlertState=-1, @Priority=-1      

        -- create new exception
        IF @act <> 'remove'
          EXEC asisp_createEXCP @name=@evtname, @parent_cid=@cid, @parent_id=@oid, @DATE=@time, 
	  		        @TIME=@time, @EXCP_NAME=@evtname, @EXCP_CD=@evtdesc, 
                                @AlertState=@currState, @Priority=@currPriority

        SET @disp = @disp + ', ' + @act + ' EXCP with ' + @apstr + ' for (' + @cid + ', ' + CAST(@oid as NVARCHAR) + ')'
      END
    END
    FETCH NEXT FROM id_c INTO @cid, @oid
  END

