PRINT 'Creating procedure cl_processLink'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processLink')
  DROP PROCEDURE cl_processLink
GO

CREATE PROCEDURE cl_processLink
@action TINYINT,
@disp VARCHAR(8000) OUTPUT
AS
  -- All the attributes are in #attrs table
  RAISERROR('cl_processLink: @action=%d, @disp OUT', 0, 1, @action, @disp)
  
  -- 4/1 is ADD and 5/2 is DELETE
  IF @action = 4 SET @action = 1
  IF @action = 5 SET @action = 2

  -- table for parsing and getting all link combinations
  CREATE TABLE #contd_links (
    src_id ObjID,
    dst_cid ClassID,
    dst_kn NVARCHAR(64),
    dst_kv NVARCHAR(255)
  )

  -- input table for link rules
  CREATE TABLE #linkEventData ( 
    cid 		ClassID,
    iid	 		ObjID,
    dest_cid 		ClassID NULL,
    linkname 		NVARCHAR(128),
    src_direction 	TINYINT,  
    dest_direction 	TINYINT,
    dest_keyname 	NVARCHAR(32),
    dest_condition 	NVARCHAR(512),
    enabled 		BIT,
    excl 		BIT,
    priority 		INT,
    action 		INT
  )

  -- input table to process containment links
  CREATE TABLE #linkCandidates ( 
    name               NVARCHAR(128),
    src_direction      TINYINT,    
    src_cid            ClassID,
    src_id             ObjID,
    dst_direction      TINYINT,    
    dst_cid            ClassID,
    dst_id             ObjID,
    rule_id            INT 
  )

  -- table of dst_oids if they exist
  CREATE TABLE #dst_oids (
    oid ObjID
  )

  DECLARE @ret INT, @type NVARCHAR(255), @dir NVARCHAR(255), @src_cid ClassID
  DECLARE @dst_cid ClassID, @kn NVARCHAR(64), @kv NVARCHAR(255)

  -- Convert link direction from string to integer
  -- tosrc(1), todst(2), 2way(3), none(0)
  UPDATE #attrs 
    SET value=CASE LOWER(value)
		WHEN 'totgt' THEN '1' 
		WHEN 'tosrc' THEN '2' 
		WHEN '2way' THEN '3' 
		ELSE '0' 
	      END
    WHERE root='link' and keyword='dir'

  -- gets all the valid combinations of links that are to be processed
  EXEC @ret = cl_parseLinks @type OUTPUT, @dir OUTPUT, @src_cid OUTPUT, @disp OUTPUT
  IF @ret < 0 RETURN -1

  IF EXISTS (SELECT * FROM link_type WHERE link_type=@type AND (flag_containment=1 OR flag_parent_child=1))
  BEGIN
    DECLARE dst_c CURSOR FOR SELECT DISTINCT dst_cid, dst_kn, dst_kv FROM #contd_links
    OPEN dst_c
    FETCH NEXT FROM dst_c INTO @dst_cid, @kn, @kv

    WHILE @@FETCH_STATUS <> -1 BEGIN
      EXEC cl_getLinkDsts @dst_cid, @kn, @kv, 1, @src_cid

      --Process containment link if destinaion oids can be found
      IF EXISTS (SELECT * FROM #dst_oids) BEGIN
        INSERT #linkCandidates SELECT @type, @dir, @src_cid, a.src_id, 0, @dst_cid, b.oid, 0
          FROM #contd_links a CROSS JOIN #dst_oids b        
          WHERE dst_cid=@dst_cid AND dst_kn=@kn AND dst_kv=@kv

        DELETE FROM #contd_links WHERE dst_cid=@dst_cid AND dst_kn=@kn AND dst_kv=@kv
      END

      FETCH NEXT FROM dst_c INTO @dst_cid, @kn, @kv
    END
    DEALLOCATE dst_c

   -- Delete containment links if they exist (and dst oids are found)
   -- No action taken when no dst oids can be found 
   IF @action = 2 BEGIN
      DECLARE @src_id ObjID, @dst_id ObjID, @name NVARCHAR(255)
      DECLARE del_c CURSOR FOR SELECT a.name, a.src_cid, a.src_id, a.dst_cid, a.dst_id 
        FROM #linkCandidates a, link b WHERE a.src_cid=b.dst_cid AND a.src_id=b.dst_id AND 
        a.dst_cid=b.src_cid AND a.dst_id=b.src_id AND a.name=b.link_type AND b.deleted=0
      OPEN del_c
      FETCH NEXT FROM del_c INTO @name, @src_cid, @src_id, @dst_cid, @dst_id
 
      WHILE @@FETCH_STATUS <> -1 BEGIN 
        EXEC asisp_deletelink @type=@name, @src_cid=@dst_cid, @src_id=@dst_id, 
	     @dst_cid=@src_cid, @dst_id=@src_id
        FETCH NEXT FROM del_c INTO @name, @src_cid, @src_id, @dst_cid, @dst_id
      END
      DEALLOCATE del_c
    END
    ELSE BEGIN
      --Process containment link if destinaion oids can be found
      EXEC cl_processContd @disp OUTPUT

      -- Making containment links where dst oids not found as rules
      INSERT #linkEventData SELECT @src_cid, src_id, dst_cid, @type, @dir, 0, dst_kn, dst_kv, 1, 0, NULL, @action
        FROM #contd_links
    END
  END
  ELSE
    INSERT #linkEventData SELECT @src_cid, src_id, dst_cid, @type, @dir, 0, dst_kn, 
	dst_kv, 1, 0, NULL, @action FROM #contd_links

  -- Call link rule processing
  -- Just add it as rule but dont do any processing
  IF EXISTS (SELECT * FROM #linkEventData)    
    EXEC cl_processLinks @disp OUTPUT


  



