SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_processLinkRule]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_processLinkRule]
GO




/* This procedure gets called for a single enabled link rule. The rule is processed and a temporary table populated
   with all the data needed to produce links defined by the rule.                                        

   The temporary table is defined as follows:                                                                    

     CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                                   src_direction      TINYINT,
                                   src_cid            ClassID,
                                   src_id             ObjID,
                                   dst_direction      TINYINT,
                                   dst_cid            ClassID,
                                   dst_id             ObjID,
                                   rule_id            INT )

 */

CREATE PROCEDURE cl_processLinkRule @rule_id  INT
AS

RAISERROR( 'cl_processLinkRule: @rule_id=%d', 0, 1, @rule_id )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_processLinkRule                  
ELSE
  SAVE TRAN cl_processLinkRule                  


DECLARE @name           NVARCHAR(128)
DECLARE @src_direction  TINYINT
DECLARE @src_cid        ClassID
DECLARE @src_id         ObjID
DECLARE @dst_direction  TINYINT
DECLARE @dst_cid        ClassID
DECLARE @dst_id         ObjID
DECLARE @key_name       NVARCHAR(32)
DECLARE @condition      NVARCHAR(512)


-- retrieve fields from the rule entry
SELECT @name = name,
       @src_direction = src_direction,
       @src_cid = src_cid,
       @src_id = src_id,
       @dst_direction = dst_direction,
       @dst_cid = dst_cid,
       @key_name = key_name,
       @condition = condition
FROM CL_LinkRules WHERE rule_id = @rule_id

-- add the appropriate rows to the temporary table
INSERT INTO #linkCandidates SELECT name = @name,
                                   src_direction = @src_direction,
                                   src_cid = @src_cid,
                                   src_id = @src_id,
                                   dst_direction = @dst_direction,
                                   dst_cid = CID,
                                   dst_id = IID,
                                   rule_id = @rule_id
                            FROM GEM_IDLookup
                            WHERE ( CID = @dst_cid OR @dst_cid IS NULL ) AND
                                  key_name = @key_name                   AND 
                                  key_value LIKE @condition              

IF @@ERROR <> 0 BEGIN   
  ROLLBACK TRAN cl_processLinkRule        
  RETURN -30000
END

IF @tcount = 0 
  COMMIT TRAN cl_processLinkRule

RETURN 0
           




GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

