SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_processLinkRuleID]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_processLinkRuleID]
GO




/* This procedure filters the CL_LinkRules table by a specific rule ID, then calls a procedure to process each rule 
   retrieved ( in this case a single rule ). This procedure is also responsible for creating a temporary table
   that will contain the "link candidates" determined by processing each rule. Once the rules are processed, the   
   specified "callback" procedure is invoked to process the "link candidates".                                   

   The temporary table is defined as follows:                                                                    

     CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                                   src_direction      TINYINT,    
                                   src_cid            ClassID,
                                   src_id             ObjID,
                                   dst_direction      TINYINT,     
                                   dst_cid            ClassID,
                                   dst_id             ObjID,
                                   rule_id            INT )

 */

CREATE PROCEDURE cl_processLinkRuleID @rule_id          INT,
                                      @callback_proc    NVARCHAR(128)
AS

RAISERROR( 'cl_processLinkRuleID: @rule_id=%d, @callback_proc=%s', 0, 1, @rule_id, @callback_proc )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_processLinkRuleID                  
ELSE
  SAVE TRAN cl_processLinkRuleID                  


DECLARE @return       INT

IF EXISTS ( SELECT * FROM CL_LinkRules WHERE @rule_id = rule_id AND enabled = 1 ) BEGIN

  CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                                src_direction      TINYINT,      
                                src_cid            ClassID,      
                                src_id             ObjID,        
                                dst_direction      TINYINT,    
                                dst_cid            ClassID,      
                                dst_id             ObjID,        
                                rule_id            INT )         

  -- process the single rule to produce "link candidates"
  EXEC @return = cl_processLinkRule @rule_id 

  IF @return !< 0 
    -- invoke the "callback" procedure to process the "link candidates" and produce the appropriate link instances
    EXEC @return = @callback_proc

END

IF @return < 0
  ROLLBACK TRAN cl_processLinkRuleID
ELSE BEGIN
  DROP TABLE #linkCandidates
  IF @tcount = 0
    COMMIT TRAN cl_processLinkRuleID
END

RETURN @return



GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

