SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_processLinkRulesDst]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_processLinkRulesDst]
GO




/* This procedure filters the CL_LinkRules table by a specific destination CID and instance ID, then populates a 
   temporary table that will contain the "link candidates" determined by applying each rule with the destination
   resource. Once the temporary table is populated, the specified "callback" procedure is invoked to process the 
   "link candidates".                                   

   The temporary table is defined as follows:                                                                    

     CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                                   src_direction      TINYINT,    
                                   src_cid            ClassID,
                                   src_id             ObjID,
                                   dst_direction      TINYINT,    
                                   dst_cid            ClassID,
                                   dst_id             ObjID,
                                   rule_id            INT )

 */

CREATE PROCEDURE cl_processLinkRulesDst @dst_cid          ClassID,
                                        @dst_id           ObjID,        
                                        @callback_proc    NVARCHAR(128)
AS

RAISERROR( 'cl_processLinkRulesDst: @dst_cid=%s, @dst_id=%d, @callback_proc=%s', 0, 1, 
                                    @dst_cid, @dst_id, @callback_proc )
-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_processLinkRulesDst                
ELSE
  SAVE TRAN cl_processLinkRulesDst                


DECLARE @key_name          NVARCHAR(32)
DECLARE @key_value         NVARCHAR(412)

DECLARE @return             INT


-- create the temporary table
CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                              src_direction      TINYINT,      
                              src_cid            ClassID,      
                              src_id             ObjID,        
                              dst_direction      TINYINT,     
                              dst_cid            ClassID,      
                              dst_id             ObjID,        
                              rule_id            INT )         

-- look through all the key types available for the destination resource
DECLARE keyTypeCursor CURSOR FOR 
  SELECT key_name, key_value FROM GEM_IDLookup WHERE CID = @dst_cid AND IID = @dst_id
OPEN keyTypeCursor

FETCH NEXT FROM keyTypeCursor INTO @key_name, @key_value

SET @return = 0 

WHILE @@FETCH_STATUS = 0 AND @return !< 0 BEGIN

  -- add the appropriate rows to the temporary table
  INSERT INTO #linkCandidates SELECT name,
                                     src_direction,
                                     src_cid,
                                     src_id,
                                     dst_direction,
                                     dst_cid = @dst_cid,
                                     dst_id = @dst_id,
                                     rule_id
                              FROM CL_LinkRules
                              WHERE ( dst_cid IS NULL OR dst_cid = @dst_cid ) AND
                                    @key_name = key_name                      AND 
                                    @key_value LIKE condition              

  IF @@ERROR <> 0 BEGIN 
    RAISERROR( 'error inserting into temp table #linkCandidates', 0, 1 )
    SET @return = -30000  
  END
  ELSE
    FETCH NEXT FROM keyTypeCursor INTO @key_name, @key_value

END

CLOSE keyTypeCursor
DEALLOCATE keyTypeCursor

IF @return !< 0 
  -- invoke the "callback" procedure to process the "link candidates" and produce the appropriate link instances
  EXEC @return = @callback_proc

IF @return < 0           
  ROLLBACK TRAN cl_processLinkRulesDst
ELSE BEGIN
  DROP TABLE #linkCandidates

  -- commit transaction
  IF @tcount = 0                           
    COMMIT TRAN cl_processLinks
END

RETURN @return   




GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

