SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_processLinkRulesNamed]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_processLinkRulesNamed]
GO




/* This procedure filters the CL_LinkRules table by a specific link name and optional direction, then calls a 
   procedure sharing a cursor that contains each rule retrieved. The cursor must be named ruleCursor.
 */

CREATE PROCEDURE cl_processLinkRulesNamed @name             NVARCHAR(128),
                                          @direction        TINYINT = 0,
                                          @callback_proc    NVARCHAR(128)
AS

RAISERROR( 'cl_processLinkRulesNamed: @name=%s, @direction=%d, @callback_proc=%s', 0, 1, 
                                      @name, @direction, @callback_proc )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_processLinkRulesNamed              
ELSE
  SAVE TRAN cl_processLinkRulesNamed             

DECLARE @return             INT

-- cursor through the rules indicated by the filter
DECLARE ruleCursor CURSOR FOR 
        SELECT IDENTITYCOL FROM CL_LinkRules WHERE  name = @name                AND
                                                    src_direction = @direction  AND
                                                    enabled = 1

EXEC @return = cl_processLinkRulesUsingCursor @callback_proc

DEALLOCATE ruleCursor

IF @return < 0 
  ROLLBACK TRAN cl_processLinkRulesNamed
ELSE
  IF @tcount = 0
    COMMIT TRAN cl_processLinkRulesNamed

RETURN @return



GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

