SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_processLinkRulesSrc]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_processLinkRulesSrc]
GO




/* This procedure filters the CL_LinkRules table by a specific source CID and optional instance ID, then calls a
   procedure sharing a cursor that contains each rule retrieved. The cursor must be named ruleCursor.
 */

CREATE PROCEDURE cl_processLinkRulesSrc @src_cid          ClassID,
                                        @src_id           ObjID = 0,
                                        @callback_proc    NVARCHAR(128)
AS

RAISERROR( 'cl_processLinkRulesSrc: @src_cid=%s, @src_id=%d, @callback_proc=%s', 0, 1, 
                                    @src_cid, @src_id, @callback_proc )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_processLinkRulesSrc                
ELSE
  SAVE TRAN cl_processLinkRulesSrc               

DECLARE @return             INT

-- cursor through the rules indicated by the filter
DECLARE ruleCursor CURSOR FOR 
        SELECT IDENTITYCOL FROM CL_LinkRules WHERE  src_cid = @src_cid                   AND
                                                    ( 0 = @src_id OR src_id = @src_id )  AND
                                                    enabled = 1

EXEC @return = cl_processLinkRulesUsingCursor @callback_proc

DEALLOCATE ruleCursor

IF @return < 0 
  ROLLBACK TRAN cl_processLinkRulesSrc  
ELSE
  IF @tcount = 0
    COMMIT TRAN cl_processLinkRulesSrc  

RETURN @return



GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

