SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_processLinkRulesUsingCursor]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_processLinkRulesUsingCursor]
GO



/* This procedure uses a cursor named "ruleCursor" that contains rules that should be processed for link creation.
   The "ruleCursor" should be declared and subsequently deallocated by the caller, but not opened. This procedure
   is also responsible for creating a temporary table that will contain the "link candidates" determined by processing
   each rule in the cursor. Once the rules are processed, the specified "callback" procedure is invoked to process
   the "link candidates".                                   

   The temporary table is defined as follows:                                                                    

     CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                                   src_direction      TINYINT,   
                                   src_cid            ClassID,
                                   src_id             ObjID,
                                   dst_direction      TINYINT,   
                                   dst_cid            ClassID,
                                   dst_id             ObjID,
                                   rule_id            INT )

 */

CREATE PROCEDURE cl_processLinkRulesUsingCursor  @callback_proc    NVARCHAR(128)
AS

RAISERROR( 'cl_processLinkRulesUsingCursor: @callback_proc=%s', 0, 1, @callback_proc )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_processLinkRulesUsingCursor        
ELSE
  SAVE TRAN cl_processLinkRulesUsingCursor       

DECLARE @return           INT
DECLARE @rule_id          INT


OPEN ruleCursor

FETCH NEXT FROM ruleCursor INTO @rule_id

SET @return = 0

IF @@FETCH_STATUS = 0 BEGIN

  -- create the temporary table
  CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                                src_direction      TINYINT,  
                                src_cid            ClassID,      
                                src_id             ObjID,        
                                dst_direction      TINYINT,
                                dst_cid            ClassID,      
                                dst_id             ObjID,        
                                rule_id            INT )         

  -- process each rule in the cursor
  WHILE @@FETCH_STATUS = 0 AND @return !< 0 BEGIN

    -- process the single rule to produce "link candidates"
    EXEC @return = cl_processLinkRule @rule_id 

    FETCH NEXT FROM ruleCursor INTO @rule_id

  END

  IF @return !< 0
    -- invoke the "callback" procedure to process the "link candidates" and produce the appropriate link instances
    EXEC @return = @callback_proc

END

CLOSE ruleCursor

IF @return < 0 
  ROLLBACK TRAN cl_processLinkRulesUsingCursor
ELSE
  IF @tcount = 0
    COMMIT TRAN cl_processLinkRulesUsingCursor

RETURN @return




GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

