PRINT 'Creating procedure cl_setBulkEvents'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_setBulkEvents')
  DROP PROCEDURE cl_setBulkEvents
GO

CREATE PROCEDURE cl_setBulkEvents
AS
  -- All the attributes are in #staged_evt table
  RAISERROR('cl_setBulkEvents: ', 0, 1)

  -- Processing all Messages
  -- Reproducing the code in asisp_createMESG
  UPDATE se SET se.state=md.State FROM MessageDescription_T md
    JOIN #staged_evt se ON (md.obj_cid=se.parent_cid AND md.MsgID=se.evt_name)
    WHERE se.stateid IS NULL

  -- Process Messages for CICS and STC through asisp_createMESG, has special processing 
  DECLARE @eid INT
  SELECT @eid=MIN(eid) FROM #staged_evt 
    WHERE type='MSG' AND (parent_cid='STC' OR parent_cid='CICS')

  WHILE @eid IS NOT NULL BEGIN
    -- Set all messages less than eid in set-operation
    INSERT StagedMESG (
      parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, STATE,
      MESSAGE, REASON_CD, DISCOVERY_IND, AlertState, Priority, StateID, Flags
    )
    SELECT parent_cid, parent_id, time, '', '', '', state, 
           evt_name, evt_cd, 'MON', alertstate, priority, stateid, NULL 
      FROM #staged_evt WHERE type='MSG' AND eid < @eid ORDER BY eid
    INSERT #event_log SELECT eid, 'OK', 'Created MESG with ' + CAST(alertstate AS NVARCHAR) + 
      '-' + CAST(priority AS NVARCHAR) + ' for (' + parent_cid + ', ' + CAST(parent_id AS NVARCHAR) + ')'
      FROM #staged_evt WHERE type='MSG' AND eid < @eid ORDER BY eid

    -- Set messages that needs special-processing individually
    DECLARE @pcid ClassID, @pid ObjID, @time DATETIME, @name NVARCHAR(255)
    DECLARE @desc NVARCHAR(255), @alert INT, @pri INT, @sid INT
    SELECT @pcid=parent_cid, @pid=parent_id, @time=time, @name=evt_name, @desc=evt_cd,
 	   @alert=alertstate, @pri=priority, @sid=stateid FROM #staged_evt WHERE eid=@eid
    EXEC asisp_createMESG @parent_cid=@pcid, @parent_id=@pid, @TIME=@time, @MESSAGE=@name, 
			  @REASON_CD=@desc, @AlertState=@alert, @Priority=@pri, @StateID=@sid
    INSERT #event_log SELECT @eid, 'OK', 'Created MESG with ' + CAST(@alert AS NVARCHAR) + 
      '-' + CAST(@pri AS NVARCHAR) + ' for (' + @pcid + ', ' + CAST(@pid AS NVARCHAR) + ')'

    DELETE FROM #staged_evt WHERE type='MSG' AND eid <= @eid
    SELECT @eid=MIN(eid) FROM #staged_evt 
      WHERE type='MSG' AND (parent_cid='STC' OR parent_cid='CICS')      
  END

  -- Set remaining messages
  INSERT StagedMESG (
    parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, STATE,
    MESSAGE, REASON_CD, DISCOVERY_IND, AlertState, Priority, StateID, Flags
  )
  SELECT parent_cid, parent_id, time, '', '', '', state, 
         evt_name, evt_cd, 'MON', alertstate, priority, stateid, NULL 
    FROM #staged_evt WHERE type='MSG' ORDER BY eid
  INSERT #event_log SELECT eid, 'OK', 'Created MESG with ' + CAST(alertstate AS NVARCHAR) + 
    '-' + CAST(priority AS NVARCHAR) + ' for (' + parent_cid + ', ' + CAST(parent_id AS NVARCHAR) + ')'
    FROM #staged_evt WHERE type='MSG' ORDER BY eid

  DELETE FROM #staged_evt WHERE type='MSG'


  -- Processing all Exceptions (remaining are Exceptions)
  -- When evt_name is empty (reset/remove all Exceptions for parent_cid-parent_id pair)
  INSERT #staged_evt (eid, action, parent_cid, parent_id, evt_name, evt_cd, time) 
    SELECT se.eid, se.action, se.parent_cid, se.parent_id, ev.name, 'UNKNOWN', se.time FROM #staged_evt se
    JOIN Exception_V ev ON (ev._ParentCID=se.parent_cid AND ev._ParentID=se.parent_id)
    WHERE (se.evt_name IS NULL OR se.evt_name='') AND ev.deleted=0  
  DELETE FROM #staged_evt WHERE evt_name IS NULL OR evt_name=''

  -- Creating needed Exceptions categories per distinct (cid, name)
  DECLARE @cid ClassID, @ename NVARCHAR(255)
  DECLARE ecat_c CURSOR FOR SELECT DISTINCT parent_cid, evt_name FROM #staged_evt
  OPEN ecat_c
  FETCH NEXT FROM ecat_c INTO @cid, @ename

  WHILE @@FETCH_STATUS <> -1 BEGIN
    EXEC asisp_getExceptionType @cid, @cid, @ename, 1, 5, NULL, 'CommonListener', 'Dynamically defined' 
    FETCH NEXT FROM ecat_c INTO @cid, @ename      
  END
  DEALLOCATE ecat_c

  -- Clear previous Exception
  INSERT StagedEXCP (
    parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, EXCP_OBJ_TYPE, 
    EXCP_OBJ_NAME, EXCP_NAME, EXCP_CD, nEXCH, AlertState, Priority
  )
  SELECT parent_cid, parent_id, time, '', '', '', '', '', evt_name, 'OK', 1, -1, -1
    FROM #staged_evt ORDER BY eid

  -- Set Exception
  INSERT StagedEXCP (
    parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, EXCP_OBJ_TYPE, 
    EXCP_OBJ_NAME, EXCP_NAME, EXCP_CD, nEXCH, AlertState, Priority
  )
  SELECT parent_cid, parent_id, time, '', '', '', '', '', evt_name, evt_cd, 1, alertstate, priority 
    FROM #staged_evt WHERE action <> 'remove' ORDER BY eid

  INSERT #event_log SELECT eid, 'OK', CASE WHEN action='remove' THEN 'Removed' ELSE 'Created' END + 
    ' EXCP with ' + CAST(alertstate AS NVARCHAR) + '-' + CAST(priority AS NVARCHAR) + 
    ' for (' + parent_cid + ', ' + CAST(parent_id AS NVARCHAR) + ')'
    FROM #staged_evt ORDER BY eid
