USE Object
GO

PRINT 'Creating procedure gemsp_LOBs'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_LOBs')
  DROP PROCEDURE gemsp_LOBs
GO

CREATE PROCEDURE gemsp_LOBs
@cid ClassID,
@id ObjID,
@disposition NVARCHAR(128) OUTPUT
AS
  -- This procedure primarily performs instance filtering 
  RAISERROR('gemsp_LOBs: @cid=%s, @id=%d, @disposition OUTPUT', 0, 1, @cid, @id)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_LOBs
  ELSE
    SAVE TRAN gemsp_LOBs
  
  DECLARE @sql NVARCHAR(255), @lobkey NVARCHAR(255), @name NVARCHAR(255), @excl TINYINT
  DECLARE @stat INT, @matches TINYINT, @pri TINYINT
  DECLARE gem_lob1_c CURSOR FOR SELECT DISTINCT LOBKey, priority FROM GEM_InstFiltering 
    WHERE cid=@cid AND enabled=1 ORDER BY priority DESC

  CREATE TABLE #tmpfilter (expr_value NVARCHAR(255) NULL)    
  EXEC gemsp_getAttribute @cid, @id, 'name', @name OUTPUT
  OPEN gem_lob1_c
  FETCH NEXT FROM gem_lob1_c INTO @lobkey, @pri

  WHILE @@FETCH_STATUS <> -1 BEGIN
    DECLARE @sql2 NVARCHAR(255), @opt NVARCHAR(32), @pattern NVARCHAR(32), @expr NVARCHAR(32), @test NVARCHAR(32)
    SELECT @excl=exclusive FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey
    DECLARE gem_lob2_c CURSOR FOR SELECT memOptions, memPattern, memExpr, memTest FROM GEM_InstFiltering 
      WHERE cid=@cid AND LOBKey=@lobkey
    SET @matches = 1

    OPEN gem_lob2_c
    FETCH NEXT FROM gem_lob2_c INTO @opt, @pattern, @expr, @test

    WHILE @@FETCH_STATUS <> -1 BEGIN
      IF(@opt IS NULL AND @pattern IS NULL AND @expr IS NULL AND @test IS NULL) BEGIN
        SET @matches = 1
        RAISERROR(55000, 0, 1, @cid, @id, @lobkey)
      END
      ELSE BEGIN
        DECLARE @rslt TINYINT, @tvar NVARCHAR(255), @tsym NVARCHAR(255)
        EXEC gemsp_getSymbolic @cid, @id, @expr, @tsym OUTPUT
        
        -- Enable instance filtering with SQL_LIKE command
        IF @opt='SQL_LIKE' BEGIN
          SET @rslt = 0
          SET @tvar = @tsym + ' LIKE ' + @pattern
          
          DELETE FROM #tmpfilter
          INSERT INTO #tmpfilter VALUES (@tsym)
          
          IF EXISTS (SELECT * FROM #tmpfilter WHERE expr_value LIKE @pattern)
            SET @rslt = 1
        END
        ELSE IF @opt='REG_EXPR' BEGIN	-- Instance filtering using regexpr        
          DECLARE @error NVARCHAR(255), @exprout INT
          SET @rslt = 0
          SET @tvar = 'master..xp_regexmatch ' + @pattern + ', ' + @tsym + ', @exprout OUT, @error OUT'
          
          EXEC master..xp_regexmatch @pattern, @tsym, @exprout OUT, @error OUT
          IF @exprout IS NOT NULL BEGIN
            SET @rslt = 1
            EXEC master..xp_regexmatcharrayfree @exprout
          END
        END
        ELSE BEGIN   -- Instance filtering with egrep        
          SELECT @tvar = 'echo ' + @tsym + ' | egrep -s' + @opt + ' "' + @pattern + '"'
          EXEC gemsp_cmdSh @tvar, @rslt OUTPUT
        END
  
        IF @rslt <> @test BEGIN
          SET @matches = 0
          RAISERROR(55001, 0, 1, @tvar, @cid, @id, @lobkey, 'false')
          BREAK
        END
        ELSE
          RAISERROR(55001, 0, 1, @tvar, @cid, @id, @lobkey, 'true')
      END
   
      FETCH NEXT FROM gem_lob2_c INTO @opt, @pattern, @expr, @test
    END 
   
    IF @matches <> 0 
      EXEC gemsp_putInLOB @cid, @id, @name, @lobkey, 'CREATE', @disposition OUTPUT
    ELSE 
      EXEC gemsp_putInLOB @cid, @id, @name, @lobkey, 'DELETE', @disposition OUTPUT
 
    DEALLOCATE gem_lob2_c

    IF @excl = 1 BREAK
    FETCH NEXT FROM gem_lob1_c INTO @lobkey, @pri
  END 
  DEALLOCATE gem_lob1_c

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_LOBs

  

