USE Object
GO

PRINT 'Creating procedure gemsp_action_inLOB'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_action_inLOB')
  DROP PROCEDURE gemsp_action_inLOB
GO

CREATE PROCEDURE gemsp_action_inLOB
@action NVARCHAR(8),
@cid ClassID,
@id ObjID, 
@plobid ObjID,
@disposition NVARCHAR(128) OUTPUT
AS

  -- This PROCEDURE performs the action (which can be DELETE OR EXISTS) specified.
  -- IF EXISTS, checks IF the instance represented by @cid, @id EXISTS under the LOB id @plobid
  -- IF DELETE, deletes all the LOB instances of @cid, @id under LOB id @plobid
  RAISERROR('gemsp_action_inLOB: @action=%s, @cid=%s, @id=%d, @plobid=%d, @disposition OUTPUT', 0, 1, @action, @cid, @id, @plobid)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_action_inLOB
  ELSE
    SAVE TRAN gemsp_action_inLOB

  DECLARE @sql2 NVARCHAR(510), @tblname ObjName, @lob_oid ObjID
  CREATE TABLE #tmpid (lobid ObjID NULL)
  SELECT @tblname = 'LOB_' + RTRIM(@cid) + '_LOBL'

  SELECT @sql2 = 'INSERT #tmpid SELECT src_id FROM ' + @tblname + ' WHERE dst_id=' + CAST(@id AS NVARCHAR)
  EXEC (@sql2)
    
  DECLARE lob_c CURSOR FOR SELECT lobid FROM #tmpid
  OPEN lob_c
  FETCH NEXT FROM lob_c INTO @lob_oid
  WHILE @@FETCH_STATUS <> -1 BEGIN      
    IF EXISTS (SELECT * FROM LOB_LOB_LOBC WHERE dst_id=@lob_oid AND src_id=@plobid) BEGIN
      IF EXISTS (SELECT * FROM link WHERE dst_cid=@cid AND dst_id=@id AND src_cid='LOB' AND src_id=@lob_oid AND link_type='LOBL' AND deleted=0) BEGIN
        IF @action = 'EXISTS'
	  RETURN 1
        IF @action = 'DELETE' BEGIN         
          EXEC delete_LOB @lob_oid
 
          IF @disposition IS NULL
	    SELECT @disposition = 'LOB : deletedFrom ' + CAST(@plobid AS NVARCHAR) + ' '
      	  ELSE
            SELECT @disposition = @disposition + ': deletedFrom ' + CAST(@plobid AS NVARCHAR) + ' '
        END
      END
    END

    FETCH NEXT FROM lob_c INTO @lob_oid
  END
 
  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_action_inLOB
  RETURN 0


