USE Object
GO

PRINT 'Creating procedure gemsp_addLOB'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_addLOB')
  DROP PROCEDURE gemsp_addLOB
GO

CREATE PROCEDURE gemsp_addLOB
@loblabel NVARCHAR(255),
@ploblabel NVARCHAR(255) = 'LOBC',
@plobkey NVARCHAR(450) = ''
AS
  RAISERROR('gemsp_addLOB: @loblabel=%s, @ploblabel=%s, @plobkey=%s', 0, 1, @loblabel, @ploblabel, @plobkey)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_addLOB
  ELSE
    SAVE TRAN gemsp_addLOB

  DECLARE @lobkey NVARCHAR(450), @plobid ObjID
  IF @loblabel = @ploblabel BEGIN
    RAISERROR(55007, 0, 1) 
    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_addLOB
    RETURN
  END

  IF @plobkey <> '' BEGIN
    IF EXISTS (SELECT * FROM GEM_LOBLookup WHERE LOBKey=@plobkey) BEGIN
      SELECT @plobid=LOBID FROM GEM_LOBLookup WHERE LOBKey=@plobkey
      INSERT INTO GEM_LOBLookup VALUES (0, @loblabel, @loblabel+@plobkey, @plobid, @plobkey)   
      IF @@ERROR = 0
        RAISERROR(55008, 0, 1, @loblabel, @plobkey)
      ELSE BEGIN
        ROLLBACK TRAN gemsp_addLOB        
        RETURN -30000
      END
    END
    ELSE
      RAISERROR(55009, 0, 1, @plobkey)      

    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_addLOB
    RETURN
  END 

  IF @ploblabel = 'LOBC' BEGIN
    INSERT INTO GEM_LOBLookup VALUES (0, @loblabel, @loblabel+'LOBC', 1, 'LOBC')   
    IF @@ERROR = 0 BEGIN
      RAISERROR(55010, 0, 1, @loblabel)      
      -- Finish up transaction
      IF @tcount = 0 
        COMMIT TRAN gemsp_addLOB
      RETURN
    END
    ELSE BEGIN
      ROLLBACK TRAN gemsp_addLOB        
      RETURN -30000
    END
  END
  
  SELECT * FROM GEM_LOBLookup WHERE LOBName=@ploblabel
  IF @@ROWCOUNT > 0 BEGIN
    DECLARE icur_c CURSOR FOR SELECT LOBKey FROM GEM_LOBLookup WHERE LOBName=@ploblabel
    OPEN icur_c
    FETCH NEXT FROM icur_c INTO @plobkey

    WHILE @@FETCH_STATUS <> -1 BEGIN
      SELECT @plobid=LOBID FROM GEM_LOBLookup WHERE LOBKey=@plobkey
      INSERT INTO GEM_LOBLookup VALUES (0, @loblabel, @loblabel+@plobkey, @plobid, @plobkey)      

      IF @@ERROR = 0
        RAISERROR(55012, 0, 1, @loblabel, @ploblabel)        
      ELSE BEGIN
        ROLLBACK TRAN gemsp_addLOB        
        RETURN -30000
      END
      
      FETCH NEXT FROM icur_c INTO @plobkey
    END
    DEALLOCATE icur_c
  END
  ELSE BEGIN	-- creating new parent LOB under LOBC  
    SELECT @plobid = 0
    SELECT @plobkey = @ploblabel + 'LOBC'

    INSERT INTO GEM_LOBLookup VALUES (@plobid, @ploblabel, @plobkey, 1, 'LOBC')
    IF @@ERROR = 0
      RAISERROR(55010, 0, 1, @ploblabel)      
    ELSE BEGIN
      ROLLBACK TRAN gemsp_addLOB        
      RETURN -30000
    END

    INSERT INTO GEM_LOBLookup VALUES (0, @loblabel, @loblabel+@plobkey, @plobid, @plobkey)   
    IF @@ERROR = 0
      RAISERROR(55011, 0, 1, @loblabel, @ploblabel)      
    ELSE BEGIN
      ROLLBACK TRAN gemsp_addLOB        
      RETURN -30000
    END
  END

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_addLOB
  RETURN
