USE Object
GO

PRINT 'Creating procedure gemsp_addpvToLOB'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_addpvToLOB')
  DROP PROCEDURE gemsp_addpvToLOB
GO

CREATE PROCEDURE gemsp_addpvToLOB
@product NVARCHAR(255) = '',
@version NVARCHAR(255) = '',
@loblabel NVARCHAR(255),
@lobkey NVARCHAR(450) = '',
@memOptions NVARCHAR(64) = NULL,
@memPattern NVARCHAR(64) = NULL,
@memExpr NVARCHAR(64) = NULL,
@memTest NVARCHAR(64) = NULL,
@action NVARCHAR(16) = 'INSERT'
AS  
  RAISERROR('gemsp_addpvToLOB: @product=%s, @version=%s, @loblabel=%s, @lobkey=%s, @memOptions=%s, @memPattern=%s, @memExpr=%s, @memTest=%s, @action=%s', 0, 1, 
	    @product, @version, @loblabel, @lobkey, @memOptions, @memPattern, @memExpr, @memTest, @action)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_addpvToLOB
  ELSE
    SAVE TRAN gemsp_addpvToLOB

  SET @action = UPPER(@action)  DECLARE @cid ClassID, @view INT
  SET @view = 0
  SELECT @cid=cid FROM GEMLookupCID WHERE prodname=@product AND version=@version
  IF @cid IS NULL 
    SELECT @cid=cid FROM obj_class WHERE cname=@product OR cname=@version

  IF @cid IS NOT NULL BEGIN
    IF @loblabel = 'LOBC' BEGIN
      RAISERROR(55013, 0, 1)      
      -- Finish up transaction
      IF @tcount = 0 
        COMMIT TRAN gemsp_addpvToLOB
      RETURN
    END

    -- The action (INSERT/UPDATE/DELETE) IS done ON ALL rows that match the cid AND lobname/lobkey.
    -- So the granularity IS at LOBName/LOBKey level.
    
    IF @lobkey <> '' BEGIN
      IF EXISTS (SELECT * FROM GEM_LOBLookup WHERE LOBKey=@lobkey) BEGIN
        IF @action = 'INSERT' BEGIN
          INSERT INTO GEM_InstFiltering VALUES (@cid, @lobkey, @memOptions, @memPattern, @memExpr, @memTest, 1, 0, 0)
          SET @view = 1
          RAISERROR(55014, 0, 1, @product, @version, @lobkey)          
        END
        ELSE BEGIN
          IF EXISTS (SELECT * FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey) BEGIN
            IF @action = 'UPDATE' BEGIN
              SELECT * FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey
              IF @@ROWCOUNT = 1
                UPDATE GEM_InstFiltering SET memOptions=@memOptions, memPattern=@memPattern, memExpr=@memExpr, memTest=@memTest WHERE cid=@cid AND LOBKey=@lobkey
              ELSE BEGIN
                RAISERROR(55015, 0, 1, @product, @version, 'LOBKey', @lobkey)                
		-- Finish up transaction
		IF @tcount = 0 
		  COMMIT TRAN gemsp_addpvToLOB
                RETURN
              END
            END
            ELSE IF @action = 'DELETE'
              DELETE FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey
            SET @view = 1
            RAISERROR(55016, 0, 1, @product, @version, 'LOBKey', @lobkey, @action)            
          END
          ELSE
            RAISERROR(55017, 0, 1, @action, @product, @version, 'LOBKey', @lobkey)
        END

        IF @view = 1 
          IF NOT EXISTS(SELECT * FROM GEM_UpdateLOBView WHERE cid=@cid AND id=0)
            INSERT GEM_UpdateLOBView (cid, id) SELECT @cid, 0
      END
      ELSE
        RAISERROR(55018, 0, 1, @lobkey)        

      -- Finish up transaction
      IF @tcount = 0 
        COMMIT TRAN gemsp_addpvToLOB
      RETURN
    END

    SET @view = 0
    SELECT * FROM GEM_LOBLookup WHERE LOBName=@loblabel
    IF @@ROWCOUNT > 0 BEGIN
      DECLARE icur_c CURSOR FOR SELECT LOBKey FROM GEM_LOBLookup WHERE LOBName=@loblabel
      OPEN icur_c
      FETCH NEXT FROM icur_c INTO @lobkey

      WHILE @@FETCH_STATUS <> -1 BEGIN
        IF @action = 'INSERT' BEGIN
          INSERT INTO GEM_InstFiltering VALUES (@cid, @lobkey, @memOptions, @memPattern, @memExpr, @memTest, 1, 0, 0)
          SET @view = 1
          RAISERROR(55019, 0, 1, @product, @version, @loblabel)          
        END
        ELSE BEGIN
          IF EXISTS (SELECT * FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey) BEGIN
            IF @action = 'UPDATE' BEGIN          
              SELECT * FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey
              IF @@ROWCOUNT = 1 BEGIN
                UPDATE GEM_InstFiltering SET memOptions=@memOptions, memPattern=@memPattern, memExpr=@memExpr, memTest=@memTest WHERE cid=@cid AND LOBKey=@lobkey
                RAISERROR(55016, 0, 1, @product, @version, 'LOBName', @loblabel, @action)                            
                SET @view = 1
              END
              ELSE
                RAISERROR(55015, 0, 1, @product, @version, 'LOBName', @loblabel)                                
            END
            ELSE IF @action = 'DELETE' BEGIN
              DELETE FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey
              RAISERROR(55016, 0, 1, @product, @version, 'LOBName', @loblabel, @action)                                          
              SET @view = 1
            END
          END
          ELSE
            RAISERROR(55017, 0, 1, @action, @product, @version, 'LOBName', @loblabel)            
        END     
      
        FETCH NEXT FROM icur_c INTO @lobkey
      END
      DEALLOCATE icur_c
    END
    ELSE BEGIN
      IF @action = 'INSERT' BEGIN
        SELECT @lobkey = @loblabel + 'LOBC'
        INSERT INTO GEM_LOBLookup VALUES (0, @loblabel, @lobkey, 1, 'LOBC')   
        IF @@ERROR = 0
          RAISERROR(55010, 0, 1, @loblabel)          

        INSERT INTO GEM_InstFiltering VALUES (@cid, @lobkey, @memOptions, @memPattern, @memExpr, @memTest, 1, 0, 0)
        IF @@ERROR = 0
          RAISERROR(55020, 0, 1, @product, @version, @loblabel)          
        SET @view = 1
      END
      ELSE
        RAISERROR(55017, 0, 1, @action, @product, @version, 'LOBName', @loblabel)        
    END

    -- Put in Update LOB view table for later view updation
    IF @view = 1 
      IF NOT EXISTS(SELECT * FROM GEM_UpdateLOBView WHERE cid=@cid AND id=0)
        INSERT GEM_UpdateLOBView (cid, id) SELECT @cid, 0
  END
  ELSE 
    RAISERROR(55029, 0, 1, @product, @version)    


  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_addpvToLOB

