USE Object
GO

PRINT 'Creating procedure gemsp_createLOB'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_createLOB')
  DROP PROCEDURE gemsp_createLOB
GO

CREATE PROCEDURE gemsp_createLOB
@lobname NVARCHAR(255),
@lobkey NVARCHAR(450),
@parentid ObjID,
@type NVARCHAR(4) = 'LOB'
AS
  RAISERROR('gemsp_createLOB: @lobname=%s, @lobkey=%s, @parentid=%d, @type=%s', 0, 1, @lobname, @lobkey, @parentid, @type)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_createLOB
  ELSE
    SAVE TRAN gemsp_createLOB

  DECLARE @child_id ObjID  
  -- @child_id is got from link table if it already exists, and is 
  -- created otherwise
  
  SELECT @child_id=l.dst_id FROM link l, LineOfBusiness_C b 
    WHERE l.deleted=0 AND l.link_type='LOBC' AND l.src_cid=@type AND 
          l.src_id=@parentid AND l.dst_cid='LOB' AND l.dst_id=b.id AND 
          b.name=@lobname
  
  IF @@ROWCOUNT <= 0 OR @child_id <= 0
    EXEC @child_id = asisp_createLOB @lobname, @type, @parentid
  
  IF @child_id > 0 BEGIN
    UPDATE GEM_LOBLookup SET LOBID=@child_id WHERE LOBKey=@lobkey
    UPDATE GEM_LOBLookup SET parentLOBID=@child_id WHERE parentLOBKey=@lobkey
  END
  ELSE
    RAISERROR(55006, 0, 1, 'Update tables of ')

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_createLOB
  RETURN @child_id


