USE Object
GO

PRINT 'Creating procedure gemsp_deleteLOBs'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_deleteLOBs')
  DROP PROCEDURE gemsp_deleteLOBs
GO

CREATE PROCEDURE gemsp_deleteLOBs
@cid ClassID
AS
  RAISERROR('gemsp_deleteLOBs: @cid=%s', 0, 1, @cid)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_deleteLOBs
  ELSE
    SAVE TRAN gemsp_deleteLOBs

  DECLARE @lobkey NVARCHAR(450), @plobkey NVARCHAR(450), @lobid ObjID, @error INT
  SELECT @error = 0
  DECLARE gem_lob1_c CURSOR FOR SELECT DISTINCT LOBKey FROM GEM_InstFiltering WHERE cid=@cid
  OPEN gem_lob1_c
  FETCH NEXT FROM gem_lob1_c INTO @lobkey

  WHILE @@FETCH_STATUS <> -1 BEGIN
    WHILE @lobkey IS NOT NULL BEGIN
      SELECT @plobkey=parentLOBKey FROM GEM_LOBLookup WHERE LOBKey=@lobkey
      DELETE FROM GEM_LOBLookup WHERE LOBKey=@lobkey AND parentLOBKey=@plobkey
      IF @@ERROR <> 0 SET @error = @error + 1
      
      -- Trigger on GEM_LOBLookup cleans up GEM_LOB_BCDF and GEM_InstFiltering tables
      -- DELETE FROM GEM_LOB_BCDF WHERE LOBName=@lobkey

      IF NOT EXISTS (SELECT * FROM GEM_LOBLookup WHERE parentLOBKey=@plobkey)
        SELECT @lobkey = @plobkey
      ELSE
        SELECT @lobkey = NULL
    END
    FETCH NEXT FROM gem_lob1_c INTO @lobkey
  END
  DEALLOCATE gem_lob1_c

  DELETE FROM GEM_InstFiltering WHERE cid=@cid
  IF @@ERROR <> 0 SET @error = @error + 1
  
  /* Delete the instances created by drag-drop */
  DECLARE @plobid ObjID, @pcid ClassID
  DECLARE gem_lob2_c CURSOR FOR SELECT src_id FROM link WHERE dst_cid=@cid AND src_cid='LOB' AND deleted=0
  OPEN gem_lob2_c
  FETCH NEXT FROM gem_lob2_c INTO @lobid
  
  WHILE @@FETCH_STATUS <> -1 BEGIN
    WHILE @lobid <> 0 BEGIN
      EXEC delete_LOB @lobid
      IF @@ERROR <> 0 SET @error = @error + 1
      SELECT @plobid=src_id, @pcid=src_cid FROM link WHERE dst_cid='LOB' AND dst_id=@lobid AND deleted=1

      IF @pcid = 'LOB' AND @plobid > 0 BEGIN
        IF NOT EXISTS (SELECT * FROM link WHERE src_id=@plobid AND src_cid=@pcid AND deleted=0 AND link_type='LOBC')  
          SELECT @lobid = @plobid
        ELSE
          SELECT @lobid = 0
      END
      ELSE
        SELECT @lobid = 0
    END
    FETCH NEXT FROM gem_lob2_c INTO @lobid
  END
  DEALLOCATE gem_lob2_c

  IF @error > 0
    ROLLBACK TRAN gemsp_deleteLOBs
  ELSE IF @tcount = 0 -- Finish up transaction
    COMMIT TRAN gemsp_deleteLOBs

  RETURN @error    






