USE Object
GO

PRINT 'Creating procedure gemsp_getLOBTree'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_getLOBTree')
  DROP PROCEDURE gemsp_getLOBTree
GO

CREATE PROCEDURE gemsp_getLOBTree
@lobkey NVARCHAR(450),
@outree NVARCHAR(510) OUTPUT
AS
--  RAISERROR('gemsp_getLOBTree: @lobkey=%s, @outree OUTPUT', 0, 1, @lobkey)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_getLOBTree
  ELSE
    SAVE TRAN gemsp_getLOBTree

  DECLARE @lobname NVARCHAR(255), @lobid ObjID, @plobkey NVARCHAR(450)
  SELECT @lobname=LOBName, @lobid=LOBID, @plobkey=parentLOBKey FROM GEM_LOBLookup WHERE LOBKey=@lobkey
  
  IF @plobkey = 'LOBC'
    SET @outree = @outree + @lobname + '(' + CAST(@lobid AS NVARCHAR) + ') -> LOBC'
  ELSE IF @plobkey IS NOT NULL AND @plobkey <> '' BEGIN
    SET @outree = @outree + @lobname + '(' + CAST(@lobid AS NVARCHAR) + ') -> '
    EXEC gemsp_getLOBTree @plobkey, @outree OUTPUT
  END
  
  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_getLOBTree

