USE Object
GO

PRINT 'Creating procedure gemsp_listLOBs'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_listLOBs')
  DROP PROCEDURE gemsp_listLOBs
GO

CREATE PROCEDURE gemsp_listLOBs
AS
  RAISERROR('gemsp_listLOBs: ', 0, 1)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_listLOBs
  ELSE
    SAVE TRAN gemsp_listLOBs

  DECLARE @cid ClassID, @product NVARCHAR(64), @version NVARCHAR(16), @out NVARCHAR(510)
  RAISERROR('INSTANCE FILTERING & AUTOMATIC LOB INFORMATION', 0, 1)

  DECLARE cid_c CURSOR FOR SELECT DISTINCT cid FROM GEM_InstFiltering
  OPEN cid_c 
  FETCH NEXT FROM cid_c INTO @cid
        
  WHILE @@FETCH_STATUS <> -1 BEGIN
    SELECT @product=prodname, @version=version FROM GEMLookupCID WHERE cid=@cid
    RAISERROR('----- PRODUCT=%s, VERSION=%s -----', 0, 1, @product, @version)
  
    DECLARE @lobkey NVARCHAR(450), @opt NVARCHAR(64), @patt NVARCHAR(64), @expr NVARCHAR(64), @test NVARCHAR(64), @lobname NVARCHAR(255), @lobid ObjID
    DECLARE key_c CURSOR FOR SELECT LOBKey, memOptions, memPattern, memExpr, memTest FROM GEM_InstFiltering WHERE cid=@cid
    OPEN key_c 
    FETCH NEXT FROM key_c INTO @lobkey, @opt, @patt, @expr, @test   
     
    WHILE @@FETCH_STATUS <> -1 BEGIN
      SET @out = 'Tree: ['
      SELECT @lobname=LOBName, @lobid=LOBID FROM GEM_LOBLookup WHERE LOBKey=@lobkey
      RAISERROR('  LOB:  %s', 0, 1, @lobname)
      EXEC gemsp_getLOBTree @lobkey, @out OUTPUT
      RAISERROR('  %s]', 0, 1, @out)

      IF @opt IS NULL AND @patt IS NULL AND @expr IS NULL AND @test IS NULL 
        RAISERROR('  Filtering: none', 0, 1)
      ELSE
        RAISERROR('  Filtering: Options=%s, Pattern=%s, Expression=%s and TestCondition=%s', 0, 1, @opt, @patt, @expr, @test)
      RAISERROR('', 0, 1)
      FETCH NEXT FROM key_c INTO @lobkey, @opt, @patt, @expr, @test
    END
    DEALLOCATE key_c
    FETCH NEXT FROM cid_c INTO @cid
  END
  DEALLOCATE cid_c

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_listLOBs


