USE Object
GO

PRINT 'Creating procedure gemsp_processOrphanLOBs'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_processOrphanLOBs')
  DROP PROCEDURE gemsp_processOrphanLOBs
GO

CREATE PROCEDURE gemsp_processOrphanLOBs
AS

  -- get LOB children if they exist
  RAISERROR('gemsp_processOrphanLOBs: ', 0, 1)

  IF NOT EXISTS (SELECT * FROM GEM_UpdateLOBView WHERE LOBKey <> '')
    RETURN

  CREATE TABLE #lobkeys ( 
    lkey NVARCHAR(255),
    lobid ObjID
  )

  -- when lobid is specified
  INSERT #lobkeys SELECT uv.LOBKey, li.lobid FROM GEM_LOBInstLookup li
    JOIN GEM_UpdateLOBView uv ON (li.LOBKey=uv.LOBKey)
    JOIN link l ON (l.dst_id=li.lobid)
    WHERE l.dst_cid='LOB' AND l.deleted=0

  -- when oid_where is empty and LOBName is not
  INSERT #lobkeys SELECT li.LOBKey, v.id FROM GEM_LOBInstLookup li
    JOIN GEM_UpdateLOBView uv ON (li.LOBKey=uv.LOBKey)
    JOIN LineOfBusiness_V v ON (li.LOBName=v.name)
    JOIN link l ON (l.dst_id=v.id)
    WHERE li.lobid=0 AND li.oid_where='' AND l.dst_cid='LOB' and l.deleted=0

  -- when oid_where is not empty irrespective of LOBName
  DECLARE @lobname NVARCHAR(255), @lobkey NVARCHAR(255), @where_clause NVARCHAR(255)
  DECLARE @cname NVARCHAR(24), @sql NVARCHAR(1024), @cid ClassID
  DECLARE wh_c CURSOR FOR SELECT li.cid, li.LOBKey, li.LOBName, li.oid_where FROM GEM_LOBInstLookup li
    JOIN GEM_UpdateLOBView uv ON (li.LOBKey=uv.LOBKey)
    WHERE li.lobid=0 AND li.oid_where<>''
  OPEN wh_c
  FETCH NEXT FROM wh_c INTO @cid, @lobkey, @lobname, @where_clause

  WHILE @@FETCH_STATUS <> -1 BEGIN
    SELECT @cname=cname FROM obj_class WHERE cid=@cid
    IF @lobname='' OR @lobname IS NULL
      SET @sql = 'INSERT #lobkeys SELECT ''' + @lobkey + ''', l.src_id FROM ' + @cname + '_V v 
	          JOIN link l ON (v.id=l.dst_id AND l.dst_cid=''' + @cid + ''')
	          WHERE l.src_cid=''LOB'' AND l.link_type=''LOBL'' AND l.deleted=0'
    ELSE
      SET @sql = 'INSERT #lobkeys SELECT ''' + @lobkey + ''', l.src_id FROM ' + @cname + '_V v 
	          JOIN link l ON (v.id=l.dst_id AND l.dst_cid=''' + @cid + ''')
                  JOIN LineOfBusiness_V lobv ON (lobv.id=l.src_id)
	          WHERE l.src_cid=''LOB'' AND l.link_type=''LOBL'' AND l.deleted=0 AND lobv.name=' + @lobname
    EXEC (@sql)

    FETCH NEXT FROM wh_c INTO @cid, @lobkey, @lobname, @where_clause
  END
  DEALLOCATE wh_c

  -- Create the parent/child relation 
  DECLARE @child_cid ClassID, @child_id ObjID, @action NVARCHAR(32), @parent_lobid ObjID, @idx INT
  DECLARE @name ObjName,  @tblname ObjName, @id ObjID, @exists TINYINT, @disp NVARCHAR(255)
  DECLARE upd_c CURSOR FOR SELECT uv.cid, uv.id, k.lkey, k.lobid FROM #lobkeys k
    JOIN GEM_UpdateLOBView uv ON (uv.LOBKey=k.lkey)
    WHERE UPPER(uv.action)='CREATE'
  OPEN upd_c
  FETCH NEXT FROM upd_c INTO @cid, @id, @lobkey, @parent_lobid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    IF @cid = 'LOB' BEGIN
      SET @idx = CHARINDEX(',', @lobkey, 0)
      IF @idx > 0 SET @name = SUBSTRING(@lobkey, @idx+1, LEN(@lobkey)-@idx)
      ELSE SET @name = 'UnKnown'

      IF NOT EXISTS (
	SELECT * FROM link l JOIN LineOfBusiness_V v ON (v.id=l.dst_id)
	  WHERE v.name=@name AND l.dst_cid='LOB' AND l.src_cid='LOB' AND 
		l.src_id=@parent_lobid AND l.deleted=0 AND l.link_type='LOBC'
      )       
        EXEC asisp_createLOB @name, 'LOB', @parent_lobid      
    END
    ELSE BEGIN
      EXEC @exists = gemsp_action_inLOB 'EXISTS', @cid, @id, @parent_lobid, @disp OUT
      IF @exists = 0 BEGIN
        EXEC gemsp_getAttribute @cid, @id, 'name', @name OUT
        EXEC @child_id = asisp_createLOB @name, 'LOB', @parent_lobid

        -- Attach the LOB to instance
        IF @child_id > 0 BEGIN
          SELECT @tblname = 'LOB_' + RTRIM(@cid) + '_LOBL'
          SELECT @sql = 'INSERT ' + @tblname + '(src_id, dst_id) SELECT ' + CONVERT(NCHAR(9), @child_id) + ', ' + CONVERT(NCHAR(9),@id)
          EXEC(@sql)
        END
      END

      DELETE FROM GEM_UpdateLOBView 
        WHERE cid=@cid AND id=@id AND LOBKey=@lobkey AND action='CREATE'
    END
    FETCH NEXT FROM upd_c INTO @cid, @id, @lobkey, @parent_lobid
  END
  DEALLOCATE upd_c
