USE Object
GO

PRINT 'Creating procedure gemsp_putInLOBInst'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_putInLOBInst')
  DROP PROCEDURE gemsp_putInLOBInst
GO

CREATE PROCEDURE gemsp_putInLOBInst
@cid ClassID,
@id ObjID,
@name NVARCHAR(255),
@lobkey NVARCHAR(255),
@action NVARCHAR(8),
@disposition NVARCHAR(128) OUTPUT
AS
  -- Put a trigger on this table to clean-up GEM_InstFiltering (just like GEM_LOBLookup does)
  -- Need to update/reset the table in delete_obj 
  RAISERROR('gemsp_putInLOBInst: @cid=%s, @id=%d, @name=%s, @lobkey=%s, @action=%s, @disposition OUTPUT', 0, 1, @cid, @id, @name, @lobkey, @action)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_putInLOBInst
  ELSE
    SAVE TRAN gemsp_putInLOBInst

  DECLARE @pcid ClassID, @plobname NVARCHAR(255), @plobid ObjID, @pmatch NVARCHAR(255)
  SELECT @pcid=cid, @pmatch=oid_where, @plobname=LOBName, @plobid=lobid 
    FROM GEM_LOBInstLookup WHERE LOBKey=@lobkey
  IF @@ROWCOUNT <= 0 BEGIN
    RAISERROR(55030, 0, 1, @pcid, @pmatch, @plobname, @plobid, @cid, @name)
    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_putInLOBInst
    RETURN -1
  END

  CREATE TABLE #lobids (oid ObjID)
  IF @pmatch IS NULL SET @pmatch = ''
  IF @plobname IS NULL SET @plobname = ''

  IF @plobid = 0 BEGIN
    IF @pmatch = '' 
      INSERT #lobids SELECT l.src_id FROM link l, LineOfBusiness_C b WHERE
        b.name=@plobname AND b.id=l.src_id AND l.src_cid='LOB' AND 
        l.deleted=0 AND l.link_type='LOBL' AND l.dst_cid=@pcid
    ELSE BEGIN
      CREATE TABLE #alloids ( oid ObjID )
      DECLARE @cname NVARCHAR(24), @sql NVARCHAR(1024)
      SELECT @cname=cname FROM obj_class WHERE cid=@pcid
      SET @sql = 'INSERT #alloids SELECT id FROM ' + @cname + '_V ' + @pmatch 
      EXEC (@sql)
     
      IF @plobname = ''
        INSERT #lobids SELECT l.src_id FROM link l 
          JOIN LineOfBusiness_C b ON (l.src_cid='LOB' AND b.id=l.src_id)
          JOIN #alloids o ON (l.dst_cid=@pcid AND l.dst_id=o.oid)
	  WHERE l.deleted=0 AND l.link_type='LOBL'
      ELSE
        INSERT #lobids SELECT l.src_id FROM link l 
          JOIN LineOfBusiness_C b ON (l.src_cid='LOB' AND b.id=l.src_id)
          JOIN #alloids o ON (l.dst_cid=@pcid AND l.dst_id=o.oid)
	  WHERE b.name=@plobname AND l.deleted=0 AND l.link_type='LOBL'   
    END
  END
  ELSE
    INSERT #lobids SELECT @plobid

  IF NOT EXISTS(SELECT * FROM #lobids) BEGIN
    -- Put in GEM_UpdateLOBView for later processing when parent is not yet available
    RAISERROR('Could not find parent(s) in LOB view', 0, 1)
    INSERT GEM_UpdateLOBView SELECT @cid, @id, @lobkey, @action

    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_putInLOBInst
    RETURN -1    
  END

  -- Create if does not already exist
  DECLARE @child_id ObjID, @tblname ObjName, @exists TINYINT
  DECLARE id_c CURSOR FOR SELECT oid FROM #lobids
  OPEN id_c
  FETCH NEXT FROM id_c INTO @plobid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    IF @action = 'DELETE' 
      EXEC gemsp_action_inLOB 'DELETE', @cid, @id, @plobid, @disposition OUTPUT
    ELSE BEGIN
      EXEC @exists = gemsp_action_inLOB 'EXISTS', @cid, @id, @plobid, @disposition OUTPUT

      IF @exists = 0 BEGIN
        EXEC @child_id = asisp_createLOB @name, 'LOB', @plobid
        RAISERROR(55005, 0, 1, @cid, @id, @name, @plobid)

        IF @child_id > 0 BEGIN
          SELECT @tblname = 'LOB_' + RTRIM(@cid) + '_LOBL'
          SELECT @sql = 'INSERT ' + @tblname + '(src_id, dst_id) SELECT ' + CONVERT(NCHAR(9), @child_id) + ', ' + CONVERT(NCHAR(9),@id)
          EXEC(@sql)
        
          IF @disposition IS NULL
            SELECT @disposition = 'LOB : addedTo ' + CAST(@plobid AS NVARCHAR) + ' '
          ELSE
            SELECT @disposition = @disposition + ': addedTo ' + CAST(@plobid AS NVARCHAR) + ' '
        END
        ELSE
          RAISERROR(55006, 0, 1, 'CREATE')    
      END
    END  
    FETCH NEXT FROM id_c INTO @plobid
  END
  DEALLOCATE id_c

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_putInLOBInst
