USE Object
GO

PRINT 'Creating procedure gemsp_rmLOB'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_rmLOB')
  DROP PROCEDURE gemsp_rmLOB
GO

CREATE PROCEDURE gemsp_rmLOB
@loblabel NVARCHAR(255),
@ploblabel NVARCHAR(255)
AS  
  -- Removes the LOB @loblabel (under @ploblabel) from GEM_LOBLookup table, so that 
  -- this LOB gets deleted permanently
  RAISERROR('gemsp_rmLOB: @loblabel=%s, @ploblabel=%s', 0, 1, @loblabel, @ploblabel)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_rmLOB
  ELSE
    SAVE TRAN gemsp_rmLOB

  DECLARE @lkey NVARCHAR(450)
  CREATE TABLE #keys (LOBKey NVARCHAR(450))

  IF @ploblabel = 'LOBC'
    INSERT INTO #keys SELECT LOBKey FROM GEM_LOBLookup 
      WHERE LOBName=@loblabel AND parentLOBKey = 'LOBC'
  ELSE
    INSERT INTO #keys SELECT a.LOBKey FROM GEM_LOBLookup a, GEM_LOBLookup b
      WHERE b.LOBName=@ploblabel AND b.LOBKey=a.parentLOBKey AND a.LOBName=@loblabel

  IF @@ROWCOUNT <= 0 BEGIN
    RAISERROR(55022, 0, 1, @loblabel, @ploblabel)
    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_rmLOB
    RETURN
  END

  DECLARE key_c CURSOR FOR SELECT LOBKey FROM #keys
  OPEN key_c
  FETCH NEXT FROM key_c INTO @lkey
  WHILE @@FETCH_STATUS <> -1 BEGIN
    DECLARE @lid ObjID
    SELECT @lid=LOBID FROM GEM_LOBLookup WHERE LOBKey=@lkey
    IF @lid > 0 BEGIN
      -- Check if the LOB has any children that are alive
      IF EXISTS (SELECT * FROM LOB_LOB_LOBC l, link k 
	WHERE l.src_id=@lid AND l.dst_id=k.src_id AND k.src_cid='LOB' AND (link_type='LOBL' OR link_type='LOBC') AND k.deleted=0)
      BEGIN
        RAISERROR(55023, 0, 1)      
        -- Finish up transaction
        IF @tcount = 0 
          COMMIT TRAN gemsp_rmLOB  
        RETURN
      END 
      EXEC delete_LOB @lid   
    END
    
    DELETE FROM GEM_LOBLookup WHERE LOBKey=@lkey
    RAISERROR(55024, 0, 1, @loblabel, @ploblabel)

    FETCH NEXT FROM key_c INTO @lkey
  END
  DEALLOCATE key_c

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_rmLOB

