USE Object
GO

PRINT 'Creating procedure gemsp_rmLOBInst'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_rmLOBInst')
  DROP PROCEDURE gemsp_rmLOBInst
GO

CREATE PROCEDURE gemsp_rmLOBInst
@product NVARCHAR(128) = '',
@version NVARCHAR(128) = '',
@pproduct NVARCHAR(128) = '',
@pversion NVARCHAR(128) = '',
@pmatch NVARCHAR(255) = '',
@plobname NVARCHAR(255) = '',
@plobid ObjID = 0,
@name NVARCHAR(128) = ''
AS  
  RAISERROR('gemsp_rmLOBInst: @product=%s, @version=%d, @pproduct=%s, @pversion=%s, @pmatch=%s, @plobname=%s, @plobid=%d, @name=%s', 0, 1, 
	    @product, @version, @pproduct, @pversion, @pmatch, @plobname, @plobid, @name)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_rmLOBInst
  ELSE
    SAVE TRAN gemsp_rmLOBInst

  DECLARE @cid ClassID, @pcid ClassID
  SELECT @cid=cid FROM GEMLookupCID WHERE prodname=@product AND version=@version
  SELECT @pcid=cid FROM GEMLookupCID WHERE prodname=@pproduct AND version=@pversion
  IF @cid IS NULL 
    SELECT @cid=cid FROM obj_class WHERE cname=@product OR cname=@version
  IF @pcid IS NULL 
    SELECT @pcid=cid FROM obj_class WHERE cname=@pproduct OR cname=@pversion

  IF @cid IS NOT NULL AND @pcid IS NOT NULL BEGIN  
    DECLARE @lkey NVARCHAR(255)   
    IF @cid <> 'LOB'
      SELECT @lkey=a.LOBKey FROM GEM_InstFiltering a 
        JOIN GEM_LOBInstLookup b ON (a.LOBKey=b.LOBKey)
        WHERE a.cid=@cid AND b.cid=@pcid AND b.oid_where=@pmatch 
	      AND b.LOBName=@plobname AND b.lobid=@plobid
    ELSE
      SELECT @lkey=a.LOBKey FROM GEM_InstFiltering a 
        JOIN GEM_LOBInstLookup b ON (a.LOBKey=b.LOBKey)
        WHERE a.cid=@cid AND b.cid=@pcid AND b.oid_where=@pmatch 
	      AND b.LOBName=@plobname AND b.lobid=@plobid AND a.LOBKey LIKE '%,'+@name

    IF @lkey IS NULL BEGIN
      RAISERROR(55030, 0, 1, @pcid, @pmatch, @plobname, @plobid, @cid, @name) 
      -- Finish up transaction
      IF @tcount = 0 
        COMMIT TRAN gemsp_rmLOBInst
      RETURN -1
    END

    -- Delete all the resources of @cid that are under (@pcid, @pmatch, @plobname, @plobid) in LOB view
    CREATE TABLE #plobids (poid ObjID)
    CREATE TABLE #lobids (oid ObjID)
    DECLARE @lid ObjID, @lname ObjName

    -- Populate #lobids and #plobids with potential resources (to delete) with
    -- respective parents
    IF @plobid = 0 BEGIN
      IF @pmatch = '' 
        INSERT #plobids SELECT l.src_id FROM link l, LineOfBusiness_C b WHERE
          b.name=@plobname AND b.id=l.src_id AND l.src_cid='LOB' AND 
          l.deleted=0 AND l.link_type='LOBL' AND l.dst_cid=@pcid
      ELSE BEGIN
        CREATE TABLE #alloids ( oid ObjID )
        DECLARE @cname NVARCHAR(24), @sql NVARCHAR(1024)
        SELECT @cname=cname FROM obj_class WHERE cid=@pcid
        SET @sql = 'INSERT #alloids SELECT id FROM ' + @cname + '_V ' + @pmatch 
        EXEC (@sql)
     
        IF @plobname = ''
          INSERT #plobids SELECT l.src_id FROM link l 
            JOIN LineOfBusiness_C b ON (l.src_cid='LOB' AND b.id=l.src_id)
            JOIN #alloids o ON (l.dst_cid=@pcid AND l.dst_id=o.oid)
	    WHERE l.deleted=0 AND l.link_type='LOBL'
        ELSE
          INSERT #plobids SELECT l.src_id FROM link l 
            JOIN LineOfBusiness_C b ON (l.src_cid='LOB' AND b.id=l.src_id)
            JOIN #alloids o ON (l.dst_cid=@pcid AND l.dst_id=o.oid)
	    WHERE b.name=@plobname AND l.deleted=0 AND l.link_type='LOBL'   
      END
    END
    ELSE
      INSERT #plobids SELECT @plobid

    -- Assuming here that we are adding (and thus deleting now) resources to the parent 
    -- LOB only through auto-LOB processing
    IF @cid = 'LOB'
      INSERT #lobids SELECT id FROM LineOfBusiness_V WHERE name=@name
    ELSE
      INSERT #lobids SELECT src_id FROM link 
        WHERE dst_cid=@cid AND src_cid='LOB' AND link_type='LOBL' and deleted=0

    -- Delete only those resources that are under given parent in LOB view
    DECLARE del_c CURSOR FOR SELECT a.oid FROM #lobids a, #plobids b, link l
      WHERE l.src_id=b.poid AND l.dst_id=a.oid AND l.link_type='LOBC' AND l.deleted=0
    OPEN del_c
    FETCH NEXT FROM del_c INTO @lid

    WHILE @@FETCH_STATUS <> -1 BEGIN
      EXEC delete_LOB @lid    

      SELECT @lname=name from LineOfBusiness_V WHERE id=@lid
      RAISERROR(55031, 0, 1, @lname, @lid)     
      FETCH NEXT FROM del_c INTO @lid
    END
    DEALLOCATE del_c

    -- Deletes LOB Instance rule only if there are no other cids attached to it
    -- else, deletes the filtering conditions for that cid
    DELETE FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lkey
    IF NOT EXISTS (SELECT * FROM GEM_InstFiltering WHERE LOBKey=@lkey) BEGIN
      DELETE FROM GEM_LOBInstLookup WHERE LOBKey=@lkey
      RAISERROR(55032, 0, 1, @pcid, @pmatch, @plobname, @plobid)     
    END
    ELSE
      RAISERROR(55041, 0, 1, @cid, @pcid, @pmatch, @plobname, @plobid)     
  END
  ELSE IF @cid IS NULL
    RAISERROR(55029, 0, 1, @product, @version)    
  ELSE IF @pcid IS NULL
    RAISERROR(55029, 0, 1, @pproduct, @pversion)    

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_rmLOBInst

GO




