USE Object
GO

PRINT 'Creating procedure gemsp_updateLOBview'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_updateLOBview')
  DROP PROCEDURE gemsp_updateLOBview
GO

CREATE PROCEDURE gemsp_updateLOBview
@cid ClassID,
@id ObjID,
@lobkey NVARCHAR(255) = '',
@action NVARCHAR(32) = ''
AS
  RAISERROR('gemsp_updateLOBview: @cid=%s, @id=%d, @lobkey=%s, @action=%s', 0, 1, @cid, @id, @lobkey, @action)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_updateLOBview
  ELSE
    SAVE TRAN gemsp_updateLOBview

  IF @cid IS NULL OR @cid = '' BEGIN
    RAISERROR('CID passed in is NULL OR empty', 0, 1)
    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_updateLOBview
    RETURN
  END
  IF @id < 0 SET @id = 0

  DECLARE @cname NVARCHAR(24), @sql NVARCHAR(255), @disp NVARCHAR(128), @oid ObjID
  CREATE TABLE #oids (oid ObjID)
  SELECT @cname=cname FROM obj_class WHERE cid=@cid

  IF @id > 0 INSERT #oids SELECT @id
  ELSE BEGIN
    SELECT @sql = 'INSERT #oids SELECT id FROM ' + @cname + '_V WHERE deleted=0' 
    EXEC (@sql)
    DELETE ids FROM #oids ids JOIN link l ON (ids.oid=l.dst_id)
      WHERE l.dst_cid=@cid AND l.deleted=1
  END

  -- call gemsp_LOBs for all the instance to Update LOB view
  DECLARE @name NVARCHAR(255)
  DECLARE i_c CURSOR FOR SELECT oid FROM #oids
  OPEN i_c
  FETCH NEXT FROM i_c INTO @oid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    IF @lobkey = '' OR @lobkey IS NULL 
      EXEC gemsp_LOBs @cid, @oid, @disp OUT
    ELSE BEGIN
      IF @action = '' OR @action IS NULL SET @action = 'CREATE'
      EXEC gemsp_getAttribute @cid, @id, 'name', @name OUT
      EXEC gemsp_putInLOB @cid, @id, @name, @lobkey, @action, @disp OUT
    END

    RAISERROR(55028, 0, 1, @cid, @oid)
    FETCH NEXT FROM i_c INTO @oid
  END
  DEALLOCATE i_c

  RAISERROR('Disposition from gemsp_updateLOBview is: %s', 0, 1, @disp)
  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_updateLOBview
  
