USE Object
GO

PRINT 'Creating procedure gemsp_updateLOBviews'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_updateLOBviews')
  DROP PROCEDURE gemsp_updateLOBviews
GO

CREATE PROCEDURE gemsp_updateLOBviews
AS

  -- Updates LOBs for outstanding cids in GEM_UpdateLOBView
  -- If id = 0, updates all resources of that class
  -- Done only for those whose LOBKey and action are empty

  RAISERROR('gemsp_updateLOBviews: ', 0, 1)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_updateLOBviews
  ELSE
    SAVE TRAN gemsp_updateLOBviews

  DECLARE @cid ClassID, @oid ObjID
  SELECT @cid=cid, @oid=id FROM GEM_UpdateLOBView WHERE LOBKey=''

  WHILE @@ROWCOUNT > 0 BEGIN
    EXEC gemsp_updateLOBview @cid=@cid, @id=@oid
    DELETE FROM GEM_UpdateLOBView WHERE cid=@cid AND id=@oid AND LOBKey=''

    SELECT @cid=cid, @oid=id FROM GEM_UpdateLOBView WHERE LOBKey=''
  END

  -- Process orphan LOBs (if their parents exist)
  EXEC gemsp_processOrphanLOBs

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_updateLOBviews
  RETURN

