USE Object
GO

PRINT 'Creating procedure gemsp_viewLOBInfo'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_viewLOBInfo')
  DROP PROCEDURE gemsp_viewLOBInfo
GO

CREATE PROCEDURE gemsp_viewLOBInfo
@product NVARCHAR(128),
@version NVARCHAR(128),
@pname NVARCHAR(255) = ''
AS  

  RAISERROR('gemsp_viewLOBInfo: @product=%s, @version=%s, @pname=%s', 0, 1, @product, @version, @pname)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_viewLOBInfo
  ELSE
    SAVE TRAN gemsp_viewLOBInfo

  DECLARE @cid ClassID
  SELECT @cid=cid FROM GEMLookupCID WHERE prodname=@product AND version=@version
  IF @cid IS NULL 
    SELECT @cid=cid FROM obj_class WHERE cname=@product OR cname=@version

  IF @cid IS NOT NULL BEGIN  
    IF @pname IS NOT NULL AND @pname <> '' BEGIN
      RAISERROR(55033, 0, 1)
      SELECT 'TBSM: '+CAST(a.src_id AS NVARCHAR)+', '+RTRIM(b.name)+', '+RTRIM(d.name) 
        FROM link a 
        JOIN LineOfBusiness_C b ON (b.id=a.src_id)
        JOIN link c ON (a.src_id=c.dst_id)
        JOIN LineOfBusiness_C d ON (c.src_id=d.id)
        WHERE b.name=@pname AND a.src_cid='LOB' AND 
          a.deleted=0 AND a.link_type='LOBL' AND a.dst_cid=@cid AND
	  c.dst_cid='LOB' and c.link_type='LOBC' and c.deleted=0
    END
    ELSE BEGIN
      DECLARE @sql NVARCHAR(1028), @cname NVARCHAR(24)
      SELECT @cname=cname FROM obj_class WHERE cid=@cid
      RAISERROR(55038, 0, 1, @cid, @cid)
      SET @sql = 'SELECT ''TBSM: ''+CAST(c.id AS NVARCHAR)+'', ''+RTRIM(c.name)+'', ''+CAST(a.src_id AS NVARCHAR)+'', ''+RTRIM(b.name)
        FROM link a
        JOIN LineOfBusiness_C b ON (b.id=a.src_id)
        JOIN ' + @cname + '_C c ON (a.dst_id=c.id)
        WHERE a.dst_cid=''' + @cid + ''' AND a.link_type=''LOBL'' AND a.deleted=0	 
        ORDER BY c.id'
      EXEC (@sql)
    END

    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_addpvToLOBInst
    RETURN
  END
  ELSE 
    RAISERROR(55029, 0, 1, @product, @version)    

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_addpvToLOBInst

