USE Object
GO

PRINT 'Creating procedure gemsp_where_inLOB'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_where_inLOB')
  DROP PROCEDURE gemsp_where_inLOB
GO

CREATE PROCEDURE gemsp_where_inLOB
@cid ClassID,
@oid ObjID
AS
  RAISERROR('gemsp_where_inLOB: @cid=%s, @oid=%d', 0, 1, @cid, @oid)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_where_inLOB
  ELSE
    SAVE TRAN gemsp_where_inLOB

  DECLARE @lobid ObjID, @del TINYINT
  DECLARE gem_lob_c CURSOR FOR SELECT src_id, deleted FROM link WHERE dst_cid=@cid AND dst_id=@oid AND link_type='LOBL' AND src_cid='LOB'
  OPEN gem_lob_c
  FETCH NEXT FROM gem_lob_c INTO @lobid, @del

  IF @lobid IS NULL
    RAISERROR('%s (%d) IS NOT associated with any LOB', 0, 1, @cid, @oid)

  WHILE @@FETCH_STATUS <> -1 BEGIN
    DECLARE @lobname NVARCHAR(255), @plobid ObjID, @pcid ClassID, @plobname NVARCHAR(255)
    SELECT @lobname=name FROM LineOfBusiness_C WHERE id=@lobid

    RAISERROR('************************************************************************', 0, 1)
    RAISERROR('', 0, 1)
    IF @del = 0
      RAISERROR('%s (%d) IS associated with "%s" (%d) IN LOB', 0, 1, @cid, @oid, @lobname, @lobid)
    ELSE
      RAISERROR('%s (%d) WAS associated with "%s" (%d) IN LOB', 0, 1, @cid, @oid, @lobname, @lobid)
   
    SELECT @plobid=src_id, @pcid=src_cid, @del=deleted FROM link WHERE dst_id=@lobid AND dst_cid='LOB' AND link_type='LOBC'
    SELECT @plobname=name FROM LineOfBusiness_C WHERE id=@plobid

    RAISERROR('', 0, 1)
    WHILE @pcid = 'LOB' BEGIN
      IF @del = 0
        RAISERROR('"%s" (%d) IS under LOB "%s" (%d)', 0, 1, @lobname, @lobid, @plobname, @plobid)
      ELSE
        RAISERROR('"%s" (%d) WAS under LOB "%s" (%d)', 0, 1, @lobname, @lobid, @plobname, @plobid)
      
      SELECT @lobid = @plobid
      SELECT @lobname = @plobname
      SELECT @plobid=src_id, @pcid=src_cid, @del=deleted FROM link WHERE dst_id=@lobid AND dst_cid='LOB' AND link_type='LOBC'  
      SELECT @plobname=name FROM LineOfBusiness_C WHERE id=@plobid
    END

    IF @pcid = 'LOBC' BEGIN
      IF @del = 0
        RAISERROR('"%s" (%d) IS under LOBC "%s" (%d)', 0, 1, @lobname, @lobid, @plobname, @plobid)
      ELSE
        RAISERROR('"%s" (%d) WAS under LOBC "%s" (%d)', 0, 1, @lobname, @lobid, @plobname, @plobid)
    END
    ELSE
      RAISERROR('Error: "%s" IS NOT under LOBC (Line Of Business Container) but %s', 0, 1, @lobname, @pcid)
    RAISERROR('', 0, 1)
   
    FETCH NEXT FROM gem_lob_c INTO @lobid, @del
  END
  DEALLOCATE gem_lob_c

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_where_inLOB
