-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/delete_obj.sqv   1.0   Jul 22 2001 11:27:04   BRENT  $
--
-- delete_obj.sql
--
-- Module Description
--   Defines the delete_obj stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- delete_obj
PRINT 'Creating procedure delete_obj'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'delete_obj' AND type = 'P')
  DROP PROCEDURE delete_obj
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/delete_obj.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 22 2001 11:27:04  $
-- $Author:   BRENT  $
-- Base procedure for deleting objects. Most of the delete_<CID> procedures use this procedure
-- for most/all of the work.
CREATE PROCEDURE delete_obj
  @cid  ClassID,
  @id   ObjID
AS
--  DECLARE @time NVARCHAR(40)
--  SELECT @time = CONVERT(NVARCHAR(40), GETDATE(), 13)
--  RAISERROR('%s: delete_obj "%s", %d started', 0, 1, @time, @cid, @id)
  -- TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount > 0
    SAVE TRAN delete_obj
  ELSE
    BEGIN TRAN delete_obj
  -- Notify all logically linked objects that this object has been deleted
  IF EXISTS (SELECT * FROM derived_obj_link WHERE link_type = 'LOBL' AND dst_cid = @cid)
    EXEC asisp_notifylobsofdelete @cid, @id
 
  -- Perform the DELETE
  DECLARE @id_decl NVARCHAR(80), @err INT
  SELECT @id_decl = 'DECLARE @cid ClassID, @id ObjID SELECT @cid='''+RTRIM(@cid)+''', @id='+CONVERT(NVARCHAR(9),@id)
  EXEC
  (@id_decl + "
    -- Build TABLE of objects to be deleted
    CREATE TABLE #deleted_objs
    (
      cid         ClassID,    -- cid of descendent object to be deleted
      id          ObjID,      -- id of descendent object to be deleted
      generation  INT,        -- generation of this object FROM original object
      cno         SMALLINT,   -- obj_class.cno corresponding to cid
      propagation TINYINT     -- obj_class.flag_propagation setting
    )
    DECLARE @generation INT
    SELECT @generation = 0
    INSERT #deleted_objs SELECT @cid, @id, @generation, cno, flag_propagation
    FROM obj_class
    WHERE cid = @cid
    WHILE @@ROWCOUNT > 0
    BEGIN
      SELECT @generation = @generation + 1
      INSERT #deleted_objs SELECT dst_cid, dst_id, @generation, o.cno, o.flag_propagation
      FROM containment, #deleted_objs d, obj_class o
      WHERE
        generation = @generation-1 AND src_cid=d.cid AND src_id=id AND deleted=0
        AND o.cid = dst_cid 
    END
--    SELECT * FROM #deleted_objs
    -- Mark all containment records FOR the deleted objects
    UPDATE containment
      SET deleted=1 FROM #deleted_objs WHERE dst_cid=cid AND dst_id=id AND deleted=0
    UPDATE containment
      SET deleted=1 FROM #deleted_objs WHERE src_cid=cid AND src_id=id AND deleted=0
    -- Mark all link records FOR the deleted objects
    UPDATE link
      SET deleted=1 FROM #deleted_objs WHERE dst_cid=cid AND dst_id=id AND deleted=0
    UPDATE link
      SET deleted=1 FROM #deleted_objs WHERE src_cid=cid AND src_id=id AND deleted=0

    -- BEGIN gem processing
    IF EXISTS (SELECT * FROM sysobjects WHERE name = 'GEM_IDLookup' AND type = 'U')
      DELETE g FROM GEM_IDLookup AS g JOIN #deleted_objs AS d ON d.cid = g.CID AND d.id = g.IID
    -- gem lob processing
    IF EXISTS (SELECT * FROM sysobjects WHERE name = 'GEM_LOBLookup' AND type = 'U')
    BEGIN
      UPDATE GEM_LOBLookup SET LOBID = 0 FROM #deleted_objs d WHERE d.cid = 'LOB' AND d.id = LOBID 
      UPDATE GEM_LOBLookup SET parentLOBID = 0 FROM #deleted_objs d WHERE d.cid = 'LOB' AND d.id = parentLOBID
    END
    -- END gem processing

    -- BEGIN CommonListener processing
    -- Delete from CommonListener Cache
    DELETE g FROM CL_IDCache AS g JOIN #deleted_objs AS d ON d.cid = g.cid AND d.id = g.oid
 
    -- Mark deleted flag in <cname>_V table
    DECLARE @clsql NVARCHAR(1024), @clcid ClassID, @clcname NVARCHAR(24)
    DECLARE cldel_c CURSOR FOR SELECT DISTINCT cid FROM #deleted_objs
    OPEN cldel_c
    FETCH NEXT FROM cldel_c INTO @clcid

    WHILE @@FETCH_STATUS <> -1 BEGIN
      SELECT @clcname=cname FROM obj_class WHERE cid=@clcid
      SET @clsql = 'UPDATE v SET v.deleted=1 FROM ' + @clcname + '_V v' + 
                   ' JOIN #deleted_objs d ON (d.cid=''' + @clcid + ''' AND v.id=d.id)' 
      EXEC (@clsql)
      FETCH NEXT FROM cldel_c INTO @clcid
    END
    DEALLOCATE cldel_c
    -- END CommonListener processing

    -- Send notification to the dispatcher of all propagation objects that have just been deleted
    SELECT 
      type = et.type,
      fields=
        CONVERT(NVARCHAR(9), isnull(d.cno, 0)) + '|' +
        CONVERT(NVARCHAR(9), isnull(d.id, 0)) + '|' +
        CASE generation WHEN 0 THEN '1' ELSE '0' END
    INTO #ObjectDeletedEvent
    FROM 
      #deleted_objs d,
      EventType et
    WHERE
      d.propagation = 1
      AND et.name = 'ObjectDeleted'
    INSERT PendingEvent(type, fields, detail) SELECT type, fields, '' FROM #ObjectDeletedEvent
    INSERT PendingNotification(type, fields, detail) SELECT type, fields, '' FROM #ObjectDeletedEvent
  ")
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('delete_obj ''%s'', %d: Could NOT DELETE due to database error %d, aborting', 11, 1, @cid, @id, @err)
    ROLLBACK TRAN delete_obj
    RETURN -30000
  END
  -- Decrement the child COUNT IN our parent
  DECLARE @sql NVARCHAR(255)
  SELECT
    @sql = 'UPDATE ' + cname + '_C SET nchild = nchild - 1 WHERE id = ' + CONVERT(NVARCHAR(9), src_id)
  FROM 
    obj_class c, 
    link l,
    link_type lt
  WHERE
    l.dst_cid = @cid
    AND l.dst_id = @id
    AND l.link_type = lt.link_type
    AND lt.flags & 1 = 1
    AND c.cid = l.src_cid
  IF @@ROWCOUNT = 1
  BEGIN
    EXEC(@sql)
    IF @@ERROR != 0
    BEGIN
      RAISERROR('delete_obj ''%s'', %d: could NOT decrement child COUNT IN parent', 11, 1, @cid, @id)
      ROLLBACK TRAN delete_obj
      RETURN -30000
    END
  END
  IF @tcount = 0
    COMMIT TRAN delete_obj
GO
