SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[gemsp_buildLinkRule]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[gemsp_buildLinkRule]
GO




/* Given the relevant slots from a connection change event, returns the information used to locate the destination 
   resource candidates.                                                                                               
 */

CREATE PROCEDURE gemsp_buildLinkRule           
		@linkname NVARCHAR(128),
		@src_direction TINYINT = 0,    
		@dest_subsource NVARCHAR(128),
		@dest_origin NVARCHAR(255),
		@dest_suborigin NVARCHAR(255),
                @dest_cid ClassID OUTPUT,
		@dest_direction TINYINT OUTPUT,    
                @dest_keyname NVARCHAR(32) OUTPUT,
                @dest_condition NVARCHAR(511) OUTPUT
AS

RAISERROR( 'gemsp_buildLinkRule: @linkname=%s, @src_direction=%d, @dest_subsource=%s,
           @dest_origin=%s, @dest_suborigin=%s', 
           0, 1,
           @linkname, @src_direction, @dest_subsource, @dest_origin, @dest_suborigin )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN gemsp_buildLinkRule                   
ELSE
  SAVE TRAN gemsp_buildLinkRule                   


DECLARE @productName   NVARCHAR(128)
DECLARE @version       NVARCHAR(128)

DECLARE @cid           ClassID

DECLARE @return        INT

SET @return = 0

-- must have at least the destination origin to construct a rule 
IF @dest_origin IS NOT NULL AND LTRIM( @dest_origin ) <> '' BEGIN

  SET @dest_direction = CASE @src_direction WHEN 1 THEN 2
                                            WHEN 2 THEN 1
                                            ELSE        @src_direction
                        END

  IF @dest_subsource IS NOT NULL AND LTRIM( @dest_subsource ) <> '' BEGIN  -- if we have a triplet, go get the cid

    EXEC gemsp_getProductVersion @dest_subsource, @productName OUTPUT, @version OUTPUT

    SELECT @dest_cid = cid FROM GEMLookupCID WHERE prodname = @productName AND version = @version

    -- look in CL_CIDLinks table to see if this link is supported using the destination direction
    IF NOT EXISTS (SELECT * FROM CL_CIDLinks WHERE cid = @dest_cid AND @linkname = name
                                             AND direction = @dest_direction ) 
      SET @return = 2

  END

  SET @dest_keyname = 'KeyOS'

  -- massage the key value to search for based on the parameters passed
  SET @dest_condition = '%' + @dest_origin + '%|%'
  IF @dest_suborigin IS NOT NULL AND LTRIM( @dest_suborigin ) <> ''
    SET @dest_condition = @dest_condition + @dest_suborigin + '%'

END
ELSE
  SET @return = 1 

-- commit transaction
IF @tcount = 0                           
  COMMIT TRAN gemsp_buildLinkRule

RETURN @return




GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

