SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[gemsp_processAPMConnectionChangeMsg]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[gemsp_processAPMConnectionChangeMsg]
GO




CREATE PROCEDURE gemsp_processAPMConnectionChangeMsg
		@class  NVARCHAR(128)='',
		@hostname  NVARCHAR(128)='',
		@origin NVARCHAR(255)='',
		@suborigin NVARCHAR(255)='',
		@source NVARCHAR(128)='',
		@subsource NVARCHAR(128)='',
		@linkname NVARCHAR(128)='',
		@direction NVARCHAR(2),
		@dest_source NVARCHAR(128)='',
		@dest_subsource NVARCHAR(128)='',
		@dest_origin NVARCHAR(255)='',
		@dest_suborigin NVARCHAR(255)='',
		@dest_hostname NVARCHAR(128)='',
		@change_type NVARCHAR(2) 
AS

RAISERROR( 'gemsp_processAPMConnectionChangeMsg: @class=%s, @hostname=%s, @origin=%s,
	   @suborigin=%s, @source=%s, @subsource=%s, @linkname=%s, @direction=%s,
	   @dest_source=%s, @dest_subsource=%s, @dest_origin=%s, @dest_suborigin=%s,  
	   @dest_hostname=%s, @change_type=%s',
           0, 1,
	   @class, @hostname, @origin, @suborigin, @source, @subsource, @linkname, @direction,
	   @dest_source, @dest_subsource, @dest_origin, @dest_suborigin, @dest_hostname, @change_type )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN gemsp_processAPMConnectionChange
ELSE
  SAVE TRAN gemsp_processAPMConnectionChange


DECLARE @disposition NVARCHAR(255)
SET @disposition = ''

DECLARE @productName NVARCHAR(128)
DECLARE @version NVARCHAR(128)

DECLARE @cid ClassID
DECLARE @oid ObjID

DECLARE @dest_cid        ClassID
DECLARE @src_direction   TINYINT
DECLARE @dest_direction  TINYINT
DECLARE @dest_keyname    NVARCHAR(32) 
DECLARE @dest_condition  NVARCHAR(511)  

DECLARE @return            INT

SET @return = 0

-- get product/version from triplet

EXEC gemsp_getProductVersion @subsource, @productName OUTPUT, @version OUTPUT

SELECT @cid = cid FROM GEMLookupCID WHERE prodname = @productName AND version = @version

IF @cid IS NOT NULL BEGIN

  SET @src_direction = CASE WHEN @direction IS NULL       THEN '0'                   
                            WHEN LTRIM( @direction ) = '' THEN '0'                   
                            ELSE                           @direction
                       END

  -- look in CL_CIDLinks table to see if this relationship is supported
  IF EXISTS (SELECT * FROM CL_CIDLinks WHERE cid = @cid AND name = @linkname
                                       AND direction = @src_direction ) BEGIN

    -- look FOR an existing instance of this software component
    IF SUBSTRING(@origin, len(@origin), 1) <> ';'
      SET @origin = @origin + ';'

    IF SUBSTRING(@suborigin, len(@suborigin), 1) <> ';'
      SET @suborigin = @suborigin + ';'

    SELECT @oid = IID FROM GEM_IDLookup
           WHERE CID = @cid AND key_name = 'KeyOS' AND key_value = @origin + '|' + @suborigin

    IF (@oid IS NULL) BEGIN
      SELECT @disposition = 'local resource NOT found' 
    END
    ELSE BEGIN
      -- Construct a link rule from the event data
      EXEC @return = gemsp_buildLinkRule @linkname, @direction, @dest_subsource, @dest_origin, @dest_suborigin,
                                         @dest_cid OUTPUT, @dest_direction OUTPUT, 
                                         @dest_keyname OUTPUT, @dest_condition OUTPUT

      IF ( @return = 0 ) BEGIN    

        -- call common stored procedure to add/delete link rules and links. Inputs are provided in a temporary
        -- table to accommodate bulk processing.

        CREATE TABLE #linkEventData( cid        	ClassID,
                                     iid        	ObjID,
                                     dest_cid           ClassID NULL,
                                     linkname           NVARCHAR(128),
                                     src_direction      TINYINT, 
                                     dest_direction     TINYINT, 
                                     dest_keyname       NVARCHAR(32),
                                     dest_condition     NVARCHAR(512),
                                     enabled            BIT,
                                     excl               BIT,
                                     priority           INT,
                                     action             INT )

        INSERT INTO #linkEventData VALUES( @cid,
                                           @oid,
                                           @dest_cid,
                                           @linkname,
                                           @src_direction,
                                           @dest_direction,
                                           @dest_keyname,
                                           @dest_condition,
                                           1,
                                           0,
                                           NULL,
                                           CAST( @change_type AS INT ) )

        EXEC @return = cl_processLinks @disposition OUTPUT

        IF @return < 0 BEGIN
          ROLLBACK TRAN gemsp_processAPMConnectionChange   
        END
        ELSE
          DROP TABLE #linkEventData

      END
      ELSE BEGIN
        SELECT @disposition = CASE WHEN @return = 1 THEN 'unable to build a link rule - no destination origin'
                                   WHEN @return = 2 THEN 'for destination CID=' + @dest_cid + ': linkname=' + 
                                            @linkname + ', destination direction=' + 
                                            CAST( @dest_direction AS NVARCHAR(10) ) + ' is not supported'
                                   ELSE     'unable to build a link rule'
                              END
      END                          
    END
  END
  ELSE BEGIN
    SELECT @disposition = 'for CID=' + @cid + ': linkname=' + @linkname + 
                          ', direction=' + @direction + ' is not supported' 
  END
END
ELSE BEGIN
  SELECT @disposition = 'product/version NOT found for subsource=' + @subsource 
END

IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'GEM_APMCCMsgs')
  CREATE TABLE GEM_APMCCMsgs(
    class  NVARCHAR(255) NULL,
    hostname  NVARCHAR(255) NULL,
    origin NVARCHAR(255) NULL,
    suborigin NVARCHAR(255) NULL,
    source NVARCHAR(255) NULL,
    subsource NVARCHAR(255) NULL,
    relationship NVARCHAR(128) NULL,
    direction NVARCHAR(2) NULL,
    remote_source NVARCHAR(128) NULL,
    remote_subsource NVARCHAR(128) NULL,
    remote_origin NVARCHAR(255) NULL,
    remote_suborigin NVARCHAR(255) NULL,
    remote_hostname NVARCHAR(128) NULL,
    change_type NVARCHAR(2) NULL,
    disposition NVARCHAR(255) NULL,
    time DATETIME NULL)

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'GEM_Options')
  IF EXISTS (SELECT * FROM GEM_Options WHERE _Option = 'trace events' AND _Value = 'yes') BEGIN
    INSERT INTO GEM_APMCCMsgs (class, hostname, origin, suborigin, source, subsource, relationship, direction,
                               remote_source, remote_subsource, remote_origin, remote_suborigin, remote_hostname,
                               change_type, disposition, time) 
  
      VALUES (@class, 
	@hostname ,
	@origin,
	@suborigin,
	@source,
	@subsource,
	@linkname,
	@direction,
	@dest_source,
	@dest_subsource,
        @dest_origin,
	@dest_suborigin,
	@dest_hostname,
	@change_type,
        @disposition,
	getdate()
		)
  END


-- commit transaction
IF @tcount = 0 AND @return !< 0 
  COMMIT TRAN gemsp_processAPMConnectionChange   



GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

