-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/imObjectList_diff.sqv   1.0   Jun 18 2001 16:43:30   lin  $
--
-- imObjectList_diff.sql
--
-- Module Description
--   Differ two table of version view
--
USE Object
GO

-----------------------------------------------------------------------------
-- sp_imObjectList_diff
PRINT 'Creating PROCEDURE sp_imObjectList_diff'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'sp_imObjectList_diff' AND type = 'P')
  DROP PROCEDURE sp_imObjectList_diff
GO

-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/imObjectList_diff.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jun 18 2001 16:43:30  $
-- $Author:   lin  $
CREATE PROCEDURE sp_imObjectList_diff 
  @firstver           INT,
  @lastver            INT,
  @orderbyclause      NVARCHAR(255) = NULL,
  @debug              INT = 0
AS
  SET NOCOUNT ON

  -- DECLARE block
  DECLARE @startdate datetime
  DECLARE @nowdate   datetime
  DECLARE @stepdate  datetime
  SELECT @nowdate    = GETDATE()
  SELECT @stepdate   = @nowdate
  SELECT @startdate  = @stepdate
  DECLARE @stepname  VARCHAR(20)
  SELECT @stepname   = NULL
  DECLARE @steptime  INT
  DECLARE @totaltime INT

  -- Timing header
  IF @debug >= 1
  BEGIN
    RAISERROR( 'Timings: Stepname             (ms_step/ms_total)', 0, 1)
  END

  CREATE TABLE #imObjectList_first
  (
    versionRequested  INT,
    host              NVARCHAR(64),           
    path              NVARCHAR(255),         
    name              NVARCHAR(255),        
    objectType        NVARCHAR(2), 
    checksum          INT,  
    blockcount        INT,     
    snapshotver       INT,   
    confirmed         INT,    
    deleted           INT      
  )

  CREATE TABLE #imObjectList_last
  (
    versionRequested  INT,
    host              NVARCHAR(64),           
    path              NVARCHAR(255),         
    name              NVARCHAR(255),        
    objectType        NVARCHAR(2), 
    checksum          INT,  
    blockcount        INT,     
    snapshotver       INT,   
    confirmed         INT,    
    deleted           INT      
  )

  CREATE TABLE #imObjectList_result
  (
    host              NVARCHAR(64),           
    context           NVARCHAR(128),
    path              NVARCHAR(255),         
    name              NVARCHAR(255),        
    objectType        NVARCHAR(2), 
    checksum          INT,  
    blockcount        INT,     
    base_schema_ver   INT,
    schema_ver        INT,
    crdate            DATETIME,
    mddate            DATETIME,
    snapshotver       INT,   
    confirmed         INT,    
    deleted           INT,      
    difference        NVARCHAR(20)      
  )

  -- Timing of Step Table creation
  IF @debug >= 1
  BEGIN
    SELECT @stepname   = 'Table creation'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime   = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime  = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  INSERT INTO #imObjectList_first
  SELECT * FROM imObjectList_versionView WHERE versionRequested = @firstver

  -- Timing of Step #imObjectList_first Table insersion
  IF @debug >= 1
  BEGIN
    SELECT @stepname   = '#imObjectList_first Table Insersion'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime   = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime  = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  INSERT INTO #imObjectList_last
  SELECT * FROM imObjectList_versionView WHERE versionRequested = @lastver

  -- Timing of Step #imObjectList_last Table insersion
  IF @debug >= 1
  BEGIN
    SELECT @stepname   = '#imObjectList_last Table Insersion'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime   = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime  = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  TRUNCATE TABLE #imObjectList_result

  -- Right Outer Join for Add
  INSERT INTO #imObjectList_result
  SELECT 
    l.host, 
    s.context,
    l.path, 
    l.name,  
    l.objectType, 
    l.checksum,  
    l.blockcount,     
    s.base_schema_ver,
    s.schema_ver,
    s.crdate,
    s.mddate,
    l.snapshotver,   
    l.confirmed,    
    l.deleted,
    'Add'
  FROM #imObjectList_first as f
  RIGHT OUTER JOIN #imObjectList_last as l     -- pickup newly added
    ON f.name = l.name
    AND f.path = l.path
    AND f.objectType = l.objectType
  INNER JOIN imObjectList as s                 -- Pickup l's snapshot information for the SELECT
    ON l.name = s.name
    AND l.snapshotver = s.snapshotver
    -- Match host name 'real name' to 'real name' or NULL to NULL
    AND ((l.host = s.host) OR (l.host IS NULL AND s.host IS NULL))  
    AND l.path = s.path
    AND l.objectType = s.objectType
  -- If the first version has NULL columns, other than host (which can be NULL)
  WHERE f.name is NULL 
    AND l.deleted = 0                          -- And the last record is not a delete record

  -- Timing of Step Right Outer Join
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Right Outer Join'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  -- Inner Join for Delete, Add after Delete
  INSERT INTO #imObjectList_result
  SELECT 
    l.host, 
    s.context,
    l.path, 
    l.name,  
    l.objectType, 
    l.checksum,  
    l.blockcount,     
    s.base_schema_ver,
    s.schema_ver,
    s.crdate,
    s.mddate,
    l.snapshotver,   
    l.confirmed,    
    l.deleted,
    CASE
      WHEN (f.deleted = 0 AND l.deleted = 1) THEN 'Delete'
      WHEN (f.deleted = 1 AND l.deleted = 0) THEN 'Add'
      WHEN (f.deleted = 0 AND l.deleted = 0 AND (f.checksum != l.checksum OR f.blockcount != l.blockcount))
        THEN 'Update'
    END
  FROM #imObjectList_first as f
  INNER JOIN #imObjectList_last as l
    ON f.name = l.name
    AND f.path = l.path
    AND f.objectType = l.objectType
  INNER JOIN imObjectList as s
    ON l.name = s.name
    -- Match host name 'real name' to 'real name' or NULL to NULL
    AND ((l.host = s.host) OR (l.host IS NULL AND s.host IS NULL))  
    AND l.path = s.path
    AND l.objectType = s.objectType
  WHERE l.snapshotver = s.snapshotver
    AND (
      (f.deleted = 0 AND l.deleted = 1) OR
      (f.deleted = 1 AND l.deleted = 0) OR
      (f.deleted = 0 AND l.deleted = 0 AND (f.checksum != l.checksum OR f.blockcount != l.blockcount))
    )

  -- Timing of Step Inner Join
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Inner Join'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

  DROP TABLE #imObjectList_first
  DROP TABLE #imObjectList_last

  -- Populate Attributes in #imObjectList_result
  DECLARE @sql NVARCHAR(1024)
  SELECT @sql="
  SELECT 
    s.host,  -- this might be a NULL 
    s.context, 
    s.path, 
    s.name, 
    s.objectType, 
    s.checksum, 
    s.blockcount, 
    s.base_schema_ver, 
    s.schema_ver,
    CONVERT(NVARCHAR(13), s.crdate, 107) as createDate, 
    CONVERT(NVARCHAR(8), s.crdate, 108) as createTime, 
    CONVERT(NVARCHAR(13), s.mddate, 107) as modifiedDate, 
    CONVERT(NVARCHAR(8), s.mddate, 108) as modifiedTime, 
    s.snapshotver,
    s.confirmed,
    s.deleted, 
    v.type as snapshotType,
    ''''+v.name+'''' as snapshotName,
    v.build as snapshotBuild,
    CONVERT(NVARCHAR(13), v.date, 107) as snapshotDate, 
    CONVERT(NVARCHAR(8), v.date, 108) as snapshotTime,
    s.difference
  FROM #imObjectList_result as s 
  INNER JOIN imSnapshotHistory as v
    ON s.snapshotver = v.version
  WHERE
    v.state = 'Started'
  "+@orderbyclause

  --PRINT(@sql)
  EXEC(@sql)

  -- Timing of Step Report Generation
  IF @debug >= 1
  BEGIN
    SELECT @stepname = 'Report Generation'
    SELECT @nowdate    = GETDATE()
    SELECT @steptime = DATEDIFF( ms, @stepdate, @nowdate)
    SELECT @totaltime = DATEDIFF( ms, @startdate, @nowdate)
    RAISERROR( 'Timings: %-20s (%7d/%8d)', 0, 1, @stepname, @steptime, @totaltime)
    SELECT @stepdate   = @nowdate
  END

