-- $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/imObjectList_versionView.sqv   1.0   Jun 18 2001 16:44:12   lin  $
--
-- imObjectList_versionView.sql
--
-- Module Description
--   Create a view to retrieve a version of object snapshot
--
USE Object
GO

-----------------------------------------------------------------------------
-- imObjectList_versionView
PRINT 'Creating VIEW imObjectList_versionView'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'imObjectList_versionView' AND type = 'V')
  DROP VIEW imObjectList_versionView
GO


-- $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/DB/SQL/imObjectList_versionView.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jun 18 2001 16:44:12  $
-- $Author:   lin  $
--   Create a view to retrieve the latest version of object snapshot

CREATE VIEW imObjectList_versionView ( 
  versionRequested,
  host,           
  path,         
  name,        
  objectType, 
  checksum,  
  blockcount,     
  snapshotver,   
  confirmed,    
  deleted      
  )
AS
  SELECT DISTINCT
    h.version,                              -- version requested
    o1.host,           
    o1.path,         
    o1.name,        
    o1.objectType, 
    o1.checksum,  
    o1.blockcount,     
    o1.snapshotver,   
    o1.confirmed,    
    o1.deleted      
  FROM imSnapshotHistory AS h,
    imObjectList as o1
  WHERE o1.snapshotver <= h.version         -- version is possible match
    AND NOT EXISTS (                        -- when a better match does not exist
      SELECT * FROM imObjectList AS o2      -- better(second) match detail:
      WHERE o2.snapshotver <= h.version     -- second possible match

        AND ((o2.host = o1.host)                             -- and second match is the same as the first host
         OR ( o1.host IS NOT NULL AND o2.host IS NULL )      -- or the second match is null (a wildcard) and the first host is a real host - wildcard takes precedence over real host when same version 
         OR ( o1.host IS NULL AND o2.host IS NULL))          -- or the second match and the first match are both nulls (first case redone for IS syntax)

        AND o2.path = o1.path 
        AND o2.name = o1.name
        AND o2.objectType = o1.objectType 
        AND (h.version - o2.snapshotver) < (h.version - o1.snapshotver)
                                            -- second match is closer to version requested than the first match
    )

